@@ -52,7 +52,7 @@
 #include "cpl_worker_thread_pool.h"
 #include "gdal.h"
 
-CPL_CVSID("$Id: gdalgrid.cpp 9ff327806cd64df6d73a6c91f92d12ca0c5e07df 2018-04-07 20:25:06 +0200 Even Rouault $")
+CPL_CVSID("$Id: gdalgrid.cpp 949d180a26e40650413703b2b73b213137b171e4 2018-06-25 13:43:52 +0200 Even Rouault $")
 
 constexpr double TO_RADIANS = M_PI / 180.0;
 
@@ -1414,7 +1414,8 @@ GDALGridDataMetricAverageDistancePts( const void *poOptionsIn, GUInt32 nPoints,
  * of a Delaunay triangulation the point is, and by doing interpolation from
  * its barycentric coordinates within the triangle.
  * If the point is not in any triangle, depending on the radius, the
- * algorithm will use the value of the nearest point or the nodata value.
+ * algorithm will use the value of the nearest point (radius != 0),
+ * or the nodata value (radius == 0)
  *
  * @param poOptionsIn Algorithm parameters. This should point to
  * GDALGridLinearOptions object.
@@ -1477,6 +1478,13 @@ GDALGridLinear( const void *poOptionsIn, GUInt32 nPoints,
     }
     else
     {
+        if( nOutputFacetIdx >= 0 )
+        {
+            // Also reuse this failed output facet, when valid, as seed for
+            // next search.
+            psExtraParams->nInitialFacetIdx = nOutputFacetIdx;
+        }
+
         const GDALGridLinearOptions *const poOptions =
             static_cast<const GDALGridLinearOptions *>(poOptionsIn);
         const double dfRadius = poOptions->dfRadius;
