@@ -38,15 +38,19 @@
 #define E00ReadClose        GDALE00GRIDReadClose
 #define E00ReadNextLine     GDALE00GRIDReadNextLine
 #define E00ReadRewind       GDALE00GRIDReadRewind
+
+CPL_CVSID("$Id: e00griddataset.cpp f466372a2a145bebc6c1b3c5bc26055e4de26328 2018-09-03 19:46:58 +0200 Even Rouault $")
+
+#undef NULL
+#define NULL nullptr
 #include "e00read.c"
+#undef NULL
 
-static const int E00_INT_SIZE = 10;
-// #define E00_INT14_SIZE  14
-static const int E00_FLOAT_SIZE = 14;
-static const int E00_DOUBLE_SIZE = 21;
-static const int VALS_PER_LINE = 5;
-
-CPL_CVSID("$Id: e00griddataset.cpp 36501 2016-11-25 14:09:24Z rouault $");
+constexpr int E00_INT_SIZE = 10;
+// constexpr int E00_INT14_SIZE = 14;
+constexpr int E00_FLOAT_SIZE = 14;
+constexpr int E00_DOUBLE_SIZE = 21;
+constexpr int VALS_PER_LINE = 5;
 
 /* g++ -fPIC -Wall -g frmts/e00grid/e00griddataset.cpp -shared -o gdal_E00GRID.so -Iport -Igcore -Iogr -L. -lgdal */
 
@@ -103,11 +107,11 @@ class E00GRIDDataset : public GDALPamDataset
     static void        Rewind(void* ptr);
 
   public:
-                 E00GRIDDataset();
-    virtual     ~E00GRIDDataset();
+    E00GRIDDataset();
+    ~E00GRIDDataset() override;
 
-    virtual CPLErr GetGeoTransform( double * ) override;
-    virtual const char* GetProjectionRef() override;
+    CPLErr GetGeoTransform( double * ) override;
+    const char* GetProjectionRef() override;
 
     static GDALDataset *Open( GDALOpenInfo * );
     static int          Identify( GDALOpenInfo * );
@@ -124,17 +128,17 @@ class E00GRIDRasterBand : public GDALPamRasterBand
     friend class E00GRIDDataset;
 
   public:
-                E00GRIDRasterBand( E00GRIDDataset *, int, GDALDataType );
+    E00GRIDRasterBand( E00GRIDDataset *, int, GDALDataType );
 
-    virtual CPLErr      IReadBlock( int, int, void * ) override;
+    CPLErr IReadBlock( int, int, void * ) override;
 
-    virtual double      GetNoDataValue( int *pbSuccess = NULL ) override;
-    virtual const char *GetUnitType() override;
-    virtual double      GetMinimum( int *pbSuccess = NULL ) override;
-    virtual double      GetMaximum( int *pbSuccess = NULL ) override;
-    virtual CPLErr      GetStatistics( int bApproxOK, int bForce,
-                                       double *pdfMin, double *pdfMax,
-                                       double *pdfMean, double *padfStdDev ) override;
+    double GetNoDataValue( int *pbSuccess = nullptr ) override;
+    const char *GetUnitType() override;
+    double GetMinimum( int *pbSuccess = nullptr ) override;
+    double GetMaximum( int *pbSuccess = nullptr ) override;
+    CPLErr GetStatistics( int bApproxOK, int bForce,
+                          double *pdfMin, double *pdfMax,
+                          double *pdfMean, double *padfStdDev ) override;
 };
 
 /************************************************************************/
@@ -201,7 +205,10 @@ CPLErr E00GRIDRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
         {
             //CPLDebug("E00GRID", "Forward skip to %d from %d", nBlockYOff, poGDS->nLastYOff);
             for(int i=poGDS->nLastYOff + 1; i < nBlockYOff;i++)
-                IReadBlock(0, i, pImage);
+            {
+                if( IReadBlock(0, i, pImage) != CE_None )
+                    return CE_Failure;
+            }
         }
 
         if (nBlockYOff > poGDS->nMaxYOffset)
@@ -211,14 +218,19 @@ CPLErr E00GRIDRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
             poGDS->nMaxYOffset = nBlockYOff;
         }
 
-        const char* pszLine = NULL;
+        const char* pszLine = nullptr;
         for(int i=0;i<nBlockXSize;i++)
         {
             if ((i % VALS_PER_LINE) == 0)
             {
                 pszLine = E00ReadNextLine(poGDS->e00ReadPtr);
-                if (pszLine == NULL || strlen(pszLine) < 5 * E00_FLOAT_SIZE)
+                if (pszLine == nullptr || strlen(pszLine) < 5 * E00_FLOAT_SIZE)
+                {
+                    CPLError(CE_Failure, CPLE_FileIO,
+                             "Could not find enough values for line %d",
+                             nBlockYOff);
                     return CE_Failure;
+                }
             }
             if (eDataType == GDT_Float32)
             {
@@ -247,7 +259,12 @@ CPLErr E00GRIDRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
     for(int i=0;i<nBlockXSize;i++)
     {
         if (VSIFReadL(szVal, E00_FLOAT_SIZE, 1, poGDS->fp) != 1)
+        {
+            CPLError(CE_Failure, CPLE_FileIO,
+                             "Could not find enough values for line %d",
+                             nBlockYOff);
             return CE_Failure;
+        }
 
         if (eDataType == GDT_Float32)
         {
@@ -295,7 +312,7 @@ const char * E00GRIDRasterBand::GetUnitType()
 
     poGDS->ReadMetadata();
 
-    if (poGDS->papszPrj == NULL)
+    if (poGDS->papszPrj == nullptr)
         return GDALPamRasterBand::GetUnitType();
 
     char** papszIter = poGDS->papszPrj;
@@ -333,7 +350,7 @@ double E00GRIDRasterBand::GetMinimum( int *pbSuccess )
 
     if (poGDS->bHasStats)
     {
-        if( pbSuccess != NULL )
+        if( pbSuccess != nullptr )
             *pbSuccess = TRUE;
 
         return poGDS->dfMin;
@@ -354,7 +371,7 @@ double E00GRIDRasterBand::GetMaximum( int *pbSuccess )
 
     if (poGDS->bHasStats)
     {
-        if( pbSuccess != NULL )
+        if( pbSuccess != nullptr )
             *pbSuccess = TRUE;
 
         return poGDS->dfMax;
@@ -398,16 +415,16 @@ CPLErr E00GRIDRasterBand::GetStatistics( int bApproxOK, int bForce,
 /************************************************************************/
 
 E00GRIDDataset::E00GRIDDataset() :
-    e00ReadPtr(NULL),
-    fp(NULL),
+    e00ReadPtr(nullptr),
+    fp(nullptr),
     nDataStart(0),
     nBytesEOL(1),
     nPosBeforeReadLine(0),
-    panOffsets(NULL),
+    panOffsets(nullptr),
     nLastYOff(-1),
     nMaxYOffset(-1),
     dfNoData(0),
-    papszPrj(NULL),
+    papszPrj(nullptr),
     bHasReadMetadata(FALSE),
     bHasStats(FALSE),
     dfMin(0),
@@ -452,7 +469,7 @@ int E00GRIDDataset::Identify( GDALOpenInfo * poOpenInfo )
         return FALSE;
 
     /* FIXME: handle GRD  3 if that ever exists ? */
-    if (strstr((const char*)poOpenInfo->pabyHeader, "GRD  2") == NULL)
+    if (strstr((const char*)poOpenInfo->pabyHeader, "GRD  2") == nullptr)
         return FALSE;
 
     return TRUE;
@@ -466,7 +483,7 @@ const char* E00GRIDDataset::ReadNextLine(void * ptr)
 {
     E00GRIDDataset* poDS = (E00GRIDDataset*) ptr;
     poDS->nPosBeforeReadLine = VSIFTellL(poDS->fp);
-    return CPLReadLine2L(poDS->fp, 256, NULL);
+    return CPLReadLine2L(poDS->fp, 256, nullptr);
 }
 
 /************************************************************************/
@@ -486,15 +503,14 @@ void E00GRIDDataset::Rewind(void * ptr)
 GDALDataset *E00GRIDDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    if (!Identify(poOpenInfo))
-        return NULL;
+    if (!Identify(poOpenInfo) || poOpenInfo->fpL == nullptr )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Find dataset characteristics                                    */
 /* -------------------------------------------------------------------- */
-    VSILFILE* fp = VSIFOpenL(poOpenInfo->pszFilename, "rb");
-    if (fp == NULL)
-        return NULL;
+    VSILFILE* fp = poOpenInfo->fpL;
+    poOpenInfo->fpL = nullptr;
 
     if (poOpenInfo->eAccess == GA_Update)
     {
@@ -502,38 +518,38 @@ GDALDataset *E00GRIDDataset::Open( GDALOpenInfo * poOpenInfo )
                   "The E00GRID driver does not support update access to existing"
                   " datasets.\n" );
         VSIFCloseL(fp);
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
     E00GRIDDataset *poDS = new E00GRIDDataset();
-    if (strstr((const char*)poOpenInfo->pabyHeader, "\r\n") != NULL)
+    if (strstr((const char*)poOpenInfo->pabyHeader, "\r\n") != nullptr)
         poDS->nBytesEOL = 2;
     poDS->fp = fp;
 
     /* read EXP  0 or EXP  1 line */
-    const char* pszLine = CPLReadLine2L(fp, 81, NULL);
-    if (pszLine == NULL)
+    const char* pszLine = CPLReadLine2L(fp, 81, nullptr);
+    if (pszLine == nullptr)
     {
         CPLDebug("E00GRID", "Bad 1st line");
         delete poDS;
-        return NULL;
+        return nullptr;
     }
     bool bCompressed = STARTS_WITH_CI(pszLine, "EXP  1");
 
-    E00ReadPtr e00ReadPtr = NULL;
+    E00ReadPtr e00ReadPtr = nullptr;
     if (bCompressed)
     {
         VSIRewindL(fp);
         e00ReadPtr = E00ReadCallbackOpen(poDS,
                                          E00GRIDDataset::ReadNextLine,
                                          E00GRIDDataset::Rewind);
-        if (e00ReadPtr == NULL)
+        if (e00ReadPtr == nullptr)
         {
             delete poDS;
-            return NULL;
+            return nullptr;
         }
         E00ReadNextLine(e00ReadPtr);
         poDS->e00ReadPtr = e00ReadPtr;
@@ -543,34 +559,37 @@ GDALDataset *E00GRIDDataset::Open( GDALOpenInfo * poOpenInfo )
     if (e00ReadPtr)
         pszLine = E00ReadNextLine(e00ReadPtr);
     else
-        pszLine = CPLReadLine2L(fp, 81, NULL);
-    if (pszLine == NULL || !STARTS_WITH_CI(pszLine, "GRD  2"))
+        pszLine = CPLReadLine2L(fp, 81, nullptr);
+    if (pszLine == nullptr || !STARTS_WITH_CI(pszLine, "GRD  2"))
     {
         CPLDebug("E00GRID", "Bad 2nd line");
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     /* read ncols, nrows and nodata value */
     if (e00ReadPtr)
         pszLine = E00ReadNextLine(e00ReadPtr);
     else
-        pszLine = CPLReadLine2L(fp, 81, NULL);
-    if (pszLine == NULL || strlen(pszLine) <
+        pszLine = CPLReadLine2L(fp, 81, nullptr);
+    if (pszLine == nullptr || strlen(pszLine) <
                 E00_INT_SIZE+E00_INT_SIZE+2+E00_DOUBLE_SIZE)
     {
         CPLDebug("E00GRID", "Bad 3rd line");
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     const int nRasterXSize = atoi(pszLine);
     const int nRasterYSize = atoi(pszLine + E00_INT_SIZE);
 
-    if (!GDALCheckDatasetDimensions(nRasterXSize, nRasterYSize))
+    if (!GDALCheckDatasetDimensions(nRasterXSize, nRasterYSize) ||
+        /* ASCII format. Huge rasters do not make sense */
+        nRasterXSize > 100000 ||
+        nRasterYSize > 100000 )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     GDALDataType eDT = GDT_Float32;
@@ -590,12 +609,12 @@ GDALDataset *E00GRIDDataset::Open( GDALOpenInfo * poOpenInfo )
     if (e00ReadPtr)
         pszLine = E00ReadNextLine(e00ReadPtr);
     else
-        pszLine = CPLReadLine2L(fp, 81, NULL);
-    if (pszLine == NULL || strlen(pszLine) < 2*E00_DOUBLE_SIZE)
+        pszLine = CPLReadLine2L(fp, 81, nullptr);
+    if (pszLine == nullptr || strlen(pszLine) < 2*E00_DOUBLE_SIZE)
     {
         CPLDebug("E00GRID", "Bad 4th line");
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 /*
     double dfPixelX = CPLAtof(pszLine);
@@ -606,12 +625,12 @@ GDALDataset *E00GRIDDataset::Open( GDALOpenInfo * poOpenInfo )
     if (e00ReadPtr)
         pszLine = E00ReadNextLine(e00ReadPtr);
     else
-        pszLine = CPLReadLine2L(fp, 81, NULL);
-    if (pszLine == NULL || strlen(pszLine) < 2*E00_DOUBLE_SIZE)
+        pszLine = CPLReadLine2L(fp, 81, nullptr);
+    if (pszLine == nullptr || strlen(pszLine) < 2*E00_DOUBLE_SIZE)
     {
         CPLDebug("E00GRID", "Bad 5th line");
         delete poDS;
-        return NULL;
+        return nullptr;
     }
     const double dfMinX = CPLAtof(pszLine);
     const double dfMinY = CPLAtof(pszLine + E00_DOUBLE_SIZE);
@@ -620,12 +639,12 @@ GDALDataset *E00GRIDDataset::Open( GDALOpenInfo * poOpenInfo )
     if (e00ReadPtr)
         pszLine = E00ReadNextLine(e00ReadPtr);
     else
-        pszLine = CPLReadLine2L(fp, 81, NULL);
-    if (pszLine == NULL || strlen(pszLine) < 2*E00_DOUBLE_SIZE)
+        pszLine = CPLReadLine2L(fp, 81, nullptr);
+    if (pszLine == nullptr || strlen(pszLine) < 2*E00_DOUBLE_SIZE)
     {
         CPLDebug("E00GRID", "Bad 6th line");
         delete poDS;
-        return NULL;
+        return nullptr;
     }
     const double dfMaxX = CPLAtof(pszLine);
     const double dfMaxY = CPLAtof(pszLine + E00_DOUBLE_SIZE);
@@ -644,10 +663,10 @@ GDALDataset *E00GRIDDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         poDS->panOffsets = (vsi_l_offset*)
                         VSIMalloc2(sizeof(vsi_l_offset), nRasterYSize);
-        if (poDS->panOffsets == NULL)
+        if (poDS->panOffsets == nullptr)
         {
             delete poDS;
-            return NULL;
+            return nullptr;
         }
     }
 /* -------------------------------------------------------------------- */
@@ -691,7 +710,7 @@ const char* E00GRIDDataset::ReadLine()
     if (e00ReadPtr)
         return E00ReadNextLine(e00ReadPtr);
 
-    return CPLReadLine2L(fp, 81, NULL);
+    return CPLReadLine2L(fp, 81, nullptr);
 }
 
 /************************************************************************/
@@ -717,10 +736,15 @@ void E00GRIDDataset::ReadMetadata()
 
     bHasReadMetadata = TRUE;
 
-    if (e00ReadPtr == NULL)
+    if (e00ReadPtr == nullptr)
     {
-        const int nRoundedBlockXSize = ((nRasterXSize + VALS_PER_LINE - 1) /
-                                                VALS_PER_LINE) * VALS_PER_LINE;
+        const int nRoundedBlockXSize =
+            DIV_ROUND_UP(nRasterXSize, VALS_PER_LINE) * VALS_PER_LINE;
+        if( static_cast<vsi_l_offset>(nRasterYSize) >
+                                    GUINTBIG_MAX / nRoundedBlockXSize )
+        {
+            return;
+        }
         const vsi_l_offset nValsToSkip =
                                (vsi_l_offset)nRasterYSize * nRoundedBlockXSize;
         const vsi_l_offset nLinesToSkip = nValsToSkip / VALS_PER_LINE;
@@ -812,21 +836,24 @@ void E00GRIDDataset::ReadMetadata()
             return;
     }
 
-    const char* pszLine = NULL;
+    const char* pszLine = nullptr;
     bool bPRJFound = false;
     bool bStatsFound = false;
-    while((pszLine = ReadLine()) != NULL)
+    while((pszLine = ReadLine()) != nullptr)
     {
         if (STARTS_WITH_CI(pszLine, "PRJ  2"))
         {
             bPRJFound = true;
-            while((pszLine = ReadLine()) != NULL)
+            while((pszLine = ReadLine()) != nullptr)
             {
                 if (EQUAL(pszLine, "EOP"))
                 {
                     break;
                 }
-                papszPrj = CSLAddString(papszPrj, pszLine);
+                if (!EQUAL(pszLine, "~") )
+                {
+                    papszPrj = CSLAddString(papszPrj, pszLine);
+                }
             }
 
             OGRSpatialReference oSRS;
@@ -837,8 +864,8 @@ void E00GRIDDataset::ReadMetadata()
             }
             else
             {
-                char* pszWKT = NULL;
-                if (oSRS.exportToWkt(&pszWKT) == OGRERR_NONE && pszWKT != NULL)
+                char* pszWKT = nullptr;
+                if (oSRS.exportToWkt(&pszWKT) == OGRERR_NONE && pszWKT != nullptr)
                     osProjection = pszWKT;
                 CPLFree(pszWKT);
             }
@@ -881,7 +908,7 @@ void E00GRIDDataset::ReadMetadata()
 void GDALRegister_E00GRID()
 
 {
-    if( GDALGetDriverByName( "E00GRID" ) != NULL )
+    if( GDALGetDriverByName( "E00GRID" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
