@@ -35,7 +35,7 @@
    #include "gnm_frmts.h"
 #endif
 
-CPL_CVSID("$Id: gdalallregister.cpp 36731 2016-12-07 11:08:07Z rouault $");
+CPL_CVSID("$Id: gdalallregister.cpp c77afa6dc277e9099347dcd28de7d954e619b774 2018-03-10 18:58:53Z Even Rouault $")
 
 #ifdef notdef
 // we may have a use for this some day
@@ -118,10 +118,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_SDTS();
 #endif
 
-#ifdef FRMT_ogdi
-    GDALRegister_OGDI();
-#endif
-
 #ifdef FRMT_dted
     GDALRegister_DTED();
 #endif
@@ -237,6 +233,7 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_ISIS3();
     GDALRegister_ISIS2();
     GDALRegister_PDS();
+    GDALRegister_PDS4();
     GDALRegister_VICAR();
 #endif
 
@@ -413,7 +410,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_PNM();
     GDALRegister_DOQ1();
     GDALRegister_DOQ2();
-    GDALRegister_GenBin();
     GDALRegister_PAux();
     GDALRegister_MFF();
     GDALRegister_HKV();
@@ -437,11 +433,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_KRO();
     GDALRegister_ROIPAC();
     GDALRegister_RRASTER();
-
-    // Those ones need to look for side car files so put them at end
-    GDALRegister_ENVI();
-    GDALRegister_EHdr();
-    GDALRegister_ISCE();
 #endif
 
 #ifdef FRMT_arg
@@ -554,12 +545,28 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_PRF();
 #endif
 
+#ifdef FRMT_rda
+    GDALRegister_RDA();
+#endif
+
+#ifdef FRMT_null
+    GDALRegister_NULL();
+#endif
+
 #ifdef GNM_ENABLED
     GNMRegisterAllInternal();
 #endif
 
     OGRRegisterAllInternal();
 
+#ifdef FRMT_raw
+    // Those ones need to look for side car files so put them at end
+    GDALRegister_GenBin();
+    GDALRegister_ENVI();
+    GDALRegister_EHdr();
+    GDALRegister_ISCE();
+#endif
+
 #ifdef FRMT_wcs
     GDALRegister_HTTP();
 #endif
