@@ -32,11 +32,12 @@
 #include <climits>
 
 #include <algorithm>
+#include <limits>
 #include <string>
 
 #include "gdal_frmts.h"
 
-CPL_CVSID("$Id: ilwisdataset.cpp 36979 2016-12-20 18:40:40Z rouault $");
+CPL_CVSID("$Id: ilwisdataset.cpp 22f8ae3bf7bc3cccd970992655c63fc5254d3206 2018-04-08 20:13:05 +0200 Even Rouault $")
 
 namespace GDAL
 {
@@ -68,7 +69,7 @@ static std::string TrimSpaces(const std::string& input)
 static std::string GetLine(VSILFILE* fil)
 {
     const char *p = CPLReadLineL( fil );
-    if (p == NULL)
+    if (p == nullptr)
         return std::string();
 
     CPLString osWrk = p;
@@ -166,7 +167,7 @@ void IniFile::RemoveSection(const std::string& section)
 void IniFile::Load()
 {
     VSILFILE *filIni = VSIFOpenL(filename.c_str(), "r");
-    if (filIni == NULL)
+    if (filIni == nullptr)
         return;
 
     std::string section, key, value;
@@ -229,7 +230,7 @@ void IniFile::Load()
 void IniFile::Store()
 {
     VSILFILE *filIni = VSIFOpenL(filename.c_str(), "w+");
-    if (filIni == NULL)
+    if (filIni == nullptr)
         return;
 
     Sections::iterator iterSect;
@@ -463,7 +464,7 @@ ILWISDataset::ILWISDataset() :
 ILWISDataset::~ILWISDataset()
 
 {
-    FlushCache();
+    ILWISDataset::FlushCache();
     CPLFree( pszProjection );
 }
 
@@ -668,18 +669,18 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Does this look like an ILWIS file                               */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->nHeaderBytes < 1 )
-        return NULL;
+        return nullptr;
 
     std::string sExt = CPLGetExtension( poOpenInfo->pszFilename );
     if (!EQUAL(sExt.c_str(),"mpr") && !EQUAL(sExt.c_str(),"mpl"))
-        return NULL;
+        return nullptr;
 
     if (!CheckASCII(poOpenInfo->pabyHeader, poOpenInfo->nHeaderBytes))
-        return NULL;
+        return nullptr;
 
     std::string ilwistype = ReadElement("Ilwis", "Type", poOpenInfo->pszFilename);
     if( ilwistype.empty())
-        return NULL;
+        return nullptr;
 
     std::string sFileType;  // map or map list
     int    iBandCount;
@@ -719,7 +720,7 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
                 CPLError( CE_Failure, CPLE_AppDefined,
                           "Unsupported ILWIS data file. \n"
                           "can't treat as raster.\n" );
-                return NULL;
+                return nullptr;
             }
         }
     }
@@ -736,7 +737,7 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
             //CPLError( CE_Failure, CPLE_AppDefined,
             //          "Unsupported ILWIS data file. \n"
             //          "can't treat as raster.\n" );
-            return NULL;
+            return nullptr;
         }
     }
     else
@@ -744,7 +745,7 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Unsupported ILWIS data file. \n"
                   "can't treat as raster.\n" );
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -760,12 +761,12 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
     if ( GetRowCol(mapsize, Row, Col) != CE_None)
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
     if( !GDALCheckDatasetDimensions(Col, Row) )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
     poDS->nRasterXSize = Col;
     poDS->nRasterYSize = Row;
@@ -876,7 +877,7 @@ GDALDataset *ILWISDataset::Create(const char* pszFilename,
                   "data type (%s).\n",
                   GDALGetDataTypeName(eType) );
 
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -887,7 +888,7 @@ GDALDataset *ILWISDataset::Create(const char* pszFilename,
     double stepsize = 1;
     std::string sStoreType = GDALType2ILWIS(eType);
     if( EQUAL(sStoreType.c_str(),""))
-        return NULL;
+        return nullptr;
     else if( EQUAL(sStoreType.c_str(),"Real") || EQUAL(sStoreType.c_str(),"float"))
         stepsize = 0;
 
@@ -966,11 +967,11 @@ GDALDataset *ILWISDataset::Create(const char* pszFilename,
 
         VSILFILE  *fp = VSIFOpenL( pszDataName.c_str(), "wb" );
 
-        if( fp == NULL )
+        if( fp == nullptr )
         {
             CPLError( CE_Failure, CPLE_OpenFailed,
                       "Unable to create file %s.\n", pszDataName.c_str() );
-            return NULL;
+            return nullptr;
         }
         VSIFCloseL( fp );
     }
@@ -1011,8 +1012,8 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                           GDALProgressFunc pfnProgress, void * pProgressData )
 
 {
-    if( !pfnProgress( 0.0, NULL, pProgressData ) )
-        return NULL;
+    if( !pfnProgress( 0.0, nullptr, pProgressData ) )
+        return nullptr;
 
     const int nXSize = poSrcDS->GetRasterXSize();
     const int nYSize = poSrcDS->GetRasterYSize();
@@ -1034,8 +1035,8 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                                                   nBands,
                                                   eType, papszOptions );
 
-    if( poDS == NULL )
-        return NULL;
+    if( poDS == nullptr )
+        return nullptr;
     const std::string pszBaseName = std::string(CPLGetBasename( pszFilename ));
     const std::string pszPath = std::string(CPLGetPath( pszFilename ));
 
@@ -1058,7 +1059,7 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     }
 
     const char *pszProj = poSrcDS->GetProjectionRef();
-    if( pszProj != NULL && strlen(pszProj) > 0 )
+    if( pszProj != nullptr && strlen(pszProj) > 0 )
         poDS->SetProjection( pszProj );
 
 /* -------------------------------------------------------------------- */
@@ -1067,7 +1068,7 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
     for( int iBand = 0; iBand < nBands; iBand++ )
     {
-        VSILFILE *fpData = NULL;
+        VSILFILE *fpData = nullptr;
 
         GDALRasterBand *poBand = poSrcDS->GetRasterBand( iBand+1 );
         ILWISRasterBand *desBand = (ILWISRasterBand *) poDS->GetRasterBand( iBand+1 );
@@ -1085,7 +1086,7 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         const std::string sStoreType = GDALType2ILWIS( eType );
         double stepsize = 1;
         if( EQUAL(sStoreType.c_str(),""))
-            return NULL;
+            return nullptr;
         else if( EQUAL(sStoreType.c_str(),"Real") || EQUAL(sStoreType.c_str(),"float"))
             stepsize = 0;
 
@@ -1132,12 +1133,12 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         //std::string pszDataFileName = CPLResetExtension(pszODFName.c_str(), "mp#" );
 
         fpData = desBand->fpRaw;
-        if( fpData == NULL )
+        if( fpData == nullptr )
         {
             CPLError( CE_Failure, CPLE_OpenFailed,
                       "Attempt to create file `%s' failed.\n",
                       pszFilename );
-            return NULL;
+            return nullptr;
         }
 
         GByte *pData = (GByte *) CPLMalloc( nLineSize );
@@ -1147,7 +1148,7 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         {
             eErr = poBand->RasterIO( GF_Read, 0, iLine, nXSize, 1,
                                      pData, nXSize, 1, eType,
-                                     0, 0, NULL );
+                                     0, 0, nullptr );
 
             if( eErr == CE_None )
             {
@@ -1192,11 +1193,11 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                     //CPLFree( pData32 );
                     CPLError( CE_Failure, CPLE_FileIO,
                               "Write of file failed with fwrite error.");
-                    return NULL;
+                    return nullptr;
                 }
             }
-            if( !pfnProgress(iLine / (nYSize * nBands), NULL, pProgressData ) )
-                return NULL;
+            if( !pfnProgress(iLine / (nYSize * nBands), nullptr, pProgressData ) )
+                return nullptr;
         }
         VSIFFlushL( fpData );
         CPLFree( pData );
@@ -1204,7 +1205,7 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
     poDS->FlushCache();
 
-    if( !pfnProgress( 1.0, NULL, pProgressData ) )
+    if( !pfnProgress( 1.0, nullptr, pProgressData ) )
     {
         CPLError( CE_Failure, CPLE_UserInterrupt,
                   "User terminated" );
@@ -1213,7 +1214,7 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         GDALDriver *poILWISDriver =
             (GDALDriver *) GDALGetDriverByName( "ILWIS" );
         poILWISDriver->Delete( pszFilename );
-        return NULL;
+        return nullptr;
     }
 
     poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
@@ -1226,7 +1227,7 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /************************************************************************/
 
 ILWISRasterBand::ILWISRasterBand( ILWISDataset *poDSIn, int nBandIn ) :
-    fpRaw(NULL),
+    fpRaw(nullptr),
     nSizePerPixel(0)
 {
     poDS = poDSIn;
@@ -1297,10 +1298,10 @@ ILWISRasterBand::ILWISRasterBand( ILWISDataset *poDSIn, int nBandIn ) :
 ILWISRasterBand::~ILWISRasterBand()
 
 {
-    if( fpRaw != NULL )
+    if( fpRaw != nullptr )
     {
         VSIFCloseL( fpRaw );
-        fpRaw = NULL;
+        fpRaw = nullptr;
     }
 }
 
@@ -1348,7 +1349,9 @@ void ILWISRasterBand::ReadValueDomainProperties(const std::string& pszFileName)
         }
         else // Floating point values
         {
-            if ((rMin >= -FLT_MAX) && (rMax <= FLT_MAX) && (fabs(rStep) >= FLT_EPSILON)) // is "float" good enough?
+            if ((rMin >= std::numeric_limits<float>::lowest()) &&
+                (rMax <= std::numeric_limits<float>::max()) &&
+                (fabs(rStep) >= FLT_EPSILON)) // is "float" good enough?
               eDataType = GDT_Float32;
             else
               eDataType = GDT_Float64;
@@ -1481,7 +1484,7 @@ CPLErr ILWISRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
     CPLAssert( nBlockXOff == 0 );
 
     int nBlockSize =  nBlockXSize * nBlockYSize * nSizePerPixel;
-    if( fpRaw == NULL )
+    if( fpRaw == nullptr )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "Failed to open ILWIS data file.");
@@ -1673,10 +1676,10 @@ CPLErr ILWISRasterBand::IWriteBlock(CPL_UNUSED int nBlockXOff, int nBlockYOff,
 
     ILWISDataset* dataset = (ILWISDataset*) poDS;
 
-    CPLAssert( dataset != NULL
+    CPLAssert( dataset != nullptr
                && nBlockXOff == 0
                && nBlockYOff >= 0
-               && pImage != NULL );
+               && pImage != nullptr );
 
     int nXSize = dataset->GetRasterXSize();
     int nBlockSize = nBlockXSize * nBlockYSize * nSizePerPixel;
@@ -1829,7 +1832,7 @@ double ILWISRasterBand::GetNoDataValue( int *pbSuccess )
 
 static double doubleConv(const char* s)
 {
-    if (s == NULL) return rUNDEF;
+    if (s == nullptr) return rUNDEF;
     char *begin = const_cast<char*>(s);
 
     // skip leading spaces; strtol will return 0 on a std::string with only spaces
@@ -1838,7 +1841,7 @@ static double doubleConv(const char* s)
 
     if (strlen(begin) == 0) return rUNDEF;
     errno = 0;
-    char *endptr = NULL;
+    char *endptr = nullptr;
     const double r = CPLStrtod(begin, &endptr);
     if ((0 == *endptr) && (errno==0))
         return r;
@@ -1866,7 +1869,7 @@ ValueRange::ValueRange( const std::string& sRng ) :
     sRange[sRng.length()] = 0;
 
     char *p1 = strchr(sRange, ':');
-    if( NULL == p1 )
+    if( nullptr == p1 )
     {
         delete[] sRange;
         init();
@@ -1874,10 +1877,10 @@ ValueRange::ValueRange( const std::string& sRng ) :
     }
 
     char *p3 = strstr(sRange, ",offset=");
-    if( NULL == p3 )
+    if( nullptr == p3 )
         p3 = strstr(sRange, ":offset=");
     _r0 = rUNDEF;
-    if( NULL != p3 )
+    if( nullptr != p3 )
     {
         _r0 = doubleConv(p3+8);
         *p3 = 0;
@@ -1891,7 +1894,7 @@ ValueRange::ValueRange( const std::string& sRng ) :
     }
 
     p2 = strchr(sRange, ':');
-    if( p2 != NULL )
+    if( p2 != nullptr )
     {
         *p2 = 0;
         _rLo = CPLAtof(sRange);
@@ -1998,7 +2001,7 @@ void ValueRange::init( double rRaw0 )
             iRawUndef = 0;
 }
 
-std::string ValueRange::ToString()
+std::string ValueRange::ToString() const
 {
     char buffer[200];
     if (fabs(get_rLo()) > 1.0e20 || fabs(get_rHi()) > 1.0e20)
@@ -2010,7 +2013,7 @@ std::string ValueRange::ToString()
     return std::string(buffer);
 }
 
-double ValueRange::rValue(int iRawIn)
+double ValueRange::rValue(int iRawIn) const
 {
     if (iRawIn == iUNDEF || iRawIn == iRawUndef)
         return rUNDEF;
@@ -2025,7 +2028,7 @@ double ValueRange::rValue(int iRawIn)
     return rVal;
 }
 
-int ValueRange::iRaw(double rValueIn)
+int ValueRange::iRaw(double rValueIn) const
 {
     if (rValueIn == rUNDEF) // || !fContains(rValue))
         return iUNDEF;
@@ -2049,7 +2052,7 @@ int ValueRange::iRaw(double rValueIn)
 void GDALRegister_ILWIS()
 
 {
-    if( GDALGetDriverByName( "ILWIS" ) != NULL )
+    if( GDALGetDriverByName( "ILWIS" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
@@ -2057,7 +2060,7 @@ void GDALRegister_ILWIS()
     poDriver->SetDescription( "ILWIS" );
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "ILWIS Raster Map" );
-    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "mpr/mpl" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSIONS, "mpr mpl" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
                                "Byte Int16 Int32 Float64" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
