@@ -42,7 +42,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: IntergraphBand.cpp 35929 2016-10-25 16:09:00Z goatbar $");
+CPL_CVSID("$Id: IntergraphBand.cpp e13dcd4dc171dfeed63f912ba06b9374ce4f3bb2 2018-03-18 21:37:41Z Even Rouault $")
 
 using std::fill;
 
@@ -61,8 +61,8 @@ INGR_TileHeader::INGR_TileHeader() :
     TileSize(0),
     Reserved3(0)
 {
-    fill( Reserved, Reserved + CPL_ARRAYSIZE(Reserved), 0 );
-    fill( Reserved2, Reserved2 + CPL_ARRAYSIZE(Reserved2), 0 );
+    fill( Reserved, Reserved + CPL_ARRAYSIZE(Reserved), static_cast<uint16>(0) );
+    fill( Reserved2, Reserved2 + CPL_ARRAYSIZE(Reserved2), static_cast<uint8>(0) );
     First.Start = 0;
     First.Allocated = 0;
     First.Used = 0;
@@ -81,9 +81,9 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDSIn,
     bTiled(false),
     nFullBlocksX(0),
     nFullBlocksY(0),
-    pabyBlockBuf(NULL),
+    pabyBlockBuf(nullptr),
     nTiles(0),
-    pahTiles(NULL),
+    pahTiles(nullptr),
     nRLEOffset(0)
 {
     poDS = poDSIn;
@@ -101,7 +101,7 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDSIn,
     // --------------------------------------------------------------------
     if( nBandOffset > INT_MAX - (2 + ( 2 * ( hHeaderOne.WordsToFollow + 1 ) )) )
     {
-        pabyBlockBuf = NULL;
+        pabyBlockBuf = nullptr;
         CPLError(CE_Failure, CPLE_AppDefined, "Invalid header values");
         return;
     }
@@ -115,6 +115,9 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDSIn,
 
     if( nEntries > 0 )
     {
+        // Not sure what max to use, but 65536 seems far enough
+        if( nEntries > 65536 )
+            nEntries = 65536;
         switch ( hHeaderTwo.ColorTableType )
         {
         case EnvironVColorTable:
@@ -173,7 +176,7 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDSIn,
 
     if (nBlockXSize <= 0 || nBlockYSize <= 0)
     {
-        pabyBlockBuf = NULL;
+        pabyBlockBuf = nullptr;
         CPLError(CE_Failure, CPLE_AppDefined, "Invalid block dimensions");
         return;
     }
@@ -220,7 +223,7 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDSIn,
             GDALGetDataTypeSizeBytes( eDataType ) );
     }
 
-    if (pabyBlockBuf == NULL)
+    if (pabyBlockBuf == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "Cannot allocate %d bytes", nBlockBufSize);
@@ -347,7 +350,7 @@ GDALColorTable *IntergraphRasterBand::GetColorTable()
 {
     if( poColorTable->GetColorEntryCount() == 0 )
     {
-        return NULL;
+        return nullptr;
     }
 
     return poColorTable;
@@ -359,7 +362,7 @@ GDALColorTable *IntergraphRasterBand::GetColorTable()
 
 CPLErr IntergraphRasterBand::SetColorTable( GDALColorTable *poColorTableIn )
 {
-    if( poColorTableIn == NULL )
+    if( poColorTableIn == nullptr )
     {
         return CE_None;
     }
@@ -481,7 +484,7 @@ IntergraphRGBBand::IntergraphRGBBand( IntergraphDataset *poDSIn,
                                       int nRGorB )
     : IntergraphRasterBand( poDSIn, nBandIn, nBandOffset )
 {
-    if( pabyBlockBuf == NULL )
+    if( pabyBlockBuf == nullptr )
         return;
 
     nRGBIndex = static_cast<uint8>(nRGorB);
@@ -493,7 +496,7 @@ IntergraphRGBBand::IntergraphRGBBand( IntergraphDataset *poDSIn,
     nBlockBufSize *= 3;
     CPLFree( pabyBlockBuf );
     pabyBlockBuf = static_cast<GByte *>(VSIMalloc( nBlockBufSize ));
-    if( pabyBlockBuf == NULL )
+    if( pabyBlockBuf == nullptr )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "Cannot allocate %d bytes", nBlockBufSize);
@@ -538,14 +541,14 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDSIn,
                                       int nBandOffset,
                                       int nRGorB ) :
     IntergraphRasterBand( poDSIn, nBandIn, nBandOffset ),
-    pabyRLEBlock(NULL),
+    pabyRLEBlock(nullptr),
     nRLESize(0),
     bRLEBlockLoaded(FALSE),
-    panRLELineOffset(NULL)
+    panRLELineOffset(nullptr)
 {
     nRGBIndex = static_cast<uint8>(nRGorB);
 
-    if( pabyBlockBuf == NULL )
+    if( pabyBlockBuf == nullptr )
         return;
 
     if( !bTiled )
@@ -559,9 +562,20 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDSIn,
         if( eFormat == RunLengthEncodedC || eFormat == RunLengthEncoded )
         {
             nBlockYSize = 1;
+            if( nRasterYSize > 1024 * 1024 )
+            {
+                VSIFSeekL( poDSIn->fp, 0, SEEK_END );
+                // At the very least 2 bytes per row (probably more)
+                if( VSIFTellL( poDSIn->fp ) / 2 <
+                                    static_cast<vsi_l_offset>(nRasterYSize) )
+                {
+                    CPLError(CE_Failure, CPLE_AppDefined, "File too short");
+                    return;
+                }
+            }
             panRLELineOffset = (uint32 *)
                 VSI_CALLOC_VERBOSE(sizeof(uint32),nRasterYSize);
-            if( panRLELineOffset == NULL )
+            if( panRLELineOffset == nullptr )
                 return;
             nFullBlocksY = nRasterYSize;
         }
@@ -610,10 +624,10 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDSIn,
     }
 
     CPLFree(pabyBlockBuf);
-    pabyBlockBuf = NULL;
+    pabyBlockBuf = nullptr;
     if( nBlockBufSize > 0 )
         pabyBlockBuf = (GByte*) VSIMalloc( nBlockBufSize );
-    if (pabyBlockBuf == NULL)
+    if (pabyBlockBuf == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Cannot allocate %d bytes", nBlockBufSize);
     }
@@ -625,8 +639,21 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDSIn,
     if( nRLESize == 0 )
         pabyRLEBlock = (GByte*) VSIMalloc( 1 );
     else if( nRLESize < INT_MAX )
+    {
+        if( nRLESize > 100 * 1024 * 1024 )
+        {
+            IntergraphDataset *poGDS = ( IntergraphDataset * ) poDS;
+            VSIFSeekL( poGDS->fp, 0, SEEK_END );
+            if( VSIFTellL( poGDS->fp ) < nRLESize )
+            {
+                CPLError(CE_Failure, CPLE_AppDefined, "File too short");
+                pabyRLEBlock = nullptr;
+                return;
+            }
+        }
         pabyRLEBlock = (GByte*) VSIMalloc( nRLESize );
-    if (pabyRLEBlock == NULL)
+    }
+    if (pabyRLEBlock == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Cannot allocate %d bytes", nRLESize);
     }
@@ -729,13 +756,13 @@ CPLErr IntergraphRLEBand::IReadBlock( int nBlockXOff,
     {
         nOutputBytes =
             INGR_DecodeRunLengthBitonalTiled( pabyRLEBlock, pabyBlockBuf,
-                                              nRLESize, nBlockBufSize, NULL );
+                                              nRLESize, nBlockBufSize, nullptr );
     }
-    else if( bTiled || panRLELineOffset == NULL )
+    else if( bTiled || panRLELineOffset == nullptr )
     {
         nOutputBytes = INGR_Decode( eFormat, pabyRLEBlock, pabyBlockBuf,
                                   nRLESize, nBlockBufSize,
-                                  NULL );
+                                  nullptr );
     }
     else
     {
@@ -756,7 +783,7 @@ CPLErr IntergraphRLEBand::IReadBlock( int nBlockXOff,
                 if( nRLESize < panRLELineOffset[iLine] ||
                     (uint32)INGR_Decode( eFormat,
                              pabyRLEBlock + panRLELineOffset[iLine],
-                             NULL,  nRLESize - panRLELineOffset[iLine], nBlockBufSize,
+                             nullptr,  nRLESize - panRLELineOffset[iLine], nBlockBufSize,
                              &nBytesConsumed ) < nBlockBufSize )
                 {
                     memset( pImage, 0, nBlockXSize * nBlockYSize *
@@ -839,12 +866,12 @@ IntergraphBitmapBand::IntergraphBitmapBand( IntergraphDataset *poDSIn,
                                             int nBandOffset,
                                             int nRGorB ) :
     IntergraphRasterBand( poDSIn, nBandIn, nBandOffset, GDT_Byte ),
-    pabyBMPBlock(NULL),
+    pabyBMPBlock(nullptr),
     nBMPSize(0),
     nQuality(0),
     nRGBBand(nRGorB)
 {
-    if( pabyBlockBuf == NULL )
+    if( pabyBlockBuf == nullptr )
         return;
 
     if( !bTiled )
@@ -874,11 +901,26 @@ IntergraphBitmapBand::IntergraphBitmapBand( IntergraphDataset *poDSIn,
     // Create a Bitmap buffer
     // ----------------------------------------------------------------
 
-    pabyBMPBlock = (GByte*) VSIMalloc( nBMPSize );
-    if (pabyBMPBlock == NULL)
+    if( nBMPSize > INT_MAX )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
-                 "Cannot allocate %d bytes", nBMPSize);
+                 "Too large block size: %u bytes", nBMPSize);
+        return;
+    }
+    if( nBMPSize > 10 * 1024 * 1024 )
+    {
+        VSIFSeekL( poDSIn->fp, 0, SEEK_END );
+        if( VSIFTellL( poDSIn->fp ) < nBMPSize )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "File too short");
+            return;
+        }
+    }
+    pabyBMPBlock = (GByte*) VSIMalloc( nBMPSize );
+    if (pabyBMPBlock == nullptr)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Cannot allocate %u bytes", nBMPSize);
     }
 
     // ----------------------------------------------------------------
@@ -1001,7 +1043,7 @@ CPLErr IntergraphBitmapBand::IReadBlock( int nBlockXOff,
                                               nBytesRead,
                                               nRGBBand );
 
-    if( poGDS->hVirtual.poBand == NULL )
+    if( poGDS->hVirtual.poBand == nullptr )
     {
         memset( pImage, 0, nBlockXSize * nBlockYSize *
                     (GDALGetDataTypeSize( eDataType ) / 8) );
@@ -1017,7 +1059,7 @@ CPLErr IntergraphBitmapBand::IReadBlock( int nBlockXOff,
 
     if( poGDS->hVirtual.poBand->RasterIO( GF_Read, 0, 0,
         nVirtualXSize, nVirtualYSize, pImage,
-        nVirtualXSize, nVirtualYSize, GDT_Byte, 0, 0, NULL) != CE_None )
+        nVirtualXSize, nVirtualYSize, GDT_Byte, 0, 0, nullptr) != CE_None )
     {
         INGR_ReleaseVirtual( &poGDS->hVirtual );
         return CE_Failure;
@@ -1047,7 +1089,7 @@ CPLErr IntergraphBitmapBand::IReadBlock( int nBlockXOff,
 int IntergraphRasterBand::LoadBlockBuf( int nBlockXOff,
                                         int nBlockYOff,
                                         int nBlobkBytes,
-                                        GByte *pabyBlock )
+                                        GByte *pabyBlock ) const
 {
     vsi_l_offset nSeekOffset  = 0;
     uint32 nReadSize    = 0;
@@ -1115,7 +1157,7 @@ bool IntergraphRasterBand::ReshapeBlock( int nBlockXOff,
                                          GByte *pabyBlock )
 {
     GByte *pabyTile = (GByte*) VSI_MALLOC_VERBOSE( nBlockBufSize );
-    if( pabyTile == NULL )
+    if( pabyTile == nullptr )
         return false;
 
     memcpy( pabyTile, pabyBlock, nBlockBytes );
