@@ -40,14 +40,14 @@
 #include "cpl_error.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ddfsubfielddefn.cpp 36552 2016-11-29 06:01:45Z goatbar $");
+CPL_CVSID("$Id: ddfsubfielddefn.cpp e13dcd4dc171dfeed63f912ba06b9374ce4f3bb2 2018-03-18 21:37:41Z Even Rouault $")
 
 /************************************************************************/
 /*                          DDFSubfieldDefn()                           */
 /************************************************************************/
 
 DDFSubfieldDefn::DDFSubfieldDefn() :
-    pszName(NULL),
+    pszName(nullptr),
     pszFormatString(CPLStrdup("")),
     eType(DDFString),
     eBinaryFormat(NotBinary),
@@ -55,7 +55,7 @@ DDFSubfieldDefn::DDFSubfieldDefn() :
     chFormatDelimiter(DDF_UNIT_TERMINATOR),
     nFormatWidth(0),
     nMaxBufChars(0),
-    pachBuffer(NULL)
+    pachBuffer(nullptr)
 {}
 
 /************************************************************************/
@@ -145,6 +145,9 @@ int DDFSubfieldDefn::SetFormat( const char * pszFormat )
       case 'b':
         // Is the width expressed in bits? (is it a bitstring)
         bIsVariable = FALSE;
+        if( pszFormatString[1] == '\0' )
+            return FALSE;
+
         if( pszFormatString[1] == '(' )
         {
             nFormatWidth = atoi(pszFormatString+2);
@@ -269,14 +272,14 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
                       "format string %s ... returning shortened data.",
                       nMaxBytes, pszName, pszFormatString );
 
-            if( pnConsumedBytes != NULL )
+            if( pnConsumedBytes != nullptr )
                 *pnConsumedBytes = nMaxBytes;
 
             return nMaxBytes;
         }
         else
         {
-            if( pnConsumedBytes != NULL )
+            if( pnConsumedBytes != nullptr )
                 *pnConsumedBytes = nFormatWidth;
 
             return nFormatWidth;
@@ -346,7 +349,7 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
             nLength++;
         }
 
-        if( pnConsumedBytes != NULL )
+        if( pnConsumedBytes != nullptr )
         {
             if( nMaxBytes == 0 )
                 *pnConsumedBytes = nLength + extraConsumedBytes;
@@ -478,8 +481,14 @@ DDFSubfieldDefn::ExtractFloatData( const char * pachSourceData,
                         pszName, pszFormatString, nMaxBytes );
               return 0;
           }
+          if( nFormatWidth > static_cast<int>(sizeof(abyData)) )
+          {
+              CPLError( CE_Failure, CPLE_AppDefined,
+                        "Format width %d too large", nFormatWidth );
+              return 0;
+          }
 
-          if( pnConsumedBytes != NULL )
+          if( pnConsumedBytes != nullptr )
               *pnConsumedBytes = nFormatWidth;
 
           // Byte swap the data if it isn't in machine native format.
@@ -616,7 +625,7 @@ DDFSubfieldDefn::ExtractIntData( const char * pachSourceData,
               return 0;
           }
 
-          if( pnConsumedBytes != NULL )
+          if( pnConsumedBytes != nullptr )
               *pnConsumedBytes = nFormatWidth;
 
           // Byte swap the data if it isn't in machine native format.
@@ -722,11 +731,11 @@ void DDFSubfieldDefn::DumpData( const char * pachData, int nMaxBytes,
     if( eType == DDFFloat )
         fprintf( fp, "      Subfield `%s' = %f\n",
                  pszName,
-                 ExtractFloatData( pachData, nMaxBytes, NULL ) );
+                 ExtractFloatData( pachData, nMaxBytes, nullptr ) );
     else if( eType == DDFInt )
         fprintf( fp, "      Subfield `%s' = %d\n",
                  pszName,
-                 ExtractIntData( pachData, nMaxBytes, NULL ) );
+                 ExtractIntData( pachData, nMaxBytes, nullptr ) );
     else if( eType == DDFBinaryString )
     {
         int nBytes = 0;
@@ -745,7 +754,7 @@ void DDFSubfieldDefn::DumpData( const char * pachData, int nMaxBytes,
     else
         fprintf( fp, "      Subfield `%s' = `%s'\n",
                  pszName,
-                 ExtractStringData( pachData, nMaxBytes, NULL ) );
+                 ExtractStringData( pachData, nMaxBytes, nullptr ) );
 }
 
 /************************************************************************/
@@ -772,7 +781,7 @@ void DDFSubfieldDefn::DumpData( const char * pachData, int nMaxBytes,
  */
 
 int DDFSubfieldDefn::GetDefaultValue( char *pachData, int nBytesAvailable,
-                                      int *pnBytesUsed )
+                                      int *pnBytesUsed ) const
 
 {
     int nDefaultSize;
@@ -782,10 +791,10 @@ int DDFSubfieldDefn::GetDefaultValue( char *pachData, int nBytesAvailable,
     else
         nDefaultSize = 1;
 
-    if( pnBytesUsed != NULL )
+    if( pnBytesUsed != nullptr )
         *pnBytesUsed = nDefaultSize;
 
-    if( pachData == NULL )
+    if( pachData == nullptr )
         return TRUE;
 
     if( nBytesAvailable < nDefaultSize )
@@ -827,7 +836,7 @@ int DDFSubfieldDefn::GetDefaultValue( char *pachData, int nBytesAvailable,
 int DDFSubfieldDefn::FormatStringValue( char *pachData, int nBytesAvailable,
                                         int *pnBytesUsed,
                                         const char *pszValue,
-                                        int nValueLength )
+                                        int nValueLength ) const
 
 {
     int nSize;
@@ -844,10 +853,10 @@ int DDFSubfieldDefn::FormatStringValue( char *pachData, int nBytesAvailable,
         nSize = nFormatWidth;
     }
 
-    if( pnBytesUsed != NULL )
+    if( pnBytesUsed != nullptr )
         *pnBytesUsed = nSize;
 
-    if( pachData == NULL )
+    if( pachData == nullptr )
         return TRUE;
 
     if( nBytesAvailable < nSize )
@@ -889,7 +898,7 @@ int DDFSubfieldDefn::FormatStringValue( char *pachData, int nBytesAvailable,
  */
 
 int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
-                                     int *pnBytesUsed, int nNewValue )
+                                     int *pnBytesUsed, int nNewValue ) const
 
 {
     int nSize;
@@ -909,10 +918,10 @@ int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
             return FALSE;
     }
 
-    if( pnBytesUsed != NULL )
+    if( pnBytesUsed != nullptr )
         *pnBytesUsed = nSize;
 
-    if( pachData == NULL )
+    if( pachData == nullptr )
         return TRUE;
 
     if( nBytesAvailable < nSize )
@@ -934,7 +943,7 @@ int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
           {
             char chFillChar = '0'; /* ASCII zero intended */
             memset( pachData, chFillChar, nSize );
-            strncpy( pachData + nSize - strlen(szWork), szWork,
+            memcpy( pachData + nSize - strlen(szWork), szWork,
                      strlen(szWork) );
             break;
           }
@@ -952,7 +961,7 @@ int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
                     iOut = i;
 
                 pachData[iOut] = (char)((nNewValue & nMask) >> (i*8));
-                nMask *= 256;
+                nMask <<= 8;
             }
             break;
 
@@ -981,7 +990,7 @@ int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
  */
 
 int DDFSubfieldDefn::FormatFloatValue( char *pachData, int nBytesAvailable,
-                                       int *pnBytesUsed, double dfNewValue )
+                                       int *pnBytesUsed, double dfNewValue ) const
 
 {
     int nSize;
@@ -1001,10 +1010,10 @@ int DDFSubfieldDefn::FormatFloatValue( char *pachData, int nBytesAvailable,
             return FALSE;
     }
 
-    if( pnBytesUsed != NULL )
+    if( pnBytesUsed != nullptr )
         *pnBytesUsed = nSize;
 
-    if( pachData == NULL )
+    if( pachData == nullptr )
         return TRUE;
 
     if( nBytesAvailable < nSize )
@@ -1022,7 +1031,7 @@ int DDFSubfieldDefn::FormatFloatValue( char *pachData, int nBytesAvailable,
             const char chFillZeroASCII = '0'; /* ASCII zero intended */
             /* coverity[bad_memset] */
             memset( pachData, chFillZeroASCII, nSize );
-            strncpy( pachData + nSize - strlen(szWork), szWork,
+            memcpy( pachData + nSize - strlen(szWork), szWork,
                      strlen(szWork) );
         }
         else
