@@ -35,7 +35,7 @@
 #include "gdal_pam.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: levellerdataset.cpp 36501 2016-11-25 14:09:24Z rouault $");
+CPL_CVSID("$Id: levellerdataset.cpp a542b2797f15f2ed694cfcee9ff17d86b339dfee 2018-04-02 00:24:03 +0200 Even Rouault $")
 
 static bool str_equal(const char *_s1, const char *_s2) {
     return 0 == strcmp(_s1, _s2);
@@ -52,7 +52,7 @@ static bool str_equal(const char *_s1, const char *_s2) {
 /* ==================================================================== */
 /************************************************************************/
 
-static const size_t kMaxTagNameLen = 63;
+constexpr size_t kMaxTagNameLen = 63;
 
 enum
 {
@@ -147,13 +147,13 @@ typedef struct
         UNITLABEL   oemCode;
 } measurement_unit;
 
-static const double kdays_per_year = 365.25;
-static const double kdLStoM = 299792458.0;
-static const double kdLYtoM = kdLStoM * kdays_per_year * 24 * 60 * 60;
-static const double kdInch = 0.3048 / 12;
-static const double kPI = M_PI;
+constexpr double kdays_per_year = 365.25;
+constexpr double kdLStoM = 299792458.0;
+constexpr double kdLYtoM = kdLStoM * kdays_per_year * 24 * 60 * 60;
+constexpr double kdInch = 0.3048 / 12;
+constexpr double kPI = M_PI;
 
-static const int kFirstLinearMeasureIdx = 9;
+constexpr int kFirstLinearMeasureIdx = 9;
 
 static const measurement_unit kUnits[] =
 {
@@ -289,7 +289,7 @@ class LevellerDataset : public GDALPamDataset
     UNITLABEL id_to_code(const char*) const;
     UNITLABEL meter_measure_to_code(double) const;
     bool compute_elev_scaling(const OGRSpatialReference&);
-    void raw_to_proj(double, double, double&, double&);
+    void raw_to_proj(double, double, double&, double&) const;
 
 public:
     LevellerDataset();
@@ -407,8 +407,8 @@ public:
 
     // Geomeasure support.
     virtual const char* GetUnitType() override;
-    virtual double GetScale(int* pbSuccess = NULL) override;
-    virtual double GetOffset(int* pbSuccess = NULL) override;
+    virtual double GetScale(int* pbSuccess = nullptr) override;
+    virtual double GetOffset(int* pbSuccess = nullptr) override;
 
     virtual CPLErr IReadBlock( int, int, void * ) override;
     virtual CPLErr IWriteBlock( int, int, void * ) override;
@@ -420,7 +420,7 @@ public:
 /************************************************************************/
 
 LevellerRasterBand::LevellerRasterBand( LevellerDataset *poDSIn ) :
-    m_pLine(NULL),
+    m_pLine(nullptr),
     m_bFirstTime(true)
 {
     poDS = poDSIn;
@@ -440,7 +440,7 @@ bool LevellerRasterBand::Init()
 {
     m_pLine = reinterpret_cast<float*>(
         VSI_MALLOC2_VERBOSE( sizeof(float), nBlockXSize) );
-    return m_pLine != NULL;
+    return m_pLine != nullptr;
 }
 
 LevellerRasterBand::~LevellerRasterBand()
@@ -460,8 +460,8 @@ CPLErr LevellerRasterBand::IWriteBlock
 )
 {
     CPLAssert( nBlockXOff == 0  );
-    CPLAssert( pImage != NULL );
-    CPLAssert( m_pLine != NULL );
+    CPLAssert( pImage != nullptr );
+    CPLAssert( m_pLine != nullptr );
 
 /*  #define sgn(_n) ((_n) < 0 ? -1 : ((_n) > 0 ? 1 : 0) )
     #define sround(_f)                          \
@@ -524,7 +524,7 @@ CPLErr LevellerRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
 {
     CPLAssert( sizeof(float) == sizeof(GInt32) );
     CPLAssert( nBlockXOff == 0  );
-    CPLAssert( pImage != NULL );
+    CPLAssert( pImage != nullptr );
 
     LevellerDataset *poGDS = reinterpret_cast<LevellerDataset *>( poDS );
 
@@ -603,7 +603,7 @@ const char *LevellerRasterBand::GetUnitType()
 double LevellerRasterBand::GetScale(int* pbSuccess)
 {
     LevellerDataset *poGDS = reinterpret_cast<LevellerDataset *>( poDS );
-    if(pbSuccess != NULL)
+    if(pbSuccess != nullptr)
         *pbSuccess = TRUE;
     return poGDS->m_dElevScale;
 }
@@ -615,7 +615,7 @@ double LevellerRasterBand::GetScale(int* pbSuccess)
 double LevellerRasterBand::GetOffset(int* pbSuccess)
 {
     LevellerDataset *poGDS = reinterpret_cast<LevellerDataset *>( poDS );
-    if(pbSuccess != NULL)
+    if(pbSuccess != nullptr)
         *pbSuccess = TRUE;
     return poGDS->m_dElevBase;
 }
@@ -632,11 +632,11 @@ double LevellerRasterBand::GetOffset(int* pbSuccess)
 
 LevellerDataset::LevellerDataset() :
     m_version(0),
-    m_pszFilename(NULL),
-    m_pszProjection(NULL),
+    m_pszFilename(nullptr),
+    m_pszProjection(nullptr),
     m_dElevScale(),
     m_dElevBase(),
-    m_fp(NULL),
+    m_fp(nullptr),
     m_nDataOffset()
 {
     memset( m_szElevUnits, 0, sizeof(m_szElevUnits) );
@@ -655,7 +655,7 @@ LevellerDataset::~LevellerDataset()
     CPLFree(m_pszProjection);
     CPLFree(m_pszFilename);
 
-    if( m_fp != NULL )
+    if( m_fp != nullptr )
         VSIFCloseL( m_fp );
 }
 
@@ -669,7 +669,7 @@ static double average(double a, double b)
     return 0.5 * (a + b);
 }
 
-void LevellerDataset::raw_to_proj(double x, double y, double& xp, double& yp)
+void LevellerDataset::raw_to_proj(double x, double y, double& xp, double& yp) const
 {
     xp = x * m_adfTransform[1] + m_adfTransform[0];
     yp = y * m_adfTransform[5] + m_adfTransform[3];
@@ -680,7 +680,7 @@ bool LevellerDataset::compute_elev_scaling
     const OGRSpatialReference& sr
 )
 {
-    const char* pszGroundUnits = NULL;
+    const char* pszGroundUnits = nullptr;
 
     if(!sr.IsGeographic())
     {
@@ -690,7 +690,7 @@ bool LevellerDataset::compute_elev_scaling
 
         const double dfLinear = sr.GetLinearUnits();
         const measurement_unit* pu = this->get_uom(dfLinear);
-        if(pu == NULL)
+        if(pu == nullptr)
             return false;
 
         pszGroundUnits = pu->pszID;
@@ -728,7 +728,7 @@ bool LevellerDataset::compute_elev_scaling
     const measurement_unit* puG = this->get_uom(pszGroundUnits);
     const measurement_unit* puE = this->get_uom(m_szElevUnits);
 
-    if(puG == NULL || puE == NULL)
+    if(puG == nullptr || puE == nullptr)
         return false;
 
     const double g_to_e = puG->dScale / puE->dScale;
@@ -754,7 +754,7 @@ bool LevellerDataset::write_header()
     m_dElevBase = 0.0;
     m_dElevScale = 1.0;
 
-    if(m_pszProjection == NULL || m_pszProjection[0] == 0)
+    if(m_pszProjection == nullptr || m_pszProjection[0] == 0)
     {
         write_tag("csclass", LEV_COORDSYS_RASTER);
     }
@@ -863,19 +863,19 @@ GDALDataset* LevellerDataset::Create
     if(nBands != 1)
     {
         CPLError( CE_Failure, CPLE_IllegalArg, "Band count must be 1" );
-        return NULL;
+        return nullptr;
     }
 
     if(eType != GDT_Float32)
     {
         CPLError( CE_Failure, CPLE_IllegalArg, "Pixel type must be Float32" );
-        return NULL;
+        return nullptr;
     }
 
     if(nXSize < 2 || nYSize < 2)
     {
         CPLError( CE_Failure, CPLE_IllegalArg, "One or more raster dimensions too small" );
-        return NULL;
+        return nullptr;
     }
 
     LevellerDataset* poDS = new LevellerDataset;
@@ -886,13 +886,13 @@ GDALDataset* LevellerDataset::Create
 
     poDS->m_fp = VSIFOpenL( pszFilename, "wb+" );
 
-    if( poDS->m_fp == NULL )
+    if( poDS->m_fp == nullptr )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "Attempt to create file `%s' failed.",
                   pszFilename );
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     // Header will be written the first time IWriteBlock
@@ -903,19 +903,19 @@ GDALDataset* LevellerDataset::Create
 
     const char* pszValue = CSLFetchNameValue(
         papszOptions,"MINUSERPIXELVALUE");
-    if( pszValue != NULL )
+    if( pszValue != nullptr )
         poDS->m_dLogSpan[0] = CPLAtof( pszValue );
     else
     {
         delete poDS;
         CPLError( CE_Failure, CPLE_IllegalArg,
                   "MINUSERPIXELVALUE must be specified." );
-        return NULL;
+        return nullptr;
     }
 
     pszValue = CSLFetchNameValue(
         papszOptions,"MAXUSERPIXELVALUE");
-    if( pszValue != NULL )
+    if( pszValue != nullptr )
         poDS->m_dLogSpan[1] = CPLAtof( pszValue );
 
     if(poDS->m_dLogSpan[1] < poDS->m_dLogSpan[0])
@@ -934,7 +934,7 @@ GDALDataset* LevellerDataset::Create
     if( !poBand->Init() )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     return poDS;
@@ -1040,7 +1040,7 @@ bool LevellerDataset::locate_data(vsi_l_offset& offset, size_t& len, VSILFILE* f
         if(1 != VSIFReadL(&datalen, sizeof(datalen), 1, fp))
             return false;
 
-        datalen = CPL_LSBWORD32(datalen);
+        CPL_LSBPTR32(&datalen);
         descriptor[descriptorLen] = 0;
         if(str_equal(descriptor, pszTag))
         {
@@ -1135,7 +1135,7 @@ UNITLABEL LevellerDataset::meter_measure_to_code(double dM) const
     // require exact equality, otherwise be loose.
 
     const measurement_unit* pu = this->get_uom(dM);
-    return pu != NULL ? pu->oemCode : UNITLABEL_UNKNOWN;
+    return pu != nullptr ? pu->oemCode : UNITLABEL_UNKNOWN;
 }
 
 UNITLABEL LevellerDataset::id_to_code(const char* pszUnits) const
@@ -1143,7 +1143,7 @@ UNITLABEL LevellerDataset::id_to_code(const char* pszUnits) const
     // Convert a readable UOM to its OEM code.
 
     const measurement_unit* pu = this->get_uom(pszUnits);
-    return pu != NULL ? pu->oemCode : UNITLABEL_UNKNOWN;
+    return pu != nullptr ? pu->oemCode : UNITLABEL_UNKNOWN;
 }
 
 const char* LevellerDataset::code_to_id(UNITLABEL code) const
@@ -1151,7 +1151,7 @@ const char* LevellerDataset::code_to_id(UNITLABEL code) const
     // Convert a measurement unit's OEM ID to its readable ID.
 
     const measurement_unit* pu = this->get_uom(code);
-    return pu != NULL ? pu->pszID : NULL;
+    return pu != nullptr ? pu->pszID : nullptr;
 }
 
 const measurement_unit* LevellerDataset::get_uom(const char* pszUnits) const
@@ -1163,7 +1163,7 @@ const measurement_unit* LevellerDataset::get_uom(const char* pszUnits) const
     }
     CPLError( CE_Failure, CPLE_AppDefined,
               "Unknown measurement units: %s", pszUnits );
-    return NULL;
+    return nullptr;
 }
 
 const measurement_unit* LevellerDataset::get_uom(UNITLABEL code) const
@@ -1175,7 +1175,7 @@ const measurement_unit* LevellerDataset::get_uom(UNITLABEL code) const
     }
     CPLError( CE_Failure, CPLE_AppDefined,
               "Unknown measurement unit code: %08x", code );
-    return NULL;
+    return nullptr;
 }
 
 const measurement_unit* LevellerDataset::get_uom(double dM) const
@@ -1192,7 +1192,7 @@ const measurement_unit* LevellerDataset::get_uom(double dM) const
     }
     CPLError( CE_Failure, CPLE_AppDefined,
               "Unknown measurement conversion factor: %f", dM );
-    return NULL;
+    return nullptr;
 }
 
 /************************************************************************/
@@ -1235,7 +1235,7 @@ bool LevellerDataset::make_local_coordsys(const char* pszName, const char* pszUn
 bool LevellerDataset::make_local_coordsys(const char* pszName, UNITLABEL code)
 {
     const char* pszUnitID = code_to_id(code);
-    return pszUnitID != NULL && make_local_coordsys(pszName, pszUnitID);
+    return pszUnitID != nullptr && make_local_coordsys(pszName, pszUnitID);
 }
 
 /************************************************************************/
@@ -1304,7 +1304,7 @@ bool LevellerDataset::load_from_file(VSILFILE* file, const char* pszFilename)
         if(csclass != LEV_COORDSYS_RASTER)
         {
             // Get projection details and units.
-            CPLAssert(m_pszProjection == NULL);
+            CPLAssert(m_pszProjection == nullptr);
 
             if(csclass == LEV_COORDSYS_LOCAL)
             {
@@ -1367,7 +1367,7 @@ bool LevellerDataset::load_from_file(VSILFILE* file, const char* pszFilename)
             {
                 unitcode = static_cast<UNITLABEL>( unitcode_int );
                 const char* pszUnitID = code_to_id(unitcode);
-                if(pszUnitID != NULL)
+                if(pszUnitID != nullptr)
                 {
                     strncpy(m_szElevUnits, pszUnitID, sizeof(m_szElevUnits));
                     m_szElevUnits[sizeof(m_szElevUnits) - 1] = '\0';
@@ -1399,7 +1399,7 @@ bool LevellerDataset::load_from_file(VSILFILE* file, const char* pszFilename)
           {
               // Drop long name, if present.
               char* p = strchr(szWorldUnits, ' ');
-              if(p != NULL)
+              if(p != nullptr)
                   *p = 0;
           }
 
@@ -1445,7 +1445,7 @@ bool LevellerDataset::load_from_file(VSILFILE* file, const char* pszFilename)
 
 const char* LevellerDataset::GetProjectionRef(void)
 {
-    return m_pszProjection == NULL ? "" : m_pszProjection;
+    return m_pszProjection == nullptr ? "" : m_pszProjection;
 }
 
 /************************************************************************/
@@ -1480,15 +1480,15 @@ GDALDataset *LevellerDataset::Open( GDALOpenInfo * poOpenInfo )
     // The file should have at least 5 header bytes
     // and hf_w, hf_b, and hf_data tags.
 
-    if( poOpenInfo->nHeaderBytes < 5+13+13+16 )
-        return NULL;
+    if( poOpenInfo->nHeaderBytes < 5+13+13+16 || poOpenInfo->fpL == nullptr )
+        return nullptr;
 
     if( !LevellerDataset::Identify(poOpenInfo))
-        return NULL;
+        return nullptr;
 
     const int version = poOpenInfo->pabyHeader[4];
     if(version < 4 || version > 9)
-        return NULL;
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
@@ -1498,20 +1498,8 @@ GDALDataset *LevellerDataset::Open( GDALOpenInfo * poOpenInfo )
 
     poDS->m_version = version;
 
-    // Reopen for large file access.
-    if( poOpenInfo->eAccess == GA_Update )
-        poDS->m_fp = VSIFOpenL( poOpenInfo->pszFilename, "rb+" );
-    else
-        poDS->m_fp = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
-
-    if( poDS->m_fp == NULL )
-    {
-        CPLError( CE_Failure, CPLE_OpenFailed,
-                  "Failed to re-open %s within Leveller driver.",
-                  poOpenInfo->pszFilename );
-        delete poDS;
-        return NULL;
-    }
+    poDS->m_fp = poOpenInfo->fpL;
+    poOpenInfo->fpL = nullptr;
     poDS->eAccess = poOpenInfo->eAccess;
 
 /* -------------------------------------------------------------------- */
@@ -1520,7 +1508,7 @@ GDALDataset *LevellerDataset::Open( GDALOpenInfo * poOpenInfo )
     if( !poDS->load_from_file( poDS->m_fp, poOpenInfo->pszFilename ) )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1531,7 +1519,7 @@ GDALDataset *LevellerDataset::Open( GDALOpenInfo * poOpenInfo )
     if( !poBand->Init() )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     poDS->SetMetadataItem( GDALMD_AREA_OR_POINT, GDALMD_AOP_POINT );
@@ -1557,7 +1545,7 @@ GDALDataset *LevellerDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_Leveller()
 
 {
-    if( GDALGetDriverByName( "Leveller" ) != NULL )
+    if( GDALGetDriverByName( "Leveller" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
@@ -1567,6 +1555,7 @@ void GDALRegister_Leveller()
     poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "ter" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Leveller heightfield" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_leveller.html" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
     poDriver->pfnIdentify = LevellerDataset::Identify;
     poDriver->pfnOpen = LevellerDataset::Open;
