@@ -33,12 +33,12 @@
  ****************************************************************************/
 
 // Set up PDS NULL values
-static const int NULL1 = 0;
-static const int NULL2 = -32768;
+constexpr int NULL1 = 0;
+constexpr int NULL2 = -32768;
 //#define NULL3 -0.3402822655089E+39
 //Same as ESRI_GRID_FLOAT_NO_DATA
 //#define NULL3 -340282346638528859811704183484516925440.0
-static const double NULL3 = -3.4028226550889044521e+38;
+constexpr double NULL3 = -3.4028226550889044521e+38;
 
 #include "cpl_string.h"
 #include "gdal_frmts.h"
@@ -46,8 +46,9 @@ static const double NULL3 = -3.4028226550889044521e+38;
 #include "nasakeywordhandler.h"
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
+#include "cpl_safemaths.hpp"
 
-CPL_CVSID("$Id: pdsdataset.cpp 36981 2016-12-20 19:46:41Z rouault $");
+CPL_CVSID("$Id: pdsdataset.cpp 22f8ae3bf7bc3cccd970992655c63fc5254d3206 2018-04-08 20:13:05 +0200 Even Rouault $")
 
 enum PDSLayout
 {
@@ -62,7 +63,7 @@ enum PDSLayout
 /* ==================================================================== */
 /************************************************************************/
 
-class PDSDataset : public RawDataset
+class PDSDataset final: public RawDataset
 {
     VSILFILE    *fpImage;  // image data file.
     GDALDataset *poCompressedDS;
@@ -126,8 +127,8 @@ public:
 /************************************************************************/
 
 PDSDataset::PDSDataset() :
-    fpImage(NULL),
-    poCompressedDS(NULL),
+    fpImage(nullptr),
+    poCompressedDS(nullptr),
     bGotTransform(FALSE)
 {
     adfGeoTransform[0] = 0.0;
@@ -145,11 +146,11 @@ PDSDataset::PDSDataset() :
 PDSDataset::~PDSDataset()
 
 {
-    FlushCache();
-    if( fpImage != NULL )
+    PDSDataset::FlushCache();
+    if( fpImage != nullptr )
         VSIFCloseL( fpImage );
 
-    CloseDependentDatasets();
+    PDSDataset::CloseDependentDatasets();
 }
 
 /************************************************************************/
@@ -164,7 +165,7 @@ int PDSDataset::CloseDependentDatasets()
     {
         bHasDroppedRef = FALSE;
         delete poCompressedDS;
-        poCompressedDS = NULL;
+        poCompressedDS = nullptr;
     }
 
     for( int iBand = 0; iBand < nBands; iBand++ )
@@ -185,7 +186,7 @@ char **PDSDataset::GetFileList()
 {
     char **papszFileList = RawDataset::GetFileList();
 
-    if( poCompressedDS != NULL )
+    if( poCompressedDS != nullptr )
     {
         char **papszCFileList = poCompressedDS->GetFileList();
 
@@ -212,7 +213,7 @@ CPLErr PDSDataset::IBuildOverviews( const char *pszResampling,
                                     GDALProgressFunc pfnProgress,
                                     void * pProgressData )
 {
-    if( poCompressedDS != NULL )
+    if( poCompressedDS != nullptr )
         return poCompressedDS->BuildOverviews( pszResampling,
                                                nOverviews, panOverviewList,
                                                nListBands, panBandList,
@@ -238,7 +239,7 @@ CPLErr PDSDataset::IRasterIO( GDALRWFlag eRWFlag,
                               GDALRasterIOExtraArg* psExtraArg)
 
 {
-    if( poCompressedDS != NULL )
+    if( poCompressedDS != nullptr )
         return poCompressedDS->RasterIO( eRWFlag,
                                          nXOff, nYOff, nXSize, nYSize,
                                          pData, nBufXSize, nBufYSize,
@@ -463,6 +464,9 @@ void PDSDataset::ParseSRS()
     } else if (EQUAL( map_proj_name, "MERCATOR" )) {
         oSRS.SetMercator ( center_lat, center_lon, 1, 0, 0 );
     } else if (EQUAL( map_proj_name, "STEREOGRAPHIC" )) {
+        if ( (fabs(center_lat)-90) < 0.0000001 ) {
+                oSRS.SetPS ( center_lat, center_lon, 1, 0, 0 );
+        } else
         oSRS.SetStereographic ( center_lat, center_lon, 1, 0, 0 );
     } else if (EQUAL( map_proj_name, "POLAR_STEREOGRAPHIC")) {
         oSRS.SetPS ( center_lat, center_lon, 1, 0, 0 );
@@ -568,7 +572,7 @@ void PDSDataset::ParseSRS()
         }
 
         // translate back into a projection string.
-        char *pszResult = NULL;
+        char *pszResult = nullptr;
         oSRS.exportToWkt( &pszResult );
         osProjection = pszResult;
         CPLFree( pszResult );
@@ -583,7 +587,7 @@ void PDSDataset::ParseSRS()
         const char  *pszPrjFile = CPLFormCIFilename( osPath, osName, "prj" );
 
         VSILFILE *fp = VSIFOpenL( pszPrjFile, "r" );
-        if( fp != NULL )
+        if( fp != nullptr )
         {
             OGRSpatialReference oSRS2;
 
@@ -593,7 +597,7 @@ void PDSDataset::ParseSRS()
 
             if( oSRS2.importFromESRI( papszLines ) == OGRERR_NONE )
             {
-                char *pszResult = NULL;
+                char *pszResult = nullptr;
                 oSRS2.exportToWkt( &pszResult );
                 osProjection = pszResult;
                 CPLFree( pszResult );
@@ -688,11 +692,13 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
         osQube = "\"";
         osQube += GetKeywordSub( osPrefix + "^" + osImageKeyword, 1 );
         osQube +=  "\"";
-        nDetachedOffset = atoi(GetKeywordSub( osPrefix + "^" + osImageKeyword, 2, "1")) - 1;
+        nDetachedOffset = atoi(GetKeywordSub( osPrefix + "^" + osImageKeyword, 2, "1"));
+        if( nDetachedOffset >= 1 )
+            nDetachedOffset -= 1;
 
         // If this is not explicitly in bytes, then it is assumed to be in
         // records, and we need to translate to bytes.
-        if (strstr(GetKeywordSub(osPrefix + "^" + osImageKeyword, 2),"<BYTES>") != NULL)
+        if (strstr(GetKeywordSub(osPrefix + "^" + osImageKeyword, 2),"<BYTES>") != nullptr)
             bDetachedOffsetInBytes = true;
     }
 
@@ -707,7 +713,7 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
         else
         {
             CPLString osTPath = CPLGetPath(GetDescription());
-            osTargetFile = CPLFormCIFilename( osTPath, osFilename, NULL );
+            osTargetFile = CPLFormCIFilename( osTPath, osFilename, nullptr );
             osExternalCube = osTargetFile;
         }
     }
@@ -777,23 +783,36 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
         record_bytes = atoi(GetKeyword(osPrefix+"RECORD_BYTES"));
 
     // this can happen with "record_type = undefined".
+    if( record_bytes < 0 )
+        return FALSE;
     if( record_bytes == 0 )
         record_bytes = 1;
 
     int nSkipBytes = 0;
-    if( nQube >0 && osQube.find("<BYTES>") != CPLString::npos )
-        nSkipBytes = nQube - 1;
-    else if (nQube > 0 )
-        nSkipBytes = (nQube - 1) * record_bytes;
-    else if( nDetachedOffset > 0 )
+    try
     {
-        if (bDetachedOffsetInBytes)
-            nSkipBytes = nDetachedOffset;
+        if( osQube.find("<BYTES>") != CPLString::npos )
+            nSkipBytes = (CPLSM(nQube) - CPLSM(1)).v();
+        else if (nQube > 0 )
+        {
+            nSkipBytes = (CPLSM(nQube - 1) * CPLSM(record_bytes)).v();
+        }
+        else if( nDetachedOffset > 0 )
+        {
+            if (bDetachedOffsetInBytes)
+                nSkipBytes = nDetachedOffset;
+            else
+            {
+                nSkipBytes = (CPLSM(nDetachedOffset) * CPLSM(record_bytes)).v();
+            }
+        }
         else
-            nSkipBytes = nDetachedOffset * record_bytes;
+            nSkipBytes = 0;
+    }
+    catch( const CPLSafeIntOverflow& )
+    {
+        return FALSE;
     }
-    else
-        nSkipBytes = 0;
 
     nSkipBytes += atoi(GetKeyword(osPrefix+"IMAGE.LINE_PREFIX_BYTES",""));
 
@@ -826,8 +845,8 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
     double dfNoData = 0.0;
     double dfScale = 1.0;
     double dfOffset = 0.0;
-    const char *pszUnit = NULL;
-    const char *pszDesc = NULL;
+    const char *pszUnit = nullptr;
+    const char *pszDesc = nullptr;
 
     CPLString osSB = GetKeyword(osPrefix+"IMAGE.SAMPLE_BITS","");
     if ( !osSB.empty() )
@@ -839,7 +858,7 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
             dfNoData = NULL1;
             break;
           case 16 :
-            if( strstr(osST,"UNSIGNED") != NULL )
+            if( strstr(osST,"UNSIGNED") != nullptr )
             {
                 dfNoData = NULL1;
                 eDataType = GDT_UInt16;
@@ -877,7 +896,7 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
             eDataType = GDT_Byte;
             break;
           case 2 :
-            if( strstr(osST,"UNSIGNED") != NULL )
+            if( strstr(osST,"UNSIGNED") != nullptr )
                 eDataType = GDT_UInt16;
             else
                 eDataType = GDT_Int16;
@@ -914,8 +933,8 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
             GetKeyword(osPrefix + "SPECTRAL_QUBE.CORE_BASE", "0.0") );
         dfScale = CPLAtofM(
             GetKeyword(osPrefix + "SPECTRAL_QUBE.CORE_MULTIPLIER", "1.0") );
-        pszUnit = GetKeyword(osPrefix + "SPECTRAL_QUBE.CORE_UNIT", NULL);
-        pszDesc = GetKeyword(osPrefix + "SPECTRAL_QUBE.CORE_NAME", NULL);
+        pszUnit = GetKeyword(osPrefix + "SPECTRAL_QUBE.CORE_UNIT", nullptr);
+        pszDesc = GetKeyword(osPrefix + "SPECTRAL_QUBE.CORE_NAME", nullptr);
     }
 
 /* -------------------------------------------------------------------- */
@@ -923,11 +942,11 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
 /*      MISSING or MISSING_CONSTANT keywords are nodata.                */
 /* -------------------------------------------------------------------- */
 
-    const char* pszMissing = GetKeyword( osPrefix+"IMAGE.MISSING", NULL );
-    if( pszMissing == NULL )
-        pszMissing = GetKeyword( osPrefix+"IMAGE.MISSING_CONSTANT", NULL );
+    const char* pszMissing = GetKeyword( osPrefix+"IMAGE.MISSING", nullptr );
+    if( pszMissing == nullptr )
+        pszMissing = GetKeyword( osPrefix+"IMAGE.MISSING_CONSTANT", nullptr );
 
-    if( pszMissing != NULL )
+    if( pszMissing != nullptr )
     {
         if( *pszMissing == '"' )
             pszMissing ++;
@@ -950,11 +969,9 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
 /*      this never having been considered to be a match. This isn't     */
 /*      an error!                                                       */
 /* -------------------------------------------------------------------- */
-    if( nRows < 1 || nCols < 1 || l_nBands < 1 )
+    if( !GDALCheckDatasetDimensions(nCols, nRows) ||
+        !GDALCheckBandCount(l_nBands, false) )
     {
-        CPLError( CE_Failure, CPLE_AppDefined,
-                  "File %s appears to be a PDS file, but failed to find some required keywords.",
-                  GetDescription() );
         return FALSE;
     }
 
@@ -971,7 +988,7 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
     if( eAccess == GA_ReadOnly )
     {
         fpImage = VSIFOpenL( osTargetFile, "rb" );
-        if( fpImage == NULL )
+        if( fpImage == nullptr )
         {
             CPLError( CE_Failure, CPLE_OpenFailed,
                     "Failed to open %s.\n%s",
@@ -983,7 +1000,7 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
     else
     {
         fpImage = VSIFOpenL( osTargetFile, "r+b" );
-        if( fpImage == NULL )
+        if( fpImage == nullptr )
         {
             CPLError( CE_Failure, CPLE_OpenFailed,
                     "Failed to open %s with write permission.\n%s",
@@ -1001,27 +1018,37 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
     int nPixelOffset;
     int nBandOffset;
 
-    if( eLayout == PDS_BIP )
+    try
     {
-        nPixelOffset = nItemSize * l_nBands;
-        nBandOffset = nItemSize;
-        nLineOffset = ((nPixelOffset * nCols + record_bytes - 1)/record_bytes)
-            * record_bytes;
+        if( eLayout == PDS_BIP )
+        {
+            nPixelOffset = (CPLSM(nItemSize) * CPLSM(l_nBands)).v();
+            nBandOffset = nItemSize;
+            nLineOffset = (CPLSM(nPixelOffset) * CPLSM(nCols)).v();
+            nLineOffset = (CPLSM(DIV_ROUND_UP(nLineOffset, record_bytes ))
+                * CPLSM(record_bytes)).v();
+        }
+        else if( eLayout == PDS_BSQ )
+        {
+            nPixelOffset = nItemSize;
+            nLineOffset = (CPLSM(nPixelOffset) * CPLSM(nCols)).v();
+            nLineOffset = (CPLSM(DIV_ROUND_UP(nLineOffset, record_bytes ))
+                * CPLSM(record_bytes)).v();
+            nBandOffset = (CPLSM(nLineOffset) * CPLSM(nRows)
+                + CPLSM(nSuffixLines) * (CPLSM(nCols) + CPLSM(nSuffixItems)) * CPLSM(nSuffixBytes)).v();
+        }
+        else /* assume BIL */
+        {
+            nPixelOffset = nItemSize;
+            nBandOffset = (CPLSM(nItemSize) * CPLSM(nCols)).v();
+            nLineOffset = (CPLSM(nBandOffset) * CPLSM(nCols)).v();
+            nLineOffset = (CPLSM(DIV_ROUND_UP(nLineOffset, record_bytes))
+                * CPLSM(record_bytes)).v();
+        }
     }
-    else if( eLayout == PDS_BSQ )
+    catch( const CPLSafeIntOverflow& )
     {
-        nPixelOffset = nItemSize;
-        nLineOffset = ((nPixelOffset * nCols + record_bytes - 1)/record_bytes)
-            * record_bytes;
-        nBandOffset = nLineOffset * nRows
-            + nSuffixLines * (nCols + nSuffixItems) * nSuffixBytes;
-    }
-    else /* assume BIL */
-    {
-        nPixelOffset = nItemSize;
-        nBandOffset = nItemSize * nCols;
-        nLineOffset = ((nBandOffset * nCols + record_bytes - 1)/record_bytes)
-            * record_bytes;
+        return FALSE;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1031,7 +1058,7 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
     {
         RawRasterBand *poBand =
             new RawRasterBand( this, i+1, fpImage,
-                               nSkipBytes + nBandOffset * i,
+                               nSkipBytes + static_cast<vsi_l_offset>(nBandOffset) * i,
                                nPixelOffset, nLineOffset, eDataType,
 #ifdef CPL_LSB
                                chByteOrder == 'I' || chByteOrder == 'L',
@@ -1042,12 +1069,12 @@ int PDSDataset::ParseImage( CPLString osPrefix, CPLString osFilenamePrefix )
 
         if( l_nBands == 1 )
         {
-            const char* pszMin = GetKeyword(osPrefix+"IMAGE.MINIMUM", NULL);
-            const char* pszMax = GetKeyword(osPrefix+"IMAGE.MAXIMUM", NULL);
-            const char* pszMean = GetKeyword(osPrefix+"IMAGE.MEAN", NULL);
-            const char* pszStdDev= GetKeyword(osPrefix+"IMAGE.STANDARD_DEVIATION", NULL);
-            if (pszMin != NULL && pszMax != NULL &&
-                pszMean != NULL && pszStdDev != NULL)
+            const char* pszMin = GetKeyword(osPrefix+"IMAGE.MINIMUM", nullptr);
+            const char* pszMax = GetKeyword(osPrefix+"IMAGE.MAXIMUM", nullptr);
+            const char* pszMean = GetKeyword(osPrefix+"IMAGE.MEAN", nullptr);
+            const char* pszStdDev= GetKeyword(osPrefix+"IMAGE.STANDARD_DEVIATION", nullptr);
+            if (pszMin != nullptr && pszMax != nullptr &&
+                pszMean != nullptr && pszStdDev != nullptr)
             {
                 poBand->SetStatistics( CPLAtofM(pszMin),
                                        CPLAtofM(pszMax),
@@ -1107,12 +1134,12 @@ int PDSDataset::ParseCompressedImage()
     CleanString( osFileName );
 
     const CPLString osPath = CPLGetPath(GetDescription());
-    const CPLString osFullFileName = CPLFormFilename( osPath, osFileName, NULL );
+    const CPLString osFullFileName = CPLFormFilename( osPath, osFileName, nullptr );
 
     poCompressedDS = reinterpret_cast<GDALDataset *>(
         GDALOpen( osFullFileName, GA_ReadOnly ) );
 
-    if( poCompressedDS == NULL )
+    if( poCompressedDS == nullptr )
         return FALSE;
 
     nRasterXSize = poCompressedDS->GetRasterXSize();
@@ -1133,13 +1160,13 @@ int PDSDataset::ParseCompressedImage()
 int PDSDataset::Identify( GDALOpenInfo * poOpenInfo )
 
 {
-    if( poOpenInfo->pabyHeader == NULL )
+    if( poOpenInfo->pabyHeader == nullptr )
         return FALSE;
 
     return strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ),
-                  "PDS_VERSION_ID") != NULL ||
+                  "PDS_VERSION_ID") != nullptr ||
            strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ),
-                  "ODL_VERSION_ID") != NULL;
+                  "ODL_VERSION_ID") != nullptr;
 }
 
 /************************************************************************/
@@ -1148,26 +1175,24 @@ int PDSDataset::Identify( GDALOpenInfo * poOpenInfo )
 
 GDALDataset *PDSDataset::Open( GDALOpenInfo * poOpenInfo )
 {
-    if( !Identify( poOpenInfo ) )
-        return NULL;
+    if( !Identify( poOpenInfo ) || poOpenInfo->fpL == nullptr )
+        return nullptr;
 
     if( strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ),
-                  "PDS_VERSION_ID") != NULL &&
-        strstr((const char *)poOpenInfo->pabyHeader,"PDS3") == NULL )
+                  "PDS_VERSION_ID") != nullptr &&
+        strstr((const char *)poOpenInfo->pabyHeader,"PDS3") == nullptr )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "It appears this is an older PDS image type.  Only PDS_VERSION_ID = PDS3 are currently supported by this gdal PDS reader.");
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
-/*      Open and parse the keyword header.  Sometimes there is stuff    */
+/*      Parse the keyword header.  Sometimes there is stuff             */
 /*      before the PDS_VERSION_ID, which we want to ignore.             */
 /* -------------------------------------------------------------------- */
-    VSILFILE *fpQube = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
-
-    if( fpQube == NULL )
-        return NULL;
+    VSILFILE *fpQube = poOpenInfo->fpL;
+    poOpenInfo->fpL = nullptr;
 
     PDSDataset *poDS = new PDSDataset();
     poDS->SetDescription( poOpenInfo->pszFilename );
@@ -1182,7 +1207,7 @@ GDALDataset *PDSDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         delete poDS;
         VSIFCloseL( fpQube );
-        return NULL;
+        return nullptr;
     }
     VSIFCloseL( fpQube );
 
@@ -1210,8 +1235,8 @@ GDALDataset *PDSDataset::Open( GDALOpenInfo * poOpenInfo )
         !osUncompressedFilename.empty() )
     {
         const CPLString osPath = CPLGetPath(poDS->GetDescription());
-        osCompressedFilename = CPLFormFilename( osPath, osCompressedFilename, NULL );
-        osUncompressedFilename = CPLFormFilename( osPath, osUncompressedFilename, NULL );
+        osCompressedFilename = CPLFormFilename( osPath, osCompressedFilename, nullptr );
+        osUncompressedFilename = CPLFormFilename( osPath, osUncompressedFilename, nullptr );
         if( VSIStatExL(osCompressedFilename, &sStat, VSI_STAT_EXISTS_FLAG) == 0 &&
             VSIStatExL(osUncompressedFilename, &sStat, VSI_STAT_EXISTS_FLAG) != 0 )
         {
@@ -1226,7 +1251,7 @@ GDALDataset *PDSDataset::Open( GDALOpenInfo * poOpenInfo )
         if( !poDS->ParseCompressedImage() )
         {
             delete poDS;
-            return NULL;
+            return nullptr;
         }
     }
     else
@@ -1239,7 +1264,7 @@ GDALDataset *PDSDataset::Open( GDALOpenInfo * poOpenInfo )
         if( !poDS->ParseImage(osPrefix, osFilenamePrefix) )
         {
             delete poDS;
-            return NULL;
+            return nullptr;
         }
     }
 
@@ -1257,13 +1282,13 @@ GDALDataset *PDSDataset::Open( GDALOpenInfo * poOpenInfo )
           "SPACECRAFT_NAME", "INSTRUMENT_NAME", "INSTRUMENT_ID",
           "TARGET_NAME", "CENTER_FILTER_WAVELENGTH", "BANDWIDTH",
           "PRODUCT_CREATION_TIME", "NOTE",
-          NULL };
+          nullptr };
 
-    for( int i = 0; apszKeywords[i] != NULL; i++ )
+    for( int i = 0; apszKeywords[i] != nullptr; i++ )
     {
         const char *pszKeywordValue = poDS->GetKeyword( apszKeywords[i] );
 
-        if( pszKeywordValue != NULL )
+        if( pszKeywordValue != nullptr )
             poDS->SetMetadataItem( apszKeywords[i], pszKeywordValue );
     }
 
@@ -1300,9 +1325,9 @@ const char *PDSDataset::GetKeywordSub( std::string osPath,
                                        const char *pszDefault )
 
 {
-    const char *pszResult = oKeywords.GetKeyword( osPath.c_str(), NULL );
+    const char *pszResult = oKeywords.GetKeyword( osPath.c_str(), nullptr );
 
-    if( pszResult == NULL )
+    if( pszResult == nullptr )
         return pszDefault;
 
     if( pszResult[0] != '(' )
@@ -1331,9 +1356,9 @@ const char *PDSDataset::GetKeywordUnit( const char *pszPath,
                                          const char *pszDefault )
 
 {
-    const char *pszResult = oKeywords.GetKeyword( pszPath, NULL );
+    const char *pszResult = oKeywords.GetKeyword( pszPath, nullptr );
 
-    if( pszResult == NULL )
+    if( pszResult == nullptr )
         return pszDefault;
 
     char **papszTokens = CSLTokenizeString2( pszResult, "</>",
@@ -1385,7 +1410,7 @@ void PDSDataset::CleanString( CPLString &osInput )
 void GDALRegister_PDS()
 
 {
-    if( GDALGetDriverByName( "PDS" ) != NULL )
+    if( GDALGetDriverByName( "PDS" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
