@@ -32,7 +32,7 @@
 #include "ogr_srs_api.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: ctable2dataset.cpp 36501 2016-11-25 14:09:24Z rouault $");
+CPL_CVSID("$Id: ctable2dataset.cpp f946b03da9b425043fed0a0c58a329295240840f 2017-12-15 13:36:23Z Kurt Schwehr $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -48,13 +48,13 @@ class CTable2Dataset : public RawDataset
     double      adfGeoTransform[6];
 
   public:
-                CTable2Dataset();
-    virtual ~CTable2Dataset();
+    CTable2Dataset();
+    ~CTable2Dataset() override;
 
-    virtual CPLErr SetGeoTransform( double * padfTransform ) override;
-    virtual CPLErr GetGeoTransform( double * padfTransform ) override;
-    virtual const char *GetProjectionRef() override;
-    virtual void   FlushCache(void) override;
+    CPLErr SetGeoTransform( double * padfTransform ) override;
+    CPLErr GetGeoTransform( double * padfTransform ) override;
+    const char *GetProjectionRef() override;
+    void   FlushCache(void) override;
 
     static GDALDataset *Open( GDALOpenInfo * );
     static int          Identify( GDALOpenInfo * );
@@ -74,7 +74,7 @@ class CTable2Dataset : public RawDataset
 /************************************************************************/
 
 CTable2Dataset::CTable2Dataset() :
-    fpImage(NULL)
+    fpImage(nullptr)
 {
     memset( adfGeoTransform, 0, sizeof(adfGeoTransform) );
 }
@@ -88,7 +88,7 @@ CTable2Dataset::~CTable2Dataset()
 {
     FlushCache();
 
-    if( fpImage != NULL )
+    if( fpImage != nullptr )
     {
         if( VSIFCloseL( fpImage ) != 0 )
         {
@@ -133,7 +133,7 @@ GDALDataset *CTable2Dataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
     if( !Identify( poOpenInfo ) )
-        return NULL;
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
@@ -151,10 +151,10 @@ GDALDataset *CTable2Dataset::Open( GDALOpenInfo * poOpenInfo )
     else
         poDS->fpImage = VSIFOpenL( osFilename, "rb+" );
 
-    if( poDS->fpImage == NULL )
+    if( poDS->fpImage == nullptr )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -187,10 +187,12 @@ GDALDataset *CTable2Dataset::Open( GDALOpenInfo * poOpenInfo )
     int nRasterXSize, nRasterYSize;
     memcpy( &nRasterXSize, achHeader + 128, 4 );
     memcpy( &nRasterYSize, achHeader + 132, 4 );
-    if (!GDALCheckDatasetDimensions(nRasterXSize, nRasterYSize))
+    if (!GDALCheckDatasetDimensions(nRasterXSize, nRasterYSize) ||
+        /* to avoid overflow in later -8 * nRasterXSize computation */
+        nRasterXSize >= INT_MAX / 8 )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     poDS->nRasterXSize = nRasterXSize;
@@ -214,7 +216,8 @@ GDALDataset *CTable2Dataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     RawRasterBand *poBand =
         new RawRasterBand( poDS, 1, poDS->fpImage,
-                           160 + 4 + nRasterXSize * (nRasterYSize-1) * 2 * 4,
+                           160 + 4 + static_cast<vsi_l_offset>(nRasterXSize) *
+                                (nRasterYSize-1) * 2 * 4,
                            8, -8 * nRasterXSize,
                            GDT_Float32, CPL_IS_LSB, TRUE, FALSE );
     poBand->SetDescription( "Latitude Offset (radians)" );
@@ -222,7 +225,8 @@ GDALDataset *CTable2Dataset::Open( GDALOpenInfo * poOpenInfo )
 
     poBand =
         new RawRasterBand( poDS, 2, poDS->fpImage,
-                           160 + nRasterXSize * (nRasterYSize-1) * 2 * 4,
+                           160 + static_cast<vsi_l_offset>(nRasterXSize) *
+                                (nRasterYSize-1) * 2 * 4,
                            8, -8 * nRasterXSize,
                            GDT_Float32, CPL_IS_LSB, TRUE, FALSE );
     poBand->SetDescription( "Longitude Offset (radians)" );
@@ -343,7 +347,7 @@ GDALDataset *CTable2Dataset::Create( const char * pszFilename,
             CE_Failure, CPLE_AppDefined,
             "Attempt to create CTable2 file with unsupported data type '%s'.",
             GDALGetDataTypeName( eType ) );
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -351,12 +355,12 @@ GDALDataset *CTable2Dataset::Create( const char * pszFilename,
 /* -------------------------------------------------------------------- */
     VSILFILE *fp = VSIFOpenL( pszFilename, "wb" );
 
-    if( fp == NULL )
+    if( fp == nullptr )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "Attempt to create file `%s' failed.\n",
                   pszFilename );
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -368,7 +372,7 @@ GDALDataset *CTable2Dataset::Create( const char * pszFilename,
 
     memcpy( achHeader+0, "CTABLE V2.0     ", 16 );
 
-    if( CSLFetchNameValue( papszOptions, "DESCRIPTION" ) != NULL )
+    if( CSLFetchNameValue( papszOptions, "DESCRIPTION" ) != nullptr )
         strncpy( achHeader + 16,
                  CSLFetchNameValue( papszOptions, "DESCRIPTION" ),
                  80 );
@@ -418,7 +422,7 @@ GDALDataset *CTable2Dataset::Create( const char * pszFilename,
             CPLError( CE_Failure, CPLE_FileIO,
                       "Write failed at line %d, perhaps the disk is full?",
                       i );
-            return NULL;
+            return nullptr;
         }
     }
 
@@ -430,7 +434,7 @@ GDALDataset *CTable2Dataset::Create( const char * pszFilename,
     if( VSIFCloseL( fp ) != 0 )
     {
         CPLError(CE_Failure, CPLE_FileIO, "I/O error");
-        return NULL;
+        return nullptr;
     }
 
     return (GDALDataset *) GDALOpen( pszFilename, GA_Update );
@@ -443,7 +447,7 @@ GDALDataset *CTable2Dataset::Create( const char * pszFilename,
 void GDALRegister_CTable2()
 
 {
-    if( GDALGetDriverByName( "CTable2" ) != NULL )
+    if( GDALGetDriverByName( "CTable2" ) != nullptr )
       return;
 
     GDALDriver *poDriver = new GDALDriver();
