@@ -32,7 +32,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: eirdataset.cpp 36501 2016-11-25 14:09:24Z rouault $");
+CPL_CVSID("$Id: eirdataset.cpp c0a1e4f5188da2426ccf0533ecc45aa0c9c468b1 2018-03-10 22:01:02Z Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -52,15 +52,17 @@ class EIRDataset : public RawDataset
     char **papszExtraFiles;
 
     void        ResetKeyValue( const char *pszKey, const char *pszValue );
+#ifdef unused
     const char *GetKeyValue( const char *pszKey, const char *pszDefault = "" );
+#endif
 
   public:
     EIRDataset();
-    virtual ~EIRDataset();
+    ~EIRDataset() override;
 
-    virtual CPLErr GetGeoTransform( double * padfTransform ) override;
+    CPLErr GetGeoTransform( double * padfTransform ) override;
 
-    virtual char **GetFileList() override;
+    char **GetFileList() override;
 
     static int          Identify( GDALOpenInfo * );
     static GDALDataset *Open( GDALOpenInfo * );
@@ -77,11 +79,11 @@ class EIRDataset : public RawDataset
 /************************************************************************/
 
 EIRDataset::EIRDataset() :
-    fpImage(NULL),
+    fpImage(nullptr),
     bGotTransform(false),
     bHDRDirty(false),
-    papszHDR(NULL),
-    papszExtraFiles(NULL)
+    papszHDR(nullptr),
+    papszExtraFiles(nullptr)
 {
     memset( adfGeoTransform, 0, sizeof(adfGeoTransform) );
 }
@@ -109,13 +111,14 @@ EIRDataset::~EIRDataset()
         }
     }
 
-    if( fpImage != NULL )
+    if( fpImage != nullptr )
         CPL_IGNORE_RET_VAL(VSIFCloseL( fpImage ));
 
     CSLDestroy( papszHDR );
     CSLDestroy( papszExtraFiles );
 }
 
+#ifdef unused
 /************************************************************************/
 /*                            GetKeyValue()                             */
 /************************************************************************/
@@ -124,7 +127,7 @@ const char *EIRDataset::GetKeyValue( const char *pszKey,
                                      const char *pszDefault )
 
 {
-    for( int i = 0; papszHDR[i] != NULL; i++ )
+    for( int i = 0; papszHDR[i] != nullptr; i++ )
     {
         if( EQUALN(pszKey,papszHDR[i],strlen(pszKey))
             && isspace((unsigned char)papszHDR[i][strlen(pszKey)]) )
@@ -139,6 +142,7 @@ const char *EIRDataset::GetKeyValue( const char *pszKey,
 
     return pszDefault;
 }
+#endif
 
 /************************************************************************/
 /*                           ResetKeyValue()                            */
@@ -224,7 +228,7 @@ int EIRDataset::Identify( GDALOpenInfo * poOpenInfo )
         return FALSE;
 
     if( strstr((const char *) poOpenInfo->pabyHeader,
-               "IMAGINE_RAW_FILE" ) == NULL )
+               "IMAGINE_RAW_FILE" ) == nullptr )
         return FALSE;
 
     return TRUE;
@@ -237,12 +241,8 @@ int EIRDataset::Identify( GDALOpenInfo * poOpenInfo )
 GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    if( !Identify( poOpenInfo ) )
-        return NULL;
-
-    VSILFILE *fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
-    if( fp == NULL )
-        return NULL;
+    if( !Identify( poOpenInfo ) || poOpenInfo->fpL == nullptr )
+        return nullptr;
 
     /* header example and description
 
@@ -281,7 +281,7 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
     int nBits = 8;
     char chByteOrder = 'M';
     char szLayout[10] = "BIL";
-    char **papszHDR = NULL;
+    char **papszHDR = nullptr;
 
     // default raster file: same name with no extension
     const CPLString osPath = CPLGetPath( poOpenInfo->pszFilename );
@@ -289,13 +289,14 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
     CPLString osRasterFilename = CPLFormCIFilename( osPath, osName, "" );
 
     // parse the header file
-    const char *pszLine = NULL;
-    while( (pszLine = CPLReadLineL( fp )) != NULL )
+    const char *pszLine = nullptr;
+    VSIRewindL(poOpenInfo->fpL);
+    while( (pszLine = CPLReadLineL( poOpenInfo->fpL )) != nullptr )
     {
         nLineCount++;
 
         if ( (nLineCount == 1) && !EQUAL(pszLine, "IMAGINE_RAW_FILE") ) {
-            return NULL;
+            return nullptr;
         }
 
         if ( (nLineCount > 50) || EQUAL(pszLine, "END_RAW_FILE") ) {
@@ -376,8 +377,7 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
                     papszTokens[1] );
                 CSLDestroy( papszTokens );
                 CSLDestroy( papszHDR );
-                CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
-                return NULL;
+                return nullptr;
             }
         }
         else if( EQUAL(papszTokens[0], "BYTE_ORDER") )
@@ -393,8 +393,6 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
         CSLDestroy( papszTokens );
     }
 
-    CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
-
 /* -------------------------------------------------------------------- */
 /*      Did we get the required keywords?  If not we return with        */
 /*      this never having been considered to be a match. This isn't     */
@@ -403,14 +401,14 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
     if( nRows == -1 || nCols == -1 )
     {
         CSLDestroy( papszHDR );
-        return NULL;
+        return nullptr;
     }
 
     if (!GDALCheckDatasetDimensions(nCols, nRows) ||
         !GDALCheckBandCount(nBands, FALSE))
     {
         CSLDestroy( papszHDR );
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -422,7 +420,7 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_NotSupported,
                   "The EIR driver does not support update access to existing"
                   " datasets." );
-        return NULL;
+        return nullptr;
     }
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
@@ -440,13 +438,13 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Open target binary file.                                        */
 /* -------------------------------------------------------------------- */
     poDS->fpImage = VSIFOpenL( osRasterFilename.c_str(), "rb" );
-    if( poDS->fpImage == NULL )
+    if( poDS->fpImage == nullptr )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "Failed to open %s: %s",
                   osRasterFilename.c_str(), VSIStrerror( errno ) );
         delete poDS;
-        return NULL;
+        return nullptr;
     }
     poDS->papszExtraFiles =
             CSLAddString( poDS->papszExtraFiles,
@@ -465,18 +463,33 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
     if( EQUAL(szLayout, "BIP") )
     {
         nPixelOffset = nItemSize * nBands;
+        if( nPixelOffset > INT_MAX / nCols )
+        {
+            delete poDS;
+            return nullptr;
+        }
         nLineOffset = nPixelOffset * nCols;
         nBandOffset = static_cast<vsi_l_offset>(nItemSize);
     }
     else if( EQUAL(szLayout, "BSQ") )
     {
         nPixelOffset = nItemSize;
+        if( nPixelOffset > INT_MAX / nCols )
+        {
+            delete poDS;
+            return nullptr;
+        }
         nLineOffset = nPixelOffset * nCols;
         nBandOffset = static_cast<vsi_l_offset>(nLineOffset) * nRows;
     }
     else /* assume BIL */
     {
         nPixelOffset = nItemSize;
+        if( nItemSize > INT_MAX / nBands || nItemSize * nBands > INT_MAX / nCols )
+        {
+            delete poDS;
+            return nullptr;
+        }
         nLineOffset = nItemSize * nBands * nCols;
         nBandOffset = static_cast<vsi_l_offset>(nItemSize) * nCols;
     }
@@ -510,7 +523,7 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if( !poDS->bGotTransform )
         poDS->bGotTransform = CPL_TO_BOOL(
-            GDALReadWorldFile( poOpenInfo->pszFilename, NULL,
+            GDALReadWorldFile( poOpenInfo->pszFilename, nullptr,
                                poDS->adfGeoTransform ) );
 
     if( !poDS->bGotTransform )
@@ -538,7 +551,7 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_EIR()
 
 {
-    if( GDALGetDriverByName( "EIR" ) != NULL )
+    if( GDALGetDriverByName( "EIR" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
