@@ -32,7 +32,7 @@
 #include "ogr_srs_api.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: loslasdataset.cpp 36501 2016-11-25 14:09:24Z rouault $");
+CPL_CVSID("$Id: loslasdataset.cpp a542b2797f15f2ed694cfcee9ff17d86b339dfee 2018-04-02 00:24:03 +0200 Even Rouault $")
 
 /**
 
@@ -81,11 +81,11 @@ class LOSLASDataset : public RawDataset
     double      adfGeoTransform[6];
 
   public:
-                LOSLASDataset();
-    virtual ~LOSLASDataset();
+    LOSLASDataset();
+    ~LOSLASDataset() override;
 
-    virtual CPLErr GetGeoTransform( double * padfTransform ) override;
-    virtual const char *GetProjectionRef() override;
+    CPLErr GetGeoTransform( double * padfTransform ) override;
+    const char *GetProjectionRef() override;
 
     static GDALDataset *Open( GDALOpenInfo * );
     static int          Identify( GDALOpenInfo * );
@@ -101,7 +101,7 @@ class LOSLASDataset : public RawDataset
 /*                             LOSLASDataset()                          */
 /************************************************************************/
 
-LOSLASDataset::LOSLASDataset() : fpImage(NULL), nRecordLength(0)
+LOSLASDataset::LOSLASDataset() : fpImage(nullptr), nRecordLength(0)
 {
     memset( adfGeoTransform, 0, sizeof(adfGeoTransform) );
 }
@@ -115,7 +115,7 @@ LOSLASDataset::~LOSLASDataset()
 {
     FlushCache();
 
-    if( fpImage != NULL )
+    if( fpImage != nullptr )
         CPL_IGNORE_RET_VAL(VSIFCloseL( fpImage ));
 }
 
@@ -129,9 +129,11 @@ int LOSLASDataset::Identify( GDALOpenInfo *poOpenInfo )
     if( poOpenInfo->nHeaderBytes < 64 )
         return FALSE;
 
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     if( !EQUAL(CPLGetExtension(poOpenInfo->pszFilename),"las")
         && !EQUAL(CPLGetExtension(poOpenInfo->pszFilename),"los") )
         return FALSE;
+#endif
 
     if( !STARTS_WITH_CI((const char *)poOpenInfo->pabyHeader + 56, "NADGRD") )
         return FALSE;
@@ -146,24 +148,26 @@ int LOSLASDataset::Identify( GDALOpenInfo *poOpenInfo )
 GDALDataset *LOSLASDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    if( !Identify( poOpenInfo ) )
-        return NULL;
+    if( !Identify( poOpenInfo ) || poOpenInfo->fpL == nullptr )
+        return nullptr;
+
+/* -------------------------------------------------------------------- */
+/*      Confirm the requested access is supported.                      */
+/* -------------------------------------------------------------------- */
+    if( poOpenInfo->eAccess == GA_Update )
+    {
+        CPLError( CE_Failure, CPLE_NotSupported,
+                  "The LOSLAS driver does not support update access to existing"
+                  " datasets." );
+        return nullptr;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
     LOSLASDataset *poDS = new LOSLASDataset();
-
-/* -------------------------------------------------------------------- */
-/*      Open the file.                                                  */
-/* -------------------------------------------------------------------- */
-    poDS->fpImage = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
-
-    if( poDS->fpImage == NULL )
-    {
-        delete poDS;
-        return NULL;
-    }
+    poDS->fpImage = poOpenInfo->fpL;
+    poOpenInfo->fpL = nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Read the header.                                                */
@@ -176,10 +180,11 @@ GDALDataset *LOSLASDataset::Open( GDALOpenInfo * poOpenInfo )
     CPL_LSBPTR32( &(poDS->nRasterXSize) );
     CPL_LSBPTR32( &(poDS->nRasterYSize) );
 
-    if (!GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize))
+    if (!GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize) ||
+        poDS->nRasterXSize > (INT_MAX - 4) / 4 )
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     CPL_IGNORE_RET_VAL(VSIFSeekL( poDS->fpImage, 76, SEEK_SET ));
@@ -207,7 +212,8 @@ GDALDataset *LOSLASDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->SetBand(
         1, new RawRasterBand( poDS, 1, poDS->fpImage,
-                              poDS->nRasterYSize * poDS->nRecordLength + 4,
+                              static_cast<vsi_l_offset>(poDS->nRasterYSize) *
+                                    poDS->nRecordLength + 4,
                               4, -1 * poDS->nRecordLength,
                               GDT_Float32,
                               CPL_IS_LSB, TRUE, FALSE ) );
@@ -264,7 +270,7 @@ const char *LOSLASDataset::GetProjectionRef()
 void GDALRegister_LOSLAS()
 
 {
-    if( GDALGetDriverByName( "LOSLAS" ) != NULL )
+    if( GDALGetDriverByName( "LOSLAS" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
