@@ -32,7 +32,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: ndfdataset.cpp 36682 2016-12-04 20:34:45Z rouault $");
+CPL_CVSID("$Id: ndfdataset.cpp a542b2797f15f2ed694cfcee9ff17d86b339dfee 2018-04-02 00:24:03 +0200 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -51,14 +51,15 @@ class NDFDataset : public RawDataset
     const char  *Get( const char *pszKey, const char *pszDefault);
 
   public:
-                NDFDataset();
-    virtual ~NDFDataset();
+    NDFDataset();
+    ~NDFDataset() override;
 
-    virtual CPLErr  GetGeoTransform( double * padfTransform ) override;
-    virtual const char *GetProjectionRef(void) override;
-    virtual char **GetFileList(void) override;
+    CPLErr  GetGeoTransform( double * padfTransform ) override;
+    const char *GetProjectionRef(void) override;
+    char **GetFileList(void) override;
 
     static GDALDataset *Open( GDALOpenInfo * );
+    static int Identify( GDALOpenInfo * );
 };
 
 /************************************************************************/
@@ -67,8 +68,8 @@ class NDFDataset : public RawDataset
 
 NDFDataset::NDFDataset() :
     pszProjection(CPLStrdup("")),
-    papszExtraFiles(NULL),
-    papszHeader(NULL)
+    papszExtraFiles(nullptr),
+    papszHeader(nullptr)
 {
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
@@ -129,7 +130,7 @@ const char *NDFDataset::Get( const char *pszKey, const char *pszDefault )
 {
     const char *pszResult = CSLFetchNameValue( papszHeader, pszKey );
 
-    if( pszResult == NULL )
+    if( pszResult == nullptr )
         return pszDefault;
 
     return pszResult;
@@ -152,6 +153,21 @@ char **NDFDataset::GetFileList()
     return papszFileList;
 }
 
+/************************************************************************/
+/*                            Identify()                                */
+/************************************************************************/
+
+int NDFDataset::Identify( GDALOpenInfo * poOpenInfo )
+
+{
+/* -------------------------------------------------------------------- */
+/*      The user must select the header file (i.e. .H1).                */
+/* -------------------------------------------------------------------- */
+    return poOpenInfo->nHeaderBytes >= 50 &&
+           (STARTS_WITH_CI(reinterpret_cast<const char *>( poOpenInfo->pabyHeader ), "NDF_REVISION=2")
+            || STARTS_WITH_CI(reinterpret_cast<const char *>( poOpenInfo->pabyHeader ), "NDF_REVISION=0")) ;
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -162,35 +178,37 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      The user must select the header file (i.e. .H1).                */
 /* -------------------------------------------------------------------- */
-    if( poOpenInfo->nHeaderBytes < 50 )
-        return NULL;
-
-    if( !STARTS_WITH_CI(reinterpret_cast<const char *>( poOpenInfo->pabyHeader ), "NDF_REVISION=2")
-        && !STARTS_WITH_CI(reinterpret_cast<const char *>( poOpenInfo->pabyHeader ), "NDF_REVISION=0") )
-        return NULL;
+    if( !Identify(poOpenInfo) || poOpenInfo->fpL == nullptr )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
+/*      Confirm the requested access is supported.                      */
+/* -------------------------------------------------------------------- */
+    if( poOpenInfo->eAccess == GA_Update )
+    {
+        CPLError( CE_Failure, CPLE_NotSupported,
+                  "The NDF driver does not support update access to existing"
+                  " datasets." );
+        return nullptr;
+    }
+/* -------------------------------------------------------------------- */
 /*      Read and process the header into a local name/value             */
 /*      stringlist.  We just take off the trailing semicolon.  The      */
 /*      keyword is already separated from the value by an equal         */
 /*      sign.                                                           */
 /* -------------------------------------------------------------------- */
 
-    VSILFILE* fp = VSIFOpenL(poOpenInfo->pszFilename, "rb");
-    if (fp == NULL)
-        return NULL;
-
-    const char *pszLine = NULL;
+    const char *pszLine = nullptr;
     const int nHeaderMax = 1000;
     int nHeaderLines = 0;
     char **papszHeader = static_cast<char **>(
         CPLMalloc( sizeof(char *) * (nHeaderMax+1) ) );
 
     while( nHeaderLines < nHeaderMax
-           && (pszLine = CPLReadLineL( fp )) != NULL
+           && (pszLine = CPLReadLineL( poOpenInfo->fpL )) != nullptr
            && !EQUAL(pszLine,"END_OF_HDR;") )
     {
-        if( strstr(pszLine,"=") == NULL )
+        if( strstr(pszLine,"=") == nullptr )
             break;
 
         char *pszFixed = CPLStrdup( pszLine );
@@ -198,20 +216,20 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
             pszFixed[strlen(pszFixed)-1] = '\0';
 
         papszHeader[nHeaderLines++] = pszFixed;
-        papszHeader[nHeaderLines] = NULL;
+        papszHeader[nHeaderLines] = nullptr;
     }
-    CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
-    fp = NULL;
+    CPL_IGNORE_RET_VAL(VSIFCloseL(poOpenInfo->fpL));
+    poOpenInfo->fpL = nullptr;
 
-    if( CSLFetchNameValue( papszHeader, "PIXELS_PER_LINE" ) == NULL
-        || CSLFetchNameValue( papszHeader, "LINES_PER_DATA_FILE" ) == NULL
-        || CSLFetchNameValue( papszHeader, "BITS_PER_PIXEL" ) == NULL
-        || CSLFetchNameValue( papszHeader, "PIXEL_FORMAT" ) == NULL )
+    if( CSLFetchNameValue( papszHeader, "PIXELS_PER_LINE" ) == nullptr
+        || CSLFetchNameValue( papszHeader, "LINES_PER_DATA_FILE" ) == nullptr
+        || CSLFetchNameValue( papszHeader, "BITS_PER_PIXEL" ) == nullptr
+        || CSLFetchNameValue( papszHeader, "PIXEL_FORMAT" ) == nullptr )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
               "Dataset appears to be NDF but is missing a required field.");
         CSLDestroy( papszHeader );
-        return NULL;
+        return nullptr;
     }
 
     if( !EQUAL(CSLFetchNameValue( papszHeader, "PIXEL_FORMAT"), "BYTE" )
@@ -220,7 +238,7 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Currently NDF driver supports only 8bit BYTE format." );
         CSLDestroy( papszHeader );
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -232,7 +250,7 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_NotSupported,
                   "The NDF driver does not support update access to existing"
                   " datasets.\n" );
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -249,11 +267,11 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     const char* pszBand = CSLFetchNameValue(papszHeader,
                                             "NUMBER_OF_BANDS_IN_VOLUME");
-    if (pszBand == NULL)
+    if (pszBand == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Cannot find band count");
         delete poDS;
-        return NULL;
+        return nullptr;
     }
     const int nBands = atoi(pszBand);
 
@@ -261,7 +279,7 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
         !GDALCheckBandCount(nBands, FALSE))
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
     for( int iBand = 0; iBand < nBands; iBand++ )
@@ -281,17 +299,17 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
         else
         {
             CPLString osBasePath = CPLGetPath(poOpenInfo->pszFilename);
-            osFilename = CPLFormFilename( osBasePath, osFilename, NULL);
+            osFilename = CPLFormFilename( osBasePath, osFilename, nullptr);
         }
 
         VSILFILE *fpRaw = VSIFOpenL( osFilename, "rb" );
-        if( fpRaw == NULL )
+        if( fpRaw == nullptr )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
                       "Failed to open band file: %s",
                       osFilename.c_str() );
             delete poDS;
-            return NULL;
+            return nullptr;
         }
         poDS->papszExtraFiles =
             CSLAddString( poDS->papszExtraFiles,
@@ -328,7 +346,7 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
             adfUSGSParms[i] = CPLAtof(papszParmTokens[i]);
     }
     CSLDestroy(papszParmTokens);
-    papszParmTokens = NULL;
+    papszParmTokens = nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Minimal georef support ... should add full USGS style           */
@@ -359,10 +377,10 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
         oSRS.SetWellKnownGeogCS( "WGS84" );
     }
 
-    if( oSRS.GetRoot() != NULL )
+    if( oSRS.GetRoot() != nullptr )
     {
         CPLFree( poDS->pszProjection );
-        poDS->pszProjection = NULL;
+        poDS->pszProjection = nullptr;
         oSRS.exportToWkt( &(poDS->pszProjection) );
     }
 
@@ -424,7 +442,7 @@ GDALDataset *NDFDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_NDF()
 
 {
-    if( GDALGetDriverByName( "NDF" ) != NULL )
+    if( GDALGetDriverByName( "NDF" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
@@ -435,6 +453,7 @@ void GDALRegister_NDF()
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#NDF" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
+    poDriver->pfnIdentify = NDFDataset::Identify;
     poDriver->pfnOpen = NDFDataset::Open;
 
     GetGDALDriverManager()->RegisterDriver( poDriver );
