@@ -32,7 +32,7 @@
 #include "gdal_frmts.h"
 #include "gdal_pam.h"
 
-CPL_CVSID("$Id: rikdataset.cpp 36763 2016-12-09 22:10:55Z rouault $");
+CPL_CVSID("$Id: rikdataset.cpp 33c893fc72f32e43c9ed2dea9b1621a16f46d6a3 2018-05-04 09:33:05 +0200 Even Rouault $")
 
 #define RIK_HEADER_DEBUG 0
 #define RIK_CLEAR_DEBUG 0
@@ -183,7 +183,8 @@ RIKRasterBand::RIKRasterBand( RIKDataset *poDSIn, int nBandIn )
 /************************************************************************/
 
 static int GetNextLZWCode( int codeBits,
-                           GByte *blockData,
+                           const GByte *blockData,
+                           const GUInt32 blockSize,
                            GUInt32 &filePos,
                            GUInt32 &fileAlign,
                            int &bitsTaken )
@@ -203,6 +204,9 @@ static int GetNextLZWCode( int codeBits,
 
     while( bitsLeftToGo > 0 )
     {
+        if( filePos >= blockSize )
+            return -1;
+
         int tmp = blockData[filePos];
         tmp = tmp >> bitsTaken;
 
@@ -301,8 +305,7 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 #endif
         )
     {
-        for( GUInt32 i = 0; i < pixels; i++ )
-            reinterpret_cast<GByte *>( pImage )[i] = 0;
+        memset(pImage, 0, pixels);
         return CE_None;
     }
 
@@ -320,13 +323,14 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
     // Read block to memory
     GByte *blockData = reinterpret_cast<GByte *>( VSI_MALLOC_VERBOSE(nBlockSize) );
-    if( blockData == NULL )
+    if( blockData == nullptr )
         return CE_Failure;
     if( VSIFReadL( blockData, 1, nBlockSize, poRDS->fp ) != nBlockSize )
     {
         VSIFree(blockData);
         return CE_Failure;
     }
+    memset(pImage, 0, pixels);
 
 /* -------------------------------------------------------------------- */
 /*      Read RLE block.                                                 */
@@ -355,15 +359,18 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
     else if( poRDS->options == 0x0b )
     {
+      try
+      {
+        if( nBlockSize < 5 )
+        {
+            throw "Not enough bytes";
+        }
+
         const bool LZW_HAS_CLEAR_CODE = !!(blockData[4] & 0x80);
         const int LZW_MAX_BITS = blockData[4] & 0x1f; // Max 13
         if( LZW_MAX_BITS > 13 )
         {
-            CPLFree( blockData );
-            CPLError( CE_Failure, CPLE_AppDefined,
-                      "RIK decompression failed. "
-                      "Invalid LZW_MAX_BITS." );
-            return CE_Failure;
+            throw "Invalid LZW_MAX_BITS";
         }
         const int LZW_BITS_PER_PIXEL = 8;
         const int LZW_OFFSET = 5;
@@ -398,8 +405,12 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         lineBreak += 3;
         lineBreak &= 0xfffffffc;
 
-        code = GetNextLZWCode( codeBits, blockData, filePos,
+        code = GetNextLZWCode( codeBits, blockData, nBlockSize, filePos,
                                fileAlign, bitsTaken );
+        if( code < 0 )
+        {
+            throw "Not enough bytes";
+        }
 
         OutputPixel( static_cast<GByte>( code ), pImage, poRDS->nBlockXSize,
                      lineBreak, imageLine, imagePos );
@@ -407,18 +418,14 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
         while( imageLine >= 0 &&
                (imageLine || imagePos < poRDS->nBlockXSize) &&
-               filePos < nBlockSize ) try
+               filePos < nBlockSize )
         {
             lastCode = code;
-            code = GetNextLZWCode( codeBits, blockData,
+            code = GetNextLZWCode( codeBits, blockData, nBlockSize,
                                    filePos, fileAlign, bitsTaken );
-            if( VSIFEofL( poRDS->fp ) )
+            if( code < 0 )
             {
-                CPLFree( blockData );
-                CPLError( CE_Failure, CPLE_AppDefined,
-                          "RIK decompression failed. "
-                          "Read past end of file.\n" );
-                return CE_Failure;
+                throw "Not enough bytes";
             }
 
             if( LZW_HAS_CLEAR_CODE && code == LZW_CLEAR )
@@ -442,8 +449,12 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                 filePos = fileAlign;
                 bitsTaken = 0;
 
-                code = GetNextLZWCode( codeBits, blockData,
+                code = GetNextLZWCode( codeBits, blockData, nBlockSize,
                                        filePos, fileAlign, bitsTaken );
+                if( code < 0 )
+                {
+                    throw "Not enough bytes";
+                }
 
                 if( code > lastAdded )
                 {
@@ -507,7 +518,12 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                 if( lastCode != LZW_NO_SUCH_CODE &&
                     lastAdded != LZW_CODES - 1 )
                 {
-                    prefix[++lastAdded] = lastCode;
+                    ++lastAdded;
+                    if( lastAdded >= 8192 )
+                    {
+                        throw "Decode error";
+                    }
+                    prefix[lastAdded] = lastCode;
                     character[lastAdded] = lastOutput;
                 }
 
@@ -523,8 +539,9 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                 }
             }
         }
-        catch (const char *errStr)
-        {
+      }
+      catch (const char *errStr)
+      {
 #if RIK_ALLOW_BLOCK_ERRORS
                 CPLDebug( "RIK",
                           "LZW Decompress Failed: %s\n"
@@ -534,15 +551,14 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                           " blocksize: %d\n",
                           errStr, blocks, nBlockIndex,
                           nBlockOffset, nBlockSize );
-                break;
 #else
                 CPLFree( blockData );
                 CPLError( CE_Failure, CPLE_AppDefined,
-                          "RIK decompression failed. "
-                          "Corrupt image block." );
+                          "RIK decompression failed: %s",
+                          errStr );
                 return CE_Failure;
 #endif
-        }
+      }
     }
 
 /* -------------------------------------------------------------------- */
@@ -605,15 +621,15 @@ GDALColorTable *RIKRasterBand::GetColorTable()
 /************************************************************************/
 
 RIKDataset::RIKDataset() :
-    fp( NULL ),
+    fp( nullptr ),
     nBlockXSize( 0 ),
     nBlockYSize( 0 ),
     nHorBlocks( 0 ),
     nVertBlocks( 0 ),
     nFileSize( 0 ),
-    pOffsets( NULL ),
+    pOffsets( nullptr ),
     options( 0 ),
-    poColorTable( NULL )
+    poColorTable( nullptr )
 
 {
     memset( adfTransform, 0, sizeof(adfTransform) );
@@ -628,7 +644,7 @@ RIKDataset::~RIKDataset()
 {
     FlushCache();
     CPLFree( pOffsets );
-    if( fp != NULL )
+    if( fp != nullptr )
         VSIFCloseL( fp );
     delete poColorTable;
 }
@@ -690,7 +706,7 @@ static GUInt16 GetRikString( VSILFILE *fp,
 int RIKDataset::Identify( GDALOpenInfo * poOpenInfo )
 
 {
-    if( poOpenInfo->fpL == NULL || poOpenInfo->nHeaderBytes < 50 )
+    if( poOpenInfo->fpL == nullptr || poOpenInfo->nHeaderBytes < 50 )
         return FALSE;
 
     if( STARTS_WITH_CI((const char *) poOpenInfo->pabyHeader, "RIK3") )
@@ -733,7 +749,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
     if( Identify(poOpenInfo) == FALSE )
-        return NULL;
+        return nullptr;
 
     bool rik3header = false;
 
@@ -755,13 +771,13 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if( nameLength > sizeof(name) - 1 )
     {
-        return NULL;
+        return nullptr;
     }
 
     if( !rik3header )
     {
         if( nameLength == 0 || nameLength != strlen(name) )
-            return NULL;
+            return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -789,7 +805,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         if( projLength > sizeof(projection) - 1 )
         {
             // Unreasonable string length, assume wrong format
-            return NULL;
+            return nullptr;
         }
 
         // Read unknown string
@@ -806,7 +822,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         if( tmpLength > sizeof(tmpStr) - 1 )
         {
             // Unreasonable string length, assume wrong format
-            return NULL;
+            return nullptr;
         }
 
         header.fNorth = CPLAtof( tmpStr );
@@ -819,7 +835,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         if( tmpLength > sizeof(tmpStr) - 1 )
         {
             // Unreasonable string length, assume wrong format
-            return NULL;
+            return nullptr;
         }
 
         header.fWest = CPLAtof( tmpStr );
@@ -840,6 +856,8 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         CPL_SWAP32PTR( &header.iHorBlocks );
         CPL_SWAP32PTR( &header.iVertBlocks );
 #endif
+        if ( header.iMPPNum == 0 )
+            return nullptr;
 
         VSIFReadL( &header.iBitsPerPixel, 1, sizeof(header.iBitsPerPixel), poOpenInfo->fpL );
         VSIFReadL( &header.iOptions, 1, sizeof(header.iOptions), poOpenInfo->fpL );
@@ -878,8 +896,11 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         if (!CPLIsFinite(header.fSouth) ||
             !CPLIsFinite(header.fWest) ||
             !CPLIsFinite(header.fNorth) ||
-            !CPLIsFinite(header.fEast))
-            return NULL;
+            !CPLIsFinite(header.fEast) ||
+            header.iMPPNum == 0)
+        {
+            return nullptr;
+        }
 
         const bool offsetBounds = header.fSouth < 4000000;
 
@@ -896,6 +917,8 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 #ifdef CPL_MSB
             CPL_SWAP32PTR( &header.iMPPDen );
 #endif
+            if( header.iMPPDen == 0 )
+                return nullptr;
 
             headerType = "RIK1";
         }
@@ -917,7 +940,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
         if(( header.iBlockWidth > 2000 ) || ( header.iBlockWidth < 10 ) ||
            ( header.iBlockHeight > 2000 ) || ( header.iBlockHeight < 10 ))
-           return NULL;
+           return nullptr;
 
         if( !offsetBounds )
         {
@@ -932,7 +955,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
             double dfVertBlocks = ceil( (header.fNorth - header.fSouth) /
                       (header.iBlockHeight * metersPerPixel) );
             if( dfVertBlocks < 1 || dfVertBlocks > INT_MAX )
-                return NULL;
+                return nullptr;
             header.iVertBlocks = static_cast<GUInt32>(dfVertBlocks);
         }
 
@@ -949,7 +972,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
             CPLError( CE_Failure, CPLE_OpenFailed,
                       "File %s has unsupported number of bits per pixel.\n",
                       poOpenInfo->pszFilename );
-            return NULL;
+            return nullptr;
         }
 
         VSIFReadL( &header.iOptions, 1, sizeof(header.iOptions), poOpenInfo->fpL );
@@ -964,7 +987,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
             CPLError( CE_Failure, CPLE_OpenFailed,
                       "File %s. Unknown map options.\n",
                       poOpenInfo->pszFilename );
-            return NULL;
+            return nullptr;
         }
     }
 
@@ -978,7 +1001,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         header.iVertBlocks >= INT_MAX / (int)sizeof(GUInt32) ||
         header.iHorBlocks >= INT_MAX / (header.iVertBlocks * (int)sizeof(GUInt32)) )
     {
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1007,7 +1030,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "File %s. Unable to allocate offset table.\n",
                   poOpenInfo->pszFilename );
-        return NULL;
+        return nullptr;
     }
 
     if( header.iOptions == 0x00 )
@@ -1020,13 +1043,13 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
                     "File %s. Read past end of file.\n",
                     poOpenInfo->pszFilename );
             CPLFree(offsets);
-            return NULL;
+            return nullptr;
         }
 
         VSIFSeekL( poOpenInfo->fpL, 0, SEEK_END );
         vsi_l_offset nBigFileSize = VSIFTellL( poOpenInfo->fpL );
-        if( nBigFileSize > 0xFFFFFFFFU )
-            nBigFileSize = 0xFFFFFFFFU;
+        if( nBigFileSize > UINT_MAX )
+            nBigFileSize = UINT_MAX;
         GUInt32 fileSize = static_cast<GUInt32>(nBigFileSize);
 
         GUInt32 nBlocksFromFileSize = (fileSize - offsets[0]) / (header.iBlockWidth * header.iBlockHeight);
@@ -1042,7 +1065,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
                         "File %s too short.\n",
                         poOpenInfo->pszFilename );
             CPLFree( offsets );
-            return NULL;
+            return nullptr;
         }
 
         for( GUInt32 i = 1; i < blocks; i++ )
@@ -1084,7 +1107,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
                   "File %s. Read past end of file.\n",
                   poOpenInfo->pszFilename );
         CPLFree(offsets);
-        return NULL;
+        return nullptr;
     }
 
     VSIFSeekL( poOpenInfo->fpL, 0, SEEK_END );
@@ -1117,7 +1140,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
                               "File %s too short.\n",
                               poOpenInfo->pszFilename );
                     CPLFree( offsets );
-                    return NULL;
+                    return nullptr;
                 }
                 header.iVertBlocks = y;
                 break;
@@ -1131,7 +1154,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
                               "File %s. Corrupt offset table.\n",
                               poOpenInfo->pszFilename );
                     CPLFree( offsets );
-                    return NULL;
+                    return nullptr;
                 }
                 header.iVertBlocks = y;
                 break;
@@ -1191,7 +1214,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
     RIKDataset *poDS = new RIKDataset();
 
     poDS->fp = poOpenInfo->fpL;
-    poOpenInfo->fpL = NULL;
+    poOpenInfo->fpL = nullptr;
 
     poDS->adfTransform[0] = header.fWest - metersPerPixel / 2.0;
     poDS->adfTransform[1] = metersPerPixel;
@@ -1252,7 +1275,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_NotSupported,
                   "The RIK driver does not support update access to existing"
                   " datasets.\n" );
-        return NULL;
+        return nullptr;
     }
 
     return poDS;
@@ -1265,7 +1288,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_RIK()
 
 {
-    if( GDALGetDriverByName( "RIK" ) != NULL )
+    if( GDALGetDriverByName( "RIK" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
