@@ -30,7 +30,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: sdtstransfer.cpp 35897 2016-10-24 11:54:24Z goatbar $");
+CPL_CVSID("$Id: sdtstransfer.cpp e13dcd4dc171dfeed63f912ba06b9374ce4f3bb2 2018-03-18 21:37:41Z Even Rouault $")
 
 /************************************************************************/
 /*                            SDTSTransfer()                            */
@@ -38,8 +38,8 @@ CPL_CVSID("$Id: sdtstransfer.cpp 35897 2016-10-24 11:54:24Z goatbar $");
 
 SDTSTransfer::SDTSTransfer() :
     nLayers(0),
-    panLayerCATDEntry(NULL),
-    papoLayerReader(NULL)
+    panLayerCATDEntry(nullptr),
+    papoLayerReader(nullptr)
 {}
 
 /************************************************************************/
@@ -77,7 +77,7 @@ int SDTSTransfer::Open( const char * pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Read the IREF file.                                             */
 /* -------------------------------------------------------------------- */
-    if( oCATD.GetModuleFilePath( "IREF" ) == NULL )
+    if( oCATD.GetModuleFilePath( "IREF" ) == nullptr )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Can't find IREF module in transfer `%s'.\n",
@@ -91,7 +91,7 @@ int SDTSTransfer::Open( const char * pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Read the XREF file.                                             */
 /* -------------------------------------------------------------------- */
-    if( oCATD.GetModuleFilePath( "XREF" ) == NULL )
+    if( oCATD.GetModuleFilePath( "XREF" ) == nullptr )
     {
         CPLError( CE_Warning, CPLE_AppDefined,
                   "Can't find XREF module in transfer `%s'.\n",
@@ -146,13 +146,13 @@ void SDTSTransfer::Close()
 {
     for( int i = 0; i < nLayers; i++ )
     {
-        if( papoLayerReader[i] != NULL )
+        if( papoLayerReader[i] != nullptr )
             delete papoLayerReader[i];
     }
     CPLFree( papoLayerReader );
-    papoLayerReader = NULL;
+    papoLayerReader = nullptr;
     CPLFree( panLayerCATDEntry );
-    panLayerCATDEntry = NULL;
+    panLayerCATDEntry = nullptr;
     nLayers = 0;
 }
 
@@ -186,7 +186,7 @@ void SDTSTransfer::Close()
   </ul>
  */
 
-SDTSLayerType SDTSTransfer::GetLayerType( int iEntry )
+SDTSLayerType SDTSTransfer::GetLayerType( int iEntry ) const
 
 {
     if( iEntry < 0 || iEntry >= nLayers )
@@ -210,7 +210,7 @@ SDTSLayerType SDTSTransfer::GetLayerType( int iEntry )
   methods.
  */
 
-int SDTSTransfer::GetLayerCATDEntry( int iEntry )
+int SDTSTransfer::GetLayerCATDEntry( int iEntry ) const
 
 {
     if( iEntry < 0 || iEntry >= nLayers )
@@ -230,7 +230,7 @@ SDTSLineReader *SDTSTransfer::GetLayerLineReader( int iEntry )
         || iEntry >= nLayers
         || oCATD.GetEntryType( panLayerCATDEntry[iEntry] ) != SLTLine )
     {
-        return NULL;
+        return nullptr;
     }
 
     SDTSLineReader *poLineReader = new SDTSLineReader( &oIREF );
@@ -238,8 +238,9 @@ SDTSLineReader *SDTSTransfer::GetLayerLineReader( int iEntry )
     if( !poLineReader->Open(
                         oCATD.GetEntryFilePath( panLayerCATDEntry[iEntry] ) ) )
     {
+        panLayerCATDEntry[iEntry] = SLTUnknown; // to prevent further attempt
         delete poLineReader;
-        return NULL;
+        return nullptr;
     }
 
     return poLineReader;
@@ -256,7 +257,7 @@ SDTSPointReader *SDTSTransfer::GetLayerPointReader( int iEntry )
         || iEntry >= nLayers
         || oCATD.GetEntryType( panLayerCATDEntry[iEntry] ) != SLTPoint )
     {
-        return NULL;
+        return nullptr;
     }
 
     SDTSPointReader *poPointReader = new SDTSPointReader( &oIREF );
@@ -264,8 +265,9 @@ SDTSPointReader *SDTSTransfer::GetLayerPointReader( int iEntry )
     if( !poPointReader->Open(
                         oCATD.GetEntryFilePath( panLayerCATDEntry[iEntry] ) ) )
     {
+        panLayerCATDEntry[iEntry] = SLTUnknown; // to prevent further attempt
         delete poPointReader;
-        return NULL;
+        return nullptr;
     }
 
     return poPointReader;
@@ -282,7 +284,7 @@ SDTSPolygonReader *SDTSTransfer::GetLayerPolygonReader( int iEntry )
         || iEntry >= nLayers
         || oCATD.GetEntryType( panLayerCATDEntry[iEntry] ) != SLTPoly )
     {
-        return NULL;
+        return nullptr;
     }
 
     SDTSPolygonReader *poPolyReader = new SDTSPolygonReader();
@@ -290,8 +292,9 @@ SDTSPolygonReader *SDTSTransfer::GetLayerPolygonReader( int iEntry )
     if( !poPolyReader->Open(
                         oCATD.GetEntryFilePath( panLayerCATDEntry[iEntry] ) ) )
     {
+        panLayerCATDEntry[iEntry] = SLTUnknown; // to prevent further attempt
         delete poPolyReader;
-        return NULL;
+        return nullptr;
     }
 
     return poPolyReader;
@@ -308,7 +311,7 @@ SDTSAttrReader *SDTSTransfer::GetLayerAttrReader( int iEntry )
         || iEntry >= nLayers
         || oCATD.GetEntryType( panLayerCATDEntry[iEntry] ) != SLTAttr )
     {
-        return NULL;
+        return nullptr;
     }
 
     SDTSAttrReader *poAttrReader = new SDTSAttrReader();
@@ -316,8 +319,9 @@ SDTSAttrReader *SDTSTransfer::GetLayerAttrReader( int iEntry )
     if( !poAttrReader->Open(
                         oCATD.GetEntryFilePath( panLayerCATDEntry[iEntry] ) ) )
     {
+        panLayerCATDEntry[iEntry] = SLTUnknown; // to prevent further attempt
         delete poAttrReader;
-        return NULL;
+        return nullptr;
     }
 
     return poAttrReader;
@@ -349,7 +353,7 @@ SDTSRasterReader *SDTSTransfer::GetLayerRasterReader( int iEntry )
         || iEntry >= nLayers
         || oCATD.GetEntryType( panLayerCATDEntry[iEntry] ) != SLTRaster )
     {
-        return NULL;
+        return nullptr;
     }
 
     SDTSRasterReader *poRasterReader = new SDTSRasterReader();
@@ -357,8 +361,9 @@ SDTSRasterReader *SDTSTransfer::GetLayerRasterReader( int iEntry )
     if( !poRasterReader->Open( &oCATD, &oIREF,
                          oCATD.GetEntryModule(panLayerCATDEntry[iEntry] ) ) )
     {
+        panLayerCATDEntry[iEntry] = SLTUnknown; // to prevent further attempt
         delete poRasterReader;
-        return NULL;
+        return nullptr;
     }
 
     return poRasterReader;
@@ -373,7 +378,7 @@ DDFModule *SDTSTransfer::GetLayerModuleReader( int iEntry )
 {
     if( iEntry < 0 || iEntry >= nLayers )
     {
-        return NULL;
+        return nullptr;
     }
 
     DDFModule *poModuleReader = new DDFModule;
@@ -381,8 +386,9 @@ DDFModule *SDTSTransfer::GetLayerModuleReader( int iEntry )
     if( !poModuleReader->Open(
                         oCATD.GetEntryFilePath( panLayerCATDEntry[iEntry] ) ) )
     {
+        panLayerCATDEntry[iEntry] = SLTUnknown; // to prevent further attempt
         delete poModuleReader;
-        return NULL;
+        return nullptr;
     }
 
     return poModuleReader;
@@ -422,7 +428,7 @@ DDFModule *SDTSTransfer::GetLayerModuleReader( int iEntry )
 SDTSIndexedReader *SDTSTransfer::GetLayerIndexedReader( int iEntry )
 
 {
-    if( papoLayerReader[iEntry] == NULL )
+    if( papoLayerReader[iEntry] == nullptr )
     {
         switch( oCATD.GetEntryType( panLayerCATDEntry[iEntry] ) )
         {
@@ -493,19 +499,19 @@ SDTSFeature *SDTSTransfer::GetIndexedFeatureRef( SDTSModId *poModId,
 /* -------------------------------------------------------------------- */
     const int iLayer = FindLayer( poModId->szModule );
     if( iLayer == -1 )
-        return NULL;
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Get the reader, and read a feature from it.                     */
 /* -------------------------------------------------------------------- */
     SDTSIndexedReader *poReader = GetLayerIndexedReader( iLayer );
-    if( poReader == NULL )
-        return NULL;
+    if( poReader == nullptr )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      return type, if requested.                                      */
 /* -------------------------------------------------------------------- */
-    if( peType != NULL )
+    if( peType != nullptr )
         *peType = GetLayerType(iLayer);
 
     return poReader->GetIndexedFeatureRef( poModId->nRecord );
@@ -531,11 +537,11 @@ SDTSFeature *SDTSTransfer::GetIndexedFeatureRef( SDTSModId *poModId,
 DDFField *SDTSTransfer::GetAttr( SDTSModId *poModId )
 
 {
-    SDTSAttrRecord *poAttrRecord = reinterpret_cast<SDTSAttrRecord *>(
+    SDTSAttrRecord *poAttrRecord = dynamic_cast<SDTSAttrRecord *>(
         GetIndexedFeatureRef( poModId ) );
 
-    if( poAttrRecord == NULL )
-        return NULL;
+    if( poAttrRecord == nullptr )
+        return nullptr;
 
     return poAttrRecord->poATTR;
 }
@@ -574,14 +580,14 @@ int SDTSTransfer::GetBounds( double *pdfMinX, double *pdfMinY,
 
             SDTSPointReader *poLayer = reinterpret_cast<SDTSPointReader *>(
                 GetLayerIndexedReader( iLayer ) );
-            if( poLayer == NULL )
+            if( poLayer == nullptr )
                 continue;
 
             poLayer->Rewind();
 
-            SDTSRawPoint *poPoint = NULL;
+            SDTSRawPoint *poPoint = nullptr;
             while( (poPoint = reinterpret_cast<SDTSRawPoint *>(
-                      poLayer->GetNextFeature() ) ) != NULL )
+                      poLayer->GetNextFeature() ) ) != nullptr )
             {
                 if( bFirst )
                 {
@@ -606,7 +612,7 @@ int SDTSTransfer::GetBounds( double *pdfMinX, double *pdfMinY,
         else if( GetLayerType( iLayer ) == SLTRaster )
         {
             SDTSRasterReader *poRL = GetLayerRasterReader( iLayer );
-            if( poRL == NULL )
+            if( poRL == nullptr )
                 continue;
 
             double adfGeoTransform[6];
