@@ -32,14 +32,14 @@
 
 #include <cstddef>
 #include <algorithm>
+#include <set>
 #include <vector>
 
 #include "cpl_config.h"
 #include "cpl_error.h"
-#include "cpl_hash_set.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdalhashsetbandblockcache.cpp 36523 2016-11-27 04:13:26Z goatbar $");
+CPL_CVSID("$Id: gdalhashsetbandblockcache.cpp 1bffff2fa520d625ab7b649d007da770c937a56a 2018-04-08 19:18:09 +0200 Even Rouault $")
 
 //! @cond Doxygen_Suppress
 
@@ -47,24 +47,40 @@ CPL_CVSID("$Id: gdalhashsetbandblockcache.cpp 36523 2016-11-27 04:13:26Z goatbar
 /*                        GDALHashSetBandBlockCache                     */
 /* ******************************************************************** */
 
-class GDALHashSetBandBlockCache CPL_FINAL : public GDALAbstractBandBlockCache
+class GDALHashSetBandBlockCache final : public GDALAbstractBandBlockCache
 {
-    CPLHashSet     *hSet;
+    struct BlockComparator
+    {
+        // Do not change this comparator, because this order is assumed by
+        // tests like tiff_write_133 for flushing from top to bottom, left
+        // to right.
+        bool operator() (const GDALRasterBlock* const& lhs,
+                         const GDALRasterBlock* const& rhs) const
+        {
+            if( lhs->GetYOff() < rhs->GetYOff() )
+                return true;
+            if( lhs->GetYOff() > rhs->GetYOff() )
+                return false;
+            return lhs->GetXOff() < rhs->GetXOff();
+        }
+    };
+
+    std::set<GDALRasterBlock*, BlockComparator> m_oSet;
     CPLLock        *hLock;
 
-    public:
-           explicit GDALHashSetBandBlockCache( GDALRasterBand* poBand );
-           virtual ~GDALHashSetBandBlockCache();
+  public:
+    explicit GDALHashSetBandBlockCache( GDALRasterBand* poBand );
+    ~GDALHashSetBandBlockCache() override;
 
-           virtual bool             Init() override;
-           virtual bool             IsInitOK() override;
-           virtual CPLErr           FlushCache() override;
-           virtual CPLErr           AdoptBlock( GDALRasterBlock * ) override;
-           virtual GDALRasterBlock *TryGetLockedBlockRef( int nXBlockOff,
-                                                          int nYBlockYOff ) override;
-           virtual CPLErr           UnreferenceBlock( GDALRasterBlock* poBlock ) override;
-           virtual CPLErr           FlushBlock( int nXBlockOff, int nYBlockOff,
-                                                int bWriteDirtyBlock ) override;
+    bool Init() override;
+    bool IsInitOK() override;
+    CPLErr FlushCache() override;
+    CPLErr AdoptBlock( GDALRasterBlock * ) override;
+    GDALRasterBlock *TryGetLockedBlockRef( int nXBlockOff,
+                                           int nYBlockYOff ) override;
+    CPLErr UnreferenceBlock( GDALRasterBlock* poBlock ) override;
+    CPLErr FlushBlock( int nXBlockOff, int nYBlockOff,
+                       int bWriteDirtyBlock ) override;
 };
 
 /************************************************************************/
@@ -77,43 +93,6 @@ GDALAbstractBandBlockCache* GDALHashSetBandBlockCacheCreate(
     return new GDALHashSetBandBlockCache(poBand);
 }
 
-/************************************************************************/
-/*                      GDALRasterBlockHashFunc()                       */
-/************************************************************************/
-
-// Calculate hash value.
-static unsigned long GDALRasterBlockHashFunc( const void * const elt )
-{
-    const GDALRasterBlock * const poBlock =
-        static_cast<const GDALRasterBlock *>(elt);
-#if SIZEOF_UNSIGNED_LONG == 8
-    return static_cast<unsigned long>(
-        poBlock->GetXOff() |
-        (static_cast<unsigned long>(poBlock->GetYOff()) << 32) );
-#else
-    return static_cast<unsigned long>(
-        ((poBlock->GetXOff() & 0xFFFF) ^ (poBlock->GetYOff() >> 16)) |
-        (((poBlock->GetYOff() & 0xFFFF) ^ (poBlock->GetXOff() >> 16)) << 16));
-#endif
-}
-
-/************************************************************************/
-/*                      GDALRasterBlockEqualFunc()                      */
-/************************************************************************/
-
-// Test equality.
-// Must return an int rather than a bool to work with CPLHashSetNew.
-static int GDALRasterBlockEqualFunc( const void * const elt1,
-                                     const void * const elt2 )
-{
-    const GDALRasterBlock * const poBlock1 =
-        static_cast<const GDALRasterBlock *>(elt1);
-    const GDALRasterBlock * const poBlock2 =
-        static_cast<const GDALRasterBlock *>(elt2);
-    return poBlock1->GetXOff() == poBlock2->GetXOff() &&
-           poBlock1->GetYOff() == poBlock2->GetYOff();
-}
-
 /************************************************************************/
 /*                       GDALHashSetBandBlockCache()                    */
 /************************************************************************/
@@ -121,8 +100,7 @@ static int GDALRasterBlockEqualFunc( const void * const elt1,
 GDALHashSetBandBlockCache::GDALHashSetBandBlockCache(
     GDALRasterBand* poBandIn ) :
     GDALAbstractBandBlockCache(poBandIn),
-    hSet(CPLHashSetNew(GDALRasterBlockHashFunc,
-                       GDALRasterBlockEqualFunc, NULL)),
+
     hLock(CPLCreateLock(LOCK_ADAPTIVE_MUTEX))
 {}
 
@@ -133,7 +111,6 @@ GDALHashSetBandBlockCache::GDALHashSetBandBlockCache(
 GDALHashSetBandBlockCache::~GDALHashSetBandBlockCache()
 {
     FlushCache();
-    CPLHashSetDestroy(hSet);
     CPLDestroyLock(hLock);
 }
 
@@ -165,40 +142,11 @@ CPLErr GDALHashSetBandBlockCache::AdoptBlock( GDALRasterBlock * poBlock )
     FreeDanglingBlocks();
 
     CPLLockHolderOptionalLockD( hLock );
-    CPLHashSetInsert(hSet, poBlock);
+    m_oSet.insert(poBlock);
 
     return CE_None;
 }
 
-/************************************************************************/
-/*              GDALHashSetBandBlockCacheFlushCacheIterFunc()           */
-/************************************************************************/
-
-// Must return an int to work with CPLHashSetForeach.
-static int GDALHashSetBandBlockCacheFlushCacheIterFunc( void* elt,
-                                                        void* user_data )
-{
-    // TODO(schwehr): Do all of the HashSet stuff be done in a type safe manner.
-    std::vector<GDALRasterBlock*>* papoBlocks =
-        static_cast<std::vector<GDALRasterBlock *> *>(user_data);
-    GDALRasterBlock* poBlock = static_cast<GDALRasterBlock *>(elt);
-    papoBlocks->push_back(poBlock);
-    return true;
-}
-
-/************************************************************************/
-/*                  GDALHashSetBandBlockCacheSortBlocks()               */
-/************************************************************************/
-
-// TODO: Both args should be const.
-static bool GDALHashSetBandBlockCacheSortBlocks( GDALRasterBlock* poBlock1,
-                                                 GDALRasterBlock* poBlock2 )
-{
-    return poBlock1->GetYOff() < poBlock2->GetYOff() ||
-           ( poBlock1->GetYOff() == poBlock2->GetYOff() &&
-             poBlock1->GetXOff() < poBlock2->GetXOff() );
-}
-
 /************************************************************************/
 /*                            FlushCache()                              */
 /************************************************************************/
@@ -209,25 +157,14 @@ CPLErr GDALHashSetBandBlockCache::FlushCache()
 
     CPLErr eGlobalErr = poBand->eFlushBlockErr;
 
-    std::vector<GDALRasterBlock*> apoBlocks;
+    std::set<GDALRasterBlock*, BlockComparator> oOldSet;
     {
         CPLLockHolderOptionalLockD( hLock );
-        CPLHashSetForeach(hSet,
-                          GDALHashSetBandBlockCacheFlushCacheIterFunc,
-                          &apoBlocks);
-
-        CPLHashSetClear(hSet);
+        oOldSet = std::move(m_oSet);
     }
 
-    // Sort blocks by increasing y and then x in order to please some tests
-    // like tiff_write_133
-    std::sort(apoBlocks.begin(), apoBlocks.end(),
-              GDALHashSetBandBlockCacheSortBlocks);
-
-    for( size_t i = 0; i < apoBlocks.size(); ++i )
+    for( auto& poBlock: oOldSet )
     {
-        GDALRasterBlock* const poBlock = apoBlocks[i];
-
         if( poBlock->DropLockForRemovalFromStorage() )
         {
             CPLErr eErr = CE_None;
@@ -256,7 +193,7 @@ CPLErr GDALHashSetBandBlockCache::UnreferenceBlock( GDALRasterBlock* poBlock )
     UnreferenceBlockBase();
 
     CPLLockHolderOptionalLockD( hLock );
-    CPLHashSetRemoveDeferRehash(hSet, poBlock);
+    m_oSet.erase(poBlock);
     return CE_None;
 }
 
@@ -269,14 +206,14 @@ CPLErr GDALHashSetBandBlockCache::FlushBlock( int nXBlockOff, int nYBlockOff,
 
 {
     GDALRasterBlock oBlockForLookup(nXBlockOff, nYBlockOff);
-    GDALRasterBlock* poBlock = NULL;
+    GDALRasterBlock* poBlock = nullptr;
     {
         CPLLockHolderOptionalLockD( hLock );
-        poBlock = reinterpret_cast<GDALRasterBlock*>(
-            CPLHashSetLookup(hSet, &oBlockForLookup) );
-        if( poBlock == NULL )
+        auto oIter = m_oSet.find(&oBlockForLookup);
+        if( oIter == m_oSet.end() )
             return CE_None;
-        CPLHashSetRemove(hSet, poBlock);
+        poBlock = *oIter;
+        m_oSet.erase(oIter);
     }
 
     if( !poBlock->DropLockForRemovalFromStorage() )
@@ -301,21 +238,16 @@ GDALRasterBlock *GDALHashSetBandBlockCache::TryGetLockedBlockRef(
 
 {
     GDALRasterBlock oBlockForLookup(nXBlockOff, nYBlockOff);
-    GDALRasterBlock* poBlock = NULL;
-    while( true )
+    GDALRasterBlock* poBlock;
     {
-        {
-            CPLLockHolderOptionalLockD( hLock );
-            poBlock = reinterpret_cast<GDALRasterBlock*>(
-                CPLHashSetLookup(hSet, &oBlockForLookup) );
-        }
-        if( poBlock == NULL )
-            return NULL;
-        if( poBlock->TakeLock()  )
-            break;
+        CPLLockHolderOptionalLockD( hLock );
+        auto oIter = m_oSet.find(&oBlockForLookup);
+        if( oIter == m_oSet.end() )
+            return nullptr;
+        poBlock = *oIter;
     }
-
-    poBlock->Touch();
+    if( !poBlock->TakeLock()  )
+        return nullptr;
     return poBlock;
 }
 
