@@ -37,6 +37,8 @@
 #endif
 
 #include <algorithm>
+#include <map>
+#include <mutex>
 #include <vector>
 
 #include "cpl_config.h"
@@ -46,7 +48,109 @@
 #include "cpl_vsi.h"
 #include "gdal.h"
 
-CPL_CVSID("$Id: gdalopeninfo.cpp 37005 2016-12-23 15:03:59Z goatbar $");
+CPL_CVSID("$Id: gdalopeninfo.cpp 3bc98ac2e895141cb38924f83d1182b59d711148 2018-06-14 16:22:26 +0200 Even Rouault $")
+
+// Keep in sync prototype of those 2 functions between gdalopeninfo.cpp,
+// ogrsqlitedatasource.cpp and ogrgeopackagedatasource.cpp
+void GDALOpenInfoDeclareFileNotToOpen(const char* pszFilename,
+                                       const GByte* pabyHeader,
+                                       int nHeaderBytes);
+void GDALOpenInfoUnDeclareFileNotToOpen(const char* pszFilename);
+
+/************************************************************************/
+
+/* This whole section helps for SQLite/GPKG, especially with write-ahead
+ * log enabled. The issue is that sqlite3 relies on POSIX advisory locks to
+ * properly work and decide when to create/delete the wal related files.
+ * One issue with POSIX advisory locks is that if within the same process
+ * you do
+ * f1 = open('somefile')
+ * set locks on f1
+ * f2 = open('somefile')
+ * close(f2)
+ * The close(f2) will cancel the locks set on f1. The work on f1 is done by
+ * libsqlite3 whereas the work on f2 is done by GDALOpenInfo.
+ * So as soon as sqlite3 has opened a file we should make sure not to re-open
+ * it (actually close it) ourselves.
+ */
+
+namespace {
+struct FileNotToOpen
+{
+    CPLString osFilename{};
+    int       nRefCount{};
+    GByte    *pabyHeader{nullptr};
+    int       nHeaderBytes{0};
+};
+}
+
+static std::mutex sFNTOMutex;
+static std::map<CPLString, FileNotToOpen>* pMapFNTO = nullptr;
+
+void GDALOpenInfoDeclareFileNotToOpen(const char* pszFilename,
+                                       const GByte* pabyHeader,
+                                       int nHeaderBytes)
+{
+    std::lock_guard<std::mutex> oLock(sFNTOMutex);
+    if( pMapFNTO == nullptr )
+        pMapFNTO = new std::map<CPLString, FileNotToOpen>();
+    auto oIter = pMapFNTO->find(pszFilename);
+    if( oIter != pMapFNTO->end() )
+    {
+        oIter->second.nRefCount ++;
+    }
+    else
+    {
+        FileNotToOpen fnto;
+        fnto.osFilename = pszFilename;
+        fnto.nRefCount = 1;
+        fnto.pabyHeader = static_cast<GByte*>(CPLMalloc(nHeaderBytes + 1));
+        memcpy(fnto.pabyHeader, pabyHeader, nHeaderBytes);
+        fnto.pabyHeader[nHeaderBytes] = 0;
+        fnto.nHeaderBytes = nHeaderBytes;
+        (*pMapFNTO)[pszFilename] = fnto;
+    }
+}
+
+void GDALOpenInfoUnDeclareFileNotToOpen(const char* pszFilename)
+{
+    std::lock_guard<std::mutex> oLock(sFNTOMutex);
+    CPLAssert(pMapFNTO);
+    auto oIter = pMapFNTO->find(pszFilename);
+    CPLAssert( oIter != pMapFNTO->end() );
+    oIter->second.nRefCount --;
+    if( oIter->second.nRefCount == 0 )
+    {
+        CPLFree(oIter->second.pabyHeader);
+        pMapFNTO->erase(oIter);
+    }
+    if( pMapFNTO->empty() )
+    {
+        delete pMapFNTO;
+        pMapFNTO = nullptr;
+    }
+}
+
+static GByte* GDALOpenInfoGetFileNotToOpen(const char* pszFilename,
+                                           int* pnHeaderBytes)
+{
+    std::lock_guard<std::mutex> oLock(sFNTOMutex);
+    *pnHeaderBytes = 0;
+    if( pMapFNTO == nullptr )
+    {
+        return nullptr;
+    }
+    auto oIter = pMapFNTO->find(pszFilename);
+    if( oIter == pMapFNTO->end() )
+    {
+        return nullptr;
+    }
+    *pnHeaderBytes = oIter->second.nHeaderBytes;
+    GByte* pabyHeader = static_cast<GByte*>(CPLMalloc(*pnHeaderBytes + 1));
+    memcpy(pabyHeader, oIter->second.pabyHeader, *pnHeaderBytes);
+    pabyHeader[*pnHeaderBytes] = 0;
+    return pabyHeader;
+}
 
 /************************************************************************/
 /* ==================================================================== */
@@ -64,21 +168,23 @@ CPL_CVSID("$Id: gdalopeninfo.cpp 37005 2016-12-23 15:03:59Z goatbar $");
  * @param papszSiblingsIn list of sibling files, or NULL.
  */
 GDALOpenInfo::GDALOpenInfo( const char * pszFilenameIn, int nOpenFlagsIn,
-                            char **papszSiblingsIn ) :
+                            const char * const * papszSiblingsIn ) :
     bHasGotSiblingFiles(false),
-    papszSiblingFiles(NULL),
+    papszSiblingFiles(nullptr),
     nHeaderBytesTried(0),
     pszFilename(CPLStrdup(pszFilenameIn)),
-    papszOpenOptions(NULL),
+    papszOpenOptions(nullptr),
     eAccess(nOpenFlagsIn & GDAL_OF_UPDATE ? GA_Update : GA_ReadOnly),
     nOpenFlags(nOpenFlagsIn),
     bStatOK(FALSE),
     bIsDirectory(FALSE),
-    fpL(NULL),
+    fpL(nullptr),
     nHeaderBytes(0),
-    pabyHeader(NULL),
-    papszAllowedDrivers(NULL)
+    pabyHeader(nullptr),
+    papszAllowedDrivers(nullptr)
 {
+    if( STARTS_WITH(pszFilename, "MVT:/vsi") )
+        return;
 
 /* -------------------------------------------------------------------- */
 /*      Ensure that C: is treated as C:\ so we can stat it on           */
@@ -120,6 +226,7 @@ retry:  // TODO(schwehr): Stop using goto.
     {
         const char* pszExt = CPLGetExtension(pszFilename);
         if( EQUAL(pszExt, "zip") || EQUAL(pszExt, "tar") || EQUAL(pszExt, "gz")
+            || pszFilename[strlen(pszFilename)-1] == '}'
 #ifdef DEBUG
             // For AFL, so that .cur_input is detected as the archive filename.
             || EQUAL( CPLGetFilename(pszFilename), ".cur_input" )
@@ -152,15 +259,27 @@ retry:  // TODO(schwehr): Stop using goto.
         }
     }
 
-    if( !bIsDirectory ) {
+    pabyHeader = GDALOpenInfoGetFileNotToOpen(pszFilename, &nHeaderBytes);
+
+    if( !bIsDirectory && pabyHeader == nullptr ) {
         fpL = VSIFOpenExL( pszFilename, (eAccess == GA_Update) ? "r+b" : "rb", (nOpenFlagsIn & GDAL_OF_VERBOSE_ERROR) > 0);
     }
-    if( fpL != NULL )
+    if( pabyHeader )
     {
         bStatOK = TRUE;
-        const int nBufSize = 1025;
-        pabyHeader = static_cast<GByte *>( CPLCalloc(nBufSize, 1) );
-        nHeaderBytesTried = nBufSize - 1;
+        nHeaderBytesTried = nHeaderBytes;
+    }
+    else if( fpL != nullptr )
+    {
+        bStatOK = TRUE;
+        int nBufSize =
+            atoi(CPLGetConfigOption("GDAL_INGESTED_BYTES_AT_OPEN", "1024"));
+        if( nBufSize < 1024 )
+            nBufSize = 1024;
+        else if( nBufSize > 10 * 1024 * 1024)
+            nBufSize = 10 * 1024 * 1024;
+        pabyHeader = static_cast<GByte *>( CPLCalloc(nBufSize+1, 1) );
+        nHeaderBytesTried = nBufSize;
         nHeaderBytes = static_cast<int>(
             VSIFReadL( pabyHeader, 1, nHeaderBytesTried, fpL ) );
         VSIRewindL( fpL );
@@ -173,9 +292,9 @@ retry:  // TODO(schwehr): Stop using goto.
             VSI_ISDIR( sStat.st_mode ) )
         {
             CPL_IGNORE_RET_VAL(VSIFCloseL(fpL));
-            fpL = NULL;
+            fpL = nullptr;
             CPLFree(pabyHeader);
-            pabyHeader = NULL;
+            pabyHeader = nullptr;
             bIsDirectory = TRUE;
         }
     }
@@ -207,7 +326,7 @@ retry:  // TODO(schwehr): Stop using goto.
                 szPointerFilename[std::min(nBytes, nBufSize - 1)] = 0;
                 CPLFree(pszFilename);
                 pszFilename = CPLStrdup(szPointerFilename);
-                papszSiblingsIn = NULL;
+                papszSiblingsIn = nullptr;
                 bHasRetried = true;
                 goto retry;
             }
@@ -219,7 +338,7 @@ retry:  // TODO(schwehr): Stop using goto.
 /*      Capture sibling list either from passed in values, or by        */
 /*      scanning for them only if requested through GetSiblingFiles().  */
 /* -------------------------------------------------------------------- */
-    if( papszSiblingsIn != NULL )
+    if( papszSiblingsIn != nullptr )
     {
         papszSiblingFiles = CSLDuplicate( papszSiblingsIn );
         bHasGotSiblingFiles = true;
@@ -231,25 +350,25 @@ retry:  // TODO(schwehr): Stop using goto.
         if (EQUAL(pszOptionVal, "EMPTY_DIR"))
         {
             papszSiblingFiles =
-                CSLAddString( NULL, CPLGetFilename(pszFilename) );
+                CSLAddString( nullptr, CPLGetFilename(pszFilename) );
             bHasGotSiblingFiles = true;
         }
         else if( CPLTestBool(pszOptionVal) )
         {
             /* skip reading the directory */
-            papszSiblingFiles = NULL;
+            papszSiblingFiles = nullptr;
             bHasGotSiblingFiles = true;
         }
         else
         {
             /* will be lazy loaded */
-            papszSiblingFiles = NULL;
+            papszSiblingFiles = nullptr;
             bHasGotSiblingFiles = false;
         }
     }
     else
     {
-        papszSiblingFiles = NULL;
+        papszSiblingFiles = nullptr;
         bHasGotSiblingFiles = true;
     }
 }
@@ -264,7 +383,7 @@ GDALOpenInfo::~GDALOpenInfo()
     VSIFree( pabyHeader );
     CPLFree( pszFilename );
 
-    if( fpL != NULL )
+    if( fpL != nullptr )
         CPL_IGNORE_RET_VAL(VSIFCloseL( fpL ));
     CSLDestroy( papszSiblingFiles );
 }
@@ -291,16 +410,16 @@ char** GDALOpenInfo::GetSiblingFiles()
         CPLDebug("GDAL", "GDAL_READDIR_LIMIT_ON_OPEN reached on %s",
                  osDir.c_str());
         CSLDestroy(papszSiblingFiles);
-        papszSiblingFiles = NULL;
+        papszSiblingFiles = nullptr;
     }
 
     /* Small optimization to avoid unnecessary stat'ing from PAux or ENVI */
     /* drivers. The MBTiles driver needs no companion file. */
-    if( papszSiblingFiles == NULL &&
+    if( papszSiblingFiles == nullptr &&
         STARTS_WITH(pszFilename, "/vsicurl/") &&
         EQUAL(CPLGetExtension( pszFilename ),"mbtiles") )
     {
-        papszSiblingFiles = CSLAddString( NULL, CPLGetFilename(pszFilename) );
+        papszSiblingFiles = CSLAddString( nullptr, CPLGetFilename(pszFilename) );
     }
 
     return papszSiblingFiles;
@@ -320,7 +439,7 @@ char** GDALOpenInfo::GetSiblingFiles()
 char** GDALOpenInfo::StealSiblingFiles()
 {
     char** papszRet = GetSiblingFiles();
-    papszSiblingFiles = NULL;
+    papszSiblingFiles = nullptr;
     return papszRet;
 }
 
@@ -346,7 +465,7 @@ bool GDALOpenInfo::AreSiblingFilesLoaded() const
  */
 int GDALOpenInfo::TryToIngest(int nBytes)
 {
-    if( fpL == NULL )
+    if( fpL == nullptr )
         return FALSE;
     if( nHeaderBytes < nHeaderBytesTried )
         return TRUE;
