@@ -145,7 +145,7 @@ GDALVirtualMem::GDALVirtualMem( GDALDatasetH hDSIn,
     nLineSpace(nLineSpaceIn),
     nBandSpace(nBandSpaceIn)
 {
-    if( hDS != NULL )
+    if( hDS != nullptr )
     {
         panBandMap = static_cast<int *>( CPLMalloc(nBandCount * sizeof(int)) );
         if( panBandMapIn )
@@ -160,7 +160,7 @@ GDALVirtualMem::GDALVirtualMem( GDALDatasetH hDSIn,
     }
     else
     {
-        panBandMap = NULL;
+        panBandMap = nullptr;
         nBandCount = 1;
     }
 
@@ -423,7 +423,7 @@ void GDALVirtualMem::DoIOPixelInterleaved(
     }
 
     // How many whole lines can we store/load ?
-    coord_type nLineCount = (coord_type)((nBytes - nOffsetShift) / nLineSpace);
+    coord_type nLineCount = static_cast<coord_type>((nBytes - nOffsetShift) / nLineSpace);
     if( y + nLineCount > nBufYSize )
         nLineCount = nBufYSize - y;
     if( nLineCount > 0 )
@@ -516,7 +516,7 @@ void GDALVirtualMem::DoIOBandSequential(
     }
 
     // Yes, enough place to read/write until end of line
-    if( x > 0 || nBytes - nOffsetShift < (size_t)nLineSpace )
+    if( x > 0 || nBytes - nOffsetShift < static_cast<size_t>(nLineSpace) )
     {
         CPL_IGNORE_RET_VAL(GDALRasterIO(
             hBand ? hBand : GDALGetRasterBand(hDS, panBandMap[band]),
@@ -537,7 +537,7 @@ void GDALVirtualMem::DoIOBandSequential(
     }
 
     // How many whole lines can we store/load ?
-    coord_type nLineCount = (coord_type)((nBytes - nOffsetShift) / nLineSpace);
+    coord_type nLineCount = static_cast<coord_type>((nBytes - nOffsetShift) / nLineSpace);
     if( y + nLineCount > nBufYSize )
         nLineCount = nBufYSize - y;
     if( nLineCount > 0 )
@@ -566,7 +566,7 @@ void GDALVirtualMem::DoIOBandSequential(
     if( nOffsetShift < nBytes )
     {
         DoIOBandSequential( eRWFlag, nOffsetRecompute,
-               (char*) pPage + nOffsetShift, nBytes - nOffsetShift );
+               static_cast<char*>(pPage) + nOffsetShift, nBytes - nOffsetShift );
     }
 }
 
@@ -655,7 +655,7 @@ static bool GDALCheckBandParameters( GDALDatasetH hDS,
         return false;
     }
 
-    if( panBandMap != NULL )
+    if( panBandMap != nullptr )
     {
         for( int i = 0; i < nBandCount; i++ )
         {
@@ -695,17 +695,17 @@ static CPLVirtualMem* GDALGetVirtualMem( GDALDatasetH hDS,
                                          size_t nCacheSize,
                                          size_t nPageSizeHint,
                                          int bSingleThreadUsage,
-                                         char ** /*papszOptions*/ )
+                                         CSLConstList /*papszOptions*/ )
 {
-    CPLVirtualMem* view = NULL;
-    GDALVirtualMem* psParams = NULL;
+    CPLVirtualMem* view = nullptr;
+    GDALVirtualMem* psParams = nullptr;
     GUIntBig nReqMem = 0;
 
     if( nXSize != nBufXSize || nYSize != nBufYSize )
     {
         CPLError( CE_Failure, CPLE_NotSupported,
                   "nXSize != nBufXSize || nYSize != nBufYSize" );
-        return NULL;
+        return nullptr;
     }
 
     int nRasterXSize =
@@ -720,18 +720,18 @@ static CPLVirtualMem* GDALGetVirtualMem( GDALDatasetH hDS,
         nYOff + nYSize > nRasterYSize )
     {
         CPLError( CE_Failure, CPLE_AppDefined, "Invalid window request" );
-        return NULL;
+        return nullptr;
     }
 
     if( nPixelSpace < 0 || nLineSpace < 0 || nBandSpace < 0)
     {
         CPLError( CE_Failure, CPLE_NotSupported,
                   "nPixelSpace < 0 || nLineSpace < 0 || nBandSpace < 0" );
-        return NULL;
+        return nullptr;
     }
 
-    if( hDS != NULL && !GDALCheckBandParameters(hDS, nBandCount, panBandMap ) )
-        return NULL;
+    if( hDS != nullptr && !GDALCheckBandParameters(hDS, nBandCount, panBandMap ) )
+        return nullptr;
 
     const int nDataTypeSize = GDALGetDataTypeSizeBytes(eBufType);
     if( nPixelSpace == 0 )
@@ -771,7 +771,7 @@ static CPLVirtualMem* GDALGetVirtualMem( GDALDatasetH hDS,
         CPLError(
             CE_Failure, CPLE_NotSupported,
             "Only pixel interleaving or band interleaving are supported" );
-        return NULL;
+        return nullptr;
     }
 
     /* Avoid odd spacings that would complicate I/O operations */
@@ -783,7 +783,7 @@ static CPLVirtualMem* GDALGetVirtualMem( GDALDatasetH hDS,
     {
         CPLError( CE_Failure, CPLE_NotSupported,
                   "Unsupported spacing" );
-        return NULL;
+        return nullptr;
     }
 
     bool bIsBandSequential = nBandSpace >= nBufYSize * nLineSpace;
@@ -791,11 +791,11 @@ static CPLVirtualMem* GDALGetVirtualMem( GDALDatasetH hDS,
         nReqMem = nBandCount * nBandSpace;
     else
         nReqMem = nBufYSize * nLineSpace;
-    if( nReqMem != (GUIntBig)(size_t)nReqMem )
+    if( nReqMem != static_cast<GUIntBig>(static_cast<size_t>(nReqMem)) )
     {
         CPLError( CE_Failure, CPLE_OutOfMemory,
                   "Cannot reserve " CPL_FRMT_GUIB " bytes", nReqMem );
-        return NULL;
+        return nullptr;
     }
 
     psParams = new GDALVirtualMem( hDS, hBand, nXOff, nYOff,
@@ -821,7 +821,7 @@ static CPLVirtualMem* GDALGetVirtualMem( GDALDatasetH hDS,
         GDALVirtualMem::Destroy,
         psParams );
 
-    if( view == NULL )
+    if( view == nullptr )
     {
         delete psParams;
     }
@@ -961,9 +961,9 @@ CPLVirtualMem* GDALDatasetGetVirtualMem( GDALDatasetH hDS,
                                          size_t nCacheSize,
                                          size_t nPageSizeHint,
                                          int bSingleThreadUsage,
-                                         char **papszOptions )
+                                         CSLConstList papszOptions )
 {
-    return GDALGetVirtualMem( hDS, NULL, eRWFlag, nXOff, nYOff, nXSize, nYSize,
+    return GDALGetVirtualMem( hDS, nullptr, eRWFlag, nXOff, nYOff, nXSize, nYSize,
                               nBufXSize, nBufYSize, eBufType,
                               nBandCount, panBandMap,
                               nPixelSpace, nLineSpace, nBandSpace,
@@ -1086,12 +1086,12 @@ CPLVirtualMem* GDALRasterBandGetVirtualMem( GDALRasterBandH hBand,
                                             size_t nCacheSize,
                                             size_t nPageSizeHint,
                                             int bSingleThreadUsage,
-                                            char **papszOptions )
+                                            CSLConstList papszOptions )
 {
-    return GDALGetVirtualMem( NULL, hBand, eRWFlag, nXOff, nYOff,
+    return GDALGetVirtualMem( nullptr, hBand, eRWFlag, nXOff, nYOff,
                               nXSize, nYSize,
                               nBufXSize, nBufYSize, eBufType,
-                              1, NULL,
+                              1, nullptr,
                               nPixelSpace, nLineSpace, 0,
                               nCacheSize, nPageSizeHint, bSingleThreadUsage,
                               papszOptions );
@@ -1165,7 +1165,7 @@ GDALTiledVirtualMem::GDALTiledVirtualMem(
     nBandCount(nBandCountIn),
     eTileOrganization(eTileOrganizationIn)
 {
-    if( hDS != NULL )
+    if( hDS != nullptr )
     {
         panBandMap = static_cast<int*>(CPLMalloc(nBandCount * sizeof(int)));
         if( panBandMapIn )
@@ -1180,7 +1180,7 @@ GDALTiledVirtualMem::GDALTiledVirtualMem(
     }
     else
     {
-        panBandMap = NULL;
+        panBandMap = nullptr;
         nBandCount = 1;
     }
 }
@@ -1251,7 +1251,7 @@ void GDALTiledVirtualMem::DoIO( GDALRWFlag eRWFlag, size_t nOffset,
     if( eRWFlag == GF_Read && (nReqXSize < nTileXSize ||
                                nReqYSize < nTileYSize) )
         memset(pPage, 0, nBytes);
-    if( hDS != NULL )
+    if( hDS != nullptr )
     {
         CPL_IGNORE_RET_VAL(GDALDatasetRasterIO(
             hDS, eRWFlag,
@@ -1337,7 +1337,7 @@ static CPLVirtualMem* GDALGetTiledVirtualMem(
     GDALTileOrganization eTileOrganization,
     size_t nCacheSize,
     int bSingleThreadUsage,
-    char ** /* papszOptions */ )
+    CSLConstList /* papszOptions */ )
 {
     CPLVirtualMem* view;
     GDALTiledVirtualMem* psParams;
@@ -1349,7 +1349,7 @@ static CPLVirtualMem* GDALGetTiledVirtualMem(
             CE_Failure, CPLE_NotSupported,
             "GDALGetTiledVirtualMem() unsupported on this "
             "operating system / configuration" );
-        return NULL;
+        return nullptr;
     }
 
     int nRasterXSize =
@@ -1363,22 +1363,22 @@ static CPLVirtualMem* GDALGetTiledVirtualMem(
         nYOff + nYSize > nRasterYSize )
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Invalid window request");
-        return NULL;
+        return nullptr;
     }
 
-    if( hDS != NULL && !GDALCheckBandParameters(hDS, nBandCount, panBandMap ) )
-        return NULL;
+    if( hDS != nullptr && !GDALCheckBandParameters(hDS, nBandCount, panBandMap ) )
+        return nullptr;
 
     const int nDataTypeSize = GDALGetDataTypeSizeBytes(eBufType);
     int nTilesPerRow = (nXSize + nTileXSize - 1) / nTileXSize;
     int nTilesPerCol = (nYSize + nTileYSize - 1) / nTileYSize;
-    GUIntBig nReqMem = (GUIntBig)nTilesPerRow * nTilesPerCol *
+    GUIntBig nReqMem = static_cast<GUIntBig>(nTilesPerRow) * nTilesPerCol *
                         nTileXSize * nTileYSize * nBandCount * nDataTypeSize;
-    if( nReqMem != (GUIntBig)(size_t)nReqMem )
+    if( nReqMem != static_cast<GUIntBig>(static_cast<size_t>(nReqMem)) )
     {
         CPLError( CE_Failure, CPLE_OutOfMemory,
                   "Cannot reserve " CPL_FRMT_GUIB " bytes", nReqMem );
-        return NULL;
+        return nullptr;
     }
 
     size_t nPageSizeHint = nTileXSize * nTileYSize * nDataTypeSize;
@@ -1388,7 +1388,7 @@ static CPLVirtualMem* GDALGetTiledVirtualMem(
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Tile dimensions incompatible with page size");
-        return NULL;
+        return nullptr;
     }
 
     psParams = new GDALTiledVirtualMem( hDS, hBand, nXOff, nYOff,
@@ -1410,7 +1410,7 @@ static CPLVirtualMem* GDALGetTiledVirtualMem(
         GDALTiledVirtualMem::Destroy,
         psParams );
 
-    if( view == NULL )
+    if( view == nullptr )
     {
         delete psParams;
     }
@@ -1422,7 +1422,7 @@ static CPLVirtualMem* GDALGetTiledVirtualMem(
             static_cast<int>(CPLVirtualMemGetPageSize(view)),
             static_cast<int>(nPageSizeHint) );
         CPLVirtualMemFree(view);
-        return NULL;
+        return nullptr;
     }
 
     return view;
@@ -1552,9 +1552,9 @@ CPLVirtualMem* GDALDatasetGetTiledVirtualMem(
     GDALTileOrganization eTileOrganization,
     size_t nCacheSize,
     int bSingleThreadUsage,
-    char **papszOptions )
+    CSLConstList papszOptions )
 {
-    return GDALGetTiledVirtualMem( hDS, NULL, eRWFlag, nXOff, nYOff,
+    return GDALGetTiledVirtualMem( hDS, nullptr, eRWFlag, nXOff, nYOff,
                                    nXSize, nYSize, nTileXSize, nTileYSize,
                                    eBufType, nBandCount, panBandMap,
                                    eTileOrganization,
@@ -1664,11 +1664,11 @@ CPLVirtualMem* GDALRasterBandGetTiledVirtualMem( GDALRasterBandH hBand,
                                                  GDALDataType eBufType,
                                                  size_t nCacheSize,
                                                  int bSingleThreadUsage,
-                                                 char **papszOptions )
+                                                 CSLConstList papszOptions )
 {
-    return GDALGetTiledVirtualMem( NULL, hBand, eRWFlag, nXOff, nYOff,
+    return GDALGetTiledVirtualMem( nullptr, hBand, eRWFlag, nXOff, nYOff,
                                    nXSize, nYSize, nTileXSize, nTileYSize,
-                                   eBufType, 1, NULL,
+                                   eBufType, 1, nullptr,
                                    GTO_BSQ,
                                    nCacheSize, bSingleThreadUsage,
                                    papszOptions );
