@@ -40,7 +40,7 @@
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrcircularstring.cpp 40454 2017-10-16 19:14:13Z rouault $");
+CPL_CVSID("$Id: ogrcircularstring.cpp 1b132b40204d500e99c91f6a7986b44b6a065de8 2018-04-10 15:04:42 +0200 Even Rouault $")
 
 static inline double dist(double x0, double y0, double x1, double y1)
 {
@@ -137,12 +137,15 @@ const char * OGRCircularString::getGeometryName() const
 /*      format.                                                         */
 /************************************************************************/
 
-OGRErr OGRCircularString::importFromWkb( unsigned char * pabyData,
+OGRErr OGRCircularString::importFromWkb( const unsigned char * pabyData,
                                          int nSize,
-                                         OGRwkbVariant eWkbVariant )
+                                         OGRwkbVariant eWkbVariant,
+                                         int& nBytesConsumedOut )
 
 {
-    OGRErr eErr = OGRSimpleCurve::importFromWkb(pabyData, nSize, eWkbVariant);
+    OGRErr eErr = OGRSimpleCurve::importFromWkb(pabyData, nSize,
+                                                eWkbVariant,
+                                                nBytesConsumedOut);
     if( eErr == OGRERR_NONE )
     {
         if( !IsValidFast() )
@@ -183,7 +186,7 @@ OGRErr OGRCircularString::exportToWkb( OGRwkbByteOrder eByteOrder,
 /*      `CIRCULARSTRING [Z] ( x y [z], x y [z], ...)',                  */
 /************************************************************************/
 
-OGRErr OGRCircularString::importFromWkt( char ** ppszInput )
+OGRErr OGRCircularString::importFromWkt( const char ** ppszInput )
 
 {
     const OGRErr eErr = OGRSimpleCurve::importFromWkt(ppszInput);
@@ -267,7 +270,7 @@ void OGRCircularString::ExtendEnvelopeWithCircular(
     // extremities of the circle.
     for( int i = 0; i < nPointCount - 2; i += 2 )
     {
-        double x0 = paoPoints[i].x;
+        const double x0 = paoPoints[i].x;
         const double y0 = paoPoints[i].y;
         const double x1 = paoPoints[i+1].x;
         const double y1 = paoPoints[i+1].y;
@@ -283,6 +286,11 @@ void OGRCircularString::ExtendEnvelopeWithCircular(
                                                   R, cx, cy,
                                                   alpha0, alpha1, alpha2))
         {
+            if( CPLIsNan(alpha0) || CPLIsNan(alpha2) ) {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "GetCurveParmeters returned NaN");
+                continue;
+            }
             int quadrantStart =
                 static_cast<int>(std::floor(alpha0 / (M_PI / 2)));
             int quadrantEnd = static_cast<int>(std::floor(alpha2 / (M_PI / 2)));
@@ -392,10 +400,18 @@ void OGRCircularString::segmentize( double dfMaxLength )
             if( dfSegmentLength1 > dfMaxLength ||
                 dfSegmentLength2 > dfMaxLength )
             {
-                const int nIntermediatePoints =
-                    1 +
-                    2 * static_cast<int>(std::floor(dfSegmentLength1
-                                                    / dfMaxLength / 2.0));
+                const double dfVal =
+                    1 + 2 * std::floor(dfSegmentLength1 / dfMaxLength / 2.0);
+                if ( dfVal >= std::numeric_limits<int>::max() ||
+                     dfVal < 0.0 ||
+                     CPLIsNan(dfVal) )
+                {
+                    CPLError(
+                        CE_Failure, CPLE_AppDefined,
+                        "segmentize nIntermediatePoints invalid: %lf", dfVal);
+                    break;
+                }
+                const int nIntermediatePoints = static_cast<int>(dfVal);
                 const double dfStep =
                     (alpha1 - alpha0) / (nIntermediatePoints + 1);
                 for( int j = 1; j <= nIntermediatePoints; ++j )
@@ -421,10 +437,18 @@ void OGRCircularString::segmentize( double dfMaxLength )
             if( dfSegmentLength1 > dfMaxLength ||
                 dfSegmentLength2 > dfMaxLength )
             {
-                int nIntermediatePoints =
-                    1 +
-                    2 * static_cast<int>(std::floor(dfSegmentLength2
-                                                    / dfMaxLength / 2.0));
+                const double dfVal =
+                    1 + 2 * std::floor(dfSegmentLength2 / dfMaxLength / 2.0);
+                if ( dfVal >= std::numeric_limits<int>::max() ||
+                     dfVal < 0.0 ||
+                     CPLIsNan(dfVal) )
+                {
+                    CPLError(
+                        CE_Failure, CPLE_AppDefined,
+                        "segmentize nIntermediatePoints invalid 2: %lf", dfVal);
+                    break;
+                }
+                int nIntermediatePoints = static_cast<int>(dfVal);
                 const double dfStep =
                     (alpha2 - alpha1) / (nIntermediatePoints + 1);
                 for( int j = 1; j <= nIntermediatePoints; ++j )
@@ -452,9 +476,18 @@ void OGRCircularString::segmentize( double dfMaxLength )
             if( dfSegmentLength1 > dfMaxLength ||
                 dfSegmentLength2 > dfMaxLength )
             {
-                int nIntermediatePoints =
-                    1 + 2 * static_cast<int>(ceil(dfSegmentLength1 /
-                                                  dfMaxLength / 2.0));
+                const double dfVal =
+                    1 + 2 * std::ceil(dfSegmentLength1 / dfMaxLength / 2.0);
+                if ( dfVal >= std::numeric_limits<int>::max() ||
+                     dfVal < 0.0 ||
+                     CPLIsNan(dfVal) )
+                {
+                    CPLError(
+                        CE_Failure, CPLE_AppDefined,
+                        "segmentize nIntermediatePoints invalid 2: %lf", dfVal);
+                    break;
+                }
+                int nIntermediatePoints = static_cast<int>(dfVal);
                 for( int j = 1; j <= nIntermediatePoints; ++j )
                 {
                     aoRawPoint.push_back(OGRRawPoint(
@@ -473,9 +506,19 @@ void OGRCircularString::segmentize( double dfMaxLength )
             if( dfSegmentLength1 > dfMaxLength ||
                 dfSegmentLength2 > dfMaxLength )
             {
-                const int nIntermediatePoints =
-                    1 + 2 * static_cast<int>(ceil(dfSegmentLength2 /
-                                                  dfMaxLength / 2.0));
+                const double dfVal =
+                    1 + 2 * std::ceil(dfSegmentLength2 / dfMaxLength / 2.0);
+                if ( dfVal >= std::numeric_limits<int>::max() ||
+                     dfVal < 0.0 ||
+                     CPLIsNan(dfVal) )
+                {
+                    CPLError(
+                        CE_Failure, CPLE_AppDefined,
+                        "segmentize nIntermediatePoints invalid 3: %lf", dfVal);
+                     break;
+                }
+                const int nIntermediatePoints = static_cast<int>(dfVal);
+
                 for( int j = 1; j <= nIntermediatePoints; ++j )
                 {
                     aoRawPoint.push_back(OGRRawPoint(
@@ -693,16 +736,32 @@ OGRCircularString::getLinearGeometry( double dfMaxAngleStepSizeDegrees,
 /*                     GetCasterToLineString()                          */
 /************************************************************************/
 
+static OGRLineString* CasterToLineString(OGRCurve* poGeom)
+{
+    CPLError(CE_Failure, CPLE_AppDefined,
+             "%s found. Conversion impossible", poGeom->getGeometryName());
+    delete poGeom;
+    return nullptr;
+}
+
 OGRCurveCasterToLineString OGRCircularString::GetCasterToLineString() const {
-    return (OGRCurveCasterToLineString) OGRGeometry::CastToError;
+    return ::CasterToLineString;
 }
 
 /************************************************************************/
 /*                        GetCasterToLinearRing()                       */
 /************************************************************************/
 
+static OGRLinearRing* CasterToLinearRing(OGRCurve* poGeom)
+{
+    CPLError(CE_Failure, CPLE_AppDefined,
+             "%s found. Conversion impossible", poGeom->getGeometryName());
+    delete poGeom;
+    return nullptr;
+}
+
 OGRCurveCasterToLinearRing OGRCircularString::GetCasterToLinearRing() const {
-    return (OGRCurveCasterToLinearRing) OGRGeometry::CastToError;
+    return ::CasterToLinearRing;
 }
 //! @endcond
 
@@ -838,11 +897,12 @@ double OGRCircularString::get_Area() const
     return dfArea;
 }
 
+//! @cond Doxygen_Suppress
+
 /************************************************************************/
 /*                           ContainsPoint()                            */
 /************************************************************************/
 
-//! @cond Doxygen_Suppress
 int OGRCircularString::ContainsPoint( const OGRPoint* p ) const
 {
     double cx = 0.0;
@@ -858,6 +918,10 @@ int OGRCircularString::ContainsPoint( const OGRPoint* p ) const
     return -1;
 }
 
+/************************************************************************/
+/*                       IntersectsPoint()                              */
+/************************************************************************/
+
 int OGRCircularString::IntersectsPoint( const OGRPoint* p ) const
 {
     double cx = 0.0;
