@@ -39,7 +39,7 @@
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrlinearring.cpp 37113 2017-01-12 11:47:03Z rouault $");
+CPL_CVSID("$Id: ogrlinearring.cpp 99132fd3803311ecc233f6d375b94f8c7a016aef 2018-03-28 01:33:40 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                           OGRLinearRing()                            */
@@ -81,7 +81,7 @@ OGRLinearRing::~OGRLinearRing() {}
 OGRLinearRing::OGRLinearRing( OGRLinearRing * poSrcRing )
 
 {
-    if( poSrcRing == NULL )
+    if( poSrcRing == nullptr )
     {
         CPLDebug( "OGR",
                   "OGRLinearRing::OGRLinearRing(OGRLinearRing*poSrcRing) - "
@@ -151,9 +151,10 @@ int OGRLinearRing::WkbSize() const
 /*      Disable method for this class.                                  */
 /************************************************************************/
 
-OGRErr OGRLinearRing::importFromWkb( CPL_UNUSED unsigned char *pabyData,
-                                     CPL_UNUSED int nSize,
-                                     CPL_UNUSED OGRwkbVariant eWkbVariant )
+OGRErr OGRLinearRing::importFromWkb( const unsigned char * /*pabyData*/,
+                                     int /*nSize*/,
+                                     OGRwkbVariant /*eWkbVariant*/,
+                                     int& /* nBytesConsumedOut */ )
 
 {
     return OGRERR_UNSUPPORTED_OPERATION;
@@ -182,10 +183,12 @@ OGRErr OGRLinearRing::exportToWkb( CPL_UNUSED OGRwkbByteOrder eByteOrder,
 
 //! @cond Doxygen_Suppress
 OGRErr OGRLinearRing::_importFromWkb( OGRwkbByteOrder eByteOrder, int _flags,
-                                      unsigned char * pabyData,
-                                      int nBytesAvailable )
+                                      const unsigned char * pabyData,
+                                      int nBytesAvailable,
+                                      int& nBytesConsumedOut )
 
 {
+    nBytesConsumedOut = -1;
     if( nBytesAvailable < 4 && nBytesAvailable != -1 )
         return OGRERR_NOT_ENOUGH_DATA;
 
@@ -233,6 +236,9 @@ OGRErr OGRLinearRing::_importFromWkb( OGRwkbByteOrder eByteOrder, int _flags,
     else
         RemoveM();
 
+
+    nBytesConsumedOut =  4 + nPointCount * nPointSize;
+
 /* -------------------------------------------------------------------- */
 /*      Get the vertices                                                */
 /* -------------------------------------------------------------------- */
@@ -321,11 +327,11 @@ OGRErr OGRLinearRing::_exportToWkb( OGRwkbByteOrder eByteOrder, int _flags,
         {
             memcpy( pabyData+4+i*32, &(paoPoints[i].x), 8 );
             memcpy( pabyData+4+i*32+8, &(paoPoints[i].y), 8 );
-            if( padfZ == NULL )
+            if( padfZ == nullptr )
                 memset( pabyData+4+i*32+16, 0, 8 );
             else
                 memcpy( pabyData+4+i*32+16, padfZ + i, 8 );
-            if( padfM == NULL )
+            if( padfM == nullptr )
                 memset( pabyData+4+i*32+24, 0, 8 );
             else
                 memcpy( pabyData+4+i*32+24, padfM + i, 8 );
@@ -338,7 +344,7 @@ OGRErr OGRLinearRing::_exportToWkb( OGRwkbByteOrder eByteOrder, int _flags,
         {
             memcpy( pabyData+4+i*24, &(paoPoints[i].x), 8 );
             memcpy( pabyData+4+i*24+8, &(paoPoints[i].y), 8 );
-            if( padfM == NULL )
+            if( padfM == nullptr )
                 memset( pabyData+4+i*24+16, 0, 8 );
             else
                 memcpy( pabyData+4+i*24+16, padfM + i, 8 );
@@ -351,7 +357,7 @@ OGRErr OGRLinearRing::_exportToWkb( OGRwkbByteOrder eByteOrder, int _flags,
         {
             memcpy( pabyData+4+i*24, &(paoPoints[i].x), 8 );
             memcpy( pabyData+4+i*24+8, &(paoPoints[i].y), 8 );
-            if( padfZ == NULL )
+            if( padfZ == nullptr )
                 memset( pabyData+4+i*24+16, 0, 8 );
             else
                 memcpy( pabyData+4+i*24+16, padfZ + i, 8 );
@@ -421,11 +427,11 @@ OGRGeometry *OGRLinearRing::clone() const
 /*                            epsilonEqual()                            */
 /************************************************************************/
 
-static const double EPSILON = 1E-5;
+constexpr double EPSILON = 1.0E-5;
 
 static inline bool epsilonEqual(double a, double b, double eps)
 {
-    return (::fabs(a - b) < eps);
+    return ::fabs(a - b) < eps;
 }
 
 /************************************************************************/
@@ -588,19 +594,23 @@ void OGRLinearRing::closeRings()
 OGRBoolean OGRLinearRing::isPointInRing(const OGRPoint* poPoint,
                                         int bTestEnvelope) const
 {
-    if( NULL == poPoint )
+    if( nullptr == poPoint )
     {
         CPLDebug( "OGR",
                   "OGRLinearRing::isPointInRing(const OGRPoint* poPoint) - "
                   "passed point is NULL!" );
-        return 0;
+        return FALSE;
+    }
+    if( poPoint->IsEmpty() )
+    {
+        return FALSE;
     }
 
     const int iNumPoints = getNumPoints();
 
     // Simple validation
     if( iNumPoints < 4 )
-        return 0;
+        return FALSE;
 
     const double dfTestX = poPoint->getX();
     const double dfTestY = poPoint->getY();
@@ -613,7 +623,7 @@ OGRBoolean OGRLinearRing::isPointInRing(const OGRPoint* poPoint,
         if( !( dfTestX >= extent.MinX && dfTestX <= extent.MaxX
                && dfTestY >= extent.MinY && dfTestY <= extent.MaxY ) )
         {
-            return 0;
+            return FALSE;
         }
     }
 
@@ -665,7 +675,7 @@ OGRBoolean OGRLinearRing::isPointInRing(const OGRPoint* poPoint,
 OGRBoolean OGRLinearRing::isPointOnRingBoundary( const OGRPoint* poPoint,
                                                  int bTestEnvelope ) const
 {
-    if( NULL == poPoint )
+    if( nullptr == poPoint )
     {
         CPLDebug( "OGR",
                   "OGRLinearRing::isPointOnRingBoundary(const OGRPoint* "
@@ -777,17 +787,27 @@ OGRLineString* OGRLinearRing::CastToLineString( OGRLinearRing* poLR )
 /*                     GetCasterToLineString()                          */
 /************************************************************************/
 
+OGRLineString* OGRLinearRing::CasterToLineString( OGRCurve* poCurve )
+{
+    return OGRLinearRing::CastToLineString(poCurve->toLinearRing());
+}
+
 OGRCurveCasterToLineString OGRLinearRing::GetCasterToLineString() const
 {
-    return (OGRCurveCasterToLineString) OGRLinearRing::CastToLineString;
+    return OGRLinearRing::CasterToLineString;
 }
 
 /************************************************************************/
 /*                        GetCasterToLinearRing()                       */
 /************************************************************************/
 
+static OGRLinearRing* CasterToLinearRing(OGRCurve* poCurve)
+{
+    return poCurve->toLinearRing();
+}
+
 OGRCurveCasterToLinearRing OGRLinearRing::GetCasterToLinearRing() const
 {
-    return (OGRCurveCasterToLinearRing) OGRGeometry::CastToIdentity;
+    return ::CasterToLinearRing;
 }
 //! @endcond
