@@ -41,7 +41,7 @@
 #include "ogr_p.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: ogrpoint.cpp 40622 2017-11-03 10:07:36Z rouault $");
+CPL_CVSID("$Id: ogrpoint.cpp 1b132b40204d500e99c91f6a7986b44b6a065de8 2018-04-10 15:04:42 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                              OGRPoint()                              */
@@ -51,10 +51,9 @@ CPL_CVSID("$Id: ogrpoint.cpp 40622 2017-11-03 10:07:36Z rouault $");
  * \brief Create an empty point.
  */
 
-OGRPoint::OGRPoint()
+OGRPoint::OGRPoint(): x(0.0), y(0.0), z(0.0), m(0.0)
 
 {
-    empty();
     flags = 0;
 }
 
@@ -183,8 +182,8 @@ OGRGeometry *OGRPoint::clone() const
 
 {
     OGRPoint *poNewPoint = new (std::nothrow) OGRPoint( x, y, z, m );
-    if( poNewPoint == NULL )
-        return NULL;
+    if( poNewPoint == nullptr )
+        return nullptr;
 
     poNewPoint->assignSpatialReference( getSpatialReference() );
     poNewPoint->flags = flags;
@@ -295,16 +294,18 @@ int OGRPoint::WkbSize() const
 /*      format.                                                         */
 /************************************************************************/
 
-OGRErr OGRPoint::importFromWkb( unsigned char * pabyData,
+OGRErr OGRPoint::importFromWkb( const unsigned char *pabyData,
                                 int nSize,
-                                OGRwkbVariant eWkbVariant )
+                                OGRwkbVariant eWkbVariant,
+                                int& nBytesConsumedOut )
 
 {
+    nBytesConsumedOut = -1;
     OGRwkbByteOrder eByteOrder = wkbNDR;
 
     flags = 0;
     OGRErr eErr =
-        importPreambuleFromWkb( pabyData, nSize, eByteOrder, eWkbVariant );
+        importPreambleFromWkb( pabyData, nSize, eByteOrder, eWkbVariant );
     pabyData += 5;
     if( eErr != OGRERR_NONE )
         return eErr;
@@ -320,6 +321,9 @@ OGRErr OGRPoint::importFromWkb( unsigned char * pabyData,
             return OGRERR_NOT_ENOUGH_DATA;
     }
 
+    nBytesConsumedOut = 5 + 8 * (2 + ((flags & OGR_G_3D) ? 1 : 0)+
+                                     ((flags & OGR_G_MEASURED) ? 1 : 0));
+
 /* -------------------------------------------------------------------- */
 /*      Get the vertex.                                                 */
 /* -------------------------------------------------------------------- */
@@ -406,9 +410,13 @@ OGRErr OGRPoint::exportToWkb( OGRwkbByteOrder eByteOrder,
     }
 
     if( eByteOrder == wkbNDR )
-        nGType = CPL_LSBWORD32( nGType );
+    {
+        CPL_LSBPTR32( &nGType );
+    }
     else
-        nGType = CPL_MSBWORD32( nGType );
+    {
+        CPL_MSBPTR32( &nGType );
+    }
 
     memcpy( pabyData, &nGType, 4 );
     pabyData += 4;
@@ -474,13 +482,13 @@ OGRErr OGRPoint::exportToWkb( OGRwkbByteOrder eByteOrder,
 /*      (x,y)''.                                                        */
 /************************************************************************/
 
-OGRErr OGRPoint::importFromWkt( char ** ppszInput )
+OGRErr OGRPoint::importFromWkt( const char ** ppszInput )
 
 {
     int bHasZ = FALSE;
     int bHasM = FALSE;
     bool bIsEmpty = false;
-    OGRErr eErr = importPreambuleFromWkt(ppszInput, &bHasZ, &bHasM, &bIsEmpty);
+    OGRErr eErr = importPreambleFromWkt(ppszInput, &bHasZ, &bHasM, &bIsEmpty);
     flags = 0;
     if( eErr != OGRERR_NONE )
         return eErr;
@@ -500,9 +508,9 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
 /* -------------------------------------------------------------------- */
 /*      Read the point list which should consist of exactly one point.  */
 /* -------------------------------------------------------------------- */
-    OGRRawPoint *poPoints = NULL;
-    double *padfZ = NULL;
-    double *padfM = NULL;
+    OGRRawPoint *poPoints = nullptr;
+    double *padfZ = nullptr;
+    double *padfM = nullptr;
     int nMaxPoint = 0;
     int nPoints = 0;
     int flagsFromInput = flags;
@@ -510,7 +518,7 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
     pszInput = OGRWktReadPointsM( pszInput, &poPoints, &padfZ, &padfM,
                                   &flagsFromInput,
                                   &nMaxPoint, &nPoints );
-    if( pszInput == NULL || nPoints != 1 )
+    if( pszInput == nullptr || nPoints != 1 )
     {
         CPLFree( poPoints );
         CPLFree( padfZ );
@@ -535,19 +543,19 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
 
     if( bHasZ )
     {
-        if( padfZ != NULL )
+        if( padfZ != nullptr )
             z = padfZ[0];
     }
     if( bHasM )
     {
-        if( padfM != NULL )
+        if( padfM != nullptr )
             m = padfM[0];
     }
 
     CPLFree( padfZ );
     CPLFree( padfM );
 
-    *ppszInput = const_cast<char *>(pszInput);
+    *ppszInput = pszInput;
 
     return OGRERR_NONE;
 }
@@ -555,7 +563,7 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
 /************************************************************************/
 /*                            exportToWkt()                             */
 /*                                                                      */
-/*      Translate this structure into it's well known text format       */
+/*      Translate this structure into its well known text format       */
 /*      equivalent.                                                     */
 /************************************************************************/
 
@@ -703,7 +711,7 @@ void OGRPoint::getEnvelope( OGREnvelope3D * psEnvelope ) const
 /*                               Equal()                                */
 /************************************************************************/
 
-OGRBoolean OGRPoint::Equals( OGRGeometry * poOther ) const
+OGRBoolean OGRPoint::Equals( const OGRGeometry * poOther ) const
 
 {
     if( poOther== this )
@@ -712,13 +720,7 @@ OGRBoolean OGRPoint::Equals( OGRGeometry * poOther ) const
     if( poOther->getGeometryType() != getGeometryType() )
         return FALSE;
 
-    OGRPoint *poOPoint = dynamic_cast<OGRPoint *>(poOther);
-    if( poOPoint == NULL )
-    {
-        CPLError(CE_Fatal, CPLE_AppDefined,
-                 "dynamic_cast failed.  Expected OGRPoint.");
-        return FALSE;
-    }
+    const auto poOPoint = poOther->toPoint();
     if( flags != poOPoint->flags )
         return FALSE;
 
@@ -766,17 +768,10 @@ void OGRPoint::swapXY()
 OGRBoolean OGRPoint::Within( const OGRGeometry *poOtherGeom ) const
 
 {
-    if( !IsEmpty() && poOtherGeom != NULL &&
+    if( !IsEmpty() && poOtherGeom != nullptr &&
         wkbFlatten(poOtherGeom->getGeometryType()) == wkbCurvePolygon )
     {
-        const OGRCurvePolygon *poCurve =
-            dynamic_cast<const OGRCurvePolygon *>(poOtherGeom);
-        if( poCurve == NULL )
-        {
-            CPLError(CE_Fatal, CPLE_AppDefined,
-                     "dynamic_cast failed.  Expected OGRCurvePolygon.");
-            return FALSE;
-        }
+        const auto poCurve = poOtherGeom->toCurvePolygon();
         return poCurve->Contains(this);
     }
 
@@ -790,17 +785,10 @@ OGRBoolean OGRPoint::Within( const OGRGeometry *poOtherGeom ) const
 OGRBoolean OGRPoint::Intersects( const OGRGeometry *poOtherGeom ) const
 
 {
-    if( !IsEmpty() && poOtherGeom != NULL &&
+    if( !IsEmpty() && poOtherGeom != nullptr &&
         wkbFlatten(poOtherGeom->getGeometryType()) == wkbCurvePolygon )
     {
-        const OGRCurvePolygon *poCurve =
-            dynamic_cast<const OGRCurvePolygon *>(poOtherGeom);
-        if( poCurve == NULL )
-        {
-            CPLError(CE_Fatal, CPLE_AppDefined,
-                     "dynamic_cast failed.  Expected OGRCurvePolygon.");
-            return FALSE;
-        }
+        const auto poCurve = poOtherGeom->toCurvePolygon();
         return poCurve->Intersects(this);
     }
 
