@@ -30,7 +30,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: dgnread.cpp 38747 2017-05-31 22:12:52Z rouault $");
+CPL_CVSID("$Id: dgnread.cpp 01d97526bd8cf1da723b27dc55ccc73887012973 2018-03-08 20:36:54Z Even Rouault $")
 
 static DGNElemCore *DGNParseTCB( DGNInfo * );
 static DGNElemCore *DGNParseColorTable( DGNInfo * );
@@ -74,7 +74,7 @@ int DGNGotoElement( DGNHandle hDGN, int element_id )
     if( element_id < 0 || element_id >= psDGN->element_count )
         return FALSE;
 
-    if( VSIFSeek( psDGN->fp, psDGN->element_index[element_id].offset,
+    if( VSIFSeekL( psDGN->fp, psDGN->element_index[element_id].offset,
                   SEEK_SET ) != 0 )
         return FALSE;
 
@@ -95,7 +95,7 @@ int DGNLoadRawElement( DGNInfo *psDGN, int *pnType, int *pnLevel )
 /*      Read the first four bytes to get the level, type, and word      */
 /*      count.                                                          */
 /* -------------------------------------------------------------------- */
-    if( VSIFRead( psDGN->abyElem, 1, 4, psDGN->fp ) != 4 )
+    if( VSIFReadL( psDGN->abyElem, 1, 4, psDGN->fp ) != 4 )
         return FALSE;
 
     /* Is this an 0xFFFF endof file marker? */
@@ -109,12 +109,14 @@ int DGNLoadRawElement( DGNInfo *psDGN, int *pnType, int *pnLevel )
 /* -------------------------------------------------------------------- */
 /*      Read the rest of the element data into the working buffer.      */
 /* -------------------------------------------------------------------- */
-    if( nWords * 2 + 4 > (int) sizeof(psDGN->abyElem) )
+    if( nWords * 2 + 4 >= (int) sizeof(psDGN->abyElem) )
         return FALSE;
 
     /* coverity[tainted_data] */
-    if( (int) VSIFRead( psDGN->abyElem + 4, 2, nWords, psDGN->fp ) != nWords )
+    if( (int) VSIFReadL( psDGN->abyElem + 4, 2, nWords, psDGN->fp ) != nWords )
         return FALSE;
+    psDGN->abyElem[4 + 2 * nWords] = 0;
+    psDGN->abyElem[sizeof(psDGN->abyElem)-1] = 0;
 
     psDGN->nElemBytes = nWords * 2 + 4;
 
@@ -123,10 +125,10 @@ int DGNLoadRawElement( DGNInfo *psDGN, int *pnType, int *pnLevel )
 /* -------------------------------------------------------------------- */
 /*      Return requested info.                                          */
 /* -------------------------------------------------------------------- */
-    if( pnType != NULL )
+    if( pnType != nullptr )
         *pnType = nType;
 
-    if( pnLevel != NULL )
+    if( pnLevel != nullptr )
         *pnLevel = nLevel;
 
     return TRUE;
@@ -148,7 +150,7 @@ DGNGetRawExtents( DGNInfo *psDGN, int nType, unsigned char *pabyRawData,
                   GUInt32 *pnXMax, GUInt32 *pnYMax, GUInt32 *pnZMax )
 
 {
-    if( pabyRawData == NULL )
+    if( pabyRawData == nullptr )
         pabyRawData = psDGN->abyElem + 0;
 
     switch( nType )
@@ -171,12 +173,12 @@ DGNGetRawExtents( DGNInfo *psDGN, int nType, unsigned char *pabyRawData,
       case DGNT_3DSOLID_HEADER:
         *pnXMin = DGN_INT32( pabyRawData + 4 );
         *pnYMin = DGN_INT32( pabyRawData + 8 );
-        if( pnZMin != NULL )
+        if( pnZMin != nullptr )
             *pnZMin = DGN_INT32( pabyRawData + 12 );
 
         *pnXMax = DGN_INT32( pabyRawData + 16 );
         *pnYMax = DGN_INT32( pabyRawData + 20 );
-        if( pnZMax != NULL )
+        if( pnZMax != nullptr )
             *pnZMax = DGN_INT32( pabyRawData + 24 );
         return true;
 
@@ -228,7 +230,7 @@ int DGNGetElementExtents( DGNHandle hDGN, DGNElemCore *psElement,
 /*      Get the extents if we have raw data in the element, or          */
 /*      loaded in the file buffer.                                      */
 /* -------------------------------------------------------------------- */
-    if( psElement->raw_data != NULL )
+    if( psElement->raw_data != nullptr )
         bResult = DGNGetRawExtents( psDGN, psElement->type,
                                     psElement->raw_data,
                                     anMin + 0, anMin + 1, anMin + 2,
@@ -277,7 +279,7 @@ int DGNGetElementExtents( DGNHandle hDGN, DGNElemCore *psElement,
 static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
 
 {
-    DGNElemCore *psElement = NULL;
+    DGNElemCore *psElement = nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Handle based on element type.                                   */
@@ -426,7 +428,7 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
       case DGNT_LINE:
       {
           DGNElemMultiPoint *psLine = static_cast<DGNElemMultiPoint *>(
-              CPLCalloc(sizeof(DGNElemMultiPoint), 1));
+              CPLCalloc(sizeof(DGNElemMultiPoint) + sizeof(DGNPoint), 1));
           psElement = (DGNElemCore *) psLine;
           psElement->stype = DGNST_MULTIPOINT;
           DGNParseCore( psDGN, psElement );
@@ -493,10 +495,10 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
           if( count < 2 )
           {
               CPLError(CE_Failure, CPLE_AssertionFailed, "count < 2");
-              return NULL;
+              return nullptr;
           }
           DGNElemMultiPoint *psLine = static_cast<DGNElemMultiPoint *>(
-              CPLCalloc(sizeof(DGNElemMultiPoint)+(count-2)*sizeof(DGNPoint),
+              CPLCalloc(sizeof(DGNElemMultiPoint)+(count-1)*sizeof(DGNPoint),
                         1));
           psElement = (DGNElemCore *) psLine;
           psElement->stype = DGNST_MULTIPOINT;
@@ -509,7 +511,7 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
               {
                   CPLError(CE_Failure, CPLE_AssertionFailed, "new_count < 2");
                   DGNFreeElement(psDGN, psElement);
-                  return NULL;
+                  return nullptr;
               }
               CPLError( CE_Warning, CPLE_AppDefined,
                         "Trimming multipoint vertices to %d from %d because\n"
@@ -533,8 +535,9 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
                     }
               }
           }
-          psLine->num_vertices = count;
-          for( int i = 0; i < psLine->num_vertices; i++ )
+          for( int i = 0; i < count &&
+                          (( psDGN->dimension == 3 ) ? 46 : 42) +
+                                i*pntsize + 4 <= psDGN->nElemBytes; i++ )
           {
               psLine->vertices[i].x =
                   DGN_INT32( psDGN->abyElem + 38 + i*pntsize );
@@ -552,6 +555,7 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
                 psLine->vertices[i].y += dy / 32767.0;
               }
               DGNTransformPoint( psDGN, psLine->vertices + i );
+              psLine->num_vertices = i+1;
           }
       }
       break;
@@ -861,8 +865,8 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
         if( nLevel == 24 )
         {
             psElement = DGNParseTagSet( psDGN );
-            if( psElement == NULL )
-                return NULL;
+            if( psElement == nullptr )
+                return nullptr;
         }
         else
         {
@@ -878,7 +882,7 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
           if( psDGN->dimension != 3 )
           {
               CPLError(CE_Failure, CPLE_AssertionFailed, "psDGN->dimension != 3");
-              return NULL;
+              return nullptr;
           }
 
           DGNElemCone *psCone =
@@ -991,7 +995,7 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
           if( numverts <= 0 )
           {
               CPLError(CE_Failure, CPLE_AssertionFailed, "numverts <= 0");
-              return NULL;
+              return nullptr;
           }
 
           DGNElemBSplineSurfaceBoundary *psBounds =
@@ -1019,9 +1023,9 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
           }
           // Read B-Spline surface boundary
           psBounds->number = psDGN->abyElem[36] + psDGN->abyElem[37]*256;
-          psBounds->numverts = numverts;
 
-          for (int i=0;i<psBounds->numverts;i++) {
+          for (int i=0;i<numverts &&
+                       44 + i * 8 + 4 <= psDGN->nElemBytes;i++) {
             psBounds->vertices[i].x = DGN_INT32( psDGN->abyElem + 40 + i*8 );
             psBounds->vertices[i].y = DGN_INT32( psDGN->abyElem + 44 + i*8 );
             psBounds->vertices[i].z = 0;
@@ -1033,6 +1037,7 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
                 psBounds->vertices[i].x += dx / 32767.0;
                 psBounds->vertices[i].y += dy / 32767.0;
             }
+            psBounds->numverts = static_cast<short>(i+1);
           }
         }
       break;
@@ -1103,7 +1108,7 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
     psElement->element_id = psDGN->next_element_id - 1;
 
     psElement->offset =
-        static_cast<int>(VSIFTell( psDGN->fp )) - psDGN->nElemBytes;
+        static_cast<int>(VSIFTellL( psDGN->fp )) - psDGN->nElemBytes;
     psElement->size = psDGN->nElemBytes;
 
     return psElement;
@@ -1147,7 +1152,7 @@ DGNElemCore *DGNReadElement( DGNHandle hDGN )
         bInsideFilter = true;
 
         if( !DGNLoadRawElement( psDGN, &nType, &nLevel ) )
-            return NULL;
+            return nullptr;
 
         if( psDGN->has_spatial_filter )
         {
@@ -1158,9 +1163,9 @@ DGNElemCore *DGNReadElement( DGNHandle hDGN )
             GUInt32 nXMax = 0;
             GUInt32 nYMin = 0;
             GUInt32 nYMax = 0;
-            if( !DGNGetRawExtents( psDGN, nType, NULL,
-                                   &nXMin, &nYMin, NULL,
-                                   &nXMax, &nYMax, NULL ) )
+            if( !DGNGetRawExtents( psDGN, nType, nullptr,
+                                   &nXMin, &nYMin, nullptr,
+                                   &nXMax, &nYMax, nullptr ) )
             {
                 /* If we don't have spatial characteristics for the element
                    we will pass it through. */
@@ -1380,12 +1385,40 @@ static DGNElemCore *DGNParseTagSet( DGNInfo * psDGN )
     {
         DGNTagDef *tagDef = psTagSet->tagList + iTag;
 
-        if( nDataOffset >= static_cast<size_t>(psDGN->nElemBytes) )
+        // Check the buffer is large enough to read all tagDef components
+        size_t nDataOffsetEnd = nDataOffset;
+        if( nDataOffsetEnd <= static_cast<size_t>(psDGN->nElemBytes) )
+        {
+            nDataOffsetEnd += strlen((char *)psDGN->abyElem + nDataOffsetEnd)+1 +
+                              2;
+        }
+        if( nDataOffsetEnd <= static_cast<size_t>(psDGN->nElemBytes) )
+        {
+            nDataOffsetEnd += strlen((char *)psDGN->abyElem + nDataOffsetEnd)+1 +
+                              2 + 5;
+            if( tagDef->type == 1 )
+            {
+                nDataOffsetEnd += strlen(tagDef->defaultValue.string)+1;
+            }
+            else if( tagDef->type == 3 || tagDef->type == 5 )
+            {
+                nDataOffsetEnd += 4;
+            }
+            else if( tagDef->type == 4 )
+            {
+                nDataOffsetEnd += 8;
+            }
+            else
+            {
+                nDataOffsetEnd += 4;
+            }
+        }
+        if( nDataOffsetEnd > static_cast<size_t>(psDGN->nElemBytes) )
         {
             CPLError(CE_Failure, CPLE_AssertionFailed,
                      "nDataOffset >= static_cast<size_t>(psDGN->nElemBytes)");
             DGNFreeElement(psDGN, psElement);
-            return NULL;
+            return nullptr;
         }
 
         /* collect tag name. */
@@ -1554,10 +1587,10 @@ static DGNElemCore *DGNParseTCB( DGNInfo * psDGN )
 
 void DGNFreeElement( CPL_UNUSED DGNHandle hDGN, DGNElemCore *psElement )
 {
-    if( psElement->attr_data != NULL )
+    if( psElement->attr_data != nullptr )
         VSIFree( psElement->attr_data );
 
-    if( psElement->raw_data != NULL )
+    if( psElement->raw_data != nullptr )
         VSIFree( psElement->raw_data );
 
     if( psElement->stype == DGNST_TAG_SET )
@@ -1603,7 +1636,7 @@ void DGNRewind( DGNHandle hDGN )
 {
     DGNInfo *psDGN = (DGNInfo *) hDGN;
 
-    VSIRewind( psDGN->fp );
+    VSIRewindL( psDGN->fp );
 
     psDGN->next_element_id = 0;
     psDGN->in_complex_group = false;
@@ -1697,7 +1730,7 @@ int DGNLoadTCB( DGNHandle hDGN )
     while( !psDGN->got_tcb )
     {
         DGNElemCore *psElem = DGNReadElement( hDGN );
-        if( psElem == NULL )
+        if( psElem == nullptr )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
                       "DGNLoadTCB() - unable to find TCB in file." );
@@ -1744,7 +1777,7 @@ const DGNElementInfo *DGNGetElementIndex( DGNHandle hDGN, int *pnElementCount )
 
     DGNBuildIndex( psDGN );
 
-    if( pnElementCount != NULL )
+    if( pnElementCount != nullptr )
         *pnElementCount = psDGN->element_count;
 
     return psDGN->element_index;
@@ -1828,7 +1861,7 @@ void DGNBuildIndex( DGNInfo *psDGN )
 
     int nMaxElements = 0;
 
-    long nLastOffset = VSIFTell( psDGN->fp );
+    vsi_l_offset nLastOffset = VSIFTellL( psDGN->fp );
     while( DGNLoadRawElement( psDGN, &nType, &nLevel ) )
     {
         if( psDGN->element_count == nMaxElements )
@@ -1844,7 +1877,7 @@ void DGNBuildIndex( DGNInfo *psDGN )
         psEI->level = (unsigned char) nLevel;
         psEI->type = (unsigned char) nType;
         psEI->flags = 0;
-        psEI->offset = (long) nLastOffset;
+        psEI->offset = nLastOffset;
 
         if( psDGN->abyElem[0] & 0x80 )
             psEI->flags |= DGNEIF_COMPLEX;
@@ -1898,7 +1931,7 @@ void DGNBuildIndex( DGNInfo *psDGN )
 
         if( !(psEI->flags & DGNEIF_DELETED)
             && !(psEI->flags & DGNEIF_COMPLEX)
-            && DGNGetRawExtents( psDGN, nType, NULL,
+            && DGNGetRawExtents( psDGN, nType, nullptr,
                                  anRegion+0, anRegion+1, anRegion+2,
                                  anRegion+3, anRegion+4, anRegion+5 ) )
         {
@@ -1935,7 +1968,7 @@ void DGNBuildIndex( DGNInfo *psDGN )
 
         psDGN->element_count++;
 
-        nLastOffset = VSIFTell( psDGN->fp );
+        nLastOffset = VSIFTellL( psDGN->fp );
     }
 
     DGNRewind( psDGN );
