@@ -35,6 +35,7 @@
 #include "cpl_conv.h"
 #include "cpl_error.h"
 #include "cpl_http.h"
+#include "cpl_multiproc.h"
 #include "cpl_string.h"
 #include "cpl_vsi.h"
 // #include "json_object.h"
@@ -45,7 +46,11 @@
 #include "ogrgeojsonutils.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrgeojsondriver.cpp 40541 2017-10-23 20:46:00Z rouault $");
+CPL_CVSID("$Id: ogrgeojsondriver.cpp 8f13232b6c2445e31b5e813c898af7dbfc4ea759 2018-09-15 14:52:33 +0200 Even Rouault $")
+
+static CPLMutex* ghMutex = nullptr;
+static char* gpszSource = nullptr;
+static char* gpszText = nullptr;
 
 class OGRESRIFeatureServiceDataset;
 
@@ -99,7 +104,7 @@ class OGRESRIFeatureServiceDataset: public GDALDataset
 
     int GetLayerCount() override { return 1; }
     OGRLayer* GetLayer( int nLayer ) override
-        { return (nLayer == 0) ? poLayer : NULL; }
+        { return (nLayer == 0) ? poLayer : nullptr; }
 
     OGRLayer* GetUnderlyingLayer() { return poCurrent->GetLayer(0); }
 
@@ -167,13 +172,13 @@ OGRFeature* OGRESRIFeatureServiceLayer::GetNextFeature()
     {
         const bool bWasInFirstPage = !bOtherPage;
         OGRFeature* poSrcFeat = poDS->GetUnderlyingLayer()->GetNextFeature();
-        if( poSrcFeat == NULL )
+        if( poSrcFeat == nullptr )
         {
             if( !poDS->LoadNextPage() )
-                return NULL;
+                return nullptr;
             poSrcFeat = poDS->GetUnderlyingLayer()->GetNextFeature();
-            if( poSrcFeat == NULL )
-                return NULL;
+            if( poSrcFeat == nullptr )
+                return nullptr;
             bOtherPage = true;
             if( bWasInFirstPage && poSrcFeat->GetFID() != 0 &&
                 poSrcFeat->GetFID() == nFirstFID )
@@ -181,7 +186,7 @@ OGRFeature* OGRESRIFeatureServiceLayer::GetNextFeature()
                 // End-less looping
                 CPLDebug("ESRIJSON", "Scrolling not working. Stopping");
                 delete poSrcFeat;
-                return NULL;
+                return nullptr;
             }
             if( bWasInFirstPage && poSrcFeat->GetFID() == 0 &&
                 nLastFID == nFeaturesRead - 1 )
@@ -202,9 +207,9 @@ OGRFeature* OGRESRIFeatureServiceLayer::GetNextFeature()
         nFeaturesRead ++;
         delete poSrcFeat;
 
-        if( (m_poFilterGeom == NULL
+        if( (m_poFilterGeom == nullptr
              || FilterGeometry( poFeature->GetGeometryRef() ) )
-            && (m_poAttrQuery == NULL
+            && (m_poAttrQuery == nullptr
                 || m_poAttrQuery->Evaluate( poFeature )) )
         {
             return poFeature;
@@ -220,7 +225,7 @@ OGRFeature* OGRESRIFeatureServiceLayer::GetNextFeature()
 int OGRESRIFeatureServiceLayer::TestCapability( const char* pszCap )
 {
     if( EQUAL(pszCap, OLCFastFeatureCount) )
-        return m_poAttrQuery == NULL && m_poFilterGeom == NULL;
+        return m_poAttrQuery == nullptr && m_poFilterGeom == nullptr;
     if( EQUAL(pszCap, OLCFastGetExtent) )
         return FALSE;
     return poDS->GetUnderlyingLayer()->TestCapability(pszCap);
@@ -233,14 +238,13 @@ int OGRESRIFeatureServiceLayer::TestCapability( const char* pszCap )
 GIntBig OGRESRIFeatureServiceLayer::GetFeatureCount( int bForce )
 {
     GIntBig nFeatureCount = -1;
-    if( m_poAttrQuery == NULL && m_poFilterGeom == NULL )
+    if( m_poAttrQuery == nullptr && m_poFilterGeom == nullptr )
     {
         const CPLString osNewURL =
             CPLURLAddKVP(poDS->GetURL(), "returnCountOnly", "true");
-        CPLHTTPResult* pResult = NULL;
         CPLErrorReset();
-        pResult = CPLHTTPFetch( osNewURL, NULL );
-        if( pResult != NULL &&
+        CPLHTTPResult* pResult = CPLHTTPFetch( osNewURL, nullptr );
+        if( pResult != nullptr &&
             pResult->nDataLen != 0 &&
             CPLGetLastErrorNo() == 0 &&
             pResult->nStatus == 0 )
@@ -276,8 +280,8 @@ OGRErr OGRESRIFeatureServiceLayer::GetExtent( OGREnvelope *psExtent,
         CPLURLAddKVP(poDS->GetURL(), "returnExtentOnly", "true");
     osNewURL = CPLURLAddKVP(osNewURL, "f", "geojson");
     CPLErrorReset();
-    CPLHTTPResult* pResult = CPLHTTPFetch( osNewURL, NULL );
-    if( pResult != NULL && pResult->nDataLen != 0 && CPLGetLastErrorNo() == 0 &&
+    CPLHTTPResult* pResult = CPLHTTPFetch( osNewURL, nullptr );
+    if( pResult != nullptr && pResult->nDataLen != 0 && CPLGetLastErrorNo() == 0 &&
         pResult->nStatus == 0 )
     {
         const char* pszBBox =
@@ -396,11 +400,17 @@ int OGRESRIFeatureServiceDataset::LoadPage()
                                       CPLSPrintf(CPL_FRMT_GIB, nLastOffset));
     OGRGeoJSONDataSource* poDS = new OGRGeoJSONDataSource();
     GDALOpenInfo oOpenInfo(osNewURL, GA_ReadOnly);
-    if( !poDS->Open( &oOpenInfo, GeoJSONGetSourceType( &oOpenInfo ) ) ||
+    GeoJSONSourceType nSrcType;
+    if( EQUAL(poCurrent->GetJSonFlavor(), "GeoJSON") )
+        nSrcType = GeoJSONGetSourceType( &oOpenInfo );
+    else
+        nSrcType = ESRIJSONDriverGetSourceType( &oOpenInfo );
+    if( !poDS->Open( &oOpenInfo, nSrcType,
+                     poCurrent->GetJSonFlavor() ) ||
         poDS->GetLayerCount() == 0 )
     {
         delete poDS;
-        poDS = NULL;
+        poDS = nullptr;
         return FALSE;
     }
     delete poCurrent;
@@ -423,8 +433,11 @@ static int OGRGeoJSONDriverIdentifyInternal( GDALOpenInfo* poOpenInfo,
     nSrcType = GeoJSONGetSourceType( poOpenInfo );
     if( nSrcType == eGeoJSONSourceUnknown )
         return FALSE;
-    if( nSrcType == eGeoJSONSourceService )
+    if( nSrcType == eGeoJSONSourceService &&
+        !STARTS_WITH_CI(poOpenInfo->pszFilename, "GeoJSON:") )
+    {
         return -1;
+    }
     return TRUE;
 }
 
@@ -446,8 +459,20 @@ static GDALDataset* OGRGeoJSONDriverOpen( GDALOpenInfo* poOpenInfo )
 {
     GeoJSONSourceType nSrcType;
     if( OGRGeoJSONDriverIdentifyInternal(poOpenInfo, nSrcType) == FALSE )
-        return NULL;
+    {
+        return nullptr;
+    }
+    return OGRGeoJSONDriverOpenInternal(poOpenInfo, nSrcType, "GeoJSON");
+}
 
+/************************************************************************/
+/*                     OGRGeoJSONDriverOpenInternal()                   */
+/************************************************************************/
+
+GDALDataset* OGRGeoJSONDriverOpenInternal( GDALOpenInfo* poOpenInfo,
+                                           GeoJSONSourceType nSrcType,
+                                           const char* pszJSonFlavor )
+{
     OGRGeoJSONDataSource* poDS = new OGRGeoJSONDataSource();
 
 /* -------------------------------------------------------------------- */
@@ -459,16 +484,16 @@ static GDALDataset* OGRGeoJSONDriverOpen( GDALOpenInfo* poOpenInfo )
     //       described in document 'RFC 10: OGR Open Parameters'.
 
     poDS->SetGeometryTranslation( OGRGeoJSONDataSource::eGeometryPreserve );
-    const char* pszOpt = CPLGetConfigOption("GEOMETRY_AS_COLLECTION", NULL);
-    if( NULL != pszOpt && STARTS_WITH_CI(pszOpt, "YES") )
+    const char* pszOpt = CPLGetConfigOption("GEOMETRY_AS_COLLECTION", nullptr);
+    if( nullptr != pszOpt && STARTS_WITH_CI(pszOpt, "YES") )
     {
         poDS->SetGeometryTranslation(
             OGRGeoJSONDataSource::eGeometryAsCollection );
     }
 
     poDS->SetAttributesTranslation( OGRGeoJSONDataSource::eAttributesPreserve );
-    pszOpt = CPLGetConfigOption("ATTRIBUTES_SKIP", NULL);
-    if( NULL != pszOpt && STARTS_WITH_CI(pszOpt, "YES") )
+    pszOpt = CPLGetConfigOption("ATTRIBUTES_SKIP", nullptr);
+    if( nullptr != pszOpt && STARTS_WITH_CI(pszOpt, "YES") )
     {
         poDS->SetAttributesTranslation(
             OGRGeoJSONDataSource::eAttributesSkip );
@@ -477,13 +502,13 @@ static GDALDataset* OGRGeoJSONDriverOpen( GDALOpenInfo* poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Open and start processing GeoJSON datasource to OGR objects.    */
 /* -------------------------------------------------------------------- */
-    if( !poDS->Open( poOpenInfo, nSrcType ) )
+    if( !poDS->Open( poOpenInfo, nSrcType, pszJSonFlavor ) )
     {
         delete poDS;
-        poDS = NULL;
+        poDS = nullptr;
     }
 
-    if( poDS != NULL && poDS->HasOtherPages() &&
+    if( poDS != nullptr && poDS->HasOtherPages() &&
         (STARTS_WITH(poOpenInfo->pszFilename, "http") ||
          STARTS_WITH(poOpenInfo->pszFilename, "/vsimem/")) )
     {
@@ -491,8 +516,8 @@ static GDALDataset* OGRGeoJSONDriverOpen( GDALOpenInfo* poOpenInfo )
                                                "FEATURE_SERVER_PAGING");
         const bool bHasResultOffset =
           !CPLURLGetValue(poOpenInfo->pszFilename, "resultOffset").empty();
-        if( (!bHasResultOffset && (pszFSP == NULL || CPLTestBool(pszFSP))) ||
-            (bHasResultOffset && pszFSP != NULL && CPLTestBool(pszFSP)) )
+        if( (!bHasResultOffset && (pszFSP == nullptr || CPLTestBool(pszFSP))) ||
+            (bHasResultOffset && pszFSP != nullptr && CPLTestBool(pszFSP)) )
         {
             return new OGRESRIFeatureServiceDataset(poOpenInfo->pszFilename,
                                                     poDS);
@@ -518,7 +543,7 @@ static GDALDataset *OGRGeoJSONDriverCreate( const char * pszName,
     if( !poDS->Create( pszName, papszOptions ) )
     {
         delete poDS;
-        poDS = NULL;
+        poDS = nullptr;
     }
 
     return poDS;
@@ -540,6 +565,55 @@ static CPLErr OGRGeoJSONDriverDelete( const char *pszFilename )
     return CE_Failure;
 }
 
+/************************************************************************/
+/*                      OGRGeoJSONDriverStoreContent()                  */
+/************************************************************************/
+
+void OGRGeoJSONDriverStoreContent( const char* pszSource, char* pszText )
+{
+    CPLMutexHolderD(&ghMutex);
+    CPLAssert( pszSource );
+    CPLAssert( pszText );
+
+    CPLFree(gpszSource);
+    CPLFree(gpszText);
+    gpszSource = CPLStrdup(pszSource);
+    gpszText = pszText;
+}
+
+/************************************************************************/
+/*                    OGRGeoJSONDriverStealStoredContent()              */
+/************************************************************************/
+
+char* OGRGeoJSONDriverStealStoredContent( const char* pszSource )
+{
+    CPLMutexHolderD(&ghMutex);
+    if( gpszSource && EQUAL(pszSource, gpszSource) )
+    {
+        char* pszRet = gpszText;
+        CPLFree(gpszSource);
+        gpszSource = nullptr;
+        gpszText = nullptr;
+        return pszRet;
+    }
+    return nullptr;
+}
+
+/************************************************************************/
+/*                        OGRGeoJSONDriverUnload()                      */
+/************************************************************************/
+
+static void OGRGeoJSONDriverUnload( GDALDriver* )
+{
+    if( ghMutex )
+        CPLDestroyMutex(ghMutex);
+    ghMutex = nullptr;
+    CPLFree(gpszSource);
+    CPLFree(gpszText);
+    gpszSource = nullptr;
+    gpszText = nullptr;
+}
+
 /************************************************************************/
 /*                           RegisterOGRGeoJSON()                       */
 /************************************************************************/
@@ -549,7 +623,7 @@ void RegisterOGRGeoJSON()
     if( !GDAL_CHECK_VERSION("OGR/GeoJSON driver") )
         return;
 
-    if( GDALGetDriverByName( "GeoJSON" ) != NULL )
+    if( GDALGetDriverByName( "GeoJSON" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
@@ -557,7 +631,7 @@ void RegisterOGRGeoJSON()
     poDriver->SetDescription( "GeoJSON" );
     poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "GeoJSON" );
-    poDriver->SetMetadataItem( GDAL_DMD_EXTENSIONS, "json geojson topojson" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSIONS, "json geojson" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drv_geojson.html" );
 
     poDriver->SetMetadataItem( GDAL_DMD_OPENOPTIONLIST,
@@ -582,17 +656,25 @@ void RegisterOGRGeoJSON()
 "  <Option name='RFC7946' type='boolean' description='Whether to use RFC 7946 standard. Otherwise GeoJSON 2008 initial version will be used' default='NO'/>"
 "  <Option name='WRITE_NAME' type='boolean' description='Whether to write a &quot;name&quot; property at feature collection level with layer name' default='YES'/>"
 "  <Option name='DESCRIPTION' type='string' description='(Long) description to write in a &quot;description&quot; property at feature collection level'/>"
+"  <Option name='ID_FIELD' type='string' description='Name of the source field that must be used as the id member of Feature features'/>"
+"  <Option name='ID_TYPE' type='string-select' description='Type of the id member of Feature features'>"
+"    <Value>AUTO</Value>"
+"    <Value>String</Value>"
+"    <Value>Integer</Value>"
+"  </Option>"
 "</LayerCreationOptionList>");
 
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONFIELDDATATYPES,
                                "Integer Integer64 Real String IntegerList "
                                "Integer64List RealList StringList" );
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONFIELDDATASUBTYPES, "Boolean" );
 
     poDriver->pfnOpen = OGRGeoJSONDriverOpen;
     poDriver->pfnIdentify = OGRGeoJSONDriverIdentify;
     poDriver->pfnCreate = OGRGeoJSONDriverCreate;
     poDriver->pfnDelete = OGRGeoJSONDriverDelete;
+    poDriver->pfnUnloadDriver = OGRGeoJSONDriverUnload;
 
     GetGDALDriverManager()->RegisterDriver( poDriver );
 }
