@@ -36,7 +36,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: ogrgmlasdatasource.cpp 39652 2017-07-24 17:32:40Z rouault $");
+CPL_CVSID("$Id: ogrgmlasdatasource.cpp 0b629564aaf603602a76d48b0d0e1e501727ad59 2017-12-17 20:55:57Z Even Rouault $")
 
 /************************************************************************/
 /*                          OGRGMLASDataSource()                        */
@@ -46,8 +46,8 @@ OGRGMLASDataSource::OGRGMLASDataSource()
 {
     OGRInitializeXerces();
 
-    m_fpGML = NULL;
-    m_fpGMLParser = NULL;
+    m_fpGML = nullptr;
+    m_fpGMLParser = nullptr;
     m_bLayerInitFinished = false;
     m_bValidate = false;
     m_bSchemaFullChecking = false;
@@ -57,11 +57,11 @@ OGRGMLASDataSource::OGRGMLASDataSource()
     m_bFirstPassDone = false;
     m_eSwapCoordinates = GMLAS_SWAP_AUTO;
     m_nFileSize = 0;
-    m_poReader = NULL;
+    m_poReader = nullptr;
     m_bEndOfReaderLayers = false;
     m_nCurMetadataLayerIdx = -1;
     m_poFieldsMetadataLayer = new OGRMemLayer
-                                    (szOGR_FIELDS_METADATA, NULL, wkbNone );
+                                    (szOGR_FIELDS_METADATA, nullptr, wkbNone );
     m_bFoundSWE = false;
 
     {
@@ -128,7 +128,7 @@ OGRGMLASDataSource::OGRGMLASDataSource()
     }
 
     m_poLayersMetadataLayer = new OGRMemLayer
-                                    (szOGR_LAYERS_METADATA, NULL, wkbNone );
+                                    (szOGR_LAYERS_METADATA, nullptr, wkbNone );
     {
         OGRFieldDefn oFieldDefn(szLAYER_NAME, OFTString);
         m_poLayersMetadataLayer->CreateField(&oFieldDefn);
@@ -155,7 +155,7 @@ OGRGMLASDataSource::OGRGMLASDataSource()
     }
 
     m_poRelationshipsLayer = new OGRMemLayer(szOGR_LAYER_RELATIONSHIPS,
-                                             NULL, wkbNone );
+                                             nullptr, wkbNone );
     {
         OGRFieldDefn oFieldDefn(szPARENT_LAYER, OFTString);
         m_poRelationshipsLayer->CreateField(&oFieldDefn);
@@ -177,7 +177,7 @@ OGRGMLASDataSource::OGRGMLASDataSource()
         m_poRelationshipsLayer->CreateField(&oFieldDefn);
     }
     m_poOtherMetadataLayer = new OGRMemLayer(szOGR_OTHER_METADATA,
-                                             NULL, wkbNone );
+                                             nullptr, wkbNone );
     {
         OGRFieldDefn oFieldDefn(szKEY, OFTString);
         m_poOtherMetadataLayer->CreateField(&oFieldDefn);
@@ -200,9 +200,9 @@ OGRGMLASDataSource::~OGRGMLASDataSource()
     delete m_poLayersMetadataLayer;
     delete m_poRelationshipsLayer;
     delete m_poOtherMetadataLayer;
-    if( m_fpGML != NULL )
+    if( m_fpGML != nullptr )
         VSIFCloseL(m_fpGML);
-    if( m_fpGMLParser != NULL )
+    if( m_fpGMLParser != nullptr )
         VSIFCloseL(m_fpGMLParser);
     delete m_poReader;
 
@@ -227,12 +227,13 @@ OGRLayer    *OGRGMLASDataSource::GetLayer(int i)
     const int nBaseLayers = static_cast<int>(m_apoLayers.size());
     if( i >= nBaseLayers )
     {
+        RunFirstPassIfNeeded(nullptr, nullptr, nullptr);
         if( i - nBaseLayers < static_cast<int>(m_apoRequestedMetadataLayers.size()) )
             return m_apoRequestedMetadataLayers[i - nBaseLayers];
     }
 
     if( i < 0 || i >= nBaseLayers )
-        return NULL;
+        return nullptr;
     return m_apoLayers[i];
 }
 
@@ -261,11 +262,12 @@ OGRLayer    *OGRGMLASDataSource::GetLayerByName(const char* pszName)
             {
                 m_apoRequestedMetadataLayers.push_back(apoLayers[i]);
             }
+            RunFirstPassIfNeeded(nullptr, nullptr, nullptr);
             return apoLayers[i];
         }
     }
 
-    return NULL;
+    return nullptr;
 }
 
 /************************************************************************/
@@ -495,11 +497,11 @@ void OGRGMLASDataSource::FillOtherMetadataLayer(
             poFeature->SetField( szKEY, szCONFIGURATION_FILENAME );
             char* pszCurDir = CPLGetCurrentDir();
             if( !bKeepRelativePathsForMetadata &&
-                CPLIsFilenameRelative(osConfigFile) && pszCurDir != NULL)
+                CPLIsFilenameRelative(osConfigFile) && pszCurDir != nullptr)
             {
                 poFeature->SetField( szVALUE,
                                      CPLFormFilename(pszCurDir,
-                                                     osConfigFile, NULL));
+                                                     osConfigFile, nullptr));
             }
             else
             {
@@ -510,8 +512,8 @@ void OGRGMLASDataSource::FillOtherMetadataLayer(
                             m_poOtherMetadataLayer->CreateFeature(poFeature) );
             delete poFeature;
 
-            GByte* pabyRet = NULL;
-            if( VSIIngestFile( NULL, osConfigFile, &pabyRet, NULL, -1 ) )
+            GByte* pabyRet = nullptr;
+            if( VSIIngestFile( nullptr, osConfigFile, &pabyRet, nullptr, -1 ) )
             {
                 poFeature = new OGRFeature(poFDefn);
                 poFeature->SetField( szKEY, szCONFIGURATION_INLINED );
@@ -554,10 +556,10 @@ void OGRGMLASDataSource::FillOtherMetadataLayer(
         poFeature->SetField( szKEY, szDOCUMENT_FILENAME );
         char* pszCurDir = CPLGetCurrentDir();
         if( !bKeepRelativePathsForMetadata &&
-            CPLIsFilenameRelative(m_osGMLFilename) && pszCurDir != NULL)
+            CPLIsFilenameRelative(m_osGMLFilename) && pszCurDir != nullptr)
         {
             osAbsoluteGMLFilename = CPLFormFilename(pszCurDir,
-                                                    m_osGMLFilename, NULL);
+                                                    m_osGMLFilename, nullptr);
         }
         else
             osAbsoluteGMLFilename = m_osGMLFilename;
@@ -594,7 +596,7 @@ void OGRGMLASDataSource::FillOtherMetadataLayer(
              osXSDFilename.find("https://") != 0 &&
              CPLIsFilenameRelative(osXSDFilename)) ?
                 CPLString(CPLFormFilename(CPLGetDirname(osAbsoluteGMLFilename),
-                                        osXSDFilename, NULL)) :
+                                        osXSDFilename, nullptr)) :
                 osXSDFilename );
         poFeature->SetField( szVALUE, osAbsoluteXSDFilename.c_str());
         CPL_IGNORE_RET_VAL( m_poOtherMetadataLayer->CreateFeature(poFeature) );
@@ -614,12 +616,10 @@ void OGRGMLASDataSource::FillOtherMetadataLayer(
         nNSIdx ++;
     }
 
-    std::map<CPLString,CPLString>::const_iterator oIter =
-                                                m_oMapURIToPrefix.begin();
-    for( ; oIter != m_oMapURIToPrefix.end(); ++oIter )
+    for( const auto& oIter: m_oMapURIToPrefix )
     {
-        const CPLString& osURI( oIter->first );
-        const CPLString& osPrefix( oIter->second );
+        const CPLString& osURI( oIter.first );
+        const CPLString& osPrefix( oIter.second );
 
         if( oSetVisitedURI.find( osURI ) == oSetVisitedURI.end() &&
             osURI != szXML_URI &&
@@ -657,18 +657,19 @@ void OGRGMLASDataSource::FillOtherMetadataLayer(
         delete poFeature;
     }
 
-    std::set<CPLString>::const_iterator oIterSet = oSetSchemaURLs.begin();
     int nSchemaIdx = 1;
-    for( ; bExposeSchemaNames &&
-           oIterSet != oSetSchemaURLs.end(); ++oIterSet )
+    if( bExposeSchemaNames )
     {
-        OGRFeature* poFeature = new OGRFeature(poFDefn);
-        poFeature->SetField( szKEY, CPLSPrintf(szSCHEMA_NAME_FMT, nSchemaIdx) );
-        poFeature->SetField( szVALUE, (*oIterSet).c_str() );
-        CPL_IGNORE_RET_VAL( m_poOtherMetadataLayer->CreateFeature(poFeature) );
-        delete poFeature;
+        for( const auto& osSchemaURL: oSetSchemaURLs )
+        {
+            OGRFeature* poFeature = new OGRFeature(poFDefn);
+            poFeature->SetField( szKEY, CPLSPrintf(szSCHEMA_NAME_FMT, nSchemaIdx) );
+            poFeature->SetField( szVALUE, osSchemaURL.c_str() );
+            CPL_IGNORE_RET_VAL( m_poOtherMetadataLayer->CreateFeature(poFeature) );
+            delete poFeature;
 
-        nSchemaIdx ++;
+            nSchemaIdx ++;
+        }
     }
 }
 
@@ -682,15 +683,15 @@ std::vector<PairURIFilename> OGRGMLASDataSource::BuildXSDVector(
     std::vector<PairURIFilename> aoXSDs;
     char** papszTokens = CSLTokenizeString2(osXSDFilenames," ,",0);
     char* pszCurDir = CPLGetCurrentDir();
-    for( int i=0; papszTokens != NULL && papszTokens[i] != NULL; i++ )
+    for( int i=0; papszTokens != nullptr && papszTokens[i] != nullptr; i++ )
     {
         if( !STARTS_WITH(papszTokens[i], "http://") &&
             !STARTS_WITH(papszTokens[i], "https://") &&
             CPLIsFilenameRelative(papszTokens[i]) &&
-            pszCurDir != NULL )
+            pszCurDir != nullptr )
         {
             aoXSDs.push_back(PairURIFilename("",
-                        CPLFormFilename(pszCurDir, papszTokens[i], NULL)));
+                        CPLFormFilename(pszCurDir, papszTokens[i], nullptr)));
         }
         else
         {
@@ -745,11 +746,9 @@ bool OGRGMLASDataSource::Open(GDALOpenInfo* poOpenInfo)
                                     m_oConf.m_aosIgnoredXPaths );
 
     {
-        std::map<CPLString, std::vector<CPLString> >::const_iterator oIter =
-                            m_oConf.m_oMapChildrenElementsConstraints.begin();
         std::vector<CPLString> oVector;
-        for( ; oIter != m_oConf.m_oMapChildrenElementsConstraints.end(); ++oIter )
-            oVector.push_back(oIter->first);
+        for(const auto& oIter: m_oConf.m_oMapChildrenElementsConstraints )
+            oVector.push_back(oIter.first);
         m_oChildrenElementsConstraintsXPathMatcher.SetRefXPaths(
             m_oConf.m_oMapPrefixToURITypeConstraints,
             oVector );
@@ -777,6 +776,7 @@ bool OGRGMLASDataSource::Open(GDALOpenInfo* poOpenInfo)
                                         m_oConf.m_bCaseInsensitiveIdentifier);
     oAnalyzer.SetPGIdentifierLaundering(m_oConf.m_bPGIdentifierLaundering);
     oAnalyzer.SetMaximumFieldsForFlattening(m_oConf.m_nMaximumFieldsForFlattening);
+    oAnalyzer.SetAlwaysGenerateOGRId(m_oConf.m_bAlwaysGenerateOGRId);
 
     m_osGMLFilename = STARTS_WITH_CI(poOpenInfo->pszFilename, szGMLAS_PREFIX) ?
         CPLExpandTilde(poOpenInfo->pszFilename + strlen(szGMLAS_PREFIX)) :
@@ -785,11 +785,11 @@ bool OGRGMLASDataSource::Open(GDALOpenInfo* poOpenInfo)
     CPLString osXSDFilenames = CSLFetchNameValueDef(
                                 poOpenInfo->papszOpenOptions, szXSD_OPTION, "");
 
-    VSILFILE* fpGML = NULL;
+    VSILFILE* fpGML = nullptr;
     if( !m_osGMLFilename.empty() )
     {
         fpGML = VSIFOpenL(m_osGMLFilename, "rb");
-        if( fpGML == NULL )
+        if( fpGML == nullptr )
         {
             CPLError(CE_Failure, CPLE_FileIO, "Cannot open %s",
                      m_osGMLFilename.c_str());
@@ -953,13 +953,13 @@ bool OGRGMLASDataSource::Open(GDALOpenInfo* poOpenInfo)
     for( size_t i=0; i<aoClasses.size(); ++i )
     {
         if( aoClasses[i].GetParentXPath().empty() )
-            TranslateClasses( NULL, aoClasses[i] );
+            TranslateClasses( nullptr, aoClasses[i] );
     }
     // Then junction tables
     for( size_t i=0; i<aoClasses.size(); ++i )
     {
         if( !aoClasses[i].GetParentXPath().empty() )
-            TranslateClasses( NULL, aoClasses[i] );
+            TranslateClasses( nullptr, aoClasses[i] );
     }
 
     // And now do initialization since we need to have instantiated everything
@@ -991,7 +991,7 @@ bool OGRGMLASDataSource::Open(GDALOpenInfo* poOpenInfo)
                          m_oConf.m_bSWEProcessDataArray)) )
     {
         CPLErrorReset();
-        RunFirstPassIfNeeded( NULL, NULL, NULL );
+        RunFirstPassIfNeeded( nullptr, nullptr, nullptr );
         if( CPLFetchBool( poOpenInfo->papszOpenOptions,
                           szFAIL_IF_VALIDATION_ERROR_OPTION,
                           m_oConf.m_bFailIfValidationError ) &&
@@ -1025,14 +1025,14 @@ GMLASReader* OGRGMLASDataSource::CreateReader( VSILFILE*& fpGML,
                                                GDALProgressFunc pfnProgress,
                                                void* pProgressData )
 {
-    if( fpGML == NULL )
+    if( fpGML == nullptr )
     {
         // Try recycling an already opened and unused file pointer
         fpGML = PopUnusedGMLFilePointer();
-        if( fpGML == NULL )
+        if( fpGML == nullptr )
             fpGML = VSIFOpenL(GetGMLFilename(), "rb");
-        if( fpGML == NULL )
-            return NULL;
+        if( fpGML == nullptr )
+            return nullptr;
     }
 
     GMLASReader* poReader = new GMLASReader( GetCache(),
@@ -1054,12 +1054,12 @@ GMLASReader* OGRGMLASDataSource::CreateReader( VSILFILE*& fpGML,
     if( !RunFirstPassIfNeeded( poReader, pfnProgress, pProgressData ) )
     {
         delete poReader;
-        return NULL;
+        return nullptr;
     }
 
     poReader->SetMapIgnoredXPathToWarn( GetMapIgnoredXPathToWarn());
 
-    poReader->SetHash( GetHash() );
+    poReader->SetHash( m_osHash );
 
     return poReader;
 }
@@ -1071,7 +1071,7 @@ GMLASReader* OGRGMLASDataSource::CreateReader( VSILFILE*& fpGML,
 void OGRGMLASDataSource::ResetReading()
 {
     delete m_poReader;
-    m_poReader = NULL;
+    m_poReader = nullptr;
     for(size_t i=0; i<m_apoRequestedMetadataLayers.size();++i)
         m_apoRequestedMetadataLayers[i]->ResetReading();
     m_bEndOfReaderLayers = false;
@@ -1098,11 +1098,11 @@ OGRFeature* OGRGMLASDataSource::GetNextFeature( OGRLayer** ppoBelongingLayer,
                 OGRLayer* poLayer =
                     m_apoRequestedMetadataLayers[m_nCurMetadataLayerIdx];
                 OGRFeature* poFeature = poLayer->GetNextFeature();
-                if( poFeature != NULL )
+                if( poFeature != nullptr )
                 {
-                    if( pdfProgressPct != NULL )
+                    if( pdfProgressPct != nullptr )
                         *pdfProgressPct = 1.0;
-                    if( ppoBelongingLayer != NULL )
+                    if( ppoBelongingLayer != nullptr )
                         *ppoBelongingLayer = poLayer;
                     return poFeature;
                 }
@@ -1119,32 +1119,32 @@ OGRFeature* OGRGMLASDataSource::GetNextFeature( OGRLayer** ppoBelongingLayer,
             }
         }
 
-        if( pdfProgressPct != NULL )
+        if( pdfProgressPct != nullptr )
             *pdfProgressPct = 1.0;
-        if( ppoBelongingLayer != NULL )
-            *ppoBelongingLayer = NULL;
-        return NULL;
+        if( ppoBelongingLayer != nullptr )
+            *ppoBelongingLayer = nullptr;
+        return nullptr;
     }
 
     const double dfInitialScanRatio = 0.1;
-    if( m_poReader == NULL )
+    if( m_poReader == nullptr )
     {
         void* pScaledProgress = GDALCreateScaledProgress( 0.0, dfInitialScanRatio,
                                                           pfnProgress,
                                                           pProgressData );
 
         m_poReader = CreateReader(m_fpGMLParser,
-                                  pScaledProgress ? GDALScaledProgress : NULL,
+                                  pScaledProgress ? GDALScaledProgress : nullptr,
                                   pScaledProgress);
 
         GDALDestroyScaledProgress(pScaledProgress);
 
-        if( m_poReader == NULL )
+        if( m_poReader == nullptr )
         {
-            if( pdfProgressPct != NULL )
+            if( pdfProgressPct != nullptr )
                 *pdfProgressPct = 1.0;
-            if( ppoBelongingLayer != NULL )
-                *ppoBelongingLayer = NULL;
+            if( ppoBelongingLayer != nullptr )
+                *ppoBelongingLayer = nullptr;
             m_bEndOfReaderLayers = true;
             if( !m_apoRequestedMetadataLayers.empty() )
             {
@@ -1154,7 +1154,7 @@ OGRFeature* OGRGMLASDataSource::GetNextFeature( OGRLayer** ppoBelongingLayer,
             }
             else
             {
-                return NULL;
+                return nullptr;
             }
         }
     }
@@ -1165,17 +1165,17 @@ OGRFeature* OGRGMLASDataSource::GetNextFeature( OGRLayer** ppoBelongingLayer,
 
     while( true )
     {
-        OGRGMLASLayer* poBelongingLayer = NULL;
+        OGRGMLASLayer* poBelongingLayer = nullptr;
         OGRFeature* poFeature = m_poReader->GetNextFeature(
                     &poBelongingLayer,
-                    pScaledProgress ? GDALScaledProgress : NULL,
+                    pScaledProgress ? GDALScaledProgress : nullptr,
                     pScaledProgress);
-        if( poFeature == NULL ||
+        if( poFeature == nullptr ||
             poBelongingLayer->EvaluateFilter(poFeature) )
         {
-            if( ppoBelongingLayer != NULL )
+            if( ppoBelongingLayer != nullptr )
                 *ppoBelongingLayer = poBelongingLayer;
-            if( pdfProgressPct != NULL )
+            if( pdfProgressPct != nullptr )
             {
                 const vsi_l_offset nOffset = VSIFTellL(m_fpGMLParser);
                 if( nOffset == m_nFileSize )
@@ -1185,7 +1185,7 @@ OGRFeature* OGRGMLASDataSource::GetNextFeature( OGRLayer** ppoBelongingLayer,
                         (1.0 - dfInitialScanRatio) * nOffset / m_nFileSize;
             }
             GDALDestroyScaledProgress(pScaledProgress);
-            if( poFeature == NULL )
+            if( poFeature == nullptr )
             {
                 m_bEndOfReaderLayers = true;
                 if( !m_apoRequestedMetadataLayers.empty() )
@@ -1196,7 +1196,7 @@ OGRFeature* OGRGMLASDataSource::GetNextFeature( OGRLayer** ppoBelongingLayer,
                 }
                 else
                 {
-                    return NULL;
+                    return nullptr;
                 }
             }
             else
@@ -1219,7 +1219,7 @@ OGRGMLASLayer* OGRGMLASDataSource::GetLayerByXPath( const CPLString& osXPath )
             return m_apoLayers[i];
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 /************************************************************************/
@@ -1228,7 +1228,7 @@ OGRGMLASLayer* OGRGMLASDataSource::GetLayerByXPath( const CPLString& osXPath )
 
 void OGRGMLASDataSource::PushUnusedGMLFilePointer( VSILFILE* fpGML )
 {
-    if( m_fpGML == NULL )
+    if( m_fpGML == nullptr )
         m_fpGML = fpGML;
     else
     {
@@ -1243,10 +1243,27 @@ void OGRGMLASDataSource::PushUnusedGMLFilePointer( VSILFILE* fpGML )
 VSILFILE* OGRGMLASDataSource::PopUnusedGMLFilePointer()
 {
     VSILFILE* fpGML = m_fpGML;
-    m_fpGML = NULL;
+    m_fpGML = nullptr;
     return fpGML;
 }
 
+/************************************************************************/
+/*                    InitReaderWithFirstPassElements()                 */
+/************************************************************************/
+
+void OGRGMLASDataSource::InitReaderWithFirstPassElements(GMLASReader* poReader)
+{
+    if( poReader != nullptr )
+    {
+        poReader->SetMapSRSNameToInvertedAxis(m_oMapSRSNameToInvertedAxis);
+        poReader->SetMapGeomFieldDefnToSRSName(m_oMapGeomFieldDefnToSRSName);
+        poReader->SetProcessDataRecord(m_bFoundSWE && m_oConf.m_bSWEProcessDataRecord);
+        poReader->SetSWEDataArrayLayers(m_apoSWEDataArrayLayers);
+        poReader->SetMapElementIdToLayer(m_oMapElementIdToLayer);
+        poReader->SetMapElementIdToPKID(m_oMapElementIdToPKID);
+    }
+}
+
 /************************************************************************/
 /*                          RunFirstPassIfNeeded()                      */
 /************************************************************************/
@@ -1257,13 +1274,7 @@ bool OGRGMLASDataSource::RunFirstPassIfNeeded( GMLASReader* poReader,
 {
     if( m_bFirstPassDone )
     {
-        if( poReader != NULL )
-        {
-            poReader->SetMapSRSNameToInvertedAxis(m_oMapSRSNameToInvertedAxis);
-            poReader->SetMapGeomFieldDefnToSRSName(m_oMapGeomFieldDefnToSRSName);
-            poReader->SetProcessDataRecord(m_bFoundSWE && m_oConf.m_bSWEProcessDataRecord);
-            poReader->SetSWEDataArrayLayers(m_apoSWEDataArrayLayers);
-        }
+        InitReaderWithFirstPassElements(poReader);
         return true;
     }
 
@@ -1282,21 +1293,23 @@ bool OGRGMLASDataSource::RunFirstPassIfNeeded( GMLASReader* poReader,
         }
     }
 
+    bool bSuccess = true;
     const bool bHasURLSpecificRules =
                 !m_oXLinkResolver.GetConf().m_aoURLSpecificRules.empty();
     if( bHasGeomFields || m_bValidate || m_bRemoveUnusedLayers ||
         m_bRemoveUnusedFields || bHasURLSpecificRules ||
+        m_oXLinkResolver.GetConf().m_bResolveInternalXLinks ||
         (m_bFoundSWE && (m_oConf.m_bSWEProcessDataRecord ||
                          m_oConf.m_bSWEProcessDataArray)) )
     {
         bool bJustOpenedFiled =false;
-        VSILFILE* fp = NULL;
+        VSILFILE* fp = nullptr;
         if( poReader )
             fp = poReader->GetFP();
         else
         {
             fp = VSIFOpenL(GetGMLFilename(), "rb");
-            if( fp == NULL )
+            if( fp == nullptr )
             {
                 return false;
             }
@@ -1322,11 +1335,14 @@ bool OGRGMLASDataSource::RunFirstPassIfNeeded( GMLASReader* poReader,
 
         poReaderFirstPass->SetMapIgnoredXPathToWarn(
                                     m_oConf.m_oMapIgnoredXPathToWarn);
+
+        poReaderFirstPass->SetHash( m_osHash );
+
         // No need to warn afterwards
         m_oConf.m_oMapIgnoredXPathToWarn.clear();
 
         std::set<CPLString> aoSetRemovedLayerNames;
-        m_bFirstPassDone = poReaderFirstPass->RunFirstPass(
+        bSuccess = poReaderFirstPass->RunFirstPass(
             pfnProgress,
             pProgressData,
             m_bRemoveUnusedLayers,
@@ -1355,7 +1371,7 @@ bool OGRGMLASDataSource::RunFirstPassIfNeeded( GMLASReader* poReader,
             OGRFeature* poFeature;
             m_poLayersMetadataLayer->ResetReading();
             while( (poFeature = m_poLayersMetadataLayer->GetNextFeature() )
-                                                                    != NULL )
+                                                                    != nullptr )
             {
                 const char* pszLayerName =
                                     poFeature->GetFieldAsString(szLAYER_NAME);
@@ -1371,7 +1387,7 @@ bool OGRGMLASDataSource::RunFirstPassIfNeeded( GMLASReader* poReader,
 
             m_poFieldsMetadataLayer->ResetReading();
             while( (poFeature = m_poFieldsMetadataLayer->GetNextFeature() )
-                                                                    != NULL )
+                                                                    != nullptr )
             {
                 const char* pszLayerName =
                                     poFeature->GetFieldAsString(szLAYER_NAME);
@@ -1391,7 +1407,7 @@ bool OGRGMLASDataSource::RunFirstPassIfNeeded( GMLASReader* poReader,
 
             m_poRelationshipsLayer->ResetReading();
             while( (poFeature = m_poRelationshipsLayer->GetNextFeature() )
-                                                                    != NULL )
+                                                                    != nullptr )
             {
                 const char* pszParentLayerName =
                                     poFeature->GetFieldAsString(szPARENT_LAYER);
@@ -1410,26 +1426,23 @@ bool OGRGMLASDataSource::RunFirstPassIfNeeded( GMLASReader* poReader,
             m_poRelationshipsLayer->ResetReading();
         }
 
-        // Store 2 maps to reinject them in real readers
+        // Store  maps to reinject them in real readers
         m_oMapSRSNameToInvertedAxis =
                         poReaderFirstPass->GetMapSRSNameToInvertedAxis();
         m_oMapGeomFieldDefnToSRSName =
                         poReaderFirstPass->GetMapGeomFieldDefnToSRSName();
 
+        m_oMapElementIdToLayer = poReaderFirstPass->GetMapElementIdToLayer();
+        m_oMapElementIdToPKID = poReaderFirstPass->GetMapElementIdToPKID();
+
         delete poReaderFirstPass;
 
         VSIFSeekL(fp, 0, SEEK_SET);
         if( bJustOpenedFiled )
             PushUnusedGMLFilePointer(fp);
 
-        if( poReader != NULL )
-        {
-            poReader->SetMapSRSNameToInvertedAxis(m_oMapSRSNameToInvertedAxis);
-            poReader->SetMapGeomFieldDefnToSRSName(m_oMapGeomFieldDefnToSRSName);
-            poReader->SetProcessDataRecord(m_bFoundSWE && m_oConf.m_bSWEProcessDataRecord);
-            poReader->SetSWEDataArrayLayers(m_apoSWEDataArrayLayers);
-        }
+        InitReaderWithFirstPassElements(poReader);
     }
 
-    return m_bFirstPassDone;
+    return bSuccess;
 }
