@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_geopackage.h 40466 2017-10-17 12:22:04Z rouault $
+ * $Id: ogr_geopackage.h 85cc62e40607cf41aee12b371271ec570fc85c83 2018-07-24 19:46:03 +0200 Even Rouault $
  *
  * Project:  GeoPackage Translator
  * Purpose:  Definition of classes for OGR GeoPackage driver.
@@ -36,12 +36,19 @@
 #include "ogrsqliteutility.h"
 
 #include <vector>
+#include <set>
 
 #define UNKNOWN_SRID   -2
 #define DEFAULT_SRID    0
 
 #define ENABLE_GPKG_OGR_CONTENTS
 
+#if defined(DEBUG) || defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION) || defined(ALLOW_FORMAT_DUMPS)
+// Enable accepting a SQL dump (starting with a "-- SQL GPKG" line) as a valid
+// file. This makes fuzzer life easier
+#define ENABLE_SQL_GPKG_FORMAT
+#endif
+
 typedef enum
 {
     GPKG_ATTRIBUTES,
@@ -58,13 +65,31 @@ static const GUInt32 GPKG_1_2_VERSION = 0x000027D8U; // 10200
 static const size_t knApplicationIdPos = 68;
 static const size_t knUserVersionPos = 60;
 
+typedef struct
+{
+    CPLString osExtensionName;
+    CPLString osDefinition;
+    CPLString osScope;
+} GPKGExtensionDesc;
+
+typedef struct
+{
+    CPLString osDataType;
+    CPLString osIdentifier;
+    CPLString osDescription;
+    CPLString osMinX;
+    CPLString osMinY;
+    CPLString osMaxX;
+    CPLString osMaxY;
+} GPKGContentsDesc;
+
 /************************************************************************/
 /*                          GDALGeoPackageDataset                       */
 /************************************************************************/
 
 class OGRGeoPackageTableLayer;
 
-class GDALGeoPackageDataset CPL_FINAL : public OGRSQLiteBaseDataSource, public GDALGPKGMBTilesLikePseudoDataset
+class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALGPKGMBTilesLikePseudoDataset
 {
     friend class GDALGeoPackageRasterBand;
     friend class OGRGeoPackageTableLayer;
@@ -73,7 +98,6 @@ class GDALGeoPackageDataset CPL_FINAL : public OGRSQLiteBaseDataSource, public G
     GUInt32             m_nUserVersion;
     OGRGeoPackageTableLayer** m_papoLayers;
     int                 m_nLayers;
-    bool                m_bUtf8;
     void                CheckUnknownExtensions(bool bCheckRasterTable = false);
 #ifdef ENABLE_GPKG_OGR_CONTENTS
     bool                m_bHasGPKGOGRContents;
@@ -85,6 +109,7 @@ class GDALGeoPackageDataset CPL_FINAL : public OGRSQLiteBaseDataSource, public G
     bool                m_bIdentifierAsCO;
     CPLString           m_osDescription;
     bool                m_bDescriptionAsCO;
+    bool                m_bGridCellEncodingAsCO = false;
     bool                m_bHasReadMetadataFromStorage;
     bool                m_bMetadataDirty;
     char              **m_papszSubDatasets;
@@ -171,11 +196,32 @@ class GDALGeoPackageDataset CPL_FINAL : public OGRSQLiteBaseDataSource, public G
         void                    CreateOGREmptyTableIfNeeded();
         void                    RemoveOGREmptyTable();
 
+        std::map<CPLString, CPLString> m_oMapNameToType;
+        const std::map<CPLString, CPLString>&
+                                        GetNameTypeMapFromSQliteMaster();
+        void RemoveTableFromSQLiteMasterCache(const char* pszTableName);
+
+        bool                    m_bMapTableToExtensionsBuilt;
+        std::map< CPLString, std::vector<GPKGExtensionDesc> > m_oMapTableToExtensions;
+        const std::map< CPLString, std::vector<GPKGExtensionDesc> > &
+                                        GetExtensions();
+
+        bool                    m_bMapTableToContentsBuilt;
+        std::map< CPLString, GPKGContentsDesc > m_oMapTableToContents;
+        const std::map< CPLString, GPKGContentsDesc > & GetContents();
+
+        std::map<int, OGRSpatialReference*> m_oMapSrsIdToSrs;
+
+        OGRErr              DeleteLayerCommon( const char* pszLayerName );
+        OGRErr              DeleteRasterLayer( const char* pszLayerName );
+        bool                DeleteVectorOrRasterLayer(
+                                                const char* pszLayerName );
+
     public:
                             GDALGeoPackageDataset();
                             virtual ~GDALGeoPackageDataset();
 
-        virtual char **     GetMetadata( const char *pszDomain = NULL ) override;
+        virtual char **     GetMetadata( const char *pszDomain = nullptr ) override;
         virtual const char *GetMetadataItem( const char * pszName,
                                              const char * pszDomain = "" ) override;
         virtual char **     GetMetadataDomainList() override;
@@ -226,7 +272,6 @@ class GDALGeoPackageDataset CPL_FINAL : public OGRSQLiteBaseDataSource, public G
         int                 GetSrsId( const OGRSpatialReference& oSRS );
         const char*         GetSrsName( const OGRSpatialReference& oSRS );
         OGRSpatialReference* GetSpatialRef( int iSrsId );
-        bool                GetUTF8() { return m_bUtf8; }
         OGRErr              CreateExtensionsTableIfNecessary();
         bool                HasExtensionsTable();
         OGRErr              CreateGDALAspatialExtension();
@@ -266,6 +311,7 @@ class GDALGeoPackageDataset CPL_FINAL : public OGRSQLiteBaseDataSource, public G
         OGRErr              SetApplicationAndUserVersionId();
         bool                ReOpenDB();
         bool                OpenOrCreateDB( int flags );
+        void                InstallSQLFunctions();
         bool                HasGDALAspatialExtension();
 };
 
@@ -273,7 +319,7 @@ class GDALGeoPackageDataset CPL_FINAL : public OGRSQLiteBaseDataSource, public G
 /*                        GDALGeoPackageRasterBand                      */
 /************************************************************************/
 
-class GDALGeoPackageRasterBand CPL_FINAL: public GDALGPKGMBTilesLikeRasterBand
+class GDALGeoPackageRasterBand final: public GDALGPKGMBTilesLikeRasterBand
 {
         bool                    m_bStatsComputed;
 
@@ -342,10 +388,13 @@ class OGRGeoPackageLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
 /*                        OGRGeoPackageTableLayer                       */
 /************************************************************************/
 
-class OGRGeoPackageTableLayer CPL_FINAL : public OGRGeoPackageLayer
+class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
 {
     char*                       m_pszTableName;
     bool                        m_bIsTable;
+    bool                        m_bIsSpatial;
+    bool                        m_bIsInGpkgContents;
+    bool                        m_bFeatureDefnCompleted;
     int                         m_iSrs;
     OGREnvelope*                m_poExtent;
 #ifdef ENABLE_GPKG_OGR_CONTENTS
@@ -379,6 +428,7 @@ class OGRGeoPackageTableLayer CPL_FINAL : public OGRGeoPackageLayer
     bool                        m_bHasReadMetadataFromStorage;
     bool                        m_bHasTriedDetectingFID64;
     GPKGASpatialVariant         m_eASPatialVariant;
+    std::set<OGRwkbGeometryType> m_eSetBadGeomTypeWarned;
 
     virtual OGRErr      ResetStatement() override;
 
@@ -390,18 +440,28 @@ class OGRGeoPackageTableLayer CPL_FINAL : public OGRGeoPackageLayer
     OGRErr              RecreateTable(const CPLString& osColumnsForCreate,
                                       const CPLString& osFieldListForSelect);
 #ifdef ENABLE_GPKG_OGR_CONTENTS
-    void                CreateTriggers(const char* pszTableName = NULL);
+    void                CreateTriggers(const char* pszTableName = nullptr);
     void                DisableTriggers(bool bNullifyFeatureCount = true);
 #endif
 
+    void                CheckGeometryType( OGRFeature *poFeature );
+
+    OGRErr              ReadTableDefinition();
+    void                InitView();
+
     public:
                         OGRGeoPackageTableLayer( GDALGeoPackageDataset *poDS,
-                                            const char * pszTableName );
+                                                 const char * pszTableName );
                         virtual ~OGRGeoPackageTableLayer();
 
     /************************************************************************/
     /* OGR API methods */
 
+    const char*         GetName() override { return GetDescription(); }
+    const char*         GetFIDColumn() override;
+    OGRwkbGeometryType  GetGeomType() override;
+    const char*         GetGeometryColumn() override;
+    OGRFeatureDefn*     GetLayerDefn() override;
     int                 TestCapability( const char * ) override;
     OGRErr              CreateField( OGRFieldDefn *poField, int bApproxOK = TRUE ) override;
     OGRErr              CreateGeomField( OGRGeomFieldDefn *poGeomFieldIn,
@@ -429,10 +489,14 @@ class OGRGeoPackageTableLayer CPL_FINAL : public OGRGeoPackageLayer
     virtual OGRErr      GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce) override
                 { return OGRGeoPackageLayer::GetExtent(iGeomField, psExtent, bForce); }
 
-    void                PostInit();
     void                RecomputeExtent();
 
-    OGRErr              ReadTableDefinition(bool bIsSpatial, bool bIsGpkgTable);
+    void                SetOpeningParameters(bool bIsInGpkgContents,
+                                             bool bIsSpatial,
+                                             const char* pszGeomColName,
+                                             const char* pszGeomType,
+                                             bool bHasZ,
+                                             bool bHasM);
     void                SetCreationParameters( OGRwkbGeometryType eGType,
                                                const char* pszGeomColumnName,
                                                int bGeomNullable,
@@ -446,10 +510,10 @@ class OGRGeoPackageTableLayer CPL_FINAL : public OGRGeoPackageLayer
                                 { m_eASPatialVariant = eASPatialVariant; }
 
     void                CreateSpatialIndexIfNecessary();
-    bool                CreateSpatialIndex(const char* pszTableName = NULL);
+    bool                CreateSpatialIndex(const char* pszTableName = nullptr);
     bool                DropSpatialIndex(bool bCalledFromSQLFunction = false);
 
-    virtual char **     GetMetadata( const char *pszDomain = NULL ) override;
+    virtual char **     GetMetadata( const char *pszDomain = nullptr ) override;
     virtual const char *GetMetadataItem( const char * pszName,
                                              const char * pszDomain = "" ) override;
     virtual char **     GetMetadataDomainList() override;
@@ -483,7 +547,7 @@ class OGRGeoPackageTableLayer CPL_FINAL : public OGRGeoPackageLayer
     void                SetOGRFeatureCountTriggersEnabled(bool b)
                                     { m_bOGRFeatureCountTriggersEnabled = b; }
 
-    void                DisableFeatureCount( bool bInMemoryOnly = false );
+    void                DisableFeatureCount();
 #endif
 
     /************************************************************************/
@@ -511,7 +575,7 @@ class OGRGeoPackageTableLayer CPL_FINAL : public OGRGeoPackageLayer
 /*                         OGRGeoPackageSelectLayer                     */
 /************************************************************************/
 
-class OGRGeoPackageSelectLayer CPL_FINAL : public OGRGeoPackageLayer, public IOGRSQLiteSelectLayer
+class OGRGeoPackageSelectLayer final : public OGRGeoPackageLayer, public IOGRSQLiteSelectLayer
 {
     OGRSQLiteSelectLayerCommonBehaviour* poBehaviour;
 
