@@ -31,15 +31,15 @@
 #include "idrisi.h"
 #include "ogr_idrisi.h"
 
-CPL_CVSID("$Id: ogridrisidatasource.cpp 34996 2016-08-09 02:25:28Z goatbar $");
+CPL_CVSID("$Id: ogridrisidatasource.cpp 7e07230bbff24eb333608de4dbd460b7312839d0 2017-12-11 19:08:47Z Even Rouault $")
 
 /************************************************************************/
 /*                        OGRIdrisiDataSource()                         */
 /************************************************************************/
 
 OGRIdrisiDataSource::OGRIdrisiDataSource() :
-    pszName(NULL),
-    papoLayers(NULL),
+    pszName(nullptr),
+    papoLayers(nullptr),
     nLayers(0)
 {}
 
@@ -73,7 +73,7 @@ OGRLayer *OGRIdrisiDataSource::GetLayer( int iLayer )
 
 {
     if( iLayer < 0 || iLayer >= nLayers )
-        return NULL;
+        return nullptr;
 
     return papoLayers[iLayer];
 }
@@ -88,43 +88,43 @@ int OGRIdrisiDataSource::Open( const char * pszFilename )
     pszName = CPLStrdup( pszFilename );
 
     VSILFILE* fpVCT = VSIFOpenL(pszFilename, "rb");
-    if (fpVCT == NULL)
+    if (fpVCT == nullptr)
         return FALSE;
 
-    char* pszWTKString = NULL;
+    char* pszWTKString = nullptr;
 
 // --------------------------------------------------------------------
 //      Look for .vdc file
 // --------------------------------------------------------------------
     const char* pszVDCFilename = CPLResetExtension(pszFilename, "vdc");
     VSILFILE* fpVDC = VSIFOpenL(pszVDCFilename, "rb");
-    if (fpVDC == NULL)
+    if (fpVDC == nullptr)
     {
         pszVDCFilename = CPLResetExtension(pszFilename, "VDC");
         fpVDC = VSIFOpenL(pszVDCFilename, "rb");
     }
 
-    char** papszVDC = NULL;
-    if (fpVDC != NULL)
+    char** papszVDC = nullptr;
+    if (fpVDC != nullptr)
     {
         VSIFCloseL(fpVDC);
-        fpVDC = NULL;
+        fpVDC = nullptr;
 
         CPLPushErrorHandler(CPLQuietErrorHandler);
-        papszVDC = CSLLoad2(pszVDCFilename, 1024, 256, NULL);
+        papszVDC = CSLLoad2(pszVDCFilename, 1024, 256, nullptr);
         CPLPopErrorHandler();
         CPLErrorReset();
     }
 
     OGRwkbGeometryType eType = wkbUnknown;
 
-    if (papszVDC != NULL)
+    if (papszVDC != nullptr)
     {
         CSLSetNameValueSeparator( papszVDC, ":" );
 
-        const char *pszVersion = CSLFetchNameValue( papszVDC, "file format " );
+        const char *pszVersion = CSLFetchNameValue( papszVDC, "file format" );
 
-        if( pszVersion == NULL || !EQUAL( pszVersion, "IDRISI Vector A.1" ) )
+        if( pszVersion == nullptr || !EQUAL( pszVersion, "IDRISI Vector A.1" ) )
         {
             CSLDestroy( papszVDC );
             VSIFCloseL(fpVCT);
@@ -132,10 +132,10 @@ int OGRIdrisiDataSource::Open( const char * pszFilename )
         }
 
         const char *pszRefSystem
-            = CSLFetchNameValue( papszVDC, "ref. system " );
-        const char *pszRefUnits = CSLFetchNameValue( papszVDC, "ref. units  " );
+            = CSLFetchNameValue( papszVDC, "ref. system" );
+        const char *pszRefUnits = CSLFetchNameValue( papszVDC, "ref. units" );
 
-        if (pszRefSystem != NULL && pszRefUnits != NULL)
+        if (pszRefSystem != nullptr && pszRefUnits != nullptr)
             IdrisiGeoReference2Wkt( pszFilename, pszRefSystem, pszRefUnits,
                                     &pszWTKString);
     }
@@ -163,10 +163,10 @@ int OGRIdrisiDataSource::Open( const char * pszFilename )
         return FALSE;
     }
 
-    const char *pszMinX = CSLFetchNameValue( papszVDC, "min. X      " );
-    const char *pszMaxX = CSLFetchNameValue( papszVDC, "max. X      " );
-    const char *pszMinY = CSLFetchNameValue( papszVDC, "min. Y      " );
-    const char *pszMaxY = CSLFetchNameValue( papszVDC, "max. Y      " );
+    const char *pszMinX = CSLFetchNameValue( papszVDC, "min. X" );
+    const char *pszMaxX = CSLFetchNameValue( papszVDC, "max. X" );
+    const char *pszMinY = CSLFetchNameValue( papszVDC, "min. Y" );
+    const char *pszMaxY = CSLFetchNameValue( papszVDC, "max. Y" );
 
     OGRIdrisiLayer* poLayer = new OGRIdrisiLayer(pszFilename,
                                                  CPLGetBasename(pszFilename),
@@ -174,8 +174,8 @@ int OGRIdrisiDataSource::Open( const char * pszFilename )
     papoLayers = static_cast<OGRLayer**>( CPLMalloc(sizeof(OGRLayer*)) );
     papoLayers[nLayers ++] = poLayer;
 
-    if( pszMinX != NULL && pszMaxX != NULL && pszMinY != NULL &&
-        pszMaxY != NULL)
+    if( pszMinX != nullptr && pszMaxX != nullptr && pszMinY != nullptr &&
+        pszMaxY != nullptr)
     {
         poLayer->SetExtent(
             CPLAtof(pszMinX), CPLAtof(pszMinY), CPLAtof(pszMaxX),
