@@ -29,32 +29,36 @@
 #include "kmlnode.h"
 #include "kml.h"
 
-#include "cpl_conv.h"
-#include "cpl_error.h"
-
-#include <cerrno>
+#include <cstring>
 #include <cstdio>
+#include <exception>
 #include <iostream>
 #include <string>
 
-CPL_CVSID("$Id: kml.cpp 35775 2016-10-17 00:43:44Z goatbar $");
+#include "cpl_conv.h"
+#include "cpl_error.h"
+#ifdef HAVE_EXPAT
+#  include "expat.h"
+#endif
+
+CPL_CVSID("$Id: kml.cpp d8a77a7ae5788883e9231e02e52c0ae06c649a4b 2018-02-23 02:34:03Z Kurt Schwehr $")
 
 KML::KML() :
-    poTrunk_(NULL),
+    poTrunk_(nullptr),
     nNumLayers_(-1),
-    papoLayers_(NULL),
+    papoLayers_(nullptr),
     nDepth_(0),
     validity(KML_VALIDITY_UNKNOWN),
-    pKMLFile_(NULL),
-    poCurrent_(NULL),
-    oCurrentParser(NULL),
+    pKMLFile_(nullptr),
+    poCurrent_(nullptr),
+    oCurrentParser(nullptr),
     nDataHandlerCounter(0),
     nWithoutEventCounter(0)
 {}
 
 KML::~KML()
 {
-    if( NULL != pKMLFile_ )
+    if( nullptr != pKMLFile_ )
         VSIFCloseL(pKMLFile_);
     CPLFree(papoLayers_);
 
@@ -63,30 +67,30 @@ KML::~KML()
 
 bool KML::open(const char * pszFilename)
 {
-    if( NULL != pKMLFile_ )
+    if( nullptr != pKMLFile_ )
         VSIFCloseL( pKMLFile_ );
 
     pKMLFile_ = VSIFOpenL( pszFilename, "r" );
-    return pKMLFile_ != NULL;
+    return pKMLFile_ != nullptr;
 }
 
-void KML::parse()
+bool KML::parse()
 {
-    if( NULL == pKMLFile_ )
+    if( nullptr == pKMLFile_ )
     {
         sError_ = "No file given";
-        return;
+        return false;
     }
 
-    if(poTrunk_ != NULL) {
+    if(poTrunk_ != nullptr) {
         delete poTrunk_;
-        poTrunk_ = NULL;
+        poTrunk_ = nullptr;
     }
 
-    if(poCurrent_ != NULL)
+    if(poCurrent_ != nullptr)
     {
         delete poCurrent_;
-        poCurrent_ = NULL;
+        poCurrent_ = nullptr;
     }
 
     XML_Parser oParser = OGRCreateExpatXMLParser();
@@ -115,37 +119,67 @@ void KML::parse()
                       static_cast<int>(XML_GetCurrentColumnNumber(oParser)));
             XML_ParserFree(oParser);
             VSIRewindL(pKMLFile_);
-            return;
+
+            if( poCurrent_ != nullptr )
+            {
+                while( poCurrent_ )
+                {
+                    KMLNode* poTemp = poCurrent_->getParent();
+                    delete poCurrent_;
+                    poCurrent_ = poTemp;
+                }
+                // No need to destroy poTrunk_ : it has been destroyed in
+                // the last iteration
+            }
+            else
+            {
+                // Case of invalid content after closing element matching
+                // first <kml> element
+                delete poTrunk_;
+            }
+            poTrunk_ = nullptr;
+
+            return false;
         }
         nWithoutEventCounter ++;
     } while (!nDone && nLen > 0 && nWithoutEventCounter < 10);
 
     XML_ParserFree(oParser);
     VSIRewindL(pKMLFile_);
-    poCurrent_ = NULL;
 
     if (nWithoutEventCounter == 10)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "Too much data inside one element. File probably corrupted");
+        while( poCurrent_ )
+        {
+            KMLNode* poTemp = poCurrent_->getParent();
+            delete poCurrent_;
+            poCurrent_ = poTemp;
+        }
+        poTrunk_ = nullptr;
+        return false;
     }
+
+    poCurrent_ = nullptr;
+    return true;
 }
 
 void KML::checkValidity()
 {
-    if(poTrunk_ != NULL)
+    if(poTrunk_ != nullptr)
     {
         delete poTrunk_;
-        poTrunk_ = NULL;
+        poTrunk_ = nullptr;
     }
 
-    if(poCurrent_ != NULL)
+    if(poCurrent_ != nullptr)
     {
         delete poCurrent_;
-        poCurrent_ = NULL;
+        poCurrent_ = nullptr;
     }
 
-    if(pKMLFile_ == NULL)
+    if(pKMLFile_ == nullptr)
     {
         sError_ = "No file given";
         return;
@@ -153,7 +187,7 @@ void KML::checkValidity()
 
     XML_Parser oParser = OGRCreateExpatXMLParser();
     XML_SetUserData(oParser, this);
-    XML_SetElementHandler(oParser, startElementValidate, NULL);
+    XML_SetElementHandler(oParser, startElementValidate, nullptr);
     XML_SetCharacterDataHandler(oParser, dataHandlerValidate);
     int nCount = 0;
 
@@ -200,20 +234,25 @@ void KML::checkValidity()
 
     XML_ParserFree(oParser);
     VSIRewindL(pKMLFile_);
-    poCurrent_ = NULL;
+    poCurrent_ = nullptr;
 }
 
 void XMLCALL KML::startElement( void* pUserData, const char* pszName,
                                 const char** ppszAttr )
 {
-    KMLNode* poMynew = NULL;
+    KMLNode* poMynew = nullptr;
 
     KML* poKML = static_cast<KML*>(pUserData);
 
     poKML->nWithoutEventCounter = 0;
 
-    if(poKML->poTrunk_ == NULL
-    || (poKML->poCurrent_->getName()).compare("description") != 0)
+    const char* pszColumn = strchr(pszName, ':');
+    if( pszColumn)
+        pszName = pszColumn + 1;
+
+    if(poKML->poTrunk_ == nullptr
+    || (poKML->poCurrent_ != nullptr &&
+        poKML->poCurrent_->getName().compare("description") != 0))
     {
         if (poKML->nDepth_ == 1024)
         {
@@ -236,15 +275,15 @@ void XMLCALL KML::startElement( void* pUserData, const char* pszName,
             poMynew->addAttribute(poAtt);
         }
 
-        if(poKML->poTrunk_ == NULL)
+        if(poKML->poTrunk_ == nullptr)
             poKML->poTrunk_ = poMynew;
-        if(poKML->poCurrent_ != NULL)
+        if(poKML->poCurrent_ != nullptr)
             poMynew->setParent(poKML->poCurrent_);
         poKML->poCurrent_ = poMynew;
 
         poKML->nDepth_++;
     }
-    else
+    else if( poKML->poCurrent_ != nullptr )
     {
         std::string sNewContent = "<";
         sNewContent += pszName;
@@ -274,6 +313,10 @@ void XMLCALL KML::startElementValidate( void* pUserData, const char* pszName,
 
     poKML->validity = KML_VALIDITY_INVALID;
 
+    const char* pszColumn = strchr(pszName, ':');
+    if( pszColumn)
+        pszName = pszColumn + 1;
+
     if(strcmp(pszName, "kml") == 0 || strcmp(pszName, "Document") == 0)
     {
         // Check all Attributes
@@ -342,7 +385,11 @@ void XMLCALL KML::endElement(void* pUserData, const char* pszName)
 
     poKML->nWithoutEventCounter = 0;
 
-    if(poKML->poCurrent_ != NULL &&
+    const char* pszColumn = strchr(pszName, ':');
+    if( pszColumn)
+        pszName = pszColumn + 1;
+
+    if(poKML->poCurrent_ != nullptr &&
        poKML->poCurrent_->getName().compare(pszName) == 0)
     {
         poKML->nDepth_--;
@@ -439,23 +486,27 @@ void XMLCALL KML::endElement(void* pUserData, const char* pszName)
             }
         }
 
-        if(poKML->poCurrent_->getParent() != NULL)
+        if(poKML->poCurrent_->getParent() != nullptr)
             poKML->poCurrent_ = poKML->poCurrent_->getParent();
         else
-            poKML->poCurrent_ = NULL;
+            poKML->poCurrent_ = nullptr;
 
         if(!poKML->isHandled(pszName))
         {
             CPLDebug("KML", "Not handled: %s", pszName);
             delete poTmp;
+            if( poKML->poCurrent_ == poTmp )
+                poKML->poCurrent_ = nullptr;
+            if( poKML->poTrunk_ == poTmp )
+                poKML->poTrunk_ = nullptr;
         }
         else
         {
-            if(poKML->poCurrent_ != NULL)
+            if(poKML->poCurrent_ != nullptr)
                 poKML->poCurrent_->addChildren(poTmp);
         }
     }
-    else if(poKML->poCurrent_ != NULL)
+    else if(poKML->poCurrent_ != nullptr)
     {
         std::string sNewContent = "</";
         sNewContent += pszName;
@@ -473,7 +524,7 @@ void XMLCALL KML::dataHandler(void* pUserData, const char* pszData, int nLen)
 
     poKML->nWithoutEventCounter = 0;
 
-    if(nLen < 1 || poKML->poCurrent_ == NULL)
+    if(nLen < 1 || poKML->poCurrent_ == nullptr)
         return;
 
     poKML->nDataHandlerCounter ++;
@@ -518,17 +569,20 @@ std::string KML::getError() const
 
 int KML::classifyNodes()
 {
+    if( poTrunk_ == nullptr )
+        return false;
     return poTrunk_->classify(this);
 }
 
 void KML::eliminateEmpty()
 {
-    poTrunk_->eliminateEmpty(this);
+    if( poTrunk_ != nullptr )
+        poTrunk_->eliminateEmpty(this);
 }
 
 void KML::print(unsigned short nNum)
 {
-    if( poTrunk_ != NULL )
+    if( poTrunk_ != nullptr )
         poTrunk_->print(nNum);
 }
 
@@ -541,32 +595,32 @@ bool KML::isHandled(std::string const& elem) const
 bool KML::isLeaf( std::string const& /* elem */ ) const
 {
     return false;
-};
+}
 
 bool KML::isFeature( std::string const& /* elem */ ) const
 {
     return false;
-};
+}
 
 bool KML::isFeatureContainer( std::string const& /* elem */ ) const
 {
     return false;
-};
+}
 
 bool KML::isContainer( std::string const& /* elem */ ) const
 {
     return false;
-};
+}
 
 bool KML::isRest( std::string const& /* elem */ ) const
 {
     return false;
-};
+}
 
 void KML::findLayers( KMLNode* /* poNode */, int /* bKeepEmptyContainers */ )
 {
     // idle
-};
+}
 
 bool KML::hasOnlyEmpty() const
 {
@@ -588,7 +642,7 @@ bool KML::selectLayer(int nNum) {
 std::string KML::getCurrentName() const
 {
     std::string tmp;
-    if( poCurrent_ != NULL )
+    if( poCurrent_ != nullptr )
     {
         tmp = poCurrent_->getNameElement();
     }
@@ -597,7 +651,7 @@ std::string KML::getCurrentName() const
 
 Nodetype KML::getCurrentType() const
 {
-    if(poCurrent_ != NULL)
+    if(poCurrent_ != nullptr)
         return poCurrent_->getType();
 
     return Unknown;
@@ -605,7 +659,7 @@ Nodetype KML::getCurrentType() const
 
 int KML::is25D() const
 {
-    if(poCurrent_ != NULL)
+    if(poCurrent_ != nullptr)
         return poCurrent_->is25D();
 
     return Unknown;
@@ -613,7 +667,7 @@ int KML::is25D() const
 
 int KML::getNumFeatures()
 {
-    if(poCurrent_ == NULL)
+    if(poCurrent_ == nullptr)
         return -1;
 
     return static_cast<int>(poCurrent_->getNumFeatures());
@@ -621,8 +675,8 @@ int KML::getNumFeatures()
 
 Feature* KML::getFeature(std::size_t nNum, int& nLastAsked, int &nLastCount)
 {
-    if(poCurrent_ == NULL)
-        return NULL;
+    if(poCurrent_ == nullptr)
+        return nullptr;
 
     return poCurrent_->getFeature(nNum, nLastAsked, nLastCount);
 }
@@ -635,11 +689,11 @@ void KML::unregisterLayerIfMatchingThisNode(KMLNode* poNode)
         {
             if( i < nNumLayers_ - 1 )
             {
-                memcpy( papoLayers_ + i, papoLayers_ + i + 1,
+                memmove( papoLayers_ + i, papoLayers_ + i + 1,
                         (nNumLayers_ - 1 - i) * sizeof(KMLNode*) );
             }
             nNumLayers_ --;
-            continue;
+            break;
         }
         i++;
     }
