@@ -52,7 +52,7 @@
 #  include <wctype.h>      /* iswspace() */
 #endif
 
-CPL_CVSID("$Id: mitab_imapinfofile.cpp 37347 2017-02-12 03:10:54Z goatbar $");
+CPL_CVSID("$Id: mitab_imapinfofile.cpp 98dfb4b4012c5ae4621e246e8eb393b3c05a3f48 2018-04-02 22:09:55 +0200 Even Rouault $")
 
 /**********************************************************************
  *                   IMapInfoFile::IMapInfoFile()
@@ -61,9 +61,9 @@ CPL_CVSID("$Id: mitab_imapinfofile.cpp 37347 2017-02-12 03:10:54Z goatbar $");
  **********************************************************************/
 IMapInfoFile::IMapInfoFile() :
     m_nCurFeatureId(0),
-    m_poCurFeature(NULL),
+    m_poCurFeature(nullptr),
     m_bBoundsSet(FALSE),
-    m_pszCharset(NULL)
+    m_pszCharset(nullptr)
 {}
 
 /**********************************************************************
@@ -76,11 +76,11 @@ IMapInfoFile::~IMapInfoFile()
     if( m_poCurFeature )
     {
         delete m_poCurFeature;
-        m_poCurFeature = NULL;
+        m_poCurFeature = nullptr;
     }
 
     CPLFree(m_pszCharset);
-    m_pszCharset = NULL;
+    m_pszCharset = nullptr;
 }
 
 /**********************************************************************
@@ -91,13 +91,13 @@ IMapInfoFile::~IMapInfoFile()
  **********************************************************************/
 
 int IMapInfoFile::Open(const char *pszFname, const char* pszAccess,
-                       GBool bTestOpenNoError)
+                       GBool bTestOpenNoError, const char* pszCharset )
 {
     // cppcheck-suppress nullPointer
     if( STARTS_WITH_CI(pszAccess, "r") )
-        return Open(pszFname, TABRead, bTestOpenNoError);
+        return Open(pszFname, TABRead, bTestOpenNoError, pszCharset);
     else if( STARTS_WITH_CI(pszAccess, "w") )
-        return Open(pszFname, TABWrite, bTestOpenNoError);
+        return Open(pszFname, TABWrite, bTestOpenNoError, pszCharset);
     else
     {
         CPLError(CE_Failure, CPLE_FileIO,
@@ -122,7 +122,7 @@ IMapInfoFile *IMapInfoFile::SmartOpen(const char *pszFname,
                                       GBool bUpdate,
                                       GBool bTestOpenNoError /*=FALSE*/)
 {
-    IMapInfoFile *poFile = NULL;
+    IMapInfoFile *poFile = nullptr;
     int nLen = 0;
 
     if (pszFname)
@@ -149,8 +149,8 @@ IMapInfoFile *IMapInfoFile::SmartOpen(const char *pszFname,
 
         TABAdjustFilenameExtension(pszAdjFname);
         VSILFILE *fp = VSIFOpenL(pszAdjFname, "r");
-        const char *pszLine = NULL;
-        while(fp && (pszLine = CPLReadLineL(fp)) != NULL)
+        const char *pszLine = nullptr;
+        while(fp && (pszLine = CPLReadLineL(fp)) != nullptr)
         {
             while (isspace((unsigned char)*pszLine))  pszLine++;
             if (STARTS_WITH_CI(pszLine, "Fields"))
@@ -180,10 +180,10 @@ IMapInfoFile *IMapInfoFile::SmartOpen(const char *pszFname,
     if (poFile && poFile->Open(pszFname, bUpdate ? TABReadWrite : TABRead, bTestOpenNoError) != 0)
     {
         delete poFile;
-        poFile = NULL;
+        poFile = nullptr;
     }
 
-    if (!bTestOpenNoError && poFile == NULL)
+    if (!bTestOpenNoError && poFile == nullptr)
     {
         CPLError(CE_Failure, CPLE_FileIO,
                  "%s could not be opened as a MapInfo dataset.", pszFname);
@@ -204,25 +204,25 @@ OGRFeature *IMapInfoFile::GetNextFeature()
 
     while( (nFeatureId = GetNextFeatureId(m_nCurFeatureId)) != -1 )
     {
-        OGRGeometry *poGeom = NULL;
+        OGRGeometry *poGeom = nullptr;
         OGRFeature *poFeatureRef = GetFeatureRef(nFeatureId);
-        if (poFeatureRef == NULL)
-            return NULL;
-        else if( (m_poFilterGeom == NULL ||
-                  ((poGeom = poFeatureRef->GetGeometryRef()) != NULL &&
+        if (poFeatureRef == nullptr)
+            return nullptr;
+        else if( (m_poFilterGeom == nullptr ||
+                  ((poGeom = poFeatureRef->GetGeometryRef()) != nullptr &&
                    FilterGeometry( poGeom )))
-                 && (m_poAttrQuery == NULL
+                 && (m_poAttrQuery == nullptr
                      || m_poAttrQuery->Evaluate( poFeatureRef )) )
         {
             // Avoid cloning feature... return the copy owned by the class
             CPLAssert(poFeatureRef == m_poCurFeature);
-            m_poCurFeature = NULL;
-            if( poFeatureRef->GetGeometryRef() != NULL )
+            m_poCurFeature = nullptr;
+            if( poFeatureRef->GetGeometryRef() != nullptr )
                 poFeatureRef->GetGeometryRef()->assignSpatialReference(GetSpatialRef());
             return poFeatureRef;
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 /**********************************************************************
@@ -233,20 +233,19 @@ OGRFeature *IMapInfoFile::GetNextFeature()
 
 TABFeature* IMapInfoFile::CreateTABFeature(OGRFeature *poFeature)
 {
-    TABFeature *poTABFeature = NULL;
-    OGRGeometry   *poGeom = NULL;
+    TABFeature *poTABFeature = nullptr;
     OGRwkbGeometryType eGType;
-    TABPoint *poTABPointFeature = NULL;
-    TABRegion *poTABRegionFeature = NULL;
-    TABPolyline *poTABPolylineFeature = NULL;
+    TABPoint *poTABPointFeature = nullptr;
+    TABRegion *poTABRegionFeature = nullptr;
+    TABPolyline *poTABPolylineFeature = nullptr;
 
     /*-----------------------------------------------------------------
      * MITAB won't accept new features unless they are in a type derived
      * from TABFeature... so we have to do our best to map to the right
      * feature type based on the geometry type.
      *----------------------------------------------------------------*/
-    poGeom = poFeature->GetGeometryRef();
-    if( poGeom != NULL )
+    OGRGeometry* poGeom = poFeature->GetGeometryRef();
+    if( poGeom != nullptr )
         eGType = poGeom->getGeometryType();
     else
         eGType = wkbNone;
@@ -302,11 +301,12 @@ TABFeature* IMapInfoFile::CreateTABFeature(OGRFeature *poFeature)
       case wkbMultiPoint:
       {
           OGRErr eStatus = OGRERR_NONE;
-          OGRGeometryCollection *poColl = (OGRGeometryCollection*)poGeom;
+          OGRGeometryCollection *poColl = poGeom->toGeometryCollection();
           OGRFeature *poTmpFeature = poFeature->Clone();
 
           for( int i = 0;
-               eStatus==OGRERR_NONE && i<poColl->getNumGeometries();
+               eStatus==OGRERR_NONE && poColl != nullptr &&
+               i<poColl->getNumGeometries();
                i++)
           {
               poTmpFeature->SetFID(OGRNullFID);
@@ -314,7 +314,7 @@ TABFeature* IMapInfoFile::CreateTABFeature(OGRFeature *poFeature)
               eStatus = ICreateFeature(poTmpFeature);
           }
           delete poTmpFeature;
-          return NULL;
+          return nullptr;
       }
         break;
       /*-------------------------------------------------------------
@@ -326,7 +326,7 @@ TABFeature* IMapInfoFile::CreateTABFeature(OGRFeature *poFeature)
         break;
     }
 
-    if( poGeom != NULL )
+    if( poGeom != nullptr )
         poTABFeature->SetGeometryDirectly(poGeom->clone());
 
     for (int i=0; i< poFeature->GetDefnRef()->GetFieldCount();i++)
@@ -348,7 +348,7 @@ TABFeature* IMapInfoFile::CreateTABFeature(OGRFeature *poFeature)
 OGRErr     IMapInfoFile::ICreateFeature(OGRFeature *poFeature)
 {
     TABFeature *poTABFeature = CreateTABFeature(poFeature);
-    if( poTABFeature == NULL ) /* MultiGeometry */
+    if( poTABFeature == nullptr ) /* MultiGeometry */
         return OGRERR_NONE;
 
     OGRErr eErr = CreateFeature(poTABFeature);
@@ -376,12 +376,12 @@ OGRFeature *IMapInfoFile::GetFeature(GIntBig nFeatureId)
     {
         // Avoid cloning feature... return the copy owned by the class
         CPLAssert(poFeatureRef == m_poCurFeature);
-        m_poCurFeature = NULL;
+        m_poCurFeature = nullptr;
 
         return poFeatureRef;
     }
     else
-      return NULL;
+      return nullptr;
 }
 
 /************************************************************************/
@@ -511,9 +511,141 @@ int IMapInfoFile::SetCharset(const char* pszCharset)
 {
     if(pszCharset && strlen(pszCharset) > 0)
     {
+        if(pszCharset == m_pszCharset)
+        {
+            return 0;
+        }
         CPLFree(m_pszCharset);
         m_pszCharset = CPLStrdup(pszCharset);
         return 0;
     }
     return -1;
 }
+
+const char* IMapInfoFile::GetCharset() const
+{
+    return m_pszCharset;
+}
+
+// Table is adopted from
+// http://www.i-signum.com/Formation/download/MB_ReferenceGuide.pdf pp. 127-128
+static const char* const apszCharsets[][2] = {
+    { "Neutral", "" }, //No character conversions performed.
+    { "ISO8859_1", "ISO-8859-1" }, //ISO 8859-1 (UNIX)
+    { "ISO8859_2", "ISO-8859-2" }, //ISO 8859-2 (UNIX)
+    { "ISO8859_3", "ISO-8859-3" }, //ISO 8859-3 (UNIX)
+    { "ISO8859_4", "ISO-8859-4" }, //ISO 8859-4 (UNIX)
+    { "ISO8859_5", "ISO-8859-5" }, //ISO 8859-5 (UNIX)
+    { "ISO8859_6", "ISO-8859-6" }, //ISO 8859-6 (UNIX)
+    { "ISO8859_7", "ISO-8859-7" }, //ISO 8859-7 (UNIX)
+    { "ISO8859_8", "ISO-8859-8" }, //ISO 8859-8 (UNIX)
+    { "ISO8859_9", "ISO-8859-9" }, //ISO 8859-9 (UNIX)
+    { "PackedEUCJapaese", "EUC-JP" }, //UNIX, standard Japanese implementation.
+    { "WindowsLatin1", "CP1252" },
+    { "WindowsLatin2", "" },
+    { "WindowsArabic", "CP1256" },
+    { "WindowsCyrillic", "CP1251" },
+    { "WindowsGreek", "CP1253" },
+    { "WindowsHebrew", "CP1255" },
+    { "WindowsTurkish", "CP1254" }, //Windows Eastern Europe
+    { "WindowsTradChinese", "CP950" },//Windows Traditional Chinese
+    { "WindowsSimpChinese", "CP936" },//Windows Simplified Chinese
+    { "WindowsJapanese", "CP932" },
+    { "WindowsKorean", "CP949" },
+    { "CodePage437", "CP437" }, //DOS Code Page 437 = IBM Extended ASCII
+    { "CodePage850", "CP850" }, //DOS Code Page 850 = Multilingual
+    { "CodePage852", "CP852" }, //DOS Code Page 852 = Eastern Europe
+    { "CodePage855", "CP855" }, //DOS Code Page 855 = Cyrillic
+    { "CodePage857", "CP857" },
+    { "CodePage860", "CP860" }, //DOS Code Page 860 = Portuguese
+    { "CodePage861", "CP861" }, //DOS Code Page 861 = Icelandic
+    { "CodePage863", "CP863" }, //DOS Code Page 863 = French Canadian
+    { "CodePage864", "CP864" }, //DOS Code Page 864 = Arabic
+    { "CodePage865", "CP865" }, //DOS Code Page 865 = Nordic
+    { "CodePage869", "CP869" }, //DOS Code Page 869 = Modern Greek
+    { "LICS", "" }, //Lotus worksheet release 1,2 character set
+    { "LMBCS", "" },//Lotus worksheet release 3,4 character set
+    { nullptr, nullptr }
+};
+
+const char* IMapInfoFile::CharsetToEncoding( const char* pszCharset )
+{
+    if( pszCharset == nullptr )
+    {
+        return apszCharsets[0][1];
+    }
+
+    for( size_t i = 0; apszCharsets[i][0] != nullptr; ++i)
+    {
+        if( EQUAL( pszCharset, apszCharsets[i][0] ) )
+        {
+            return apszCharsets[i][1];
+        }
+    }
+
+    CPLError(CE_Warning, CPLE_NotSupported,
+             "Cannot find iconv encoding corresponding to MapInfo %s charset",
+             pszCharset);
+    return apszCharsets[0][1];
+}
+
+const char* IMapInfoFile::EncodingToCharset( const char* pszEncoding )
+{
+    if( pszEncoding == nullptr )
+    {
+        return apszCharsets[0][0];
+    }
+
+    for( size_t i = 0; apszCharsets[i][1] != nullptr; ++i)
+    {
+        if( EQUAL( pszEncoding, apszCharsets[i][1] ) )
+        {
+            return apszCharsets[i][0];
+        }
+    }
+
+    CPLError(CE_Warning, CPLE_NotSupported,
+             "Cannot find MapInfo charset corresponding to iconv %s encoding",
+             pszEncoding);
+    return apszCharsets[0][0];
+}
+
+const char* IMapInfoFile::GetEncoding() const
+{
+    return CharsetToEncoding( GetCharset() );
+}
+
+void IMapInfoFile::SetEncoding( const char* pszEncoding )
+{
+    SetCharset( EncodingToCharset( pszEncoding ) );
+}
+
+int IMapInfoFile::TestUtf8Capability() const
+{
+    const char* pszEncoding( GetEncoding() );
+    if( strlen( pszEncoding ) == 0 )
+    {
+        return FALSE;
+    }
+
+    CPLClearRecodeWarningFlags();
+    CPLErrorReset();
+
+    CPLPushErrorHandler(CPLQuietErrorHandler);
+    char* pszTest( CPLRecode( "test", GetEncoding(), CPL_ENC_UTF8 ) );
+    CPLPopErrorHandler();
+
+    if( pszTest == nullptr )
+    {
+        return FALSE;
+    }
+
+    CPLFree( pszTest );
+
+    if( CPLGetLastErrorType() != 0 )
+    {
+        return FALSE;
+    }
+
+    return TRUE;
+}
