@@ -47,7 +47,7 @@
 #include "cpl_vsi.h"
 #include "mitab_priv.h"
 
-CPL_CVSID("$Id: mitab_rawbinblock.cpp 37351 2017-02-12 05:22:20Z goatbar $");
+CPL_CVSID("$Id: mitab_rawbinblock.cpp f61bae9ed8735d8b4765f350130fb06ac3769324 2018-09-17 14:19:33 +0200 Even Rouault $")
 
 /*=====================================================================
  *                      class TABRawBinBlock
@@ -60,10 +60,10 @@ CPL_CVSID("$Id: mitab_rawbinblock.cpp 37351 2017-02-12 05:22:20Z goatbar $");
  **********************************************************************/
 TABRawBinBlock::TABRawBinBlock( TABAccess eAccessMode /*= TABRead*/,
                                 GBool bHardBlockSize /*= TRUE*/ ) :
-    m_fp(NULL),
+    m_fp(nullptr),
     m_eAccess(eAccessMode),
     m_nBlockType(0),
-    m_pabyBuf(NULL),
+    m_pabyBuf(nullptr),
     m_nBlockSize(0),
     m_nSizeUsed(0),
     m_bHardBlockSize(bHardBlockSize),
@@ -96,7 +96,7 @@ TABRawBinBlock::~TABRawBinBlock()
 int     TABRawBinBlock::ReadFromFile(VSILFILE *fpSrc, int nOffset,
                                      int nSize)
 {
-    if (fpSrc == NULL || nSize == 0)
+    if (fpSrc == nullptr || nSize == 0)
     {
         CPLError(CE_Failure, CPLE_AssertionFailed,
                  "TABRawBinBlock::ReadFromFile(): Assertion Failed!");
@@ -158,7 +158,7 @@ int     TABRawBinBlock::CommitToFile()
 {
     int nStatus = 0;
 
-    if (m_fp == NULL || m_nBlockSize <= 0 || m_pabyBuf == NULL ||
+    if (m_fp == nullptr || m_nBlockSize <= 0 || m_pabyBuf == nullptr ||
         m_nFileOffset < 0)
     {
         CPLError(CE_Failure, CPLE_AssertionFailed,
@@ -247,11 +247,9 @@ int     TABRawBinBlock::CommitToFile()
  **********************************************************************/
 int     TABRawBinBlock::CommitAsDeleted(GInt32 nNextBlockPtr)
 {
-    int nStatus = 0;
-
     CPLErrorReset();
 
-    if ( m_pabyBuf == NULL )
+    if ( m_pabyBuf == nullptr )
     {
         CPLError(CE_Failure, CPLE_AssertionFailed,
                  "CommitAsDeleted(): Block has not been initialized yet!");
@@ -265,8 +263,7 @@ int     TABRawBinBlock::CommitAsDeleted(GInt32 nNextBlockPtr)
     WriteInt16(TABMAP_GARB_BLOCK);    // Block type code
     WriteInt32(nNextBlockPtr);
 
-    if( CPLGetLastErrorType() == CE_Failure )
-        nStatus = CPLGetLastErrorNo();
+    int nStatus = CPLGetLastErrorType() == CE_Failure ? -1 : 0;
 
     /*-----------------------------------------------------------------
      * OK, call the base class to write the block to disk.
@@ -318,13 +315,13 @@ int     TABRawBinBlock::InitBlockFromData(GByte *pabyBuf,
      *---------------------------------------------------------------*/
     if (!bMakeCopy)
     {
-        if (m_pabyBuf != NULL)
+        if (m_pabyBuf != nullptr)
             CPLFree(m_pabyBuf);
         m_pabyBuf = pabyBuf;
         m_nBlockSize = nBlockSize;
         m_nSizeUsed = nSizeUsed;
     }
-    else if (m_pabyBuf == NULL || nBlockSize != m_nBlockSize)
+    else if (m_pabyBuf == nullptr || nBlockSize != m_nBlockSize)
     {
         m_pabyBuf = (GByte*)CPLRealloc(m_pabyBuf, nBlockSize*sizeof(GByte));
         m_nBlockSize = nBlockSize;
@@ -374,7 +371,7 @@ int     TABRawBinBlock::InitNewBlock(VSILFILE *fpSrc, int nBlockSize,
     else
         m_nFileOffset = 0;
 
-    if( m_fp != NULL && m_nFileSize < 0 && m_eAccess == TABReadWrite )
+    if( m_fp != nullptr && m_nFileSize < 0 && m_eAccess == TABReadWrite )
     {
         int nCurPos = (int)VSIFTellL(m_fp);
         VSIFSeekL(fpSrc, 0, SEEK_END);
@@ -401,7 +398,7 @@ int     TABRawBinBlock::InitNewBlock(VSILFILE *fpSrc, int nBlockSize,
  **********************************************************************/
 int     TABRawBinBlock::GetBlockType()
 {
-    if (m_pabyBuf == NULL)
+    if (m_pabyBuf == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "GetBlockType(): Block has not been initialized.");
@@ -530,7 +527,7 @@ int     TABRawBinBlock::GotoByteInFile(int nOffset,
     }
     else if (m_eAccess == TABReadWrite)
     {
-        // TODO: THIS IS NOT REAL read/write access (it's more extended write)
+        // TODO: THIS IS NOT REAL read/write access (it is more extended write)
         // Currently we try to read from file only if explicitly requested.
         // If we ever want true read/write mode we should implement
         // more smarts to detect whether the caller wants an existing block to
@@ -540,7 +537,7 @@ int     TABRawBinBlock::GotoByteInFile(int nOffset,
         if (bOffsetIsEndOfData &&  nOffset%m_nBlockSize == 0)
         {
             /* We're trying to go byte m_nBlockSize of a block that's full of data.
-             * In this case it's okay to place the m_nCurPos at byte m_nBlockSize
+             * In this case it is okay to place the m_nCurPos at byte m_nBlockSize
              * which is past the end of the block.
              */
 
@@ -676,7 +673,7 @@ int     TABRawBinBlock::ReadBytes(int numBytes, GByte *pabyDstBuf)
      * Make sure block is initialized with Read access and that the
      * operation won't go beyond the buffer's size.
      *---------------------------------------------------------------*/
-    if (m_pabyBuf == NULL)
+    if (m_pabyBuf == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "ReadBytes(): Block has not been initialized.");
@@ -789,7 +786,7 @@ int  TABRawBinBlock::WriteBytes(int nBytesToWrite, const GByte *pabySrcBuf)
      * Make sure block is initialized with Write access and that the
      * operation won't go beyond the buffer's size.
      *---------------------------------------------------------------*/
-    if (m_pabyBuf == NULL)
+    if (m_pabyBuf == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "WriteBytes(): Block has not been initialized.");
@@ -945,11 +942,11 @@ int  TABRawBinBlock::WritePaddedString(int nFieldSize, const char *pszString)
 
 void TABRawBinBlock::Dump(FILE *fpOut /*=NULL*/)
 {
-    if (fpOut == NULL)
+    if (fpOut == nullptr)
         fpOut = stdout;
 
     fprintf(fpOut, "----- TABRawBinBlock::Dump() -----\n");
-    if (m_pabyBuf == NULL)
+    if (m_pabyBuf == nullptr)
     {
         fprintf(fpOut, "Block has not been initialized yet.");
     }
@@ -1008,7 +1005,7 @@ void TABRawBinBlock::DumpBytes(GInt32 nValue, int nOffset /*=0*/,
     double dValue = 0.0;
     memcpy(&dValue, anVal, 8);
 
-    if (fpOut == NULL)
+    if (fpOut == nullptr)
         fpOut = stdout;
 
     fprintf(fpOut, "%d\t0x%8.8x  %-5d\t%-6d %-6d %5.3e  d=%5.3e",
@@ -1035,11 +1032,11 @@ TABRawBinBlock *TABCreateMAPBlockFromFile(VSILFILE *fpSrc, int nOffset,
                                           GBool bHardBlockSize /*= TRUE */,
                                           TABAccess eAccessMode /*= TABRead*/)
 {
-    if (fpSrc == NULL || nSize == 0)
+    if (fpSrc == nullptr || nSize == 0)
     {
         CPLError(CE_Failure, CPLE_AssertionFailed,
                  "TABCreateMAPBlockFromFile(): Assertion Failed!");
-        return NULL;
+        return nullptr;
     }
 
     /*----------------------------------------------------------------
@@ -1057,7 +1054,7 @@ TABRawBinBlock *TABCreateMAPBlockFromFile(VSILFILE *fpSrc, int nOffset,
          "TABCreateMAPBlockFromFile() failed reading %d bytes at offset %d.",
                  nSize, nOffset);
         CPLFree(pabyBuf);
-        return NULL;
+        return nullptr;
     }
 
     /*----------------------------------------------------------------
@@ -1065,7 +1062,7 @@ TABRawBinBlock *TABCreateMAPBlockFromFile(VSILFILE *fpSrc, int nOffset,
      * Header block is different: it does not start with the object
      * type byte but it is always the first block in a file
      *---------------------------------------------------------------*/
-    TABRawBinBlock *poBlock = NULL;
+    TABRawBinBlock *poBlock = nullptr;
 
     if (nOffset == 0)
     {
@@ -1102,7 +1099,7 @@ TABRawBinBlock *TABCreateMAPBlockFromFile(VSILFILE *fpSrc, int nOffset,
     {
         // Some error happened... and CPLError() has been called
         delete poBlock;
-        poBlock = NULL;
+        poBlock = nullptr;
     }
 
     return poBlock;
@@ -1120,8 +1117,8 @@ TABRawBinBlock *TABCreateMAPBlockFromFile(VSILFILE *fpSrc, int nOffset,
 TABBinBlockManager::TABBinBlockManager() :
     m_nBlockSize(0),
     m_nLastAllocatedBlock(-1),
-    m_psGarbageBlocksFirst(NULL),
-    m_psGarbageBlocksLast(NULL)
+    m_psGarbageBlocksFirst(nullptr),
+    m_psGarbageBlocksLast(nullptr)
 {
     m_szName[0] = '\0';
 }
@@ -1196,13 +1193,13 @@ void TABBinBlockManager::Reset()
     m_nLastAllocatedBlock = -1;
 
     // Flush list of garbage blocks
-    while (m_psGarbageBlocksFirst != NULL)
+    while (m_psGarbageBlocksFirst != nullptr)
     {
         TABBlockRef *psNext = m_psGarbageBlocksFirst->psNext;
         CPLFree(m_psGarbageBlocksFirst);
         m_psGarbageBlocksFirst = psNext;
     }
-    m_psGarbageBlocksLast = NULL;
+    m_psGarbageBlocksLast = nullptr;
 }
 
 /**********************************************************************
@@ -1215,13 +1212,13 @@ void TABBinBlockManager::PushGarbageBlockAsFirst(GInt32 nBlockPtr)
     TABBlockRef *psNewBlockRef = (TABBlockRef *)CPLMalloc(sizeof(TABBlockRef));
 
     psNewBlockRef->nBlockPtr = nBlockPtr;
-    psNewBlockRef->psPrev = NULL;
+    psNewBlockRef->psPrev = nullptr;
     psNewBlockRef->psNext = m_psGarbageBlocksFirst;
 
-    if( m_psGarbageBlocksFirst != NULL )
+    if( m_psGarbageBlocksFirst != nullptr )
         m_psGarbageBlocksFirst->psPrev = psNewBlockRef;
     m_psGarbageBlocksFirst = psNewBlockRef;
-    if( m_psGarbageBlocksLast == NULL )
+    if( m_psGarbageBlocksLast == nullptr )
         m_psGarbageBlocksLast = m_psGarbageBlocksFirst;
 }
 
@@ -1236,12 +1233,12 @@ void TABBinBlockManager::PushGarbageBlockAsLast(GInt32 nBlockPtr)
 
     psNewBlockRef->nBlockPtr = nBlockPtr;
     psNewBlockRef->psPrev = m_psGarbageBlocksLast;
-    psNewBlockRef->psNext = NULL;
+    psNewBlockRef->psNext = nullptr;
 
-    if( m_psGarbageBlocksLast != NULL )
+    if( m_psGarbageBlocksLast != nullptr )
         m_psGarbageBlocksLast->psNext = psNewBlockRef;
     m_psGarbageBlocksLast = psNewBlockRef;
-    if( m_psGarbageBlocksFirst == NULL )
+    if( m_psGarbageBlocksFirst == nullptr )
         m_psGarbageBlocksFirst = m_psGarbageBlocksLast;
 }
 
@@ -1275,10 +1272,10 @@ GInt32 TABBinBlockManager::PopGarbageBlock()
         nBlockPtr = m_psGarbageBlocksFirst->nBlockPtr;
         TABBlockRef *psNext = m_psGarbageBlocksFirst->psNext;
         CPLFree(m_psGarbageBlocksFirst);
-        if( psNext != NULL )
-            psNext->psPrev = NULL;
+        if( psNext != nullptr )
+            psNext->psPrev = nullptr;
         else
-            m_psGarbageBlocksLast = NULL;
+            m_psGarbageBlocksLast = nullptr;
         m_psGarbageBlocksFirst = psNext;
     }
 
