@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_sqlite.h 37810 2017-03-20 16:23:30Z rouault $
+ * $Id: ogr_sqlite.h 3bc98ac2e895141cb38924f83d1182b59d711148 2018-06-14 16:22:26 +0200 Even Rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for OGR/SQLite driver.
@@ -31,13 +31,19 @@
 #ifndef OGR_SQLITE_H_INCLUDED
 #define OGR_SQLITE_H_INCLUDED
 
-#include "ogrsf_frmts.h"
-#include "gdal_pam.h"
-#include "cpl_error.h"
+#include <cstddef>
 #include <map>
 #include <set>
+#include <utility>
 #include <vector>
 
+#include "cpl_error.h"
+#include "gdal_pam.h"
+#include "ogrsf_frmts.h"
+#include "ogrsf_frmts.h"
+#include "rasterlite2_header.h"
+#include "sqlite3.h"
+
 #ifdef HAVE_SPATIALITE
   #ifdef SPATIALITE_AMALGAMATION
     /*
@@ -61,12 +67,17 @@
 #include "sqlite3.h"
 #endif
 
-#include "rasterlite2_header.h"
-
 #ifndef DO_NOT_INCLUDE_SQLITE_CLASSES
 
 #define UNINITIALIZED_SRID  -2
 
+#if defined(DEBUG) || defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION) || defined(ALLOW_FORMAT_DUMPS)
+// Enable accepting a SQL dump (starting with a "-- SQL SQLITE" or
+// "-- SQL RASTERLITE" or "--SQL MBTILES" line) as a valid
+// file. This makes fuzzer life easier
+#define ENABLE_SQL_SQLITE_FORMAT
+#endif
+
 /************************************************************************/
 /*      Format used to store geometry data in the database.             */
 /************************************************************************/
@@ -152,7 +163,7 @@ enum OGRSpatialiteGeomType
 /*                        OGRSQLiteGeomFieldDefn                        */
 /************************************************************************/
 
-class OGRSQLiteGeomFieldDefn CPL_FINAL : public OGRGeomFieldDefn
+class OGRSQLiteGeomFieldDefn final : public OGRGeomFieldDefn
 {
     public:
         OGRSQLiteGeomFieldDefn( const char* pszNameIn, int iGeomColIn ) :
@@ -178,10 +189,10 @@ class OGRSQLiteGeomFieldDefn CPL_FINAL : public OGRGeomFieldDefn
 /*                        OGRSQLiteFeatureDefn                          */
 /************************************************************************/
 
-class OGRSQLiteFeatureDefn CPL_FINAL : public OGRFeatureDefn
+class OGRSQLiteFeatureDefn final : public OGRFeatureDefn
 {
     public:
-        explicit OGRSQLiteFeatureDefn( const char * pszName = NULL ) :
+        explicit OGRSQLiteFeatureDefn( const char * pszName = nullptr ) :
             OGRFeatureDefn(pszName)
         {
             SetGeomType(wkbNone);
@@ -283,7 +294,7 @@ class OGRSQLiteLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
                         OGRSQLiteLayer();
     virtual             ~OGRSQLiteLayer();
 
-    virtual void        Finalize();
+    void                Finalize();
 
     virtual void        ResetReading() override;
     virtual OGRFeature *GetNextRawFeature();
@@ -654,7 +665,7 @@ class OGRSQLiteSelectLayer : public OGRSQLiteLayer, public IOGRSQLiteSelectLayer
 /*                   OGRSQLiteSingleFeatureLayer                        */
 /************************************************************************/
 
-class OGRSQLiteSingleFeatureLayer CPL_FINAL : public OGRLayer
+class OGRSQLiteSingleFeatureLayer final : public OGRLayer
 {
   private:
     int                 nVal;
@@ -684,6 +695,7 @@ class OGRSQLiteBaseDataSource : public GDALPamDataset
 {
   protected:
     char               *m_pszFilename;
+    bool                m_bCallUndeclareFileNotToOpen = false;
 
     sqlite3             *hDB;
     int                 bUpdate;
@@ -749,7 +761,7 @@ class OGRSQLiteBaseDataSource : public GDALPamDataset
 /*                         OGRSQLiteDataSource                          */
 /************************************************************************/
 
-class OGRSQLiteDataSource CPL_FINAL : public OGRSQLiteBaseDataSource
+class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
 {
     OGRSQLiteLayer    **papoLayers;
     int                 nLayers;
@@ -813,8 +825,7 @@ class OGRSQLiteDataSource CPL_FINAL : public OGRSQLiteBaseDataSource
                         OGRSQLiteDataSource();
                         virtual ~OGRSQLiteDataSource();
 
-    int                 Open( const char *, int bUpdateIn,
-                              char** papszOpenOptions, int nOpenFlags );
+    int                 Open( GDALOpenInfo* poOpenInfo );
     int                 Create( const char *, char **papszOptions );
 
     int                 OpenTable( const char *pszTableName,
@@ -898,7 +909,7 @@ class OGRSQLiteDataSource CPL_FINAL : public OGRSQLiteBaseDataSource
 /*                           RL2RasterBand                              */
 /************************************************************************/
 
-class RL2RasterBand CPL_FINAL: public GDALPamRasterBand
+class RL2RasterBand final: public GDALPamRasterBand
 {
     bool            m_bHasNoData;
     double          m_dfNoDataValue;
@@ -916,7 +927,7 @@ class RL2RasterBand CPL_FINAL: public GDALPamRasterBand
                                            int nBlockYSizeIn,
                                            bool bHasNoDataIn,
                                            double dfNoDataValueIn );
-                            RL2RasterBand( const RL2RasterBand* poOther );
+        explicit            RL2RasterBand( const RL2RasterBand* poOther );
 
         virtual            ~RL2RasterBand();
 
@@ -925,7 +936,7 @@ class RL2RasterBand CPL_FINAL: public GDALPamRasterBand
         virtual CPLErr      IReadBlock( int, int, void* ) override;
         virtual GDALColorInterp GetColorInterpretation() override
                                                     { return m_eColorInterp; }
-        virtual double      GetNoDataValue( int* pbSuccess = NULL ) override;
+        virtual double      GetNoDataValue( int* pbSuccess = nullptr ) override;
         virtual GDALColorTable* GetColorTable() override;
         virtual int         GetOverviewCount() override;
         virtual GDALRasterBand* GetOverview(int) override;
