@@ -42,7 +42,7 @@
 #include "cpl_multiproc.h"
 #include "gdal_csv.h"
 
-CPL_CVSID("$Id: cpl_csv.cpp 37621 2017-03-06 11:38:25Z rouault $");
+CPL_CVSID("$Id: cpl_csv.cpp 090ecb96506b67d811929bf888e6192d4a02f1b1 2018-02-11 20:01:07Z Even Rouault $")
 
 /* ==================================================================== */
 /*      The CSVTable is a persistent set of info about an open CSV      */
@@ -55,7 +55,9 @@ typedef struct ctb {
     struct ctb *psNext;
     char       *pszFilename;
     char      **papszFieldNames;
+    int        *panFieldNamesLength;
     char      **papszRecFields;
+    int         nFields;
     int         iLastLine;
     bool        bNonUniqueKey;
 
@@ -74,7 +76,7 @@ static void CSVDeaccessInternal( CSVTable **ppsCSVTableList, bool bCanUseTLS,
 /************************************************************************/
 static void CSVFreeTLS( void* pData )
 {
-    CSVDeaccessInternal( static_cast<CSVTable **>( pData ), false, NULL );
+    CSVDeaccessInternal( static_cast<CSVTable **>( pData ), false, nullptr );
     CPLFree(pData);
 }
 
@@ -103,13 +105,13 @@ static CSVTable *CSVAccess( const char * pszFilename )
     CSVTable **ppsCSVTableList = static_cast<CSVTable **>(
         CPLGetTLSEx( CTLS_CSVTABLEPTR, &bMemoryError ) );
     if( bMemoryError )
-        return NULL;
-    if( ppsCSVTableList == NULL )
+        return nullptr;
+    if( ppsCSVTableList == nullptr )
     {
         ppsCSVTableList = static_cast<CSVTable **>(
             VSI_CALLOC_VERBOSE( 1, sizeof(CSVTable*) ) );
-        if( ppsCSVTableList == NULL )
-            return NULL;
+        if( ppsCSVTableList == nullptr )
+            return nullptr;
         CPLSetTLSWithFreeFunc( CTLS_CSVTABLEPTR, ppsCSVTableList, CSVFreeTLS );
     }
 
@@ -117,7 +119,7 @@ static CSVTable *CSVAccess( const char * pszFilename )
 /*      Is the table already in the list.                               */
 /* -------------------------------------------------------------------- */
     for( CSVTable *psTable = *ppsCSVTableList;
-         psTable != NULL;
+         psTable != nullptr;
          psTable = psTable->psNext )
     {
         if( EQUAL(psTable->pszFilename, pszFilename) )
@@ -134,8 +136,8 @@ static CSVTable *CSVAccess( const char * pszFilename )
 /*      If not, try to open it.                                         */
 /* -------------------------------------------------------------------- */
     VSILFILE *fp = VSIFOpenL( pszFilename, "rb" );
-    if( fp == NULL )
-        return NULL;
+    if( fp == nullptr )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Create an information structure about this table, and add to    */
@@ -143,19 +145,19 @@ static CSVTable *CSVAccess( const char * pszFilename )
 /* -------------------------------------------------------------------- */
     CSVTable * const psTable = static_cast<CSVTable *>(
         VSI_CALLOC_VERBOSE( sizeof(CSVTable), 1 ) );
-    if( psTable == NULL )
+    if( psTable == nullptr )
     {
         VSIFCloseL(fp);
-        return NULL;
+        return nullptr;
     }
 
     psTable->fp = fp;
     psTable->pszFilename = VSI_STRDUP_VERBOSE( pszFilename );
-    if( psTable->pszFilename == NULL )
+    if( psTable->pszFilename == nullptr )
     {
         VSIFree(psTable);
         VSIFCloseL(fp);
-        return NULL;
+        return nullptr;
     }
     psTable->bNonUniqueKey = false;  // As far as we know now.
     psTable->psNext = *ppsCSVTableList;
@@ -166,6 +168,16 @@ static CSVTable *CSVAccess( const char * pszFilename )
 /*      Read the table header record containing the field names.        */
 /* -------------------------------------------------------------------- */
     psTable->papszFieldNames = CSVReadParseLineL( fp );
+    psTable->nFields = CSLCount(psTable->papszFieldNames);
+    psTable->panFieldNamesLength = static_cast<int*>(
+        CPLMalloc(sizeof(int) * psTable->nFields));
+    for(int i = 0; i < psTable->nFields &&
+        /* null-pointer check to avoid a false positive from CLang S.A. */
+                   psTable->papszFieldNames != nullptr; i++ )
+    {
+        psTable->panFieldNamesLength[i] = static_cast<int>(
+            strlen(psTable->papszFieldNames[i]));
+    }
 
     return psTable;
 }
@@ -178,15 +190,15 @@ static void CSVDeaccessInternal( CSVTable **ppsCSVTableList, bool bCanUseTLS,
                                  const char * pszFilename )
 
 {
-    if( ppsCSVTableList == NULL )
+    if( ppsCSVTableList == nullptr )
         return;
 
 /* -------------------------------------------------------------------- */
 /*      A NULL means deaccess all tables.                               */
 /* -------------------------------------------------------------------- */
-    if( pszFilename == NULL )
+    if( pszFilename == nullptr )
     {
-        while( *ppsCSVTableList != NULL )
+        while( *ppsCSVTableList != nullptr )
             CSVDeaccessInternal( ppsCSVTableList, bCanUseTLS,
                                  (*ppsCSVTableList)->pszFilename );
 
@@ -196,16 +208,16 @@ static void CSVDeaccessInternal( CSVTable **ppsCSVTableList, bool bCanUseTLS,
 /* -------------------------------------------------------------------- */
 /*      Find this table.                                                */
 /* -------------------------------------------------------------------- */
-    CSVTable *psLast = NULL;
+    CSVTable *psLast = nullptr;
     CSVTable *psTable = *ppsCSVTableList;
     for( ;
-         psTable != NULL && !EQUAL(psTable->pszFilename, pszFilename);
+         psTable != nullptr && !EQUAL(psTable->pszFilename, pszFilename);
          psTable = psTable->psNext )
     {
         psLast = psTable;
     }
 
-    if( psTable == NULL )
+    if( psTable == nullptr )
     {
         if( bCanUseTLS )
             CPLDebug( "CPL_CSV", "CPLDeaccess( %s ) - no match.", pszFilename );
@@ -215,7 +227,7 @@ static void CSVDeaccessInternal( CSVTable **ppsCSVTableList, bool bCanUseTLS,
 /* -------------------------------------------------------------------- */
 /*      Remove the link from the list.                                  */
 /* -------------------------------------------------------------------- */
-    if( psLast != NULL )
+    if( psLast != nullptr )
         psLast->psNext = psTable->psNext;
     else
         *ppsCSVTableList = psTable->psNext;
@@ -223,10 +235,11 @@ static void CSVDeaccessInternal( CSVTable **ppsCSVTableList, bool bCanUseTLS,
 /* -------------------------------------------------------------------- */
 /*      Free the table.                                                 */
 /* -------------------------------------------------------------------- */
-    if( psTable->fp != NULL )
+    if( psTable->fp != nullptr )
         VSIFCloseL( psTable->fp );
 
     CSLDestroy( psTable->papszFieldNames );
+    CPLFree( psTable->panFieldNamesLength );
     CSLDestroy( psTable->papszRecFields );
     CPLFree( psTable->pszFilename );
     CPLFree( psTable->panLineIndex );
@@ -236,7 +249,7 @@ static void CSVDeaccessInternal( CSVTable **ppsCSVTableList, bool bCanUseTLS,
     CPLFree( psTable );
 
     if( bCanUseTLS )
-        CPLReadLine( NULL );
+        CPLReadLine( nullptr );
 }
 
 void CSVDeaccess( const char * pszFilename )
@@ -261,18 +274,32 @@ void CSVDeaccess( const char * pszFilename )
 /*      semantics.                                                      */
 /************************************************************************/
 
-static char **CSVSplitLine( const char *pszString, char chDelimiter )
+static char **CSVSplitLine( CSVTable* psTable,
+                            const char *pszString, char chDelimiter )
 
 {
 
     char *pszToken = static_cast<char *>( VSI_CALLOC_VERBOSE( 10, 1 ) );
-    if( pszToken == NULL )
-        return NULL;
+    if( pszToken == nullptr )
+        return nullptr;
 
     int nTokenMax = 10;
-    char **papszRetList = NULL;
+    char **papszRetList = nullptr;
+    int nListSize = 0;
+    int nListAlloc = 0;
+    if( psTable )
+    {
+        papszRetList = static_cast<char**>(
+            VSI_CALLOC_VERBOSE( psTable->nFields + 1, sizeof(char*) ));
+        if( papszRetList == nullptr )
+        {
+            VSIFree(pszToken);
+            return nullptr;
+        }
+        nListAlloc = psTable->nFields;
+    }
 
-    while( pszString != NULL && *pszString != '\0' )
+    while( pszString != nullptr && *pszString != '\0' )
     {
         bool bInString = false;
         int nTokenLen = 0;
@@ -306,11 +333,11 @@ static char **CSVSplitLine( const char *pszString, char chDelimiter )
                 nTokenMax = nTokenMax * 2 + 10;
                 char* pszTokenNew = static_cast<char *>(
                     VSI_REALLOC_VERBOSE( pszToken, nTokenMax ) );
-                if( pszTokenNew == NULL )
+                if( pszTokenNew == nullptr )
                 {
                     VSIFree(pszToken);
                     CSLDestroy(papszRetList);
-                    return NULL;
+                    return nullptr;
                 }
                 pszToken = pszTokenNew;
             }
@@ -320,28 +347,47 @@ static char **CSVSplitLine( const char *pszString, char chDelimiter )
         }
 
         pszToken[nTokenLen] = '\0';
-        char** papszRetListNew = CSLAddStringMayFail( papszRetList, pszToken );
-        if( papszRetListNew == NULL )
+        if( nListSize + 1 >= nListAlloc )
         {
-            VSIFree(pszToken);
-            CSLDestroy(papszRetList);
-            return NULL;
+            nListAlloc = 10 + 2 * nListAlloc;
+            char** papszRetListNew = static_cast<char**>(
+                VSI_REALLOC_VERBOSE(papszRetList,
+                                    (nListAlloc + 1) * sizeof(char*)));
+            if( papszRetListNew == nullptr )
+            {
+                CSLDestroy(papszRetList);
+                VSIFree(pszToken);
+                return nullptr;
+            }
+            papszRetList = papszRetListNew;
         }
-        papszRetList = papszRetListNew;
+
+        papszRetList[nListSize] = VSI_STRDUP_VERBOSE(pszToken);
+        if( papszRetList[nListSize] == nullptr )
+        {
+            CSLDestroy(papszRetList);
+            VSIFree(pszToken);
+            return nullptr;
+        }
+        nListSize ++;
+        papszRetList[nListSize] = nullptr;
+
 
         /* If the last token is an empty token, then we have to catch
          * it now, otherwise we won't reenter the loop and it will be lost.
          */
         if( *pszString == '\0' && *(pszString-1) == chDelimiter )
         {
-            papszRetListNew = CSLAddStringMayFail( papszRetList, "" );
-            if( papszRetListNew == NULL )
+            papszRetList[nListSize] = VSI_STRDUP_VERBOSE("");
+            if( papszRetList[nListSize] == nullptr )
             {
-                VSIFree(pszToken);
                 CSLDestroy(papszRetList);
-                return NULL;
+                VSIFree(pszToken);
+                return nullptr;
             }
-            papszRetList = papszRetListNew;
+            nListSize ++;
+            papszRetList[nListSize] = nullptr;
+            break;
         }
     }
 
@@ -379,7 +425,7 @@ static char *CSVFindNextLine( char *pszThisLine )
         pszThisLine[i++] = '\0';
 
     if( pszThisLine[i] == '\0' )
-        return NULL;
+        return nullptr;
 
     return pszThisLine + i;
 }
@@ -391,18 +437,10 @@ static char *CSVFindNextLine( char *pszThisLine )
 /************************************************************************/
 
 // TODO(schwehr): Clean up all the casting in CSVIngest.
-static void CSVIngest( const char *pszFilename )
+static void CSVIngest( CSVTable *psTable )
 
 {
-    CSVTable *psTable = CSVAccess( pszFilename );
-    if( psTable == NULL )
-    {
-        CPLError( CE_Failure, CPLE_FileIO, "Failed to open file: %s",
-                  pszFilename );
-        return;
-    }
-
-    if( psTable->pszRawData != NULL )
+    if( psTable->pszRawData != nullptr )
         return;
 
 /* -------------------------------------------------------------------- */
@@ -412,7 +450,7 @@ static void CSVIngest( const char *pszFilename )
     {
         CPLError( CE_Failure, CPLE_FileIO,
                   "Failed using seek end and tell to get file length: %s",
-                  pszFilename );
+                  psTable->pszFilename );
         return;
     }
     const vsi_l_offset nFileLen = VSIFTellL( psTable->fp );
@@ -420,21 +458,21 @@ static void CSVIngest( const char *pszFilename )
     {
         CPLError( CE_Failure, CPLE_FileIO,
                   "Failed using seek end and tell to get file length: %s",
-                  pszFilename );
+                  psTable->pszFilename );
         return;
     }
     VSIRewindL( psTable->fp );
 
     psTable->pszRawData = static_cast<char *>(
         VSI_MALLOC_VERBOSE( static_cast<size_t>(nFileLen) + 1) );
-    if( psTable->pszRawData == NULL )
+    if( psTable->pszRawData == nullptr )
         return;
     if( VSIFReadL( psTable->pszRawData, 1,
                    static_cast<size_t>(nFileLen), psTable->fp )
         != static_cast<size_t>(nFileLen) )
     {
         CPLFree( psTable->pszRawData );
-        psTable->pszRawData = NULL;
+        psTable->pszRawData = nullptr;
 
         CPLError( CE_Failure, CPLE_FileIO, "Read of file %s failed.",
                   psTable->pszFilename );
@@ -455,7 +493,7 @@ static void CSVIngest( const char *pszFilename )
 
     psTable->papszLines = static_cast<char **>(
         VSI_CALLOC_VERBOSE( sizeof(char*), nMaxLineCount ) );
-    if( psTable->papszLines == NULL )
+    if( psTable->papszLines == nullptr )
         return;
 
 /* -------------------------------------------------------------------- */
@@ -467,7 +505,7 @@ static void CSVIngest( const char *pszFilename )
     char *pszThisLine = CSVFindNextLine( psTable->pszRawData );
 
     int iLine = 0;
-    while( pszThisLine != NULL && iLine < nMaxLineCount )
+    while( pszThisLine != nullptr && iLine < nMaxLineCount )
     {
         if( pszThisLine[0] != '#' )
             psTable->papszLines[iLine++] = pszThisLine;
@@ -483,7 +521,7 @@ static void CSVIngest( const char *pszFilename )
 /* -------------------------------------------------------------------- */
     psTable->panLineIndex = static_cast<int *>(
         VSI_MALLOC_VERBOSE( sizeof(int) * psTable->nLineCount ) );
-    if( psTable->panLineIndex == NULL )
+    if( psTable->panLineIndex == nullptr )
         return;
 
     for( int i = 0; i < psTable->nLineCount; i++ )
@@ -493,7 +531,7 @@ static void CSVIngest( const char *pszFilename )
         if( i > 0 && psTable->panLineIndex[i] < psTable->panLineIndex[i-1] )
         {
             CPLFree( psTable->panLineIndex );
-            psTable->panLineIndex = NULL;
+            psTable->panLineIndex = nullptr;
             break;
         }
     }
@@ -504,7 +542,20 @@ static void CSVIngest( const char *pszFilename )
 /*      We should never need the file handle against, so close it.      */
 /* -------------------------------------------------------------------- */
     VSIFCloseL( psTable->fp );
-    psTable->fp = NULL;
+    psTable->fp = nullptr;
+}
+
+static void CSVIngest( const char *pszFilename )
+
+{
+    CSVTable *psTable = CSVAccess( pszFilename );
+    if( psTable == nullptr )
+    {
+        CPLError( CE_Failure, CPLE_FileIO, "Failed to open file: %s",
+                  pszFilename );
+        return;
+    }
+    CSVIngest(psTable);
 }
 
 /************************************************************************/
@@ -589,20 +640,20 @@ char **CSVReadParseLine( FILE * fp )
 char **CSVReadParseLine2( FILE * fp, char chDelimiter )
 
 {
-    CPLAssert( fp != NULL );
-    if( fp == NULL )
-        return NULL;
+    CPLAssert( fp != nullptr );
+    if( fp == nullptr )
+        return nullptr;
 
     const char *pszLine = CPLReadLine( fp );
-    if( pszLine == NULL )
-        return NULL;
+    if( pszLine == nullptr )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      If there are no quotes, then this is the simple case.           */
 /*      Parse, and return tokens.                                       */
 /* -------------------------------------------------------------------- */
-    if( strchr(pszLine, '\"') == NULL )
-        return CSVSplitLine( pszLine, chDelimiter );
+    if( strchr(pszLine, '\"') == nullptr )
+        return CSVSplitLine( nullptr, pszLine, chDelimiter );
 
 /* -------------------------------------------------------------------- */
 /*      We must now count the quotes in our working string, and as      */
@@ -627,7 +678,7 @@ char **CSVReadParseLine2( FILE * fp, char chDelimiter )
             break;
 
         pszLine = CPLReadLine( fp );
-        if( pszLine == NULL )
+        if( pszLine == nullptr )
             break;
 
         const size_t nLineLen = strlen(pszLine);
@@ -635,7 +686,7 @@ char **CSVReadParseLine2( FILE * fp, char chDelimiter )
         char* pszWorkLineTmp = static_cast<char *>(
             VSIRealloc(pszWorkLine,
                        nWorkLineLength + nLineLen + 2) );
-        if( pszWorkLineTmp == NULL )
+        if( pszWorkLineTmp == nullptr )
             break;
         pszWorkLine = pszWorkLineTmp;
         // The newline gets lost in CPLReadLine().
@@ -645,7 +696,7 @@ char **CSVReadParseLine2( FILE * fp, char chDelimiter )
         nWorkLineLength += nLineLen + 1;
     }
 
-    char **papszReturn = CSVSplitLine( pszWorkLine, chDelimiter );
+    char **papszReturn = CSVSplitLine( nullptr, pszWorkLine, chDelimiter );
 
     CPLFree( pszWorkLine );
 
@@ -670,20 +721,20 @@ char **CSVReadParseLineL( VSILFILE * fp )
 char **CSVReadParseLine2L( VSILFILE * fp, char chDelimiter )
 
 {
-    CPLAssert( fp != NULL );
-    if( fp == NULL )
-        return NULL;
+    CPLAssert( fp != nullptr );
+    if( fp == nullptr )
+        return nullptr;
 
     const char *pszLine = CPLReadLineL( fp );
-    if( pszLine == NULL )
-        return NULL;
+    if( pszLine == nullptr )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      If there are no quotes, then this is the simple case.           */
 /*      Parse, and return tokens.                                       */
 /* -------------------------------------------------------------------- */
-    if( strchr(pszLine, '\"') == NULL )
-        return CSVSplitLine( pszLine, chDelimiter );
+    if( strchr(pszLine, '\"') == nullptr )
+        return CSVSplitLine( nullptr, pszLine, chDelimiter );
 
 /* -------------------------------------------------------------------- */
 /*      We must now count the quotes in our working string, and as      */
@@ -708,7 +759,7 @@ char **CSVReadParseLine2L( VSILFILE * fp, char chDelimiter )
             break;
 
         pszLine = CPLReadLineL( fp );
-        if( pszLine == NULL )
+        if( pszLine == nullptr )
             break;
 
         const size_t nLineLen = strlen(pszLine);
@@ -716,7 +767,7 @@ char **CSVReadParseLine2L( VSILFILE * fp, char chDelimiter )
         char* pszWorkLineTmp = static_cast<char *>(
             VSIRealloc(pszWorkLine,
                        nWorkLineLength + nLineLen + 2) );
-        if( pszWorkLineTmp == NULL )
+        if( pszWorkLineTmp == nullptr )
             break;
 
         pszWorkLine = pszWorkLineTmp;
@@ -727,7 +778,7 @@ char **CSVReadParseLine2L( VSILFILE * fp, char chDelimiter )
         nWorkLineLength += nLineLen + 1;
     }
 
-    char **papszReturn = CSVSplitLine( pszWorkLine, chDelimiter );
+    char **papszReturn = CSVSplitLine( nullptr, pszWorkLine, chDelimiter );
 
     CPLFree( pszWorkLine );
 
@@ -776,18 +827,18 @@ char **CSVScanLines( FILE *fp, int iKeyField, const char * pszValue,
                      CSVCompareCriteria eCriteria )
 
 {
-    CPLAssert( pszValue != NULL );
+    CPLAssert( pszValue != nullptr );
     CPLAssert( iKeyField >= 0 );
-    CPLAssert( fp != NULL );
+    CPLAssert( fp != nullptr );
 
     bool bSelected = false;
     const int nTestValue = atoi(pszValue);
-    char **papszFields = NULL;
+    char **papszFields = nullptr;
 
     while( !bSelected ) {
         papszFields = CSVReadParseLine( fp );
-        if( papszFields == NULL )
-            return NULL;
+        if( papszFields == nullptr )
+            return nullptr;
 
         if( CSLCount( papszFields ) < iKeyField+1 )
         {
@@ -807,7 +858,7 @@ char **CSVScanLines( FILE *fp, int iKeyField, const char * pszValue,
         if( !bSelected )
         {
             CSLDestroy( papszFields );
-            papszFields = NULL;
+            papszFields = nullptr;
         }
     }
 
@@ -826,18 +877,18 @@ char **CSVScanLinesL( VSILFILE *fp, int iKeyField, const char * pszValue,
                       CSVCompareCriteria eCriteria )
 
 {
-    CPLAssert( pszValue != NULL );
+    CPLAssert( pszValue != nullptr );
     CPLAssert( iKeyField >= 0 );
-    CPLAssert( fp != NULL );
+    CPLAssert( fp != nullptr );
 
     bool bSelected = false;
     const int nTestValue = atoi(pszValue);
-    char **papszFields = NULL;
+    char **papszFields = nullptr;
 
     while( !bSelected ) {
         papszFields = CSVReadParseLineL( fp );
-        if( papszFields == NULL )
-            return NULL;
+        if( papszFields == nullptr )
+            return nullptr;
 
         if( CSLCount( papszFields ) < iKeyField+1 )
         {
@@ -857,7 +908,7 @@ char **CSVScanLinesL( VSILFILE *fp, int iKeyField, const char * pszValue,
         if( !bSelected )
         {
             CSLDestroy( papszFields );
-            papszFields = NULL;
+            papszFields = nullptr;
         }
     }
 
@@ -876,7 +927,7 @@ static char **
 CSVScanLinesIndexed( CSVTable *psTable, int nKeyValue )
 
 {
-    CPLAssert( psTable->panLineIndex != NULL );
+    CPLAssert( psTable->panLineIndex != nullptr );
 
 /* -------------------------------------------------------------------- */
 /*      Find target record with binary search.                          */
@@ -907,14 +958,14 @@ CSVScanLinesIndexed( CSVTable *psTable, int nKeyValue )
     }
 
     if( iResult == -1 )
-        return NULL;
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Parse target line, and update iLastLine indicator.              */
 /* -------------------------------------------------------------------- */
     psTable->iLastLine = iResult;
 
-    return CSVSplitLine( psTable->papszLines[iResult], ',' );
+    return CSVSplitLine( psTable, psTable->papszLines[iResult], ',' );
 }
 
 /************************************************************************/
@@ -930,7 +981,7 @@ CSVScanLinesIngested( CSVTable *psTable, int iKeyField, const char * pszValue,
                       CSVCompareCriteria eCriteria )
 
 {
-    CPLAssert( pszValue != NULL );
+    CPLAssert( pszValue != nullptr );
     CPLAssert( iKeyField >= 0 );
 
     const int nTestValue = atoi(pszValue);
@@ -939,18 +990,19 @@ CSVScanLinesIngested( CSVTable *psTable, int iKeyField, const char * pszValue,
 /*      Short cut for indexed files.                                    */
 /* -------------------------------------------------------------------- */
     if( iKeyField == 0 && eCriteria == CC_Integer
-        && psTable->panLineIndex != NULL )
+        && psTable->panLineIndex != nullptr )
         return CSVScanLinesIndexed( psTable, nTestValue );
 
 /* -------------------------------------------------------------------- */
 /*      Scan from in-core lines.                                        */
 /* -------------------------------------------------------------------- */
-    char **papszFields = NULL;
+    char **papszFields = nullptr;
     bool bSelected = false;
 
     while( !bSelected && psTable->iLastLine+1 < psTable->nLineCount ) {
         psTable->iLastLine++;
-        papszFields = CSVSplitLine( psTable->papszLines[psTable->iLastLine],
+        papszFields = CSVSplitLine( psTable,
+                                    psTable->papszLines[psTable->iLastLine],
                                     ',' );
 
         if( CSLCount( papszFields ) < iKeyField+1 )
@@ -971,7 +1023,7 @@ CSVScanLinesIngested( CSVTable *psTable, int iKeyField, const char * pszValue,
         if( !bSelected )
         {
             CSLDestroy( papszFields );
-            papszFields = NULL;
+            papszFields = nullptr;
         }
     }
 
@@ -993,11 +1045,11 @@ char **CSVGetNextLine( const char *pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Get access to the table.                                        */
 /* -------------------------------------------------------------------- */
-    CPLAssert( pszFilename != NULL );
+    CPLAssert( pszFilename != nullptr );
 
     CSVTable * const psTable = CSVAccess( pszFilename );
-    if( psTable == NULL )
-        return NULL;
+    if( psTable == nullptr )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      If we use CSVGetNextLine() we can pretty much assume we have    */
@@ -1010,12 +1062,12 @@ char **CSVGetNextLine( const char *pszFilename )
 /*      ingested tables I believe.                                      */
 /* -------------------------------------------------------------------- */
     if( psTable->iLastLine+1 >= psTable->nLineCount )
-        return NULL;
+        return nullptr;
 
     psTable->iLastLine++;
     CSLDestroy( psTable->papszRecFields );
     psTable->papszRecFields =
-        CSVSplitLine( psTable->papszLines[psTable->iLastLine], ',' );
+        CSVSplitLine( psTable, psTable->papszLines[psTable->iLastLine], ',' );
 
     return psTable->papszRecFields;
 }
@@ -1027,23 +1079,11 @@ char **CSVGetNextLine( const char *pszFilename )
 /*      taking care of file opening and closing.                        */
 /************************************************************************/
 
-char **CSVScanFile( const char * pszFilename, int iKeyField,
+static
+char **CSVScanFile( CSVTable * const psTable, int iKeyField,
                     const char * pszValue, CSVCompareCriteria eCriteria )
-
 {
-/* -------------------------------------------------------------------- */
-/*      Get access to the table.                                        */
-/* -------------------------------------------------------------------- */
-    CPLAssert( pszFilename != NULL );
-
-    if( iKeyField < 0 )
-        return NULL;
-
-    CSVTable * const psTable = CSVAccess( pszFilename );
-    if( psTable == NULL )
-        return NULL;
-
-    CSVIngest( pszFilename );
+    CSVIngest( psTable->pszFilename );
 
 /* -------------------------------------------------------------------- */
 /*      Does the current record match the criteria?  If so, just        */
@@ -1064,7 +1104,7 @@ char **CSVScanFile( const char * pszFilename, int iKeyField,
     psTable->iLastLine = -1;
     CSLDestroy( psTable->papszRecFields );
 
-    if( psTable->pszRawData != NULL )
+    if( psTable->pszRawData != nullptr )
         psTable->papszRecFields =
             CSVScanLinesIngested( psTable, iKeyField, pszValue, eCriteria );
     else
@@ -1079,6 +1119,25 @@ char **CSVScanFile( const char * pszFilename, int iKeyField,
     return psTable->papszRecFields;
 }
 
+char **CSVScanFile( const char * pszFilename, int iKeyField,
+                    const char * pszValue, CSVCompareCriteria eCriteria )
+
+{
+/* -------------------------------------------------------------------- */
+/*      Get access to the table.                                        */
+/* -------------------------------------------------------------------- */
+    CPLAssert( pszFilename != nullptr );
+
+    if( iKeyField < 0 )
+        return nullptr;
+
+    CSVTable * const psTable = CSVAccess( pszFilename );
+    if( psTable == nullptr )
+        return nullptr;
+
+    return CSVScanFile( psTable, iKeyField, pszValue, eCriteria );
+}
+
 /************************************************************************/
 /*                           CPLGetFieldId()                            */
 /*                                                                      */
@@ -1095,12 +1154,12 @@ char **CSVScanFile( const char * pszFilename, int iKeyField,
 int CSVGetFieldId( FILE * fp, const char * pszFieldName )
 
 {
-    CPLAssert( fp != NULL && pszFieldName != NULL );
+    CPLAssert( fp != nullptr && pszFieldName != nullptr );
 
     VSIRewind( fp );
 
     char **papszFields = CSVReadParseLine( fp );
-    for( int i = 0; papszFields != NULL && papszFields[i] != NULL; i++ )
+    for( int i = 0; papszFields != nullptr && papszFields[i] != nullptr; i++ )
     {
         if( EQUAL(papszFields[i], pszFieldName) )
         {
@@ -1128,12 +1187,12 @@ int CSVGetFieldId( FILE * fp, const char * pszFieldName )
 int CSVGetFieldIdL( VSILFILE * fp, const char * pszFieldName )
 
 {
-    CPLAssert( fp != NULL && pszFieldName != NULL );
+    CPLAssert( fp != nullptr && pszFieldName != nullptr );
 
     VSIRewindL( fp );
 
     char **papszFields = CSVReadParseLineL( fp );
-    for( int i = 0; papszFields != NULL && papszFields[i] != NULL; i++ )
+    for( int i = 0; papszFields != nullptr && papszFields[i] != nullptr; i++ )
     {
         if( EQUAL(papszFields[i], pszFieldName) )
         {
@@ -1154,27 +1213,21 @@ int CSVGetFieldIdL( VSILFILE * fp, const char * pszFieldName )
 /*      on filename, rather than having an existing handle.             */
 /************************************************************************/
 
-int CSVGetFileFieldId( const char * pszFilename, const char * pszFieldName )
+static int CSVGetFileFieldId( CSVTable * const psTable, const char * pszFieldName )
 
 {
-/* -------------------------------------------------------------------- */
-/*      Get access to the table.                                        */
-/* -------------------------------------------------------------------- */
-    CPLAssert( pszFilename != NULL );
-
-    CSVTable * const psTable = CSVAccess( pszFilename );
-    if( psTable == NULL )
-        return -1;
-
 /* -------------------------------------------------------------------- */
 /*      Find the requested field.                                       */
 /* -------------------------------------------------------------------- */
+    const int nFieldNameLength = static_cast<int>(strlen(pszFieldName));
     for( int i = 0;
-         psTable->papszFieldNames != NULL
-             && psTable->papszFieldNames[i] != NULL;
+         psTable->papszFieldNames != nullptr
+             && psTable->papszFieldNames[i] != nullptr;
          i++ )
     {
-        if( EQUAL(psTable->papszFieldNames[i], pszFieldName) )
+        if( psTable->panFieldNamesLength[i] == nFieldNameLength &&
+            EQUALN(psTable->papszFieldNames[i], pszFieldName,
+                   nFieldNameLength) )
         {
             return i;
         }
@@ -1183,6 +1236,21 @@ int CSVGetFileFieldId( const char * pszFilename, const char * pszFieldName )
     return -1;
 }
 
+int CSVGetFileFieldId( const char * pszFilename, const char * pszFieldName )
+
+{
+/* -------------------------------------------------------------------- */
+/*      Get access to the table.                                        */
+/* -------------------------------------------------------------------- */
+    CPLAssert( pszFilename != nullptr );
+
+    CSVTable * const psTable = CSVAccess( pszFilename );
+    if( psTable == nullptr )
+        return -1;
+    return CSVGetFileFieldId( psTable, pszFieldName );
+}
+
+
 /************************************************************************/
 /*                         CSVScanFileByName()                          */
 /*                                                                      */
@@ -1197,7 +1265,7 @@ char **CSVScanFileByName( const char * pszFilename,
 {
     const int iKeyField = CSVGetFileFieldId( pszFilename, pszKeyFieldName );
     if( iKeyField == -1 )
-        return NULL;
+        return nullptr;
 
     return CSVScanFile( pszFilename, iKeyField, pszValue, eCriteria );
 }
@@ -1223,25 +1291,29 @@ const char *CSVGetField( const char * pszFilename,
 /*      Find the table.                                                 */
 /* -------------------------------------------------------------------- */
     CSVTable * const psTable = CSVAccess( pszFilename );
-    if( psTable == NULL )
+    if( psTable == nullptr )
+        return "";
+
+    const int iKeyField = CSVGetFileFieldId( psTable, pszKeyFieldName );
+    if( iKeyField == -1 )
         return "";
 
 /* -------------------------------------------------------------------- */
 /*      Find the correct record.                                        */
 /* -------------------------------------------------------------------- */
-    char **papszRecord = CSVScanFileByName( pszFilename, pszKeyFieldName,
-                                            pszKeyFieldValue, eCriteria );
-    if( papszRecord == NULL )
+    char **papszRecord = CSVScanFile( psTable, iKeyField,
+                                      pszKeyFieldValue, eCriteria );
+    if( papszRecord == nullptr )
         return "";
 
 /* -------------------------------------------------------------------- */
 /*      Figure out which field we want out of this.                     */
 /* -------------------------------------------------------------------- */
-    const int iTargetField = CSVGetFileFieldId( pszFilename, pszTargetField );
+    const int iTargetField = CSVGetFileFieldId( psTable, pszTargetField );
     if( iTargetField < 0 )
         return "";
 
-    for( int i=0; papszRecord[i] != NULL; ++i )
+    for( int i=0; papszRecord[i] != nullptr; ++i )
     {
         if( i == iTargetField )
             return papszRecord[iTargetField];
@@ -1269,12 +1341,12 @@ const char * GDALDefaultCSVFilename( const char *pszBasename )
     int bMemoryError = FALSE;
     CSVTable **ppsCSVTableList = static_cast<CSVTable **>(
       CPLGetTLSEx( CTLS_CSVTABLEPTR, &bMemoryError ) );
-    if( ppsCSVTableList != NULL )
+    if( ppsCSVTableList != nullptr )
     {
         const size_t nBasenameLen = strlen(pszBasename);
 
         for( const CSVTable *psTable = *ppsCSVTableList;
-             psTable != NULL;
+             psTable != nullptr;
              psTable = psTable->psNext )
         {
             const size_t nFullLen = strlen(psTable->pszFilename);
@@ -1283,7 +1355,7 @@ const char * GDALDefaultCSVFilename( const char *pszBasename )
                 && strcmp(psTable->pszFilename+nFullLen-nBasenameLen,
                           pszBasename) == 0
                 && strchr("/\\", psTable->pszFilename[+nFullLen-nBasenameLen-1])
-                          != NULL )
+                          != nullptr )
             {
                 return psTable->pszFilename;
             }
@@ -1296,34 +1368,34 @@ const char * GDALDefaultCSVFilename( const char *pszBasename )
     DefaultCSVFileNameTLS* pTLSData =
         static_cast<DefaultCSVFileNameTLS *>(
             CPLGetTLSEx( CTLS_CSVDEFAULTFILENAME, &bMemoryError ) );
-    if( pTLSData == NULL && !bMemoryError )
+    if( pTLSData == nullptr && !bMemoryError )
     {
         pTLSData = static_cast<DefaultCSVFileNameTLS *>(
             VSI_CALLOC_VERBOSE( 1, sizeof(DefaultCSVFileNameTLS) ) );
         if( pTLSData )
             CPLSetTLS( CTLS_CSVDEFAULTFILENAME, pTLSData, TRUE );
     }
-    if( pTLSData == NULL )
+    if( pTLSData == nullptr )
         return "/not_existing_dir/not_existing_path";
 
     const char *pszResult = CPLFindFile( "epsg_csv", pszBasename );
 
-    if( pszResult != NULL )
+    if( pszResult != nullptr )
         return pszResult;
 
     if( !pTLSData->bCSVFinderInitialized )
     {
         pTLSData->bCSVFinderInitialized = true;
 
-        if( CPLGetConfigOption("GEOTIFF_CSV", NULL) != NULL )
-            CPLPushFinderLocation( CPLGetConfigOption("GEOTIFF_CSV", NULL));
+        if( CPLGetConfigOption("GEOTIFF_CSV", nullptr) != nullptr )
+            CPLPushFinderLocation( CPLGetConfigOption("GEOTIFF_CSV", nullptr));
 
-        if( CPLGetConfigOption("GDAL_DATA", NULL) != NULL )
-            CPLPushFinderLocation( CPLGetConfigOption("GDAL_DATA", NULL) );
+        if( CPLGetConfigOption("GDAL_DATA", nullptr) != nullptr )
+            CPLPushFinderLocation( CPLGetConfigOption("GDAL_DATA", nullptr) );
 
         pszResult = CPLFindFile( "epsg_csv", pszBasename );
 
-        if( pszResult != NULL )
+        if( pszResult != nullptr )
             return pszResult;
     }
 
@@ -1351,10 +1423,10 @@ const char * GDALDefaultCSVFilename( const char *pszBasename )
 #endif  // GDAL_PREFIX
 
     VSILFILE *fp = VSIFOpenL( pTLSData->szPath, "rt" );
-    if( fp == NULL )
+    if( fp == nullptr )
         CPLStrlcpy( pTLSData->szPath, pszBasename, sizeof(pTLSData->szPath) );
 
-    if( fp != NULL )
+    if( fp != nullptr )
         VSIFCloseL( fp );
 
     return pTLSData->szPath;
@@ -1369,13 +1441,13 @@ const char * GDALDefaultCSVFilename( const char *pszBasename )
 /************************************************************************/
 
 CPL_C_START
-static const char *(*pfnCSVFilenameHook)(const char *) = NULL;
+static const char *(*pfnCSVFilenameHook)(const char *) = nullptr;
 CPL_C_END
 
 const char * CSVFilename( const char *pszBasename )
 
 {
-    if( pfnCSVFilenameHook == NULL )
+    if( pfnCSVFilenameHook == nullptr )
         return GDALDefaultCSVFilename( pszBasename );
 
     return pfnCSVFilenameHook( pszBasename );
