@@ -44,23 +44,23 @@ void CPL_DLL VSIInstallCryptFileHandler();
 void CPL_DLL VSISetCryptKey( const GByte* pabyKey, int nKeySize );
 CPL_C_END
 
-CPL_CVSID("$Id: cpl_vsil_crypt.cpp 40012 2017-09-06 12:52:12Z rouault $");
+CPL_CVSID("$Id: cpl_vsil_crypt.cpp 8e4c736c5491eb2ed544aa2c51c3ac07d55bcca8 2018-05-01 05:05:17 +0800 jerry73204 $")
 
-static const char VSICRYPT_PREFIX[] = "/vsicrypt/";
+constexpr char VSICRYPT_PREFIX[] = "/vsicrypt/";
 
 #if defined(HAVE_CRYPTOPP) || defined(DOXYGEN_SKIP)
 
 //! @cond Doxygen_Suppress
 
 /* Increase Major in case of backward incompatible changes */
-static const int VSICRYPT_CURRENT_MAJOR = 1;
-static const int VSICRYPT_CURRENT_MINOR = 0;
-static const char VSICRYPT_SIGNATURE[] = "VSICRYPT";  // Must be 8 chars.
+constexpr int VSICRYPT_CURRENT_MAJOR = 1;
+constexpr int VSICRYPT_CURRENT_MINOR = 0;
+constexpr char VSICRYPT_SIGNATURE[] = "VSICRYPT";  // Must be 8 chars.
 
-static const char VSICRYPT_PREFIX_WITHOUT_SLASH[] = "/vsicrypt";
+constexpr char VSICRYPT_PREFIX_WITHOUT_SLASH[] = "/vsicrypt";
 
-static const unsigned int VSICRYPT_READ = 0x1;
-static const unsigned int VSICRYPT_WRITE = 0x2;
+constexpr unsigned int VSICRYPT_READ = 0x1;
+constexpr unsigned int VSICRYPT_WRITE = 0x2;
 
 #ifdef _MSC_VER
 #pragma warning( push )
@@ -101,6 +101,14 @@ static const unsigned int VSICRYPT_WRITE = 0x2;
 #ifdef _MSC_VER
 #pragma warning( pop )
 #endif
+
+// Fix compatibility with Crypto++
+#if CRYPTOPP_VERSION >= 600
+typedef CryptoPP::byte cryptopp_byte;
+#else
+typedef byte cryptopp_byte;
+#endif
+
 /* End of crypto++ headers */
 
 // I don't really understand why this is necessary, especially
@@ -109,8 +117,8 @@ static const unsigned int VSICRYPT_WRITE = 0x2;
 // Borrowed from dlltest.cpp of crypto++
 #if defined(WIN32) && defined(USE_ONLY_CRYPTODLL_ALG)
 
-static CryptoPP::PNew s_pNew = NULL;
-static CryptoPP::PDelete s_pDelete = NULL;
+static CryptoPP::PNew s_pNew = nullptr;
+static CryptoPP::PDelete s_pDelete = nullptr;
 
 extern "C" __declspec(dllexport)
 void __cdecl SetNewAndDeleteFromCryptoPP(
@@ -134,7 +142,7 @@ void __cdecl operator delete( void * p )
 
 #endif  // defined(WIN32) && defined(USE_ONLY_CRYPTODLL_ALG)
 
-static GByte* pabyGlobalKey = NULL;
+static GByte* pabyGlobalKey = nullptr;
 static int nGlobalKeySize = 0;
 
 typedef enum
@@ -196,15 +204,15 @@ typedef enum
  */
 void VSISetCryptKey( const GByte* pabyKey, int nKeySize )
 {
-    CPLAssert( (pabyKey != NULL && nKeySize != 0) ||
-               (pabyKey == NULL && nKeySize == 0) );
+    CPLAssert( (pabyKey != nullptr && nKeySize != 0) ||
+               (pabyKey == nullptr && nKeySize == 0) );
     if( pabyGlobalKey )
     {
         // Make some effort to clear the memory, although it could have leaked
         // elsewhere...
         memset( pabyGlobalKey, 0, nGlobalKeySize );
         CPLFree(pabyGlobalKey);
-        pabyGlobalKey = NULL;
+        pabyGlobalKey = nullptr;
         nGlobalKeySize = 0;
     }
     if( pabyKey )
@@ -297,7 +305,7 @@ static CryptoPP::BlockCipher* GetEncBlockCipher( VSICryptAlg eAlg )
         CASE_ALG(Twofish)
         CASE_ALG(XTEA)
 #endif
-        default: return NULL;
+        default: return nullptr;
     }
 }
 
@@ -342,7 +350,7 @@ static CryptoPP::BlockCipher* GetDecBlockCipher( VSICryptAlg eAlg )
         CASE_ALG(Twofish)
         CASE_ALG(XTEA)
 #endif
-        default: return NULL;
+        default: return nullptr;
     }
 }
 
@@ -443,11 +451,11 @@ VSICryptFileHeader::CryptKeyCheck( CryptoPP::BlockCipher* poEncCipher )
 
     CPLAssert( osIV.size() == poEncCipher->BlockSize() );
     // Generate a unique IV with a sector offset of 0xFFFFFFFFFFFFFFFF.
-    std::string osCheckIV(VSICryptGenerateSectorIV(osIV, ~((vsi_l_offset)0)));
+    std::string osCheckIV(VSICryptGenerateSectorIV(osIV, ~(static_cast<vsi_l_offset>(0))));
     CryptoPP::StringSink* poSink = new CryptoPP::StringSink(osKeyCheckRes);
     CryptoPP::StreamTransformation* poMode =
         new CryptoPP::CBC_Mode_ExternalCipher::Encryption(
-            *poEncCipher, (const byte*)osCheckIV.c_str());
+            *poEncCipher, reinterpret_cast<const cryptopp_byte*>(osCheckIV.c_str()));
     CryptoPP::StreamTransformationFilter* poEnc =
         new CryptoPP::StreamTransformationFilter(
             *poMode, poSink,
@@ -455,7 +463,7 @@ VSICryptFileHeader::CryptKeyCheck( CryptoPP::BlockCipher* poEncCipher )
     // Not sure if it is add extra security, but pick up something that is
     // unlikely to be a plain text (random number).
     poEnc->Put(
-        reinterpret_cast<const byte*>(
+        reinterpret_cast<const cryptopp_byte*>(
             "\xDB\x31\xB9\x1B\xD3\x1C\xFA\x3E\x84\x06\xC1\x42\xC3\xEC\xCD\x9A"
             "\x02\x36\x22\x15\x58\x88\x74\x65\x00\x2F\x98\xBC\x69\x22\xE1\x63"),
         std::min(32U, poEncCipher->BlockSize()));
@@ -531,8 +539,8 @@ int VSICryptFileHeader::ReadFromFile( VSIVirtualHandle* fp,
                  nMode);
         return FALSE;
     }
-    eAlg = (VSICryptAlg)nAlg;
-    eMode = (VSICryptMode)nMode;
+    eAlg = static_cast<VSICryptAlg>(nAlg);
+    eMode = static_cast<VSICryptMode>(nMode);
 
     GByte nIVSize;
     if( fp->Read(&nIVSize, 1, 1) == 0 )
@@ -540,7 +548,7 @@ int VSICryptFileHeader::ReadFromFile( VSIVirtualHandle* fp,
 
     osIV.resize(nIVSize);
     // TODO(schwehr): Using the const buffer of a string is a bad idea.
-    if( fp->Read((void*)osIV.c_str(), 1, nIVSize) != nIVSize )
+    if( fp->Read(reinterpret_cast<void*>(const_cast<char*>(osIV.c_str())), 1, nIVSize) != nIVSize )
         return VSICryptReadError();
 
     GUInt16 nFreeTextSize;
@@ -548,7 +556,7 @@ int VSICryptFileHeader::ReadFromFile( VSIVirtualHandle* fp,
         return VSICryptReadError();
 
     osFreeText.resize(nFreeTextSize);
-    if( fp->Read((void*)osFreeText.c_str(), 1, nFreeTextSize) != nFreeTextSize )
+    if( fp->Read(reinterpret_cast<void*>(const_cast<char*>(osFreeText.c_str())), 1, nFreeTextSize) != nFreeTextSize )
         return VSICryptReadError();
 
     GByte nKeyCheckSize;
@@ -559,11 +567,11 @@ int VSICryptFileHeader::ReadFromFile( VSIVirtualHandle* fp,
     {
         CPLString osKeyCheck;
         osKeyCheck.resize(nKeyCheckSize);
-        if( fp->Read((void*)osKeyCheck.c_str(), 1,
+        if( fp->Read(reinterpret_cast<void*>(const_cast<char*>(osKeyCheck.c_str())), 1,
                      nKeyCheckSize) != nKeyCheckSize )
             return VSICryptReadError();
 
-        if( osKey.empty() && pabyGlobalKey == NULL )
+        if( osKey.empty() && pabyGlobalKey == nullptr )
         {
             CPLError(CE_Failure, CPLE_AppDefined,
                      "Encryption key not defined as key/key_b64 parameter, "
@@ -573,7 +581,7 @@ int VSICryptFileHeader::ReadFromFile( VSIVirtualHandle* fp,
         }
 
         CryptoPP::BlockCipher* poEncCipher = GetEncBlockCipher(eAlg);
-        if( poEncCipher == NULL )
+        if( poEncCipher == nullptr )
             return FALSE;
 
         if( osIV.size() != poEncCipher->BlockSize() )
@@ -592,7 +600,7 @@ int VSICryptFileHeader::ReadFromFile( VSIVirtualHandle* fp,
             {
                 const int nKeySize =
                     std::min(nMaxKeySize, static_cast<int>(osKey.size()));
-                poEncCipher->SetKey((const byte*)osKey.c_str(), nKeySize);
+                poEncCipher->SetKey(reinterpret_cast<const cryptopp_byte*>(osKey.c_str()), nKeySize);
             }
             else if( pabyGlobalKey )
             {
@@ -634,7 +642,7 @@ int VSICryptFileHeader::ReadFromFile( VSIVirtualHandle* fp,
     nExtraContentSize = CPL_LSBWORD16(nExtraContentSize);
 
     osExtraContent.resize(nExtraContentSize);
-    if( fp->Read((void*)osExtraContent.c_str(), 1, nExtraContentSize)
+    if( fp->Read(const_cast<char*>(osExtraContent.c_str()), 1, nExtraContentSize)
         != nExtraContentSize )
         return VSICryptReadError();
 
@@ -698,7 +706,7 @@ int VSICryptFileHeader::WriteToFile( VSIVirtualHandle* fp,
     bRet &= (fp->Write(&nIVSizeToWrite, 1, 1) == 1);
     bRet &= (fp->Write(osIV.c_str(), 1, osIV.size()) == osIV.size());
 
-    GUInt16 nFreeTextSizeToWrite = CPL_LSBWORD16((GUInt16)osFreeText.size());
+    GUInt16 nFreeTextSizeToWrite = CPL_LSBWORD16(static_cast<GUInt16>(osFreeText.size()));
     bRet &= (fp->Write(&nFreeTextSizeToWrite, 2, 1) == 1);
     bRet &= (fp->Write(osFreeText.c_str(), 1,
                        osFreeText.size()) == osFreeText.size());
@@ -727,7 +735,7 @@ int VSICryptFileHeader::WriteToFile( VSIVirtualHandle* fp,
 /*                          VSICryptFileHandle                          */
 /************************************************************************/
 
-class VSICryptFileHandle CPL_FINAL : public VSIVirtualHandle
+class VSICryptFileHandle final : public VSIVirtualHandle
 {
   private:
         CPLString           osBaseFilename;
@@ -758,21 +766,19 @@ class VSICryptFileHandle CPL_FINAL : public VSIVirtualHandle
                         VSIVirtualHandle* poBaseHandle,
                         VSICryptFileHeader* poHeader,
                         int nPerms );
-    virtual ~VSICryptFileHandle();
+    ~VSICryptFileHandle() override;
 
     int                  Init( const CPLString& osKey,
                                bool bWriteHeader = false );
 
-    virtual int          Seek( vsi_l_offset nOffset, int nWhence ) override;
-    virtual vsi_l_offset Tell() override;
-    virtual size_t       Read( void *pBuffer, size_t nSize,
-                               size_t nMemb ) override;
-    virtual size_t       Write( const void *pBuffer, size_t nSize,
-                                size_t nMemb ) override;
-    virtual int          Eof() override;
-    virtual int          Flush() override;
-    virtual int          Close() override;
-    virtual int          Truncate( vsi_l_offset nNewSize ) override;
+    int Seek( vsi_l_offset nOffset, int nWhence ) override;
+    vsi_l_offset Tell() override;
+    size_t Read( void *pBuffer, size_t nSize, size_t nMemb ) override;
+    size_t Write( const void *pBuffer, size_t nSize, size_t nMemb ) override;
+    int Eof() override;
+    int Flush() override;
+    int Close() override;
+    int Truncate( vsi_l_offset nNewSize ) override;
 };
 
 /************************************************************************/
@@ -790,11 +796,11 @@ VSICryptFileHandle::VSICryptFileHandle( CPLString osBaseFilenameIn,
     bUpdateHeader(false),
     nCurPos(0),
     bEOF(false),
-    poEncCipher(NULL),
-    poDecCipher(NULL),
+    poEncCipher(nullptr),
+    poDecCipher(nullptr),
     nBlockSize(0),
     nWBOffset(0),
-    pabyWB(NULL),
+    pabyWB(nullptr),
     nWBSize(0),
     bWBDirty(false),
     bLastSectorWasModified(false)
@@ -820,7 +826,7 @@ VSICryptFileHandle::~VSICryptFileHandle()
 int VSICryptFileHandle::Init( const CPLString& osKey, bool bWriteHeader )
 {
     poEncCipher = GetEncBlockCipher(poHeader->eAlg);
-    if( poEncCipher == NULL )
+    if( poEncCipher == nullptr )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "Cipher algorithm not supported in this build: %d",
@@ -845,8 +851,8 @@ int VSICryptFileHandle::Init( const CPLString& osKey, bool bWriteHeader )
         {
             const int nKeySize =
                 std::min(nMaxKeySize, static_cast<int>(osKey.size()));
-            poEncCipher->SetKey((const byte*)osKey.c_str(), nKeySize);
-            poDecCipher->SetKey((const byte*)osKey.c_str(), nKeySize);
+            poEncCipher->SetKey(reinterpret_cast<const cryptopp_byte*>(osKey.c_str()), nKeySize);
+            poDecCipher->SetKey(reinterpret_cast<const cryptopp_byte*>(osKey.c_str()), nKeySize);
         }
         else if( pabyGlobalKey )
         {
@@ -905,19 +911,19 @@ void VSICryptFileHandle::EncryptBlock( GByte* pabyData, vsi_l_offset nOffset )
     CryptoPP::StreamTransformation* poMode;
     if( poHeader->eMode == MODE_CBC )
         poMode = new CryptoPP::CBC_Mode_ExternalCipher::Encryption(
-            *poEncCipher, reinterpret_cast<const byte *>(osIV.c_str()) );
+            *poEncCipher, reinterpret_cast<const cryptopp_byte *>(osIV.c_str()) );
     else if( poHeader->eMode == MODE_CFB )
         poMode = new CryptoPP::CFB_Mode_ExternalCipher::Encryption(
-            *poEncCipher, reinterpret_cast<const byte *>(osIV.c_str()) );
+            *poEncCipher, reinterpret_cast<const cryptopp_byte *>(osIV.c_str()) );
     else if( poHeader->eMode == MODE_OFB )
         poMode = new CryptoPP::OFB_Mode_ExternalCipher::Encryption(
-            *poEncCipher, reinterpret_cast<const byte *>(osIV.c_str()) );
+            *poEncCipher, reinterpret_cast<const cryptopp_byte *>(osIV.c_str()) );
     else if( poHeader->eMode == MODE_CTR )
         poMode = new CryptoPP::CTR_Mode_ExternalCipher::Encryption(
-            *poEncCipher, reinterpret_cast<const byte *>(osIV.c_str()) );
+            *poEncCipher, reinterpret_cast<const cryptopp_byte *>(osIV.c_str()) );
     else
         poMode = new CryptoPP::CBC_CTS_Mode_ExternalCipher::Encryption(
-            *poEncCipher, reinterpret_cast<const byte *>(osIV.c_str()) );
+            *poEncCipher, reinterpret_cast<const cryptopp_byte *>(osIV.c_str()) );
     CryptoPP::StreamTransformationFilter* poEnc =
         new CryptoPP::StreamTransformationFilter(
             *poMode, poSink, CryptoPP::StreamTransformationFilter::NO_PADDING);
@@ -941,30 +947,30 @@ bool VSICryptFileHandle::DecryptBlock( GByte* pabyData, vsi_l_offset nOffset )
     std::string osIV(VSICryptGenerateSectorIV(poHeader->osIV, nOffset));
     CPLAssert( static_cast<int>(osIV.size()) == nBlockSize );
     CryptoPP::StringSink* poSink = new CryptoPP::StringSink(osRes);
-    CryptoPP::StreamTransformation* poMode = NULL;
-    CryptoPP::StreamTransformationFilter* poDec = NULL;
+    CryptoPP::StreamTransformation* poMode = nullptr;
+    CryptoPP::StreamTransformationFilter* poDec = nullptr;
 
     try
     {
         // Yes, some modes need the encryption cipher.
         if( poHeader->eMode == MODE_CBC )
             poMode = new CryptoPP::CBC_Mode_ExternalCipher::Decryption(
-                *poDecCipher, (const byte*)osIV.c_str() );
+                *poDecCipher, reinterpret_cast<const cryptopp_byte*>(osIV.c_str()) );
         else if( poHeader->eMode == MODE_CFB )
             poMode = new CryptoPP::CFB_Mode_ExternalCipher::Decryption(
-                *poEncCipher, (const byte*)osIV.c_str() );
+                *poEncCipher, reinterpret_cast<const cryptopp_byte*>(osIV.c_str()) );
         else if( poHeader->eMode == MODE_OFB )
             poMode = new CryptoPP::OFB_Mode_ExternalCipher::Decryption(
-                *poEncCipher, (const byte*)osIV.c_str() );
+                *poEncCipher, reinterpret_cast<const cryptopp_byte*>(osIV.c_str()) );
         else if( poHeader->eMode == MODE_CTR )
             poMode = new CryptoPP::CTR_Mode_ExternalCipher::Decryption(
-                *poEncCipher, (const byte*)osIV.c_str() );
+                *poEncCipher, reinterpret_cast<const cryptopp_byte*>(osIV.c_str()) );
         else
             poMode = new CryptoPP::CBC_CTS_Mode_ExternalCipher::Decryption(
-                *poDecCipher, (const byte*)osIV.c_str() );
+                *poDecCipher, reinterpret_cast<const cryptopp_byte*>(osIV.c_str()) );
         poDec = new CryptoPP::StreamTransformationFilter(
             *poMode, poSink, CryptoPP::StreamTransformationFilter::NO_PADDING);
-        poDec->Put((const byte*)pabyData, poHeader->nSectorSize);
+        poDec->Put(reinterpret_cast<const cryptopp_byte*>(pabyData), poHeader->nSectorSize);
         poDec->MessageEnd();
         delete poDec;
         delete poMode;
@@ -1242,8 +1248,7 @@ VSICryptFileHandle::Write( const void *pBuffer, size_t nSize, size_t nMemb )
 #endif
                     // Fill with 0.
                     memset(
-                        (byte*)(pabyWB + poHeader->nPayloadFileSize -
-                                nLastSectorOffset),
+                        pabyWB + poHeader->nPayloadFileSize - nLastSectorOffset,
                         0,
                         static_cast<int>(
                             poHeader->nSectorSize -
@@ -1361,7 +1366,7 @@ int VSICryptFileHandle::Flush()
 #endif
                 CryptoPP::OS_GenerateRandomBlock(
                     false, // Do not need cryptographic randomness.
-                    (byte*)(pabyWB +
+                    reinterpret_cast<cryptopp_byte*>(pabyWB +
                             poHeader->nPayloadFileSize - nLastSectorOffset),
                     static_cast<int>(
                         poHeader->nSectorSize -
@@ -1399,13 +1404,13 @@ int VSICryptFileHandle::Flush()
 int VSICryptFileHandle::Close()
 {
     int nRet = 0;
-    if( poBaseHandle != NULL && poHeader != NULL )
+    if( poBaseHandle != nullptr && poHeader != nullptr )
     {
         if( Flush() != 0 )
             return -1;
         nRet = poBaseHandle->Close();
         delete poBaseHandle;
-        poBaseHandle = NULL;
+        poBaseHandle = nullptr;
     }
 #ifdef VERBOSE_VSICRYPT
     CPLDebug("VSICRYPT", "Close(%s)", osBaseFilename.c_str());
@@ -1417,22 +1422,20 @@ int VSICryptFileHandle::Close()
 /*                   VSICryptFilesystemHandler                          */
 /************************************************************************/
 
-class VSICryptFilesystemHandler CPL_FINAL : public VSIFilesystemHandler
+class VSICryptFilesystemHandler final : public VSIFilesystemHandler
 {
 public:
     VSICryptFilesystemHandler();
-    virtual ~VSICryptFilesystemHandler();
+    ~VSICryptFilesystemHandler() override;
 
-    virtual VSIVirtualHandle *Open( const char *pszFilename,
-                                    const char *pszAccess,
-                                    bool bSetError ) override;
-    virtual int      Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
-                           int nFlags ) override;
-    virtual int      Unlink( const char *pszFilename ) override;
-    virtual int      Rename( const char *oldpath, const char *newpath )
-        override;
-    virtual char**   ReadDirEx( const char *pszDirname, int nMaxFiles )
-        override;
+    VSIVirtualHandle *Open( const char *pszFilename,
+                            const char *pszAccess,
+                            bool bSetError ) override;
+    int Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
+              int nFlags ) override;
+    int Unlink( const char *pszFilename ) override;
+    int Rename( const char *oldpath, const char *newpath ) override;
+    char** ReadDirEx( const char *pszDirname, int nMaxFiles ) override;
 };
 
 /************************************************************************/
@@ -1464,7 +1467,7 @@ static CPLString GetFilename( const char* pszFilename )
                        strlen(VSICRYPT_PREFIX)) == 0 );
     pszFilename += strlen(VSICRYPT_PREFIX);
     const char* pszFileArg = strstr(pszFilename, "file=");
-    if( pszFileArg == NULL )
+    if( pszFileArg == nullptr )
         return pszFilename;
     CPLString osRet(pszFileArg + strlen("file="));
     return osRet;
@@ -1481,7 +1484,7 @@ static CPLString GetArgument( const char* pszFilename, const char* pszParamName,
     osParamName += "=";
 
     const char* pszNeedle = strstr(pszFilename, osParamName);
-    if( pszNeedle == NULL )
+    if( pszNeedle == nullptr )
         return pszDefault;
 
     CPLString osRet(pszNeedle + osParamName.size());
@@ -1520,13 +1523,13 @@ static CPLString GetKey( const char* pszFilename )
         }
         if( !osKeyB64.empty() )
         {
-            GByte* key = (GByte*)CPLStrdup(osKeyB64);
+            GByte* key = reinterpret_cast<GByte*>(CPLStrdup(osKeyB64));
             int nLength = CPLBase64DecodeInPlace(key);
-            osKey.assign((const char*)key, nLength);
+            osKey.assign(reinterpret_cast<const char*>(key), nLength);
             memset(key, 0, osKeyB64.size());
             CPLFree(key);
         }
-        memset((void*)osKeyB64.c_str(), 0, osKeyB64.size());
+        memset(const_cast<char*>(osKeyB64.c_str()), 0, osKeyB64.size());
     }
     return osKey;
 }
@@ -1545,32 +1548,32 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
     CPLString osFilename(GetFilename(pszFilename));
 
     CPLString osKey(GetKey(pszFilename));
-    if( osKey.empty() && pabyGlobalKey == NULL )
+    if( osKey.empty() && pabyGlobalKey == nullptr )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                 "Encryption key not defined as key/key_b64 parameter, "
                 "VSICRYPT_KEY/VSICRYPT_KEY_B64 configuration option or "
                  "VSISetCryptKey() API");
-        return NULL;
+        return nullptr;
     }
 
     if( strchr(pszAccess, 'r') )
     {
         CPLString osAccess(pszAccess);
-        if( strchr(pszAccess, 'b') == NULL )
+        if( strchr(pszAccess, 'b') == nullptr )
             osAccess += "b";
         VSIVirtualHandle* fpBase =
-            (VSIVirtualHandle*)VSIFOpenL(osFilename, osAccess);
-        if( fpBase == NULL )
-            return NULL;
+            reinterpret_cast<VSIVirtualHandle*>(VSIFOpenL(osFilename, osAccess));
+        if( fpBase == nullptr )
+            return nullptr;
         VSICryptFileHeader* poHeader = new VSICryptFileHeader();
         if( !poHeader->ReadFromFile(fpBase, osKey) )
         {
-            memset((void*)osKey.c_str(), 0, osKey.size());
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
             fpBase->Close();
             delete fpBase;
             delete poHeader;
-            return NULL;
+            return nullptr;
         }
 
         VSICryptFileHandle* poHandle =
@@ -1581,11 +1584,11 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
                 : VSICRYPT_READ);
         if( !poHandle->Init(osKey, false) )
         {
-            memset((void*)osKey.c_str(), 0, osKey.size());
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
             delete poHandle;
-            poHandle = NULL;
+            poHandle = nullptr;
         }
-        memset((void*)osKey.c_str(), 0, osKey.size());
+        memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
         return poHandle;
     }
     else if( strchr(pszAccess, 'w' ) )
@@ -1625,13 +1628,13 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
 
         /* Generate random initial vector */
         CryptoPP::BlockCipher* poBlock = GetEncBlockCipher(eAlg);
-        if( poBlock == NULL )
+        if( poBlock == nullptr )
         {
             CPLError(CE_Failure, CPLE_AppDefined,
                      "Cipher algorithm not supported in this build: %s",
                      osAlg.c_str());
-            memset((void*)osKey.c_str(), 0, osKey.size());
-            return NULL;
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
+            return nullptr;
         }
         int nMinKeySize = static_cast<int>(poBlock->MinKeyLength());
         int nMaxKeySize = static_cast<int>(poBlock->MaxKeyLength());
@@ -1646,7 +1649,7 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
                          "IV should be %d byte large",
                          nBlockSize);
                 memset(const_cast<char *>(osKey.c_str()), 0, osKey.size());
-                return NULL;
+                return nullptr;
             }
         }
         else
@@ -1654,7 +1657,7 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
             osIV.resize(nBlockSize);
             CryptoPP::OS_GenerateRandomBlock(
                 false,  // Do not need cryptographic randomness.
-                (byte*)osIV.c_str(), osIV.size());
+                reinterpret_cast<cryptopp_byte*>(const_cast<char*>(osIV.c_str())), osIV.size());
         }
 
         if( EQUAL(osKey, "GENERATE_IT") )
@@ -1667,10 +1670,10 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
                 // Config option for speeding tests.
                 CPLTestBool(CPLGetConfigOption("VSICRYPT_CRYPTO_RANDOM",
                                                "TRUE")),
-                (byte*)osKey.c_str(), osKey.size());
+                reinterpret_cast<cryptopp_byte*>(const_cast<char*>(osKey.c_str())), osKey.size());
 
             char* pszB64 = CPLBase64Encode(static_cast<int>(osKey.size()),
-                                           (const GByte*)osKey.c_str());
+                                           reinterpret_cast<const GByte*>(osKey.c_str()));
             if( CPLTestBool(CPLGetConfigOption("VSICRYPT_DISPLAY_GENERATED_KEY",
                                                "TRUE")) )
             {
@@ -1689,16 +1692,16 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
             CPLError(CE_Failure, CPLE_AppDefined,
                      "Key is too short: %d bytes. Should be at least %d bytes",
                      nKeyLength, nMinKeySize);
-            memset((void*)osKey.c_str(), 0, osKey.size());
-            return NULL;
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
+            return nullptr;
         }
 
         VSIVirtualHandle* fpBase =
             reinterpret_cast<VSIVirtualHandle *>(VSIFOpenL(osFilename, "wb+"));
-        if( fpBase == NULL )
+        if( fpBase == nullptr )
         {
-            memset((void*)osKey.c_str(), 0, osKey.size());
-            return NULL;
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
+            return nullptr;
         }
 
         VSICryptFileHeader* poHeader = new VSICryptFileHeader();
@@ -1717,30 +1720,30 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
                 : VSICRYPT_WRITE);
         if( !poHandle->Init(osKey, true) )
         {
-            memset((void*)osKey.c_str(), 0, osKey.size());
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
             delete poHandle;
-            poHandle = NULL;
+            poHandle = nullptr;
         }
-        memset((void*)osKey.c_str(), 0, osKey.size());
+        memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
         return poHandle;
     }
     else if( strchr(pszAccess, 'a') )
     {
         VSIVirtualHandle* fpBase =
             reinterpret_cast<VSIVirtualHandle *>(VSIFOpenL(osFilename, "rb+"));
-        if( fpBase == NULL )
+        if( fpBase == nullptr )
         {
-            memset((void*)osKey.c_str(), 0, osKey.size());
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
             return VSIFilesystemHandler::Open(pszFilename, "wb+");
         }
         VSICryptFileHeader* poHeader = new VSICryptFileHeader();
         if( !poHeader->ReadFromFile(fpBase, osKey) )
         {
-            memset((void*)osKey.c_str(), 0, osKey.size());
+            memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
             fpBase->Close();
             delete fpBase;
             delete poHeader;
-            return NULL;
+            return nullptr;
         }
 
         VSICryptFileHandle* poHandle =
@@ -1749,15 +1752,15 @@ VSIVirtualHandle *VSICryptFilesystemHandler::Open( const char *pszFilename,
         if( !poHandle->Init(osKey) )
         {
             delete poHandle;
-            poHandle = NULL;
+            poHandle = nullptr;
         }
-        memset((void*)osKey.c_str(), 0, osKey.size());
-        if( poHandle != NULL )
+        memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
+        if( poHandle != nullptr )
             poHandle->Seek(0, SEEK_END);
         return poHandle;
     }
 
-    return NULL;
+    return nullptr;
 }
 
 /************************************************************************/
@@ -1773,20 +1776,20 @@ int VSICryptFilesystemHandler::Stat( const char *pszFilename,
     CPLString osFilename(GetFilename(pszFilename));
     if( VSIStatExL( osFilename, pStatBuf, nFlags ) != 0 )
         return -1;
-    VSIVirtualHandle* fp = (VSIVirtualHandle*)VSIFOpenL(osFilename, "rb");
-    if( fp == NULL )
+    VSIVirtualHandle* fp = reinterpret_cast<VSIVirtualHandle*>(VSIFOpenL(osFilename, "rb"));
+    if( fp == nullptr )
         return -1;
     VSICryptFileHeader* poHeader = new VSICryptFileHeader();
     CPLString osKey(GetKey(pszFilename));
     if( !poHeader->ReadFromFile(fp, osKey) )
     {
-        memset((void*)osKey.c_str(), 0, osKey.size());
+        memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
         fp->Close();
         delete fp;
         delete poHeader;
         return -1;
     }
-    memset((void*)osKey.c_str(), 0, osKey.size());
+    memset(const_cast<char*>(osKey.c_str()), 0, osKey.size());
     fp->Close();
     delete fp;
     if( poHeader )
@@ -1860,8 +1863,8 @@ static int VSICryptIdentify(GDALOpenInfo* poOpenInfo)
 static GDALDataset* VSICryptOpen(GDALOpenInfo* poOpenInfo)
 {
     if( !VSICryptIdentify(poOpenInfo) )
-        return NULL;
-    return (GDALDataset*)GDALOpen(
+        return nullptr;
+    return GDALOpen(
         (CPLString(VSICRYPT_PREFIX) + poOpenInfo->pszFilename).c_str(),
         poOpenInfo->eAccess );
 }
@@ -2037,7 +2040,7 @@ void VSIInstallCryptFileHandler(void)
                                     new VSICryptFilesystemHandler );
 
 #ifdef VSICRYPT_DRIVER
-    if( GDALGetDriverByName( "VSICRYPT" ) != NULL )
+    if( GDALGetDriverByName( "VSICRYPT" ) != nullptr )
         return;
 
     GDALDriver *poDriver = new GDALDriver();
@@ -2058,10 +2061,6 @@ void VSIInstallCryptFileHandler(void)
 #endif
 }
 
-// #ifdef _MSC_VER
-// #pragma warning( pop ) /* 5105 */
-// #endif
-
 #else /* HAVE_CRYPTOPP */
 
 class VSIDummyCryptFilesystemHandler : public VSIFilesystemHandler
@@ -2069,17 +2068,17 @@ class VSIDummyCryptFilesystemHandler : public VSIFilesystemHandler
 public:
     VSIDummyCryptFilesystemHandler() {}
 
-    virtual VSIVirtualHandle *Open( const char * /* pszFilename */,
-                                    const char * /* pszAccess */,
-                                    bool /* bSetError */ ) override
+    VSIVirtualHandle *Open( const char * /* pszFilename */,
+                            const char * /* pszAccess */,
+                            bool /* bSetError */ ) override
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "%s support not available in this build", VSICRYPT_PREFIX);
-        return NULL;
+        return nullptr;
     }
 
-    virtual int Stat( const char * /* pszFilename */,
-                      VSIStatBufL * /*pStatBuf */, int /* nFlags */ ) override
+    int Stat( const char * /* pszFilename */,
+              VSIStatBufL * /*pStatBuf */, int /* nFlags */ ) override
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "%s support not available in this build", VSICRYPT_PREFIX);
