@@ -45,13 +45,15 @@
 #include "cpl_string.h"
 #include "cpl_vsi_virtual.h"
 
-CPL_CVSID("$Id: cpl_vsil_tar.cpp 37001 2016-12-22 18:09:05Z goatbar $");
+CPL_CVSID("$Id: cpl_vsil_tar.cpp 04558cb519c0345b92efb598af3bb44c5af65cf5 2018-06-08 19:53:00 +0200 Even Rouault $")
 
-#if defined(DEBUG) && !defined(HAVE_FUZZER_FRIENDLY_ARCHIVE)
+#if (defined(DEBUG) || defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)) && !defined(HAVE_FUZZER_FRIENDLY_ARCHIVE)
 /* This is a completely custom archive format that is rather inefficient */
 /* but supports random insertions or deletions, since it doesn't record */
 /* explicit file size or rely on files starting on a particular boundary */
 #define HAVE_FUZZER_FRIENDLY_ARCHIVE 1
+constexpr int HALF_BUFFER_SIZE = 1024;
+constexpr int BUFFER_SIZE = 2 * HALF_BUFFER_SIZE;
 #endif
 
 /************************************************************************/
@@ -60,7 +62,7 @@ CPL_CVSID("$Id: cpl_vsil_tar.cpp 37001 2016-12-22 18:09:05Z goatbar $");
 /* ==================================================================== */
 /************************************************************************/
 
-class VSITarEntryFileOffset CPL_FINAL : public VSIArchiveEntryFileOffset
+class VSITarEntryFileOffset final : public VSIArchiveEntryFileOffset
 {
 public:
         GUIntBig m_nOffset;
@@ -93,7 +95,7 @@ public:
 /* ==================================================================== */
 /************************************************************************/
 
-class VSITarReader CPL_FINAL : public VSIArchiveReader
+class VSITarReader final : public VSIArchiveReader
 {
     private:
         VSILFILE* fp;
@@ -103,25 +105,25 @@ class VSITarReader CPL_FINAL : public VSIArchiveReader
         GIntBig nModifiedTime;
 #ifdef HAVE_FUZZER_FRIENDLY_ARCHIVE
         bool m_bIsFuzzerFriendly;
-        GByte m_abyBuffer[2048];
+        GByte m_abyBuffer[BUFFER_SIZE+1] = {};
         int m_abyBufferIdx;
         int m_abyBufferSize;
         GUIntBig m_nCurOffsetOld;
 #endif
 
-    public:
-        explicit VSITarReader(const char* pszTarFileName);
-        virtual ~VSITarReader();
+  public:
+    explicit VSITarReader(const char* pszTarFileName);
+    ~VSITarReader() override;
 
-        int IsValid() { return fp != NULL; }
+    int IsValid() { return fp != nullptr; }
 
-        virtual int GotoFirstFile() override;
-        virtual int GotoNextFile() override;
-        virtual VSIArchiveEntryFileOffset* GetFileOffset() override;
-        virtual GUIntBig GetFileSize() override { return nNextFileSize; }
-        virtual CPLString GetFileName() override { return osNextFileName; }
-        virtual GIntBig GetModifiedTime() override { return nModifiedTime; }
-        virtual int GotoFileOffset(VSIArchiveEntryFileOffset* pOffset) override;
+    int GotoFirstFile() override;
+    int GotoNextFile() override;
+    VSIArchiveEntryFileOffset* GetFileOffset() override;
+    GUIntBig GetFileSize() override { return nNextFileSize; }
+    CPLString GetFileName() override { return osNextFileName; }
+    GIntBig GetModifiedTime() override { return nModifiedTime; }
+    int GotoFileOffset(VSIArchiveEntryFileOffset* pOffset) override;
 };
 
 /************************************************************************/
@@ -152,11 +154,10 @@ VSITarReader::VSITarReader(const char* pszTarFileName) :
     fp = VSIFOpenL(pszTarFileName, "rb");
 #ifdef HAVE_FUZZER_FRIENDLY_ARCHIVE
     m_bIsFuzzerFriendly = false;
-    m_abyBuffer[0] = '\0';
     m_abyBufferIdx = 0;
     m_abyBufferSize = 0;
     m_nCurOffsetOld = 0;
-    if( fp != NULL )
+    if( fp != nullptr )
     {
         GByte abySignature[24] = {};
         m_bIsFuzzerFriendly =
@@ -195,6 +196,38 @@ VSIArchiveEntryFileOffset* VSITarReader::GetFileOffset()
     return new VSITarEntryFileOffset(nCurOffset);
 }
 
+#ifdef HAVE_FUZZER_FRIENDLY_ARCHIVE
+
+/************************************************************************/
+/*                           CPLmemmem()                                */
+/************************************************************************/
+
+static void* CPLmemmem(const void *haystack, size_t haystacklen,
+                       const void *needle, size_t needlelen)
+{
+    const char* pachHaystack = reinterpret_cast<const char*>(haystack);
+    if( haystacklen < needlelen )
+        return nullptr;
+    while( true )
+    {
+        const char* pachSubstrStart = reinterpret_cast<const char*>(
+            memchr( pachHaystack,
+                reinterpret_cast<const char*>(needle)[0], haystacklen ));
+        if( pachSubstrStart == nullptr )
+            return nullptr;
+        if( static_cast<size_t>(pachSubstrStart - pachHaystack)
+                                                + needlelen > haystacklen )
+            return nullptr;
+        if( memcmp( pachSubstrStart, needle, needlelen ) == 0 )
+        {
+            return const_cast<void*>(static_cast<const void*>(pachSubstrStart));
+        }
+        haystacklen -= pachSubstrStart - pachHaystack + 1;
+        pachHaystack = pachSubstrStart + 1;
+    }
+}
+#endif
+
 /************************************************************************/
 /*                           GotoNextFile()                             */
 /************************************************************************/
@@ -204,6 +237,8 @@ int VSITarReader::GotoNextFile()
 #ifdef HAVE_FUZZER_FRIENDLY_ARCHIVE
     if( m_bIsFuzzerFriendly )
     {
+        const int nNewFileMarkerSize =
+                                static_cast<int>(strlen("***NEWFILE***:"));
         while( true )
         {
             if( m_abyBufferIdx >= m_abyBufferSize )
@@ -211,19 +246,18 @@ int VSITarReader::GotoNextFile()
                 if( m_abyBufferSize == 0 )
                 {
                     m_abyBufferSize = static_cast<int>(
-                        VSIFReadL(m_abyBuffer, 1, 2048, fp));
+                        VSIFReadL(m_abyBuffer, 1, BUFFER_SIZE, fp));
                     if( m_abyBufferSize == 0 )
                         return FALSE;
+                    m_abyBuffer[m_abyBufferSize] = '\0';
                 }
                 else
                 {
-                    if( m_abyBufferSize < 2048 )
+                    if( m_abyBufferSize < BUFFER_SIZE )
                     {
                         if( nCurOffset > 0 && nCurOffset != m_nCurOffsetOld )
                         {
                             nNextFileSize = VSIFTellL(fp);
-                            nNextFileSize -= m_abyBufferSize;
-                            nNextFileSize += m_abyBufferIdx;
                             if( nNextFileSize >= nCurOffset )
                             {
                                 nNextFileSize -= nCurOffset;
@@ -233,84 +267,98 @@ int VSITarReader::GotoNextFile()
                         }
                         return FALSE;
                     }
-                    memcpy(m_abyBuffer, m_abyBuffer + 1024, 1024);
+                    memcpy(m_abyBuffer, m_abyBuffer + HALF_BUFFER_SIZE,
+                           HALF_BUFFER_SIZE);
                     m_abyBufferSize = static_cast<int>(
-                         VSIFReadL(m_abyBuffer + 1024, 1, 1024, fp));
+                         VSIFReadL(m_abyBuffer + HALF_BUFFER_SIZE,
+                                   1, HALF_BUFFER_SIZE, fp));
                     if( m_abyBufferSize == 0 )
                         return FALSE;
                     m_abyBufferIdx = 0;
-                    m_abyBufferSize += 1024;
+                    m_abyBufferSize += HALF_BUFFER_SIZE;
+                    m_abyBuffer[m_abyBufferSize] = '\0';
                 }
             }
-            if( ((m_abyBufferSize == 2048 &&
-                  m_abyBufferIdx <
-                  m_abyBufferSize -
-                  (static_cast<int>(strlen("***NEWFILE***:")) + 64)) ||
-                 (m_abyBufferSize < 2048 &&
-                  m_abyBufferIdx < m_abyBufferSize -
-                  (static_cast<int>(strlen("***NEWFILE***:"))+2))) &&
-                m_abyBufferIdx >= 0 &&  // Make CSA happy, but useless.
-                memcmp(m_abyBuffer + m_abyBufferIdx,
-                       "***NEWFILE***:",
-                       strlen("***NEWFILE***:")) == 0 )
+
+            void* pNewFileMarker =
+                CPLmemmem( m_abyBuffer + m_abyBufferIdx,
+                        m_abyBufferSize - m_abyBufferIdx,
+                        "***NEWFILE***:", nNewFileMarkerSize );
+            if( pNewFileMarker == nullptr )
             {
-                if( nCurOffset > 0 && nCurOffset != m_nCurOffsetOld )
-                {
-                    nNextFileSize = VSIFTellL(fp);
-                    nNextFileSize -= m_abyBufferSize;
-                    nNextFileSize += m_abyBufferIdx;
-                    if( nNextFileSize >= nCurOffset )
-                    {
-                        nNextFileSize -= nCurOffset;
-                        m_nCurOffsetOld = nCurOffset;
-                        return TRUE;
-                    }
-                }
-                m_abyBufferIdx += static_cast<int>(strlen("***NEWFILE***:"));
-                const int nFilenameStartIdx = m_abyBufferIdx;
-                for( ; m_abyBufferIdx < m_abyBufferSize &&
-                       m_abyBuffer[m_abyBufferIdx] != '\n';
-                     ++m_abyBufferIdx)
-                {
-                    // Do nothing.
-                }
-                if( m_abyBufferIdx < m_abyBufferSize )
-                {
-                    osNextFileName.assign(
-                        (const char*)(m_abyBuffer + nFilenameStartIdx),
-                        m_abyBufferIdx - nFilenameStartIdx);
-                    nCurOffset = VSIFTellL(fp);
-                    nCurOffset -= m_abyBufferSize;
-                    nCurOffset += m_abyBufferIdx + 1;
-                }
+                m_abyBufferIdx = m_abyBufferSize;
             }
             else
             {
-                m_abyBufferIdx++;
+                m_abyBufferIdx = static_cast<int>(
+                    static_cast<const GByte*>(pNewFileMarker) - m_abyBuffer);
+                // 2: space for at least one-char filename and '\n'
+                if( m_abyBufferIdx < m_abyBufferSize -(nNewFileMarkerSize+2) )
+                {
+                    if( nCurOffset > 0 && nCurOffset != m_nCurOffsetOld )
+                    {
+                        nNextFileSize = VSIFTellL(fp);
+                        nNextFileSize -= m_abyBufferSize;
+                        nNextFileSize += m_abyBufferIdx;
+                        if( nNextFileSize >= nCurOffset )
+                        {
+                            nNextFileSize -= nCurOffset;
+                            m_nCurOffsetOld = nCurOffset;
+                            return TRUE;
+                        }
+                    }
+                    m_abyBufferIdx += nNewFileMarkerSize;
+                    const int nFilenameStartIdx = m_abyBufferIdx;
+                    for( ; m_abyBufferIdx < m_abyBufferSize &&
+                        m_abyBuffer[m_abyBufferIdx] != '\n';
+                        ++m_abyBufferIdx)
+                    {
+                        // Do nothing.
+                    }
+                    if( m_abyBufferIdx < m_abyBufferSize )
+                    {
+                        osNextFileName.assign(
+                            reinterpret_cast<const char*>(m_abyBuffer + nFilenameStartIdx),
+                            m_abyBufferIdx - nFilenameStartIdx);
+                        nCurOffset = VSIFTellL(fp);
+                        nCurOffset -= m_abyBufferSize;
+                        nCurOffset += m_abyBufferIdx + 1;
+                    }
+                }
+                else
+                {
+                    m_abyBufferIdx = m_abyBufferSize;
+                }
             }
         }
     }
 #endif
-    char abyHeader[512] = {};
+    GByte abyHeader[512] = {};
     if (VSIFReadL(abyHeader, 512, 1, fp) != 1)
         return FALSE;
 
-    if (abyHeader[99] != '\0' ||
-        abyHeader[107] != '\0' ||
-        abyHeader[115] != '\0' ||
-        abyHeader[123] != '\0' ||
-        (abyHeader[135] != '\0' && abyHeader[135] != ' ') ||
-        (abyHeader[147] != '\0' && abyHeader[147] != ' '))
+    if (abyHeader[99] != '\0' || /* end of filename */
+        !(abyHeader[100] == 0x80 || abyHeader[107] == '\0') || /* start/end of filemode */
+        !(abyHeader[108] == 0x80 || abyHeader[115] == '\0') || /* start/end of owner ID */
+        !(abyHeader[116] == 0x80 || abyHeader[123] == '\0') || /* start/end of group ID */
+        (abyHeader[135] != '\0' && abyHeader[135] != ' ') || /* end of file size */
+        (abyHeader[147] != '\0' && abyHeader[147] != ' ')) /* end of mtime */
     {
         return FALSE;
     }
     if( abyHeader[124] < '0' || abyHeader[124] > '7' )
         return FALSE;
 
-    osNextFileName = abyHeader;
+    osNextFileName = reinterpret_cast<const char*>(abyHeader);
     nNextFileSize = 0;
     for(int i=0;i<11;i++)
         nNextFileSize = nNextFileSize * 8 + (abyHeader[124+i] - '0');
+    if( nNextFileSize > GINTBIG_MAX )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Invalid file size for %s", osNextFileName.c_str());
+        return FALSE;
+    }
 
     nModifiedTime = 0;
     for(int i=0;i<11;i++)
@@ -383,16 +431,16 @@ int VSITarReader::GotoFileOffset( VSIArchiveEntryFileOffset* pOffset )
 /* ==================================================================== */
 /************************************************************************/
 
-class VSITarFilesystemHandler CPL_FINAL : public VSIArchiveFilesystemHandler
+class VSITarFilesystemHandler final : public VSIArchiveFilesystemHandler
 {
 public:
-    virtual const char* GetPrefix() override { return "/vsitar"; }
-    virtual std::vector<CPLString> GetExtensions() override;
-    virtual VSIArchiveReader* CreateReader(const char* pszTarFileName) override;
+    const char* GetPrefix() override { return "/vsitar"; }
+    std::vector<CPLString> GetExtensions() override;
+    VSIArchiveReader* CreateReader(const char* pszTarFileName) override;
 
-    virtual VSIVirtualHandle *Open( const char *pszFilename,
-                                    const char *pszAccess,
-                                    bool bSetError ) override;
+    VSIVirtualHandle *Open( const char *pszFilename,
+                            const char *pszAccess,
+                            bool bSetError ) override;
 };
 
 /************************************************************************/
@@ -429,13 +477,13 @@ VSIArchiveReader* VSITarFilesystemHandler::CreateReader(const char* pszTarFileNa
     if (!poReader->IsValid())
     {
         delete poReader;
-        return NULL;
+        return nullptr;
     }
 
     if (!poReader->GotoFirstFile())
     {
         delete poReader;
-        return NULL;
+        return nullptr;
     }
 
     return poReader;
@@ -450,25 +498,25 @@ VSIVirtualHandle* VSITarFilesystemHandler::Open( const char *pszFilename,
                                                  bool /* bSetError */ )
 {
 
-    if (strchr(pszAccess, 'w') != NULL ||
-        strchr(pszAccess, '+') != NULL)
+    if (strchr(pszAccess, 'w') != nullptr ||
+        strchr(pszAccess, '+') != nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "Only read-only mode is supported for /vsitar");
-        return NULL;
+        return nullptr;
     }
 
     CPLString osTarInFileName;
     char* tarFilename
         = SplitFilename(pszFilename, osTarInFileName, TRUE);
-    if (tarFilename == NULL)
-        return NULL;
+    if (tarFilename == nullptr)
+        return nullptr;
 
     VSIArchiveReader* poReader = OpenArchiveFile(tarFilename, osTarInFileName);
-    if (poReader == NULL)
+    if (poReader == nullptr)
     {
         CPLFree(tarFilename);
-        return NULL;
+        return nullptr;
     }
 
     CPLString osSubFileName("/vsisubfile/");
@@ -491,7 +539,7 @@ VSIVirtualHandle* VSITarFilesystemHandler::Open( const char *pszFilename,
     delete(poReader);
 
     CPLFree(tarFilename);
-    tarFilename = NULL;
+    tarFilename = nullptr;
 
     return reinterpret_cast<VSIVirtualHandle*>(VSIFOpenL(osSubFileName, "rb"));
 }
