@@ -29,7 +29,7 @@
 
 #include "cpl_vsi_virtual.h"
 
-CPL_CVSID("$Id: cpl_vsil_win32.cpp 37127 2017-01-12 19:07:33Z rouault $");
+CPL_CVSID("$Id: cpl_vsil_win32.cpp 0f654dda9faabf9d86a44293f0f89903a8e97dd7 2018-04-15 20:18:32 +0200 Even Rouault $")
 
 #if defined(WIN32)
 
@@ -55,7 +55,7 @@ CPL_CVSID("$Id: cpl_vsil_win32.cpp 37127 2017-01-12 19:07:33Z rouault $");
 #undef GetDiskFreeSpace
 #endif
 
-class VSIWin32FilesystemHandler CPL_FINAL : public VSIFilesystemHandler
+class VSIWin32FilesystemHandler final : public VSIFilesystemHandler
 {
 public:
     // TODO(schwehr): Fix Open call to remove the need for this using call.
@@ -82,7 +82,7 @@ public:
 /* ==================================================================== */
 /************************************************************************/
 
-class VSIWin32Handle CPL_FINAL : public VSIVirtualHandle
+class VSIWin32Handle final : public VSIVirtualHandle
 {
   public:
     HANDLE       hFile;
@@ -96,7 +96,7 @@ class VSIWin32Handle CPL_FINAL : public VSIVirtualHandle
     virtual int       Flush() override;
     virtual int       Close() override;
     virtual int       Truncate( vsi_l_offset nNewSize ) override;
-    virtual void     *GetNativeFileDescriptor() override { return (void*) hFile; }
+    virtual void     *GetNativeFileDescriptor() override { return static_cast<void*>(hFile); }
     virtual VSIRangeStatus GetRangeStatus( vsi_l_offset nOffset,
                                            vsi_l_offset nLength ) override;
 };
@@ -143,6 +143,7 @@ static int ErrnoFromGetLastError(DWORD dwError = 0)
     case ERROR_DRIVE_LOCKED:        /* The disk is in use or locked by another process. */
     case ERROR_LOCK_FAILED:         /* Unable to lock a region of a file. */
     case ERROR_SEEK_ON_DEVICE:      /* The file pointer cannot be set on the specified device or file. */
+    case ERROR_SHARING_VIOLATION:   /* The process cannot access the file because it is being used by another process. */
         err = EACCES;
         break;
     case ERROR_INVALID_HANDLE:      /* The handle is invalid. */
@@ -204,7 +205,7 @@ int VSIWin32Handle::Close()
 int VSIWin32Handle::Seek( vsi_l_offset nOffset, int nWhence )
 
 {
-    GUInt32       dwMoveMethod, dwMoveHigh;
+    LONG       dwMoveMethod, dwMoveHigh;
     GUInt32       nMoveLow;
     LARGE_INTEGER li;
 
@@ -229,24 +230,24 @@ int VSIWin32Handle::Seek( vsi_l_offset nOffset, int nWhence )
     dwMoveHigh = li.HighPart;
 
     SetLastError( 0 );
-    SetFilePointer(hFile, (LONG) nMoveLow, (PLONG)&dwMoveHigh,
+    SetFilePointer(hFile, nMoveLow, &dwMoveHigh,
                        dwMoveMethod);
 
     if( GetLastError() != NO_ERROR )
     {
 #ifdef notdef
-        LPVOID      lpMsgBuf = NULL;
+        LPVOID      lpMsgBuf = nullptr;
 
         FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER
                        | FORMAT_MESSAGE_FROM_SYSTEM
                        | FORMAT_MESSAGE_IGNORE_INSERTS,
-                       NULL, GetLastError(),
+                       nullptr, GetLastError(),
                        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-                       (LPTSTR) &lpMsgBuf, 0, NULL );
+                       (LPTSTR) &lpMsgBuf, 0, nullptr );
 
         printf( "[ERROR %d]\n %s\n", GetLastError(), (char *) lpMsgBuf );/*ok*/
         printf( "nOffset=%u, nMoveLow=%u, dwMoveHigh=%u\n",/*ok*/
-                (GUInt32) nOffset, nMoveLow, dwMoveHigh );
+                static_cast<GUInt32>(nOffset), nMoveLow, dwMoveHigh );
 #endif
         errno = ErrnoFromGetLastError();
         return -1;
@@ -265,7 +266,7 @@ vsi_l_offset VSIWin32Handle::Tell()
     LARGE_INTEGER   li;
 
     li.HighPart = 0;
-    li.LowPart = SetFilePointer( hFile, 0, (PLONG) &(li.HighPart),
+    li.LowPart = SetFilePointer( hFile, 0, &(li.HighPart),
                                  FILE_CURRENT );
 
     return (static_cast<vsi_l_offset>(li.QuadPart));
@@ -298,7 +299,7 @@ size_t VSIWin32Handle::Read( void * pBuffer, size_t nSize, size_t nCount )
     DWORD dwSizeRead = 0;
     size_t nResult = 0;
 
-    if( !ReadFile( hFile, pBuffer, (DWORD)(nSize*nCount), &dwSizeRead, NULL ) )
+    if( !ReadFile( hFile, pBuffer, static_cast<DWORD>(nSize*nCount), &dwSizeRead, nullptr ) )
     {
         nResult = 0;
         errno = ErrnoFromGetLastError();
@@ -324,8 +325,8 @@ size_t VSIWin32Handle::Write( const void *pBuffer, size_t nSize, size_t nCount)
     DWORD dwSizeWritten = 0;
     size_t nResult = 0;
 
-    if( !WriteFile(hFile, (void *)pBuffer,
-                   (DWORD)(nSize*nCount),&dwSizeWritten,NULL) )
+    if( !WriteFile(hFile, pBuffer,
+                   static_cast<DWORD>(nSize*nCount),&dwSizeWritten,nullptr) )
     {
         nResult = 0;
         errno = ErrnoFromGetLastError();
@@ -360,7 +361,7 @@ int VSIWin32Handle::Truncate( vsi_l_offset nNewSize )
     {
         // Enable sparse files if growing size
         DWORD dwTemp;
-        DeviceIoControl(hFile, FSCTL_SET_SPARSE, NULL, 0, NULL, 0, &dwTemp, NULL);
+        DeviceIoControl(hFile, FSCTL_SET_SPARSE, nullptr, 0, nullptr, 0, &dwTemp, nullptr);
     }
     Seek( nNewSize, SEEK_SET );
     BOOL bRes = SetEndOfFile( hFile );
@@ -398,7 +399,7 @@ VSIRangeStatus VSIWin32Handle::GetRangeStatus( vsi_l_offset
     if( !DeviceIoControl(hFile, FSCTL_QUERY_ALLOCATED_RANGES,
                         &sQueryRange, sizeof(sQueryRange),
                         asOutputRange, sizeof(asOutputRange),
-                        &nOutputBytes, NULL) )
+                        &nOutputBytes, nullptr) )
     {
         if( GetLastError() == ERROR_MORE_DATA )
         {
@@ -432,13 +433,13 @@ VSIRangeStatus VSIWin32Handle::GetRangeStatus( vsi_l_offset
 static const char* CPLGetWineVersion()
 {
     HMODULE hntdll = GetModuleHandle("ntdll.dll");
-    if( hntdll == NULL )
-        return NULL;
+    if( hntdll == nullptr )
+        return nullptr;
 
     static const char * (CDECL *pwine_get_version)(void);
-    pwine_get_version = ( const char* (*)(void) ) GetProcAddress(hntdll, "wine_get_version");
-    if( pwine_get_version == NULL )
-        return NULL;
+    pwine_get_version = reinterpret_cast<const char* (*)(void)>(GetProcAddress(hntdll, "wine_get_version"));
+    if( pwine_get_version == nullptr )
+        return nullptr;
 
     return pwine_get_version();
 }
@@ -549,13 +550,13 @@ VSIVirtualHandle *VSIWin32FilesystemHandler::Open( const char *pszFilename,
 
     // GENERICs are used instead of FILE_GENERIC_READ.
     dwDesiredAccess = GENERIC_READ;
-    if (strchr(pszAccess, '+') != NULL || strchr(pszAccess, 'w') != NULL)
+    if (strchr(pszAccess, '+') != nullptr || strchr(pszAccess, 'w') != nullptr)
         dwDesiredAccess |= GENERIC_WRITE;
 
     // Append mode only makes sense on files and pipes, have to use FILE_ access
     // these are very different from the GENERICs
     // Append is read and write but not overwrite data (only append data)
-    if (strchr(pszAccess, 'a') != NULL )
+    if (strchr(pszAccess, 'a') != nullptr )
     {
         dwDesiredAccess =
             FILE_GENERIC_READ | (FILE_GENERIC_WRITE ^ FILE_WRITE_DATA);
@@ -565,7 +566,7 @@ VSIVirtualHandle *VSIWin32FilesystemHandler::Open( const char *pszFilename,
         // resulting semantics isn't completely conformant.
         // See https://bugs.winehq.org/show_bug.cgi?id=33232
         const char* pszWineVersion = CPLGetWineVersion();
-        if( pszWineVersion != NULL )
+        if( pszWineVersion != nullptr )
         {
             int nVersion = atoi(pszWineVersion) * 10000;
             const char* pszDot = strchr(pszWineVersion, '.');
@@ -590,9 +591,9 @@ VSIVirtualHandle *VSIWin32FilesystemHandler::Open( const char *pszFilename,
         }
     }
 
-    if( strstr(pszAccess, "w") != NULL )
+    if( strstr(pszAccess, "w") != nullptr )
         dwCreationDisposition = CREATE_ALWAYS;
-    else if( strstr(pszAccess, "a") != NULL )
+    else if( strstr(pszAccess, "a") != nullptr )
         dwCreationDisposition = OPEN_ALWAYS;
     else
         dwCreationDisposition = OPEN_EXISTING;
@@ -605,15 +606,16 @@ VSIVirtualHandle *VSIWin32FilesystemHandler::Open( const char *pszFilename,
 /*      converting to wide characters to open.                          */
 /* -------------------------------------------------------------------- */
     DWORD nLastError = 0;
+    bool bShared = CPLTestBool(CPLGetConfigOption( "GDAL_SHARED_FILE", "YES" ) );
     if( CPLTestBool(CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
         wchar_t *pwszFilename =
             CPLRecodeToWChar( pszFilename, CPL_ENC_UTF8, CPL_ENC_UCS2 );
 
         hFile = CreateFileW( pwszFilename, dwDesiredAccess,
-                            FILE_SHARE_READ | FILE_SHARE_WRITE,
-                            NULL, dwCreationDisposition,  dwFlagsAndAttributes,
-                            NULL );
+                            bShared ? FILE_SHARE_READ | FILE_SHARE_WRITE : 0,
+                            nullptr, dwCreationDisposition,  dwFlagsAndAttributes,
+                            nullptr );
         if ( hFile == INVALID_HANDLE_VALUE &&
             !VSIWin32IsLongFilename(pwszFilename) )
         {
@@ -628,6 +630,7 @@ VSIVirtualHandle *VSIWin32FilesystemHandler::Open( const char *pszFilename,
                     case ERROR_BAD_PATHNAME:        CPLDebug("VSI", "ERROR_BAD_PATHNAME"); break;
                     case ERROR_BAD_NETPATH:         CPLDebug("VSI", "ERROR_BAD_NETPATH"); break;
                     case ERROR_FILENAME_EXCED_RANGE: CPLDebug("VSI", "ERROR_FILENAME_EXCED_RANGE"); break;
+                    case ERROR_SHARING_VIOLATION:   CPLDebug("VSI", "ERROR_SHARING_VIOLATION"); break;
                     default:  CPLDebug("VSI", "other error %d", nLastError); break;
             }
 #endif
@@ -638,29 +641,32 @@ VSIVirtualHandle *VSIWin32FilesystemHandler::Open( const char *pszFilename,
             VSIWin32TryLongFilename(pwszFilename);
             nLastError = 0;
             hFile = CreateFileW( pwszFilename, dwDesiredAccess,
-                            FILE_SHARE_READ | FILE_SHARE_WRITE,
-                            NULL, dwCreationDisposition,  dwFlagsAndAttributes,
-                            NULL );
+                            bShared ? FILE_SHARE_READ | FILE_SHARE_WRITE : 0,
+                            nullptr, dwCreationDisposition,  dwFlagsAndAttributes,
+                            nullptr );
         }
         CPLFree( pwszFilename );
     }
     else
     {
         hFile = CreateFile( pszFilename, dwDesiredAccess,
-                            FILE_SHARE_READ | FILE_SHARE_WRITE,
-                            NULL, dwCreationDisposition,  dwFlagsAndAttributes,
-                            NULL );
+                            bShared ? FILE_SHARE_READ | FILE_SHARE_WRITE : 0,
+                            nullptr, dwCreationDisposition,  dwFlagsAndAttributes,
+                            nullptr );
     }
 
     if( hFile == INVALID_HANDLE_VALUE )
     {
+        nLastError = GetLastError();
         const int nError = ErrnoFromGetLastError(nLastError);
         if( bSetError && nError != 0 )
         {
-            VSIError(VSIE_FileError, "%s: %s", pszFilename, strerror(nError));
+            VSIError(VSIE_FileError, "%s: %s", pszFilename,
+                     (nLastError == ERROR_SHARING_VIOLATION) ?
+                        "file used by other process": strerror(nError));
         }
         errno = nError;
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -671,7 +677,7 @@ VSIVirtualHandle *VSIWin32FilesystemHandler::Open( const char *pszFilename,
     poHandle->hFile = hFile;
     poHandle->bEOF = FALSE;
 
-    if (strchr(pszAccess, 'a') != 0)
+    if (strchr(pszAccess, 'a') != nullptr)
         poHandle->Seek(0, SEEK_END);
 
 /* -------------------------------------------------------------------- */
@@ -700,7 +706,7 @@ int VSIWin32FilesystemHandler::Stat( const char * pszFilename,
 {
     (void) nFlags;
 
-#if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
+#if defined(_MSC_VER) || __MSVCRT_VERSION__ >= 0x0601
     if( CPLTestBool( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
         wchar_t *pwszFilename =
@@ -735,7 +741,7 @@ int VSIWin32FilesystemHandler::Stat( const char * pszFilename,
             // nResult = _wstat64( pwszFilename, pStatBuf );
 
             VSIVirtualHandle* poHandle = Open( pszFilename, "rb");
-            if( poHandle != NULL )
+            if( poHandle != nullptr )
             {
                 nResult = 0;
                 memset( pStatBuf, 0, sizeof(VSIStatBufL) );
@@ -768,7 +774,7 @@ int VSIWin32FilesystemHandler::Stat( const char * pszFilename,
 int VSIWin32FilesystemHandler::Unlink( const char * pszFilename )
 
 {
-#if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
+#if defined(_MSC_VER) || __MSVCRT_VERSION__ >= 0x0601
     if( CPLTestBool( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
         wchar_t *pwszFilename =
@@ -793,7 +799,7 @@ int VSIWin32FilesystemHandler::Rename( const char *oldpath,
                                            const char *newpath )
 
 {
-#if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
+#if defined(_MSC_VER) || __MSVCRT_VERSION__ >= 0x0601
     if( CPLTestBool( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
         wchar_t *pwszOldPath =
@@ -822,7 +828,7 @@ int VSIWin32FilesystemHandler::Mkdir( const char * pszPathname,
 
 {
     (void) nMode;
-#if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
+#if defined(_MSC_VER) || __MSVCRT_VERSION__ >= 0x0601
     if( CPLTestBool( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
         wchar_t *pwszFilename =
@@ -846,7 +852,7 @@ int VSIWin32FilesystemHandler::Mkdir( const char * pszPathname,
 int VSIWin32FilesystemHandler::Rmdir( const char * pszPathname )
 
 {
-#if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
+#if defined(_MSC_VER) || __MSVCRT_VERSION__ >= 0x0601
     if( CPLTestBool( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
         wchar_t *pwszFilename =
@@ -871,7 +877,7 @@ char **VSIWin32FilesystemHandler::ReadDirEx( const char *pszPath,
                                              int nMaxFiles )
 
 {
-#if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
+#if defined(_MSC_VER) || __MSVCRT_VERSION__ >= 0x0601
     if( CPLTestBool( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
         struct _wfinddata_t c_file;
@@ -955,7 +961,7 @@ GIntBig VSIWin32FilesystemHandler::GetDiskFreeSpace( const char* pszDirname )
 {
     GIntBig nRet = -1;
     ULARGE_INTEGER nFreeBytesAvailable;
-    if( GetDiskFreeSpaceEx(pszDirname, &nFreeBytesAvailable, NULL, NULL) )
+    if( GetDiskFreeSpaceEx(pszDirname, &nFreeBytesAvailable, nullptr, nullptr) )
     {
         nRet = static_cast<GIntBig>(nFreeBytesAvailable.QuadPart);
     }
@@ -982,8 +988,8 @@ int VSIWin32FilesystemHandler::SupportsSparseFiles( const char* pszPath )
         osPath.resize(3);
     }
 
-    GetVolumeInformation(osPath.c_str(), NULL, 0, NULL,
-                         NULL, &dwVolFlags, NULL, 0);
+    GetVolumeInformation(osPath.c_str(), nullptr, 0, nullptr,
+                         nullptr, &dwVolFlags, nullptr, 0);
     return (dwVolFlags & FILE_SUPPORTS_SPARSE_FILES);
 }
 
