@@ -9,10 +9,6 @@ rm -rf org_patched
 mkdir org_patched
 cp -r org org_patched
 
-# Remove org_patched/org/gdal/gdal/AsyncReader.java as the Java bindings
-# is not yet ready
-rm org_patched/org/gdal/gdal/AsyncReader.java
-
 echo "Patching .java files with Javadoc from javadoc.java"
 gcc -g -Wall add_javadoc.c -o add_javadoc
 ./add_javadoc javadoc.java org_patched `find org -name "*.java" -not -name "AsyncReader.java"`
